﻿// Bitground Software common helpers library
// http://www.bitground.com/
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Security.Cryptography;

namespace Bitground.Common.Helpers
{
    internal abstract class AES256
    {
        private static bool IsDataEqual(byte[] ar1, int idx1, byte[] ar2, int idx2, int n)
        {
            if ((null == ar1) ||
                (null == ar2) ||
                (idx1 < 0) ||
                (idx2 < 0) ||
                (ar1.Length < idx1 + n) ||
                (ar2.Length < idx2 + n))
            {
                return false;
            }

            for (int i = 0; i < n; i++)
            {
                if (ar1[idx1 + i] != ar2[idx2 + i])
                {
                    return false;
                }
            }

            return true;
        }

        private static byte[] BlockConcat(byte[] ar1, byte[] ar2)
        {
            if (null == ar1)
            {
                ar1 = new byte[0];
            }
            if (null == ar2)
            {
                ar1 = new byte[0];
            }

            byte[] ar = new byte[ar1.Length + ar2.Length];

            Buffer.BlockCopy(ar1, 0, ar, 0, ar1.Length);
            Buffer.BlockCopy(ar2, 0, ar, ar1.Length, ar2.Length);

            return ar;
        }

        private static void ComputeRijndaelInput(
            byte[] key,
            byte[] salt,
            out byte[] aesKey,
            out byte[] aesIV)
        {
            MD5 md5 = MD5.Create();

            int preKeyLength = key.Length + salt.Length;
            byte[] preKey = new byte[preKeyLength];
            Buffer.BlockCopy(key, 0, preKey, 0, key.Length);
            Buffer.BlockCopy(salt, 0, preKey, key.Length, salt.Length);

            byte[] md5_hash_0 = md5.ComputeHash(preKey);
            byte[] md5_hash_1 = md5.ComputeHash(BlockConcat(md5_hash_0, preKey));
            byte[] md5_hash_2 = md5.ComputeHash(BlockConcat(md5_hash_1, preKey));

            aesKey = BlockConcat(md5_hash_0, md5_hash_1);
            aesIV = md5_hash_2;

            md5.Clear();
            md5 = null;
        }

        public static byte[] Encrypt(byte[] key, byte[] arData)
        {
            //byte[] salt = PseudoRandom.GetByteArray(8);
            //use TRNG
            byte[] salt = TrueRandom.GetByteArray(8);

            return EncryptCore(salt, key, arData);
        }

        internal static byte[] EncryptCore(byte[] salt, byte[] key, byte[] arData)
        {
            byte[] salted = Encoding.UTF8.GetBytes("Salted__");
            
            // Create Key and IV from password
            byte[] aesKey;
            byte[] aesIV;
            ComputeRijndaelInput(
                key,
                salt,
                out aesKey,
                out aesIV);

            // Encrypt the data
            byte[] encData;
            {
                Rijndael rijndael = new RijndaelManaged();
                rijndael.Mode = CipherMode.CBC;
                rijndael.Padding = PaddingMode.PKCS7;
                rijndael.KeySize = 256;
                rijndael.Key = aesKey;
                rijndael.IV = aesIV;

                ICryptoTransform rijndaelEncryptor = rijndael.CreateEncryptor();
                encData = rijndaelEncryptor.TransformFinalBlock(arData, 0, arData.Length);
            }

            byte[] enc = BlockConcat(BlockConcat(salted, salt), encData);

            return enc;
        }

        public static byte[] Decrypt(byte[] key, byte[] arData)
        {
            // Read in the data and check to see if it's salted
            bool isSalted = false;
            byte[] salt = null;
            {
                if (arData.Length > 16)
                {
                    byte[] salted = Encoding.UTF8.GetBytes("Salted__");
                    Debug.Assert(8 == salted.Length);

                    if (IsDataEqual(arData, 0, salted, 0, 8))
                    {
                        isSalted = true;

                        salt = new byte[8];
                        Buffer.BlockCopy(arData, 8, salt, 0, 8);
                    }
                }
            }

            // Remove salt from data if necessary
            byte[] aesData;
            {
                if (isSalted)
                {
                    int aesDataLength = arData.Length - 16;

                    aesData = new byte[aesDataLength];
                    Buffer.BlockCopy(arData, 16, aesData, 0, aesDataLength);
                }
                else
                {
                    salt = new byte[0];
                    aesData = arData;
                }
            }

            // Create Key and IV from password
            byte[] aesKey;
            byte[] aesIV;
            ComputeRijndaelInput(
                key,
                salt,
                out aesKey,
                out aesIV);

            // Decrypt the data
            byte[] decData;
            {
                Rijndael rijndael = new RijndaelManaged();
                rijndael.Mode = CipherMode.CBC;
                rijndael.Padding = PaddingMode.PKCS7;
                rijndael.KeySize = 256;
                rijndael.Key = aesKey;
                rijndael.IV = aesIV;

                ICryptoTransform rijndaelDecryptor = rijndael.CreateDecryptor();
                decData = rijndaelDecryptor.TransformFinalBlock(aesData, 0, aesData.Length);
            }

            return decData;
        }
    }
}
