﻿// Bitground Software common helpers library
// http://www.bitground.com/
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Bitground.Common.Helpers
{
    public abstract partial class Crypto
    {
        public abstract class SRPClient
        {
            #region Utils

            private static string HashUser(string username)
            {
                string login = Crypto.Hash("!@#<32}|{_$+)EW:>fWS@@!=39dje%^#$RF']]ew3" + username + "shaSHA@");
                return login;
            }

            private static string HashPass(string password)
            {
                string pass = Crypto.Hash("\":}[|weFC@de';{{3$$  vdRF2w^5V|\\/.w32" + password + "sshHAA!%");
                return pass;
            }

            private static string CryptoKey(string pass)
            {
                if (128 != pass.Length)
                {
                    Log.Error( "Unexpected pass length '{0}'", pass.Length);
                    return pass;
                }

                string pnext = SRP.HHex(pass);
                string ptmp;
                for (int i = 0; i < 50; i++)
                {
                    ptmp = Crypto.HMACHash(pass, pnext);
                    pass = pnext;
                    pnext = ptmp;
                }
                pass = pnext;

                byte[] ar = Hex.HexStringToByteArray(pass);
                if (64 != ar.Length)
                {
                    Log.Error("Unexpected ar.Length '{0}'", ar.Length);
                    return pass;
                }

                byte[] ret = new byte[32];
                for (int i = 0; i < 32; i++)
                {
                    ret[i] = (byte)(((int)(ar[i])) ^ ((int)(ar[i + 32])));
                }

                string cryptoKey = Hex.ByteArrayToHexString(ret);

                return cryptoKey;
            }

            private static string DecryptData(string cryptoKey, string encData)
            {
                string data = Crypto.Decrypt(cryptoKey, encData);
                return data;
            }

            private static string EncryptData(string cryptoKey, string data)
            {
                string encData = Crypto.Encrypt(cryptoKey, data);
                return encData;
            }

            #endregion

            #region Users

            #region Check user

            public static void CheckUser(
                string username,
                out string login)
            {
                login = HashUser(username);
            }

            #endregion

            #region Add user

            public static void AddUser(
                string username,
                string password,
                out string login,
                out string sHex,
                out string vHex)
            {
                login = HashUser(username);
                
                string pass = HashPass(password);

                if (Log.CanLogTrace) Log.Trace("Salting and hashing the input");
                BigInteger s = new BigInteger();
                //s.genRandomBits(SRP._nbits, new Random((int)DateTime.Now.Ticks));
                //TRNG is used here
                s.genRandomBits(SRP._nbits, null);
                sHex = SRP.To128(s.ToHexString());

                if (Log.CanLogTrace) Log.Trace("Compute the private key");
                // x - a private key derived from the password and salt
                // x = H(s || H(username) || H(password))
                BigInteger x;
                {
                    BigInteger x1 = new BigInteger(SRP.HHex(
                        sHex +
                        login +
                        pass
                        ), 16);

                    x = (x1 < SRP.N) ? x1 : (x1 % SRP.Nminus1);
                }

                if (Log.CanLogTrace) Log.Trace("Generate the server's verifier");
                // v - the server's verifier
                // v = g^x (mod N)
                BigInteger v = SRP.g.modPow(x, SRP.N);

                vHex = SRP.To128(v.ToHexString());
            }

            #endregion

            #region Authenticate user

            public static void AuthStep1(
                bool dataIsHashed,
                string username,
                out string login,
                out string AHex,
                out BigInteger a)
            {
                login = "";
                if (dataIsHashed)
                {
                    login = username;
                }
                else
                {
                    login = HashUser(username);
                }

                if (Log.CanLogTrace) Log.Trace("Generate the ephemeral private key");
                a = new BigInteger();
                // a - ephemeral private key
                // a = random between 2 and N-1
                {
                    //a.genRandomBits(SRP._nbits, new Random((int)DateTime.Now.Ticks));
                    //TRNG is used here
                    a.genRandomBits(SRP._nbits, null);
                    a = (a >= SRP.N) ? (a % SRP.Nminus1) : a;
                    a = (a < 2) ? 2 : a;
                }

                if (Log.CanLogTrace) Log.Trace("Generate the public key");
                // A - public key
                // A = g^a (mod N)
                BigInteger A = SRP.g.modPow(a, SRP.N);
                AHex = SRP.To128(A.ToHexString());
            }

            public static void AuthStep2(
                bool dataIsHashed,
                string login,
                string password,
                string sHex,
                string BHex,
                string uHex,
                string AHex,
                BigInteger a,
                out string m1Hex,
                out string KHex,
                out string pass)
            {
                pass = "";
                if (dataIsHashed)
                {
                    pass = password;
                }
                else
                {
                    pass = HashPass(password);
                }

                BigInteger B = new BigInteger(BHex, 16);
                BigInteger u = new BigInteger(uHex, 16);

                if (Log.CanLogTrace) Log.Trace("Generate the private key derived from password and the salt");
                // x - private key derived from password and the salt
                // x = H(s || H(username) || H(P))
                BigInteger x;
                {
                    BigInteger x1 = new BigInteger(SRP.HHex(
                        sHex +
                        login +
                        pass), 16);

                    x = (x1 < SRP.N) ? x1 : (x1 % SRP.Nminus1);
                }

                if (Log.CanLogTrace) Log.Trace("Computing SRP common exponential value");
                // S - common exponential value
                // S = (B - k * g^x) ^ (a + u * x) (mod N)
                //var S = powMod(mod(sub(add(B, mult(this.N, this.k)), mult(powMod(this.g, x, this.N), this.k)), this.N), add(mult(x, u), a), this.N);

                BigInteger S = 
                    ((
                        (B + SRP.N * SRP.k)
                        -
                        (SRP.g.modPow(x, SRP.N) * SRP.k)
                    
                    ) % SRP.N).modPow(
                    
                    x * u + a, 
                    
                    SRP.N);

                /*
                var S = powMod(
                    mod(
                        sub(
                            add(B, mult(this.N, this.k)), 
                            mult(powMod(this.g, x, this.N), this.k)), 
                        
                        this.N), 
                    

                    add(mult(x, u), a), 
                    
                    this.N);


                SRP.k
                */

                if (Log.CanLogTrace) Log.Trace("Computing the strong cryptographically session key");
                // K - the strong cryptographically session key
                // K = H(S)
                //var K = this.hex2bigint(this.HHex(this.bigint2hex(S)));
                
                BigInteger K = new BigInteger(SRP.HHex(S.ToHexString()), 16);

                KHex = SRP.To128(K.ToHexString());



                if (Log.CanLogTrace) Log.Trace("Client signing the login session");
                // m1 - client's proof that it has the correct key
                // m1 = H(A, B, K)
                BigInteger m1 = new BigInteger(SRP.HHex(
                    AHex + 
                    BHex +
                    KHex), 16);
                m1Hex = SRP.To128(m1.ToHexString());
            }

            public static void AuthValidate(
                string AHex,
                string m1Hex,
                string m2HexServer,
                string KHex,
                string pass,
                out string cryptoKey)
            {
                if (Log.CanLogTrace) Log.Trace("Checking server's signature");
                // m2 - expected server's proof as computed by the client
                // m2 = H(A, m1, K)
                BigInteger m2 = new BigInteger(SRP.HHex(
                    AHex +
                    m1Hex +
                    KHex), 16);
                string m2Hex = SRP.To128(m2.ToHexString());

                if (m2Hex == m2HexServer)
                {
                    if (Log.CanLogTrace) Log.Trace("SRP authentication succeded. Computing the crypto key.");

                    cryptoKey = CryptoKey(pass);

                    if (Log.CanLogTrace) Log.Trace("The crypto key was computed. The secure session was established.");
                }
                else
                {
                    cryptoKey = null;
                }
            }

            #endregion

            #region Delete user

            public static void DeleteUser(
                string username,
                string password,
                out string login,
                out string pass)
            {
                login = HashUser(username);
                pass = HashPass(password);
            }

            #endregion

            #region Logout user

            public static void LogoutUser(
                )
            {
            }

            #endregion

            #region Update user

            public static void UpdateUserOld(
                string username,
                string oldPassword,
                out string login,
                out string oldpass)
            {
                login = HashUser(username);
                oldpass = HashPass(oldPassword);
            }

            public static void UpdateUserNew(
                string login,
                string newPassword,
                out string newpass,
                out string sHex,
                out string vHex)
            {
                newpass = HashPass(newPassword);

                if (Log.CanLogTrace) Log.Trace("Salting and hashing the input");
                BigInteger s = new BigInteger();
                //s.genRandomBits(SRP._nbits, new Random((int)DateTime.Now.Ticks));
                //TRNG is used here
                s.genRandomBits(SRP._nbits, null);
                sHex = SRP.To128(s.ToHexString());

                if (Log.CanLogTrace) Log.Trace("Compute the private key");
                // x - a private key derived from the password and salt
                // x = H(s || H(username) || H(password))
                BigInteger x;
                {
                    BigInteger x1 = new BigInteger(SRP.HHex(
                        sHex +
                        login +
                        newpass
                        ), 16);

                    x = (x1 < SRP.N) ? x1 : (x1 % SRP.Nminus1);
                }

                if (Log.CanLogTrace) Log.Trace("Generate the server's verifier");
                // v - the server's verifier
                // v = g^x (mod N)
                BigInteger v = SRP.g.modPow(x, SRP.N);

                vHex = SRP.To128(v.ToHexString());
            }

            #endregion

            #endregion

            #region Objects

            #region Add object

            public static void AddObject(
                string cryptoKey,
                string data,
                out string encData)
            {
                encData = EncryptData(cryptoKey, data);
            }

            #endregion

            #region Get object

            public static void GetObject(
                string cryptoKey,
                string encData,
                out string data)
            {
                data = DecryptData(cryptoKey, encData);
            }

            #endregion

            #region Add object

            public static void UpdateObject(
                string cryptoKey,
                string data,
                out string encData)
            {
                encData = EncryptData(cryptoKey, data);
            }

            #endregion

            #endregion
        }
    }
}
