﻿// Bitground Software common helpers library
// http://www.bitground.com/
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Bitground.Common.Helpers
{
    public static partial class Extensions
    {
        /// <summary>
        /// Parses a string into an Enum
        /// </summary>
        /// <typeparam name="T">The type of the Enum</typeparam>
        /// <param name="value">String value to parse</param>
        /// <returns>The Enum corresponding to the stringExtensions</returns>
        public static T ToEnum<T>(this string value)
        {
            return Extensions.ToEnum<T>(value, false);
        }

        public static T ToEnum<T>(this string value, bool ignorecase)
        {

            if (value == null)
            {
                throw new ArgumentNullException("value");
            }

            value = value.Trim();

            if (value.Length == 0)
            {
                throw new ArgumentException("Must specify valid information for parsing in the string.", "value");
            }

            Type t = typeof(T);

            if (!t.IsEnum)
            {
                throw new ArgumentException("Type provided must be an Enum.", "T");
            }

            return (T)Enum.Parse(t, value, ignorecase);
        }

        public static string SubstringFlexible(this string value, int startIndex, int length)
        {
            if (String.IsNullOrEmpty(value))
            {
                return null;
            }
            int len = value.Length;

            int startPos = startIndex;
            int endPos = startIndex + length - 1;

            int i1 = Math.Max(0, startPos);
            int i2 = Math.Min(len - 1, endPos);

            if (i2 < i1)
            {
                return "";
            }

            return value.Substring(i1, i2 - i1 + 1);
        }
    }
}
