﻿// Bitground Software common helpers library
// http://www.bitground.com/
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace Bitground.Common.Helpers
{
    public static partial class Extensions
    {
        public static UInt32 FromBytes(this UInt32 i, byte b1, byte b2, byte b3, byte b4)
        {
            return
                (((UInt32)b1) << 24) |
                (((UInt32)b2) << 16) |
                (((UInt32)b3) << 8) |
                (((UInt32)b4) << 0);
        }

        public static UInt32 FromUnsignedShort(this UInt32 i, UInt16 u1, UInt16 u2)
        {
            return
                (((UInt32)u1) << 16) |
                (((UInt32)u2) << 0);
        }

        public static byte GetByte(this UInt32 i, int pos)
        {
            Debug.Assert((0 <= pos) && (pos <= 3));

            return
                (byte)((((UInt32)i) >> (
                    (0 == pos) ? 0 :
                        (1 == pos) ? 8 :
                            (2 == pos) ? 16 :
                                24
                )) & 255);
        }

        public static string ToBinString(this UInt32 i)
        {
            // 1 followed by 31 zeroes
            const UInt32 mui = (((UInt32)1) << 31);

            StringBuilder sb = new StringBuilder();
            for (int k = 0; k < 32; k++)
            {
                if (0 == (i & mui))
                {
                    sb.Append("0");
                }
                else
                {
                    sb.Append("1");
                }
                i <<= 1;
            }
            return sb.ToString();
        }
    }

}
