﻿// Bitground Software common helpers library
// http://www.bitground.com/
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Bitground.Common.Helpers
{
    public abstract class Misc
    {
        private static string m_appIdentifier = "";
        public static string AppIdentifier
        {
            get
            {
                if (String.IsNullOrEmpty(m_appIdentifier))
                {
                    lock (m_appIdentifier)
                    {
                        if (String.IsNullOrEmpty(m_appIdentifier))
                        {
                            try
                            {
                                string machineName = System.Environment.MachineName;
                                string processId = Strings.Padding(
                                    System.Diagnostics.Process.GetCurrentProcess().Id,
                                    6);
                                string threadId = Strings.Padding(
                                    System.Threading.Thread.CurrentThread.ManagedThreadId,
                                    4);
                                m_appIdentifier = String.Format(
                                    "{0}-{1}{2}",
                                    machineName,
                                    processId,
                                    threadId);
                            }
                            catch
                            {
                                m_appIdentifier = Strings.Padding(0, 6 + 4);
                            }
                        }
                    }
                }
                return m_appIdentifier;
            }
        }

        public static int ToInterval(double dblSeconds)
        {
            return (int)(dblSeconds * 1000);
        }

        private const double m_dblEpsilon = 0.00000001;
        public static bool IsEqualApproximate(double dbl1, double dbl2)
        {
            if ((dbl1 - m_dblEpsilon <= dbl2) && (dbl2 <= dbl1 + m_dblEpsilon))
            {
                return true;
            }
            return false;
        }

        public static string ArrayIDsToString(string[] arIDs)
        {
            return ((null == arIDs) || (arIDs.Length <= 0)) ?
                    null :
                    String.Join(",", arIDs.Select(
                        p => p.ToString()).ToArray());
        }
    }
}
