﻿// Bitground Software common helpers library
// http://www.bitground.com/
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.Diagnostics;

namespace Bitground.Common.Helpers
{
    internal abstract class PseudoRandom
    {
        private static RNGCryptoServiceProvider m_pseudoRandom = new RNGCryptoServiceProvider();

        public static UInt32 GetUInt32()
        {
            byte[] ar = new byte[4];
            m_pseudoRandom.GetBytes(ar);

            return new UInt32().FromBytes(ar[0], ar[1], ar[2], ar[3]);
        }

        public static UInt16 GetUInt16()
        {
            byte[] ar = new byte[4];
            m_pseudoRandom.GetBytes(ar);

            return new UInt16().FromBytes(ar[0], ar[1]);
        }

        internal static UInt32 ScaleRandom(UInt32 rand, UInt32 max)
        {
            return ((UInt32)Math.Truncate((((double)rand) * ((double)max)) / (((double)1.0) + ((double)UInt32.MaxValue)))) % max;
        }

        public static UInt32 GetUInt32(UInt32 max)
        {
            // if smaller or equal than 1, then return 0
            if (max <= 1)
            {
                return 0;
            }
            // so max is 2 or more

            // get 32 pseudo random bits
            UInt32 ui = GetUInt32();

            ui = PseudoRandom.ScaleRandom(ui, max);

            Debug.Assert((0 <= ui) && (ui < max), "Failure in GetUInt32 (ui={0}, max={1})", ui, max);

            return ui;
        }

        public static Int32 GetInt32(Int32 max)
        {
            // if smaller or equal than 1, then return 0
            if (max <= 1)
            {
                return 0;
            }

            return (Int32)GetUInt32((UInt32)max);
        }

        public static Int32 GetInt32(Int32 min, Int32 max)
        {
            if (max <= min)
            {
                return max;
            }

            return min + GetInt32(max - min);
        }

        public static byte[] GetByteArray(int count)
        {
            byte[] ar = new byte[count];
            m_pseudoRandom.GetBytes(ar);
            return ar;
        }
    }
}
