﻿// Bitground Software common helpers library
// http://www.bitground.com/
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Bitground.Common.Helpers
{
    public abstract class UniqueId
    {
        private class Generator
        {
            internal static string _alphabet = "0123456789abcdefghijklmnopqrstuvwxyz";
            internal static Hashtable _hAlphabet = new Hashtable();
            internal static Hashtable _hAlphabetUpper = new Hashtable();
            internal static string _first = new String(_alphabet[0], 16);

            static Generator()
            {
#if DEBUG
                if (36 != _alphabet.Length)
                {
                    throw new CException("Invalid _alphabet");
                }
#endif
                for (int i = 0; i < _alphabet.Length; i++)
                {
                    _hAlphabet[_alphabet[i]] = 1;
                    _hAlphabetUpper[(new String(_alphabet[i], 1).ToUpper())[0]] = 1;
                }
            }

            public UInt64 _ticks;
            public UInt32 _rnd;
            public DelegateGetRandomInt32 _fnGetRandomInt32;

#if DEBUG
            public bool[] _bitsTaken = new bool[80];
#endif

            public UInt16 b(Int16 p)
            {

                if ((0 <= p) && (p < 54))
                {
                    return (UInt16)((_ticks >> p) & 1);
                }

                if ((54 <= p) && (p < 54 + 16 + 10))
                {
                    return (UInt16)((_rnd >> (p - 54)) & 1);
                }

                throw new CException("Invalid argument p");
            }

            public char c(Int16 pos, Int16 p0, Int16 p1, Int16 p2, Int16 p3, Int16 p4)
            {
#if DEBUG
                if (_bitsTaken[p0] || _bitsTaken[p1] || _bitsTaken[p2] || _bitsTaken[p3] || _bitsTaken[p4])
                {
                    throw new CException("One of the ps is invalid");
                }
                _bitsTaken[p0] = true;
                _bitsTaken[p1] = true;
                _bitsTaken[p2] = true;
                _bitsTaken[p3] = true;
                _bitsTaken[p4] = true;
#endif

                UInt16 p = (UInt16)
                (
                    (b(p0) << 4) |
                    (b(p1) << 3) |
                    (b(p2) << 2) |
                    (b(p3) << 1) |
                    (b(p4) << 0)
                );

                int[] ar = 
                {
                    (Int16)((5 + pos * 2) % 32),
                    (Int16)((11 + pos * 2) % 32),
                    (Int16)((18 + pos * 2) % 32),
                    (Int16)((27 + pos * 2) % 32)
                };
                Array.Sort(ar);

                int q;

                if ((0 <= p) && (p < ar[0]))
                {
                    q = p;
                }
                else
                {
                    if (ar[0] == p)
                    {
                        q = p + _fnGetRandomInt32(2);
                    }
                    else
                    {
                        if ((ar[0] < p) && (p < ar[1]))
                        {
                            q = p + 1;
                        }
                        else
                        {
                            if (ar[1] == p)
                            {
                                q = p + 1 + _fnGetRandomInt32(2);
                            }
                            else
                            {
                                if ((ar[1] < p) && (p < ar[2]))
                                {
                                    q = p + 2;
                                }
                                else
                                {
                                    if (ar[2] == p)
                                    {
                                        q = p + 2 + _fnGetRandomInt32(2);
                                    }
                                    else
                                    {
                                        if ((ar[2] < p) && (p < ar[3]))
                                        {
                                            q = p + 3;
                                        }
                                        else
                                        {
                                            if (ar[3] == p)
                                            {
                                                q = p + 3 + _fnGetRandomInt32(2);
                                            }
                                            else
                                            {
                                                if ((ar[3] < p) && (p < 32))
                                                {
                                                    q = p + 4;
                                                }
                                                else
                                                {
                                                    throw new CException("Invalid argument");
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                return _alphabet[q];
            }

        }

        static UniqueId()
        {
        }


        private static UInt32 GetTrueRandomUInt32(UInt32 max)
        {
            return TrueRandom.GetUInt32(max);
        }
        private static Int32 GetTrueRandomInt32(Int32 max)
        {
            return TrueRandom.GetInt32(max);
        }


        private static UInt32 GetPseudoRandomUInt32(UInt32 max)
        {
            return PseudoRandom.GetUInt32(max);
        }
        private static Int32 GetPseudoRandomInt32(Int32 max)
        {
            return PseudoRandom.GetInt32(max);
        }


        private delegate UInt32 DelegateGetRandomUInt32(UInt32 max);
        private delegate Int32 DelegateGetRandomInt32(Int32 max);

        private static string Get(
            DelegateGetRandomUInt32 fnGetRandomUInt32,
            DelegateGetRandomInt32 fnGetRandomInt32)
        {
            // returns a 16 characters unique string

            Generator g = new Generator() 
            { 
                _ticks = (UInt64)(DateTime.Now.Ticks),          // 64 bits from the system ticks
                                                                // we'll ignore the first 10 bits as they
                                                                // will not change in the next tenths of years
                _rnd = fnGetRandomUInt32(65536 * 1024),         // 16 + 10 random bits

                _fnGetRandomInt32 = fnGetRandomInt32
            };

            StringBuilder sb = new StringBuilder(16);

            sb.Append(g.c( 0, 32 + 4, 64 + 3, 56 + 2,  8 + 1, 16 + 0));
            sb.Append(g.c( 1, 48 + 1, 72 + 0, 40 + 7, 24 + 6,  0 + 5));
            sb.Append(g.c( 2, 32 + 6, 64 + 5, 56 + 4,  8 + 3, 16 + 2));
            sb.Append(g.c( 3, 48 + 3, 72 + 2, 40 + 1, 24 + 0,  0 + 7));
            sb.Append(g.c( 4, 32 + 0, 64 + 7, 56 + 6,  8 + 5, 16 + 4));
            sb.Append(g.c( 5, 48 + 5, 72 + 4, 40 + 3, 24 + 2,  0 + 1));
            sb.Append(g.c( 6, 32 + 2, 64 + 1, 56 + 0,  8 + 7, 16 + 6));
            sb.Append(g.c( 7, 48 + 7, 72 + 6, 40 + 5, 24 + 4,  0 + 3));
            sb.Append(g.c( 8, 32 + 5, 64 + 4, 56 + 3,  8 + 2, 16 + 1));
            sb.Append(g.c( 9, 48 + 2, 72 + 1, 40 + 0, 24 + 7,  0 + 6));
            sb.Append(g.c(10, 32 + 7, 64 + 6, 56 + 5,  8 + 4, 16 + 3));
            sb.Append(g.c(11, 48 + 4, 72 + 3, 40 + 2, 24 + 1,  0 + 0));
            sb.Append(g.c(12, 32 + 1, 64 + 0, 56 + 7,  8 + 6, 16 + 5));
            sb.Append(g.c(13, 48 + 6, 72 + 5, 40 + 4, 24 + 3,  0 + 2));
            sb.Append(g.c(14, 32 + 3, 64 + 2, 56 + 1,  8 + 0, 16 + 7));
            sb.Append(g.c(15, 48 + 0, 72 + 7, 40 + 6, 24 + 5,  0 + 4));
            
#if DEBUG
            for (int i = 0; i < 80; i++)
            {
                if (!g._bitsTaken[i])
                {
                    throw new CException("Invalid _bitsTaken");
                }
            }
#endif

            return sb.ToString();

        }

        public static string Get()
        {
            // returns a 16 characters unique string
            // using ~100 true random bits

            return Get(
                GetTrueRandomUInt32,
                GetTrueRandomInt32);
        }

        public static string GetPseudo()
        {
            // returns a 16 characters unique string
            // using the pseudo random generator

            return Get(
                GetPseudoRandomUInt32,
                GetPseudoRandomInt32);
        }

        public static string DemoGet(string seed, int length)
        {
            int hash = Strings.GetHash(seed);
            
            Random rnd = new Random(hash);

            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < length; i++)
            {
                sb.Append(
                    Generator._alphabet[rnd.Next() % (Generator._alphabet.Length)]);
            }

            return sb.ToString().ToUpper();
        }

        public static string GetFirst()
        {
            // returns "0000000000000000"
            return Generator._first;
        }

        public static bool IsValidUpper(string unique)
        {
            // returns true if the string is a valid unique string in uppercases

            if (String.IsNullOrEmpty(unique))
            {
                return false;
            }

            if (16 != unique.Length)
            {
                return false;
            }

            for (int i = 0; i < 16; i++)
            {
                if (!Generator._hAlphabetUpper.Contains(unique[i]))
                {
                    return false;
                }
            }

            return true;
        }

        private static string GetBig(
            DelegateGetRandomUInt32 fnGetRandomUInt32,
            DelegateGetRandomInt32 fnGetRandomInt32)
        {
            // returns and 128 chars unique string

            // 128 = 16 + 3 * 6 + 94
            StringBuilder sb = new StringBuilder();

            // getting 16 uniq chars
            sb.Append(UniqueId.Get(
                fnGetRandomUInt32,
                fnGetRandomInt32));

            // getting 3 * 6 true random chars
            for (int i = 0; i < 3; i++)
            {
                // 36 ^ 6 < UInt32.MaxValue
                // getting a random to obtain 6 chars
                UInt32 ui = fnGetRandomUInt32((UInt32)(((UInt32)36) * 36 * 36 * 36 * 36 * 36));

                for (int k = 0; k < 6; k++)
                {
                    sb.Append(Generator._alphabet[(Int32)(ui % 36)]);
                    ui = ui / 36;
                }
            }

            // getting 94 pseudo random chars
            {
                for (int k = 0; k < 94; k++)
                {
                    UInt32 ui = PseudoRandom.GetUInt32(36);

                    sb.Append(Generator._alphabet[(Int32)(ui % 36)]);
                    ui = ui / 36;
                }
            }

            return sb.ToString();
        }

        public static string GetBig()
        {
            // returns and 128 chars unique string
            // using ~250 true random bits

            return GetBig(
                GetTrueRandomUInt32,
                GetTrueRandomInt32);
        }

        public static string GetBigPseudo()
        {
            // returns and 128 chars unique string
            // using the pseudo random generator

            return GetBig(
                GetPseudoRandomUInt32,
                GetPseudoRandomInt32);
        }

        public static bool IsValidBigUpper(string unique)
        {
            // returns true if the string is a valid unique string in uppercases

            if (String.IsNullOrEmpty(unique))
            {
                return false;
            }

            if (128 != unique.Length)
            {
                return false;
            }

            for (int i = 0; i < 128; i++)
            {
                if (!Generator._hAlphabetUpper.Contains(unique[i]))
                {
                    return false;
                }
            }

            return true;
        }
    }
}
