// Bitground Software common helpers library
// http://www.bitground.com/
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Text;
using System.Data;

namespace Bitground.Common.Helpers
{
    public abstract class Strings
    {
        public static string GetDBInvariantValue(object ob)
        {
            if (System.DBNull.Value == ob)
            {
                return Nulls.String;
            }
            return Localization.ConvertToInvariantString(ob);
        }

        public static int GetDBInvariantInt(object ob)
        {
            if (System.DBNull.Value == ob)
            {
                return Nulls.Int32;
            }
            return Localization.ConvertToInvariantInt32(ob);
        }

        public static int GetHash(string s)
        {
            if (String.IsNullOrEmpty(s))
            {
                return 0;
            }

            byte[] b = new byte[] { 0, 0, 0, 0 };

            byte[] ar = UTF8.StringToByteArray(s);
            for (int i = 0; i < ar.Length; i++)
            {
                b[i % 4] ^= (byte)(ar[i] ^ ((byte)(i % 256)));
            }

            return
                (((int)b[0]) << 0) |
                (((int)b[1]) << 8) |
                (((int)b[2]) << 16) |
                (((int)b[3]) << 24);
        }

        public static string Padding(int intValue, int intWidth)
        {
            return String.Format(
                String.Format(
                    "{{0:D{0}}}",
                    intWidth),
                intValue);
        }

        public static string Padding(string strValue, int intWidth)
        {
            if (intWidth < strValue.Length)
            {
                return strValue.Substring(0, intWidth);
            }
            else
            {
                return strValue + new String(' ', intWidth - strValue.Length);
            }
        }

        public static string PaddingBeginning(string strValue, int intWidth)
        {
            return PaddingBeginning(strValue, intWidth, ' ');
        }

        public static string PaddingBeginning(string strValue, int intWidth, char c)
        {
            if (intWidth < strValue.Length)
            {
                return strValue.Substring(0, intWidth);
            }
            else
            {
                return new String(c, intWidth - strValue.Length) + strValue;
            }
        }

        public static string CutTo(int intCutColumn, string strText)
        {
            if (String.IsNullOrEmpty(strText))
            {
                return strText;
            }

            if (strText.Length < intCutColumn)
                return strText;
            return strText.Substring(0, intCutColumn);
        }
    }
}
