// Bitground Software common helpers library
// http://www.bitground.com/
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Text;
using System.Data;

namespace Bitground.Common.Helpers
{
    public partial class Time
    {
        private const string strNow = "now";

        private DateTime dt;
        public Time(DateTime dt)
        {
            this.dt = dt;
        }

        // Thurday, 17 Aug 2000, 16:32:05

        private string Second                           // 5
        {
            get { return Localization.ToInvariantString(dt.Second); }
        }
        private string SecondFixedSize                  // 05
        {
            get { return Localization.ToInvariantString("ss", dt); }
        }

        private string Minute                           // 32
        {
            get { return Localization.ToInvariantString(dt.Minute); }
        }
        private string MinuteFixedSize                  // 32
        {
            get { return Localization.ToInvariantString("mm", dt); }
        }

        private string Hour                             // 16
        {
            get { return Localization.ToInvariantString(dt.Hour); }
        }
        private string HourFixedSize                    // 16
        {
            get { return Localization.ToInvariantString("HH", dt); }
        }



        private string DayNumber                        // 17
        {
            get { return Localization.ToInvariantString(dt.Day); }
        }
        private string DayNumberFixedSize               // 17
        {
            get { return Localization.ToInvariantString("dd", dt); }
        }
        private string DayOfWeekNumber                  // 4        0=Sunday,...
        {
            get { return Localization.ToInvariantString(Localization.ConvertToInvariantInt16(dt.DayOfWeek)); }
        }
        private string DayOfWeekNameShort               // Thu
        {
            get { return Localization.ToInvariantString("ddd", dt); }
        }
        private string DayOfWeekNameLong                // Thursday
        {
            get { return Localization.ToInvariantString("dddd", dt); }
        }

        private string MonthNumber                      // 8
        {
            get { return Localization.ToInvariantString(dt.Month); }
        }
        private string MonthNumberFixedSize             // 08
        {
            get { return Localization.ToInvariantString("MM", dt); }
        }
        private string MonthNameShort                   // Aug
        {
            get { return Localization.ToInvariantString("MMM", dt); }
        }
        private string MonthNameLong                    // August
        {
            get { return Localization.ToInvariantString("MMMM", dt); }
        }

        private string Year                             // 2000
        {
            get { return Localization.ToInvariantString("yyyy", dt); }
        }
        private string YearShort                        // 00
        {
            get { return Localization.ToInvariantString("yy", dt); }
        }

        public DataTable TESTGetAsDataTable()
        {
            DataTable dt = new DataTable("Time");
            dt.Columns.Add("Property", typeof(string));
            dt.Columns.Add("Value", typeof(string));

            dt.Rows.Add("Second", Second);
            dt.Rows.Add("SecondFixedSize", SecondFixedSize);
            dt.Rows.Add("Minute", Minute);
            dt.Rows.Add("MinuteFixedSize", MinuteFixedSize);
            dt.Rows.Add("Hour", Hour);
            dt.Rows.Add("HourFixedSize", HourFixedSize);
            dt.Rows.Add("DayNumber", DayNumber);
            dt.Rows.Add("DayNumberFixedSize", DayNumberFixedSize);
            dt.Rows.Add("DayOfWeekNumber", DayOfWeekNumber);
            dt.Rows.Add("DayOfWeekNameShort", DayOfWeekNameShort);
            dt.Rows.Add("DayOfWeekNameLong", DayOfWeekNameLong);
            dt.Rows.Add("MonthNumber", MonthNumber);
            dt.Rows.Add("MonthNumberFixedSize", MonthNumberFixedSize);
            dt.Rows.Add("MonthNameShort", MonthNameShort);
            dt.Rows.Add("MonthNameLong", MonthNameLong);
            dt.Rows.Add("Year", Year);
            dt.Rows.Add("YearShort", YearShort);
           
            return dt;
        }

        public static DataTable TESTGetAsDataTable(DateTime dt)
        {
            return (new Time(dt)).TESTGetAsDataTable();
        }

        public static DateTime ToDayGranularity(DateTime dt)
        {
            if (Nulls.DateTime == dt)
            {
                return dt;
            }
            return dt.Date;
        }

        public static DateTime Now
        {
            get
            {
                return DateTime.Now;
            }
        }

        public static DateTime NowWithDayGranularity
        {
            get
            {
                return ToDayGranularity(Now);
            }
        }

        public static DateTime UTCNow
        {
            get
            {
                return DateTime.UtcNow;
            }
        }

        public static DateTime UTCNowWithDayGranularity
        {
            get
            {
                return ToDayGranularity(UTCNow);
            }
        }

        public static DateTime AddDays(DateTime dt, int intDaysNo)
        {
            if (Nulls.DateTime == dt)
            {
                return dt;
            }
            return dt.AddDays(intDaysNo);
        }

        public static DateTime AddHours(DateTime dt, int intHoursNo)
        {
            if (Nulls.DateTime == dt)
            {
                return dt;
            }
            return dt.AddHours(intHoursNo);
        }

        public static DateTime AddSeconds(DateTime dt, int intSecondsNo)
        {
            if (Nulls.DateTime == dt)
            {
                return dt;
            }
            return dt.AddSeconds(intSecondsNo);
        }

        public static DateTime BeforeUTCNow(int intDaysNo)
        {
            return UTCNow.AddDays(-intDaysNo);
        }

        public static DateTime BeforeUTCNow(double dblSecondsNo)
        {
            return UTCNow.AddSeconds(-dblSecondsNo);
        }

        public static string ToCanonicalDateYearFileString(DateTime dt)
        {
            return Localization.ToInvariantString("yyyy", dt);
        }

        public static string ToCanonicalDateMonthFileString(DateTime dt)
        {
            return Localization.ToInvariantString("yyyy-MM", dt);
        }

        public static string ToCanonicalDateFileString(DateTime dt)
        {
            return Localization.ToInvariantString("yyyy-MM-dd", dt);
        }

        public static string ToCanonicalCondensedDateFileString(DateTime dt)
        {
            return Localization.ToInvariantString("yyyyMMdd", dt);
        }

        public static string ToCanonicalCondensedDateTimeFileString(DateTime dt)
        {
            return Localization.ToInvariantString("yyyyMMddHHmmss", dt);
        }

        public static DateTime FromCanonicalCondensedDateTimeFileString(string fdt)
        {
            return DateTime.ParseExact(
                fdt,
                "yyyyMMddHHmmss",
                System.Globalization.CultureInfo.InvariantCulture.DateTimeFormat);
        }

        public static string ToCanonicalFileString(DateTime dt)
        {
            return Localization.ToInvariantString("yyyy-MM-dd HH-mm-ss", dt);
        }

        public static string ToCanonicalAccurateFileString(DateTime dt)
        {
            return String.Format(
                "{0}-{1}-{2:D3}",
                Localization.ToInvariantString("yyyyMMdd", dt),
                Localization.ToInvariantString("HHmmss", dt),
                dt.Millisecond);
        }

        public static string ToCanonicalCondensedAccurateFileString(DateTime dt)
        {
            return String.Format(
                "{0}{1}{2:D3}",
                Localization.ToInvariantString("yyyyMMdd", dt),
                Localization.ToInvariantString("HHmmss", dt),
                dt.Millisecond);
        }

        public static string ToCanonicalString(DateTime dt)
        {
            return Localization.ToInvariantString("yyyy-MM-dd HH:mm:ss", dt);
        }

        public static string ToCanonicalStringUTC(DateTime dt)
        {
            return Localization.ToInvariantString("yyyy-MM-dd HH:mm:ss", dt) + " GMT";
        }

        public static string ToCanonicalStringDate(DateTime dt)
        {
            return Localization.ToInvariantString("yyyy-MM-dd", dt);
        }

        public static string ToCanonicalStringTime(DateTime dt)
        {
            return Localization.ToInvariantString("HH:mm:ss", dt);
        }

        public static string TimeSpanAsString(TimeSpan ts)
        {
            return String.Format(
                "{0}{1:D2}:{2:D2}:{3:D2}",
                (ts.Days <= 0) ? 
                    "" : 
                    (
                        (1 == ts.Days) ?
                            "1 day, " :
                            Localization.ToInvariantString(ts.Days) + " days, "
                    ),
                ts.Hours,
                ts.Minutes,
                ts.Seconds);
        }

        public static string TimeSpanMinutesSecondsAsString(TimeSpan ts)
        {
            string strPrefix = "";
            if (ts.TotalSeconds < 0)
            {
                strPrefix = "-";
                ts = ts.Negate();
            }

            return String.Format(
                "{0}{1:D2}:{2:D2}",
                strPrefix,
                (int)ts.TotalMinutes,
                ts.Seconds);
        }

        public static string TimeSpanAsStringAccurate(TimeSpan ts)
        {
            return String.Format(
                "{0}.{1:D3}",
                TimeSpanAsString(ts),
                ts.Milliseconds);
        }

        public static string TimeSpanAsStringAccurateHigh(TimeSpan ts)
        {
            return String.Format(
                "{0}.{1:D7}",
                TimeSpanAsString(ts),
                ts.Ticks % 10000000);
        }

        public static string TimeSpanAsStringMilliseconds(TimeSpan ts)
        {
            return String.Format(
                "{0}ms",
                ts.Ticks / 10000);
        }

        public static string SecondsAsString(int intSeconds)
        {
            return TimeSpanAsString(new TimeSpan(0, 0, intSeconds));
        }

        private static DataTable m_dtYears;
        private static DataTable m_dtMonths;
        private static DataTable m_dtDays;
        private static DataTable m_dtHours;
        private static DataTable m_dtMinutes;
        static Time()
        {
            int intThisYear = Time.UTCNow.Year;

            m_dtYears = new DataTable("Years");
            {
                m_dtYears.Columns.Add("Id", typeof(int));
                m_dtYears.Columns.Add("Name", typeof(string));

                for (int i = intThisYear; i <= intThisYear + 10; i++)
                {
                    m_dtYears.Rows.Add(
                        i, 
                        i.ToString());
                }
            }


            m_dtMonths = new DataTable("Months");
            {
                m_dtMonths.Columns.Add("Id", typeof(int));
                m_dtMonths.Columns.Add("Name", typeof(string));

                for (int i = 1; i <= 12; i++)
                {
                    m_dtMonths.Rows.Add(
                        i, 
                        new Time(new DateTime(intThisYear, i, 1)).MonthNameLong);
                }
            }

            m_dtDays = new DataTable("Days");
            {
                m_dtDays.Columns.Add("Id", typeof(int));
                m_dtDays.Columns.Add("Name", typeof(string));

                for (int i = 1; i <= 31; i++)
                {
                    m_dtDays.Rows.Add(
                        i, 
                        Strings.Padding(i, 2)
                    );
                }
            }

            m_dtHours = new DataTable("Hours");
            {
                m_dtHours.Columns.Add("Id", typeof(int));
                m_dtHours.Columns.Add("Name", typeof(string));

                for (int i = 0; i <= 23; i++)
                {
                    m_dtHours.Rows.Add(
                        i,
                        String.Format("{0:D2}", i));
                }
            }

            m_dtMinutes = new DataTable("Minutes");
            {
                m_dtMinutes.Columns.Add("Id", typeof(int));
                m_dtMinutes.Columns.Add("Name", typeof(string));

                for (int i = 0; i < 60; i+= 5)
                {
                    m_dtMinutes.Rows.Add(
                        i,
                        String.Format("{0:D2}", i));
                }
            }
        }

        public static DataTable GetListYears()
        {
            return m_dtYears;
        }

        public static DataTable GetListMonths()
        {
            return m_dtMonths;
        }

        public static DataTable GetListDays()
        {
            return m_dtDays;
        }

        public static DataTable GetListHours()
        {
            return m_dtHours;
        }

        public static DataTable GetListMinutes()
        {
            return m_dtMinutes;
        }

        public static DateTime Approximate(DateTime dt)
        {
            if (Nulls.DateTime  == dt)
            {
                return dt;
            }

            return new DateTime(
                dt.Year,
                dt.Month,
                dt.Day,
                dt.Hour,
                dt.Minute,
                dt.Second);
        }
    }
}
