﻿// Bitground Software common helpers library
// http://www.bitground.com/
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Script.Serialization;

namespace Bitground.Common.Helpers
{
    public abstract class Json
    {
        private static JavaScriptSerializer jss = new JavaScriptSerializer()
        {
            MaxJsonLength = 1024 * 1024 * 10,   // Default to 10Mb
            RecursionLimit = 10                 // Go maximum 10 levels deep
        };

        public static T Decode<T>(string json)
        {
            if (String.IsNullOrEmpty(json))
            {
                return default(T);
            }
            T ret;
            try
            {
                ret = jss.Deserialize<T>(json);
            }
            catch (Exception ex)
            {
                Log.Exception(ex, "Failure during json decoding. The json string is: '{0}'", json); 
                ret = default(T);
            }
            return ret;
        }

        public static string Encode(object obj)
        {
            if (obj == null)
            {
                return String.Empty;
            }
            string ret;
            try
            {
                ret = jss.Serialize(obj);
            }
            catch (Exception ex)
            {
                Log.Exception(ex, "Failure during json encoding. The object is: '{0}'", obj);
                ret = String.Empty;
            }
            return ret;
        }
    }
}
