using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Web;
using Rest;

namespace Rest.Http
{
    /// <summary>
    /// Encoding: "application/x-www-form-urlencoded"
    /// </summary>
    /// <remarks>
    /// See: http://www.w3.org/TR/REC-html40/interact/forms.html#h-17.13.4.1
    /// </remarks> 
    public class JsonUrlEncoded : HttpContentType  
    {
        #region Properties
        private String contentType = "application/json; charset=utf-8";

        /// <summary>
        /// The content type for this instance (application/x-www-form-urlencoded)
        /// </summary>
        public sealed override String ContentType
        {
            get { return contentType; }
            set { contentType = value; }
        }
        #endregion

        private string JSONEscape(string s)
        {
            StringBuilder sb = new StringBuilder();
            int l = s.Length;
            for (int i = 0; i < l; i++)
            {
                char ch = s[i];
                switch (ch)
                {
                    case '"':
                        sb.Append("\\\"");
                        break;

                    case '\\':
                        sb.Append("\\\\");
                        break;

                    case '\b':
                        sb.Append("\\b");
                        break;

                    case '\f':
                        sb.Append("\\f");
                        break;

                    case '\n':
                        sb.Append("\\n");
                        break;

                    case '\r':
                        sb.Append("\\r");
                        break;

                    case '\t':
                        sb.Append("\\t");
                        break;

                    case '/':
                        sb.Append("\\/");
                        break;

                    default:
                        //Reference: http://www.unicode.org/versions/Unicode5.1.0/
                        if ((ch >= '\u0000' && ch <= '\u001F') || (ch >= '\u007F' && ch <= '\u009F') || (ch >= '\u2000' && ch <= '\u20FF'))
                        {
                            sb.Append(
                                String.Format("\\u{0:X4}", Convert.ToInt16(ch)));
                        }
                        else
                        {
                            sb.Append(ch);
                        }
                        break;
                }

            }

            return sb.ToString();
        }

        /// <summary>
        /// Writes request body to the supplied stream
        /// </summary>
        /// <param name="stream"></param>
        /// <param name="parameters"></param>
        /// <param name="dataItems"></param>
        /// <param name="characterEncoding"></param>
        public sealed override void WriteRequestBody(
            Stream stream,
            RestParameterCollection parameters,
            RestDataItemCollection dataItems,
            Encoding characterEncoding)
        {
            if (parameters != null && parameters.Count > 0)
            {
                //String parameterString;
                //Byte[] parameterBytes;
                //Byte[] newlineBytes = characterEncoding.GetBytes("\r\n");

                IRestParameter parameter = null;
                int pc = parameters.Count;

                StringBuilder sb = new StringBuilder();
                sb.Append("{");
                for (Int32 i = 0; i < pc; i++)
                {
                    parameter = parameters[i];

                    sb.AppendFormat(
                        "\"{0}\":\"{1}\"",
                        JSONEscape(parameter.Name),
                        JSONEscape(parameter.Value));
                    if (i < pc - 1)
                    {
                        sb.Append(",");
                    }
                    /*
                    parameterString =
                        String.Format(
                        "{0}={1}{2}",
                        HttpUtility.UrlPathEncode(parameter.Name),
                        HttpUtility.UrlPathEncode(parameter.Value),
                        i < parameters.Count - 1 ? "&": String.Empty);

                    parameterBytes = characterEncoding.GetBytes(parameterString);

                    stream.Write(parameterBytes, 0, parameterBytes.Length);
                    */
                }
                sb.Append("}");
                

                //stream.Write(newlineBytes, 0, newlineBytes.Length); 
                byte[] ar = characterEncoding.GetBytes(sb.ToString());
                stream.Write(ar, 0, ar.Length);
            }
        }
    }
}
