﻿// zkBox C# client
// http://www.zkbox.com/developers/client/csharp
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Bitground.Common.Helpers;

namespace Bitground.zkBox.Client.CS
{
    internal abstract class ParamsLogging
    {
        public static bool CanLogParams
        {
            get
            {
                return Log.CanLogInfo;
            }
        }

        public static bool LogParams(string operation, params object[] args)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat(
                "[{0}]",
                operation);

            if ((null == args) || (args.Length <= 0))
            {
                sb.Append(" <none>");
            }
            else
            {
                if (0 != args.Length % 2)
                {
                    Log.Error("We should have an even number of arguments");
                    return false;
                }
                else
                {
                    for (int i = 0; i < args.Length; i += 2)
                    {
                        string name = args[i] as string;
                        object value = args[i + 1];

                        if (null == name)
                        {
                            Log.Error("Parameter #{0} should be a string", i);
                        }
                        else
                        {
                            if ("" == name)
                            {
                                sb.AppendFormat(
                                    " {0}",
                                    Params.GetObjectAsString(value));
                            }
                            else
                            {
                                if ((value is string) && ("" == (value as string)))
                                {
                                    sb.AppendFormat(
                                        " {0}",
                                        name);
                                }
                                else
                                {
                                    sb.AppendFormat(
                                        " {0}={1}",
                                        name,
                                        Params.GetObjectAsString(value));
                                }
                            }
                        }
                    }
                }
            }

            Log.Info(sb.ToString().Replace("{", "{{").Replace("}", "}}"));
            return true;
        }
    }
}
