﻿// zkBox C# client
// http://www.zkbox.com/developers/client/csharp
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Rest;
using Rest.Xml;
using Rest.Service;
using Rest.Transport;
using Bitground.Common.Helpers;

namespace Bitground.zkBox.Client.CS
{
    public class Users : ApiHandler
    {
        public Users(Handler handler)
            : base(handler)
        {
        }

        #region Check user

        public class CheckUser : WSRet<CheckUser.Data>
        {
            public class Data
            {
                public bool exists;
            }
        };

        public CheckUser Check(string username)
        {
            return api.Call<CheckUser, CheckUser.Data>(
                "Users.Check",
                new object[] 
                {   
                    "username", username
                },
                () =>
                {
                    if (Log.CanLogTrace) Log.Trace("Checking for user");

                    string login;
                    Crypto.SRPClient.CheckUser(
                        username,
                        out login);

                    return api.Authenticate_Application<CheckUser, CheckUser.Data>(
                        "PUT",
                        "user/check",
                        new object[] 
                        { 
                            "login", login 
                        },

                        (data) =>
                        {
                            bool exists = (data["exists"] as bool?) ?? false;
                            if (Log.CanLogTrace) Log.Trace("The user was checked");
                            return new CheckUser
                            {
                                error = 0,
                                data = new CheckUser.Data
                                {
                                    exists = exists
                                }
                            };
                        });
                });
        }

        #endregion

        #region Add user

        public class AddUser : WSRet<AddUser.Data>
        {
            public class Data
            {
            }
        }

        public AddUser Add(
            string username,
            string password)
        {
            return api.Call<AddUser, AddUser.Data>(
                "Users.Add",
                new object[] 
                {   
                    "username", username,
                    "password", password
                },
                () =>
                {
                    if (Log.CanLogTrace) Log.Trace("SRP user creation started");
                    string login;
                    string s;
                    string v;
                    Crypto.SRPClient.AddUser(
                        username,
                        password,
                        out login,
                        out s,
                        out v);

                    return api.Authenticate_Application<AddUser, AddUser.Data>(
                        "POST",
                        "user",
                        new object[] 
                        { 
                            "login", login,
                            "s", s,
                            "v", v
                        },

                        (data) =>
                        {
                            if (Log.CanLogTrace) Log.Trace("The user was successfully created");
                            return new AddUser
                            {
                                error = 0,
                                data = new AddUser.Data
                                {
                                }
                            };
                        });
                });
        }

        #endregion

        #region Authenticate user

        public class AuthenticateUser : WSRet<AuthenticateUser.Data>
        {
            public class Data
            {
            }
        }

        internal class AuthenticateUserCore : WSRet<AuthenticateUserCore.Data>
        {
            public class Data
            {
                public string login;
                public string pass;
                public string userId;
                public string userVersion;
                public string sessionId;
                public string sessionKey;
                public string cryptoKey;
            }
        }

        internal class Users_AuthStep1 : WSRet<Users_AuthStep1.Data>
        {
            public class Data
            {
                public string uniq;
                public string s;
                public string B;
                public string u;
            }
        }

        internal class Users_AuthStep2 : WSRet<Users_AuthStep2.Data>
        {
            public class Data
            {
                public string m2;
                public string sessionId;
                public string userId;
                public string userVersion;
            }
        }

        delegate AuthenticateUserCore AuthenticateUserCoreF();

        internal AuthenticateUser AuthenticateUserInternal(
            bool dataIsHashed,
            string username,
            string password)
        {
            AuthenticateUserCoreF fnAuthenticateUser = () =>
            {
                string login;
                string AHex;
                BigInteger a;
                Crypto.SRPClient.AuthStep1(
                    dataIsHashed,
                    username,
                    out login,
                    out AHex,
                    out a);

                Users_AuthStep1 authStep1 = api.Authenticate_Application<Users_AuthStep1, Users_AuthStep1.Data>(
                    "PUT",
                    "user/auth/step1",
                    new object[] 
                        { 
                            "login", login,
                            "A", AHex
                        },

                    (data) =>
                    {
                        if (Log.CanLogTrace) Log.Trace("Decoding the response (1)");

                        string uniq = (data["uniq"] as string) ?? "";
                        string s = (data["s"] as string) ?? "";
                        string B = (data["B"] as string) ?? "";
                        string u = (data["u"] as string) ?? "";

                        return new Users_AuthStep1
                        {
                            error = 0,
                            data = new Users_AuthStep1.Data
                            {
                                uniq = uniq,
                                s = s,
                                B = B,
                                u = u
                            }
                        };
                    });

                if ((null != authStep1) && (0 == authStep1.error))
                {
                    var data1 = authStep1.data;

                    string uniq = data1.uniq;
                    string sHex = data1.s;
                    string BHex = data1.B;
                    string uHex = data1.u;

                    string m1Hex;
                    string KHex;
                    string pass;
                    Crypto.SRPClient.AuthStep2(
                        dataIsHashed,
                        login,
                        password,
                        sHex,
                        BHex,
                        uHex,
                        AHex,
                        a,
                        out m1Hex,
                        out KHex,
                        out pass);

                    Users_AuthStep2 authStep2 = api.Authenticate_Application<Users_AuthStep2, Users_AuthStep2.Data>(
                        "PUT",
                        "user/auth/step2",
                        new object[] 
                            { 
                                "login", login,
                                "uniq", uniq,
                                "m1", m1Hex
                            },

                        (data) =>
                        {
                            if (Log.CanLogTrace) Log.Trace("Decoding the response (2)");

                            string m2server = (data["m2"] as string) ?? "";
                            string sessionId = (data["sessionId"] as string) ?? "";
                            string userId = (data["userId"] as string) ?? "";
                            string userVersion = (data["userVersion"] as string) ?? "";

                            return new Users_AuthStep2
                            {
                                error = 0,
                                data = new Users_AuthStep2.Data
                                {
                                    m2 = m2server,
                                    sessionId = sessionId,
                                    userId = userId,
                                    userVersion = userVersion
                                }
                            };
                        });

                    if ((null != authStep2) && (0 == authStep2.error))
                    {
                        var data2 = authStep2.data;

                        string cryptoKey;
                        Crypto.SRPClient.AuthValidate(
                            AHex,
                            m1Hex,
                            data2.m2,
                            KHex,
                            pass,
                            out cryptoKey);

                        if (null != cryptoKey)
                        {
                            if (Log.CanLogTrace) Log.Trace("The user was successfully authenticated");

                            return new AuthenticateUserCore
                            {
                                error = 0, // Success
                                data = new AuthenticateUserCore.Data
                                {
                                    login = login,
                                    pass = pass,
                                    userId = data2.userId,
                                    userVersion = data2.userVersion,
                                    sessionId = data2.sessionId,
                                    sessionKey = KHex,
                                    cryptoKey = cryptoKey
                                }
                            };
                        }
                        else
                        {
                            return new AuthenticateUserCore
                            {
                                error = 101 // Invalid username or password
                            };
                        }
                    }
                    else
                    {
                        return authStep2.ErrorOnly<AuthenticateUserCore, AuthenticateUserCore.Data>();
                    }

                }
                else
                {
                    return authStep1.ErrorOnly<AuthenticateUserCore, AuthenticateUserCore.Data>(); ;
                }
            };


            AuthenticateUserCore authCore = fnAuthenticateUser();
            AuthenticateUser auth;

            if ((null != authCore) && (0 == authCore.error) && (null != authCore.data))
            {
                {
                    var data = authCore.data;
                    api.status.user = data.login;
                    api.status.pass = data.pass;
                    api.status.userId = data.userId;
                    api.status.userVersion = data.userVersion;
                    api.status.sessionId = data.sessionId;
                    api.status.sessionKey = data.sessionKey;
                    api.status.cryptoKey = data.cryptoKey;
                    api.status.updateAuth();
                }

                auth = new AuthenticateUser()
                {
                    error = authCore.error,
                    message = authCore.message,
                    data = new AuthenticateUser.Data()
                    {
                    }
                };
            }
            else
            {
                api.status.clearAuth();

                if (null == authCore)
                {
                    auth = null;
                }
                else
                {
                    auth = new AuthenticateUser()
                    {
                        error = authCore.error,
                        message = authCore.message,
                    };
                }
            }

            return auth;
        }

        public AuthenticateUser Authenticate(
            string username,
            string password)
        {
            return api.Call<AuthenticateUser, AuthenticateUser.Data>(
                "Users.Authenticate",
                new object[] 
                {   
                    "username", username,
                    "password", password
                },
                () =>
                {
                    if (Log.CanLogTrace) Log.Trace("SRP authentication started");

                    AuthenticateUser auth = AuthenticateUserInternal(
                        false,
                        username,
                        password);

                    if (Log.CanLogTrace) Log.Trace("SRP authentication finished");

                    return auth;
                });
        }

        #endregion

        #region Update user

        public class UpdateUser : WSRet<UpdateUser.Data>
        {
            public class Data
            {
            }
        };

        public UpdateUser Update(
            string username,
            string oldPassword,
            string newPassword)
        {
            // TODO_LATER: finish the C# client
            throw new CException("Unimplemented");

            return api.Call<UpdateUser, UpdateUser.Data>(
                "Users.Update",
                new object[] 
                {   
                    "username", username,
                    "oldPassword", oldPassword,
                    "newPassword", newPassword
                },
                () =>
                {
                    if (Log.CanLogTrace) Log.Trace("Updating the user");

                    string login;
                    string oldpass;
                    Crypto.SRPClient.UpdateUserOld(
                        username,
                        oldPassword,
                        out login,
                        out oldpass);

                    if (!api.status.isAuthenticated)
                    {
                        return new UpdateUser { error = 2 }; // Not authenticated
                    }

                    if (api.status.user != login)
                    {
                        return new UpdateUser { error = 101 }; // Invalid username or password
                    }

                    if (api.status.pass != oldpass)
                    {
                        return new UpdateUser { error = 101 }; // Invalid username or password
                    }


                    // TODO_LATER: finish the C# client


                    string newpass;
                    string s;
                    string v;
                    Crypto.SRPClient.UpdateUserNew(
                        login,
                        newPassword,
                        out newpass,
                        out s,
                        out v);

                    return api.Authenticate_ApplicationAndUser<UpdateUser, UpdateUser.Data>(
                        "PUT",
                        "user",
                        new object[] 
                        { 
                            "s", s,
                            "v", v
                        },

                        (data) =>
                        {
                            api.status.pass = newpass;
                            api.status.updateAuth();

                            AuthenticateUser authUser = AuthenticateUserInternal(
                                true,
                                login,
                                newpass);

                            if ((null != authUser) && (0 == authUser.error) && (null != authUser.data))
                            {
                                if (Log.CanLogTrace) Log.Trace("The user was updated");
                                return new UpdateUser
                                {
                                    error = 0,
                                    data = new UpdateUser.Data
                                    {
                                    }
                                };
                            }
                            else
                            {
                                if (Log.CanLogTrace) Log.Trace("There was an problem while trying to authenticate the user after an update");

                                if (null == authUser)
                                {
                                    return null;
                                }
                                else
                                {
                                    return authUser.ErrorOnly<UpdateUser, UpdateUser.Data>();
                                }
                            }
                        });
                });
        }

        #endregion

        #region Logout user

        public class LogoutUser : WSRet<LogoutUser.Data>
        {
            public class Data
            {
            }
        };

        public LogoutUser Logout()
        {
            return api.Call<LogoutUser, LogoutUser.Data>(
                "Users.Logout",
                new object[] 
                {   
                },
                () =>
                {
                    if (Log.CanLogTrace) Log.Trace("Logging out the user");

                    Crypto.SRPClient.LogoutUser(
                        );

                    return api.Authenticate_ApplicationAndUser<LogoutUser, LogoutUser.Data>(
                        "PUT",
                        "user/logout",
                        new object[] 
                        { 
                        },

                        (data) =>
                        {
                            api.status.clearAuth();

                            if (Log.CanLogTrace) Log.Trace("The user was logged out");
                            return new LogoutUser
                            {
                                error = 0,
                                data = new LogoutUser.Data
                                {
                                }
                            };
                        });
                });
        }

        #endregion

        #region Delete user

        public class DeleteUser : WSRet<DeleteUser.Data>
        {
            public class Data
            {
            }
        };

        public DeleteUser Delete(
            string username,
            string password)
        {
            return api.Call<DeleteUser, DeleteUser.Data>(
                "User.Delete",
                new object[] 
                {   
                    "username", username,
                    "password", password
                },
                () =>
                {
                    if (Log.CanLogTrace) Log.Trace("Deleting the user");

                    string login;
                    string pass;
                    Crypto.SRPClient.DeleteUser(
                        username,
                        password,
                        out login,
                        out pass);

                    if (!api.status.isAuthenticated)
                    {
                        return new DeleteUser { error = 2 }; // Not authenticated
                    }

                    if (api.status.user != login)
                    {
                        return new DeleteUser { error = 101 }; // Invalid username or password
                    }

                    if (api.status.pass != pass)
                    {
                        return new DeleteUser { error = 101 }; // Invalid username or password
                    }

                    return api.Authenticate_ApplicationAndUser<DeleteUser, DeleteUser.Data>(
                        "DELETE",
                        "user",
                        new object[] 
                        { 
                        },

                        (data) =>
                        {
                            if (Log.CanLogTrace) Log.Trace("The user was deleted");
                            return new DeleteUser
                            {
                                error = 0,
                                data = new DeleteUser.Data
                                {
                                }
                            };
                        });
                });
        }

        #endregion
    }
}
