﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//////////////////////////////////
////////  api operations  ////////
//////////////////////////////////

// returns the list of the operations results; returns true if the batch succeeded, null otherwise
$zkBox.RunBatch = function()
{
    if ($zkBox.logging) $zkBox.utils.log('[BATCH -> RunBatch (arguments="{0}")]',
        $zkBox.utils.dump(arguments));

    var ret = null;

    var sections = arguments;

    if (sections.length < 2)
    {
        if ($zkBox.logging) $zkBox.utils.log('RunBatch: Failure because there are no sections to run');
        return ret;
    }

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("RunBatch");

    var options = sections[0];
    options = jQuery.extend(
    {
        transactionId: null,
        atStartBeginTransaction: true,
        atEndCommitTransaction: true,
        onErrorRollbackTransaction: true
    }, options);

    var operations = [];
    var operationsCache = [];
    var ctx =
    {
        operations: operations,
        operationsCache: operationsCache,
        results: null,
        transactionId: null
    };

    var fnAddObject = function(params)
    {
        return $zkBox.batch.AddObject(ctx, params);
    };
    var fnUpdateObject = function(params)
    {
        return $zkBox.batch.UpdateObject(ctx, params);
    };
    var fnGetObject = function(params)
    {
        return $zkBox.batch.GetObject(ctx, params);
    };
    var fnDeleteObject = function(params)
    {
        return $zkBox.batch.DeleteObject(ctx, params);
    };

    var fnGetObjectsList = function(params)
    {
        return $zkBox.batch.GetObjectsList(ctx, params);
    };

    var fnGetObjectsListIDs = function(params)
    {
        return $zkBox.batch.GetObjectsListIDs(ctx, params);
    };

    var fnSetSingleObject = function(params)
    {
        params = jQuery.extend(
        {
            //transactionId: null,
            type: null,
            data: null
        }, params);

        return $zkBox.batch.SetSingleObject(ctx, params);
    };

    var fnGetSingleObject = function(params)
    {
        params = jQuery.extend(
        {
            //transactionId: null,
            type: null,
            dataDefault: null
        }, params);

        return $zkBox.batch.GetSingleObject(ctx, params);
    };



    // the transaction was created
    var failed = false;
    var sectidx = 1; // the first argument are the options, so start from the 2nd (idx=1) argument
    var sectlength = sections.length;
    var results = new Array();
    var prevResult;

    while ((!failed) && (sectidx < sectlength))
    {
        // if we didn't fail and there are still sections...
        // execute the section

        // construct the transaction context object
        var batch =
        {
            transactionId: options.transactionId,
            atStartBeginTransaction: options.atStartBeginTransaction,
            atEndCommitTransaction: options.atEndCommitTransaction,
            onErrorRollbackTransaction: options.onErrorRollbackTransaction,

            results: results,
            prevResult: prevResult,
            AddObject: fnAddObject,
            UpdateObject: fnUpdateObject,
            GetObject: fnGetObject,
            DeleteObject: fnDeleteObject,
            GetObjectsList: fnGetObjectsList,
            GetObjectsListIDs: fnGetObjectsListIDs,
            SetSingleObject: fnSetSingleObject,
            GetSingleObject: fnGetSingleObject
        }

        if ($zkBox.logging) $zkBox.utils.log('RunBatch: Running section #{0}...', sectidx);
        var sectret = sections[sectidx](batch); // pass the batch context as parameter

        if (sectret)
        {
            // the section succeeded, go to the next section
            if ($zkBox.logging) $zkBox.utils.log('RunBatch: Section #{0} finished with success', sectidx);
            results.push(sectret); // add this section's result to the results list
            prevResult = sectret; // set the previous section result
            sectidx++; // move to the next section
        }
        else
        {
            // the section failed; abort
            if ($zkBox.logging) $zkBox.utils.log('RunBatch: Section #{0} failed', sectidx);
            failed = true;
        }
    }

    if (!failed)
    {
        // the sections were executed successfull; now try to execute the batch

        var tempret = null;
        if ((!operations) || (operations.length < 1))
        {
            if ($zkBox.logging) $zkBox.utils.log('RunBatch: Failure because there are no operations to run');
        }
        else
        {
            var execbatch =
            {
                transactionId: options.transactionId,
                atStartBeginTransaction: options.atStartBeginTransaction,
                atEndCommitTransaction: options.atEndCommitTransaction,
                onErrorRollbackTransaction: options.onErrorRollbackTransaction,
                operations: operations
            };

            tempret = $zkBox.batch.ExecuteBatch(execbatch);
        }

        //if ($zkBox.logging) $zkBox.utils.log('RunBatch: tempret: "{0}"', $zkBox.utils.dump(tempret));
        //if ($zkBox.logging) $zkBox.utils.log('RunBatch: tempret.results: "{0}"', $zkBox.utils.dump(tempret.results));
        //if ($zkBox.logging) $zkBox.utils.log('RunBatch: tempret.results.length: "{0}"', $zkBox.utils.dump(tempret.results.length));
        //if ($zkBox.logging) $zkBox.utils.log('RunBatch: operations.length: "{0}"', $zkBox.utils.dump(operations.length));

        if (tempret)
        {
            if ((tempret.results) && (tempret.results.length == operations.length))
            {
                ctx.results = tempret.results;
                ret = new Array();
                for (var i = 0; i < tempret.results.length; i++)
                {
                    var obret = $zkBox.batch.DecryptObject(ctx, i);
                    ret.push(obret);
                }
                ctx.results = ret;

                if (tempret.transactionId)
                {
                    ctx.transactionId = tempret.transactionId;
                    ret.transactionId = tempret.transactionId;
                }
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('RunBatch: Call was executed ok, but the response was unexpected (batch operations and batch results differ in length). The operations were "{0}" and the return was "{1}"', $zkBox.utils.dump(operations), $zkBox.utils.dump(tempret));
            }
        }
    }

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then update the objects into the cache if needed
        $zkBox.cache.batch.ExecuteBatch(
            options,
            ctx);

        if ($zkBox.logging) $zkBox.utils.log('RunBatch: Success. Will return "{0}"', $zkBox.utils.dump(ret));
    }
    ////////////

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("RunBatch");

    if (ret)
    {
        if ($zkBox.logging) $zkBox.utils.log('RunBatch: Success. Will return "{0}"', $zkBox.utils.dump(ret));
    }
    else
    {
        if ($zkBox.logging) $zkBox.utils.log('RunBatch: Failed');
    }

    return ret;
}