﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns true
$zkBox.batch.AddObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "AddObject",
            //transactionId: null,
            type: null,
            data: null
        }, params);

    var type = params.type;
    var data = params.data;

    if (!data) throw "Batch.Objects.AddObject: data was not provided";

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);
    
    $zkBox.objects.crypto.Encrypt(data,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(encdata)
        {
            var ob = $zkBox.utils.clone(params);
            ob.data = encdata;
            operations.push(ob);

            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    return ret;
}

// returns true
$zkBox.batch.UpdateObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "UpdateObject",
            //transactionId: null,
            objectId: null,
            data: null
        }, params);

    var objectId = params.objectId;
    var data = params.data;

    if (!objectId) throw "Batch.Objects.UpdateObject: objectId was not provided";
    if (!data) throw "Batch.Objects.UpdateObject: data was not provided";

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);

    $zkBox.objects.crypto.Encrypt(data,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(encdata)
        {
            var ob = $zkBox.utils.clone(params);
            ob.data = encdata;
            operations.push(ob);
            
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    return ret;
}

// returns true
$zkBox.batch.GetObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "GetObject",
            //transactionId: null,
            objectId: null
        }, params);

    var objectId = params.objectId;

    if (!objectId) throw "Batch.Objects.GetObject: objectId was not provided";

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);

    // we'll do the call nomatter if the object is already in the local cache
    operations.push(params);
    ret = true;

    return ret;
}

// returns true
$zkBox.batch.DeleteObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "DeleteObject",
            //transactionId: null,
            objectId: null
        }, params);

    var objectId = params.objectId;

    if (!objectId) throw "Batch.Objects.DeleteObject: objectId was not provided";

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);

    operations.push(params);
    ret = true;
    
    return ret;
}

// returns true
$zkBox.batch.GetObjectsList = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "GetObjectsList",
            //transactionId: null,
            type: null,
            objectsList: null
        }, params);

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);

    operations.push(params);
    ret = true;
    
    return ret;
}

// returns true
$zkBox.batch.GetObjectsListIDs = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "GetObjectsListIDs",
            //transactionId: null
            type: null
        }, params);

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);

    operations.push(params);
    ret = true;

    return ret;
}



////////  combined operations  ////////

// set a single object; returns null if fail, true otherwise
$zkBox.batch.SetSingleObject = function(ctx, params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> BatchSetSingleObject] ({0}")',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.SetSingleObject(
        ctx,
        params,
        $zkBox.transactions.GetObjectsList,     // execute reads directly
        $zkBox.batch.UpdateObject,              // cumulate the changes into the batch
        $zkBox.batch.DeleteObject);             // cumulate the changes into the batch
    
    return ret;
}

// get a single object or default; returns null if fail
$zkBox.batch.GetSingleObject = function(ctx, params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> BatchGetSingleObject] ({0})',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.GetSingleObject(
        ctx,
        params,
        $zkBox.transactions.GetObjectsList,     // execute reads directly
        $zkBox.transactions.GetObject,          // execute reads directly
        $zkBox.transactions.AddObject,          // execute reads directly
        $zkBox.transactions.DeleteObject);      // execute reads directly
    
    return ret;
}





////////  decrypting the results  ////////

// returns the decrypted object from the position idx from the results
$zkBox.batch.DecryptObject = function(ctx, idx)
{
    var ret = null;

    var operation = $zkBox.utils.clone(ctx.operations[idx]);
    var result = $zkBox.utils.clone(ctx.results[idx]);

    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> DecryptObject] (idx={0} operation={1} result={2}")',
        idx,
        $zkBox.utils.dump(operation),
        $zkBox.utils.dump(result));

    switch (operation.operation)
    {
        case "AddObject":
            // there is nothing to decrypt
            ret = result.data;
            break;

        case "UpdateObject":
            // there is nothing to decrypt
            ret = result.data;
            break;

        case "GetObject":
            // decrypt the object's data
            {
                var encdata = result.data.data;
                $zkBox.objects.crypto.Decrypt(encdata,
                    function(msg)
                    {
                        $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
                    },
                    function(ob)
                    {
                        ret = result.data;
                        ret.data = ob;
                    },
                    function(error)
                    {
                        $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
                    });
            }
            break;

        case "DeleteObject":
            // there is nothing to decrypt
            ret = result.data;
            break;

        case "GetObjectsList":
            // decrypt the objects' data
            {
                ret = result.data.objects;
                var i;
                for (i = 0; i < ret.length; i++)
                {
                    var encdata = ret[i].data;
                    $zkBox.objects.crypto.Decrypt(encdata,
                        function(msg)
                        {
                            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
                        },
                        function(ob)
                        {
                            ret[i].data = ob;
                        },
                        function(error)
                        {
                            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
                        });
                }
            }
            break;

        case "GetObjectsListIDs":
            // there is nothing to decrypt
            ret = result.data;
            break;

        default:
            throw "Batch.Objects: Unhandled operation type '" + operation.operation + "'";
    }

    if ($zkBox.logging) $zkBox.utils.log('DecryptObject: will return: {0}',
        $zkBox.utils.dump(ret));

    return ret;
}
