﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns true if the operation succeeded, null otherwise
$zkBox.cache.batch.ExecuteBatch = function(options, ctx)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Batch.ExecuteBatch (options="{0}" ctx="{1}")]',
            $zkBox.utils.dump(options),
            $zkBox.utils.dump(ctx));

        var transactionId;
        var shouldCommit;
        // compute the transaction id and if we should commit it
        {
            if (true == options.atStartBeginTransaction)
            {
                // we have a transaction at the start

                if (true == options.atEndCommitTransaction)
                {
                    // we are commiting the transaction at the end, so the transaction is internal on the server
                    // assume a transaction id for the client

                    transactionId = "TRAN" + $zkBox.utils.getUnique();
                    shouldCommit = true;
                }
                else
                {
                    // the transaction is open, so we should have it in the return
                    transactionId = ctx.transactionId;
                    shouldCommit = false;
                }
            }
            else
            {
                // we're not starting a transaction

                if (true == options.atEndCommitTransaction)
                {
                    // but we are commiting it, so the transaction id perhaps was provided
                    transactionId = options.transactionId
                    shouldCommit = true;
                }
                else
                {
                    // and we're not closing any transaction
                    transactionId = null;
                    shouldCommit = false;
                }
            }
        }

        var operationsCache = ctx.operationsCache;
        var results = ctx.results;

        for (var i = 0; i < operationsCache.length; i++)
        {
            var operation = operationsCache[i];
            var result = results[i];

            var opTransactionId = transactionId;
            if (operation.transactionId)
            {
                opTransactionId = operation.transactionId;
            }

            switch (operation.operation)
            {
                case "AddObject":
                    {
                        var ob =
                        {
                            objectId: result.objectId,
                            type: operation.type,
                            data: operation.data
                        };
                        $zkBox.cache.transactions.AddObject(
                            opTransactionId,
                            ob);
                    }
                    break;

                case "UpdateObject":
                    {
                        var ob =
                        {
                            objectId: operation.objectId,
                            data: operation.data
                        };
                        $zkBox.cache.transactions.UpdateObject(
                            opTransactionId,
                            ob);
                    }
                    break;

                case "GetObject":
                    {
                        $zkBox.cache.transactions.GetObjectUpdate(
                            opTransactionId,
                            result);
                    }
                    break;

                case "DeleteObject":
                    {
                        var ob =
                        {
                            objectId: operation.objectId
                        };
                        $zkBox.cache.transactions.DeleteObject(
                            opTransactionId,
                            ob);
                    }
                    break;

                case "GetObjectsList":
                    {
                        var ctx =
                        {
                            type: operation.type,
                            objectsList: operation.objectsList,
                            ret: result
                        };
                        $zkBox.cache.transactions.GetObjectsListUpdate(
                            opTransactionId,
                            ctx);
                    }
                    break;

                case "GetObjectsListIDs":
                    // GetObjectsListIDs is not using caching (TODO_LATER: use caching?)
                    break;

                default:
                    throw "Cache.Batch: Unhandled operation type '" + operation.operation + "'";
            }
        }

        if (shouldCommit)
        {
            // reflect the changes into the cache; ignore the result from the cache update
            $zkBox.cache.transactions.core.CommitTransaction(transactionId);
        }

        return true;
    }

    return null;
}
