﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//TODO_LATER: make the key prefix 'objects-type' (and the similar others) shorter?

$zkBox.cache.core.AddObject = function(ob)
{
    ob = jQuery.extend(
    {
        objectId: null,
        type: null
    }, ob);

    var objectId = ob.objectId;
    var type = ob.type;

    // set the single object into the cache
    $zkBox.utils.cache.setItem('object-' + objectId, ob);

    // set all objects by type
    {
        var objectIdsOfType = $zkBox.utils.cache.getItem('objects-type-' + type);
        if (null != objectIdsOfType)
        {
            objectIdsOfType.push(objectId);
            $zkBox.utils.cache.setItem('objects-type-' + type, objectIdsOfType);
        }
    }

    // set all objects
    {
        var objectIds = $zkBox.utils.cache.getItem('objects-all');
        if (null != objectIds)
        {
            objectIds.push(objectId);
            $zkBox.utils.cache.setItem('objects-all', objectIds);
        }
    }

    return true;
}



$zkBox.cache.core.UpdateObject = function(ob)
{
    ob = jQuery.extend(
    {
        objectId: null
    }, ob);

    var objectId = ob.objectId;

    // update the single object's data into the cache if the object already exists
    var obc = $zkBox.utils.cache.getItem('object-' + objectId);
    if (null != obc)
    {
        obc.data = ob.data;
        $zkBox.utils.cache.setItem('object-' + objectId, obc);
    }

    return true;
}



$zkBox.cache.core.GetObjectCheck = function(objectId)
{
    var ob = $zkBox.utils.cache.getItem('object-' + objectId);
    if (null != ob)
    {
        if ($zkBox.logging) $zkBox.utils.log('Cache hit: GetObject("{0}") will return "{1}"', objectId, $zkBox.utils.dump(ob));
        return ob;
    }

    return null;
}

$zkBox.cache.core.GetObjectUpdate = function(ob)
{
    ob = jQuery.extend(
    {
        objectId: null
    }, ob);

    var objectId = ob.objectId;

    // set the single object into the cache
    $zkBox.utils.cache.setItem('object-' + objectId, ob);

    return true;
}



$zkBox.cache.core.DeleteObject = function(ob)
{
    ob = jQuery.extend(
    {
        objectId: null,
        type: null
    }, ob);

    var objectId = ob.objectId;
    var type = ob.type;

    // delete the single object
    $zkBox.utils.cache.removeItem('object-' + objectId);

    // all objects by type
    {
        var objectIdsOfType = $zkBox.utils.cache.getItem('objects-type-' + type);
        if (null != objectIdsOfType)
        {
            var found = $zkBox.utils.removeItemFromArray(objectIdsOfType, objectId);

            if (true == found)
            {
                if ($zkBox.logging) $zkBox.utils.log('We removed the item from the objects-type list. The new value is: {0}', $zkBox.utils.dump(objectIdsOfType));
                $zkBox.utils.cache.setItem('objects-type-' + type, objectIdsOfType);
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We do not have the item in the objects-type list. The objectId is "{0}" and the list is: {1}', objectId, $zkBox.utils.dump(objectIdsOfType));
            }
        }
    }

    // all objects
    {
        var objectIds = $zkBox.utils.cache.getItem('objects-all');
        if (null != objectIds)
        {
            var found = $zkBox.utils.removeItemFromArray(objectIds, objectId);

            if (true == found)
            {
                if ($zkBox.logging) $zkBox.utils.log('We removed the item from the objects-all list. The new value is: {0}', $zkBox.utils.dump(objectIds));
                $zkBox.utils.cache.setItem('objects-all', objectIds);
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We do not have the item in the objects-all list. The objectId is "{0}" and the list is: {1}', objectId, $zkBox.utils.dump(objectIds));
            }
        }
    }
    
    return true;
}



$zkBox.cache.core.GetObjectsListInit = function(ctx)
{
    ctx.requestType = 0;
    ctx.postprocessRequest = true;
    ctx.existingObjectsList = null;
    ctx.newObjectsList = null;

    // compute the request type
    {
        if (null != ctx.type)
        {
            // we assume we want objects by type
            ctx.requestType = 2; // all objects by type
        }
        else
        {
            if (null != ctx.objectsList)
            {
                // we assume we want a list of objects
                ctx.requestType = 1; // objects list by ids
            }
            else
            {
                // we'll assume we want all objects
                ctx.requestType = 3; // all objects
            }
        }
        //if ($zkBox.logging) $zkBox.utils.log("requestType={0}", ctx.requestType);
    }

    // objects list by ids
    if (1 == ctx.requestType)
    {
        ctx.existingObjectsList = new Array();
        ctx.newObjectsList = new Array();
    }
}

$zkBox.cache.core.GetObjectsListCheck = function(ctx)
{
    if ($zkBox.logging) $zkBox.utils.log('Cache.Core.GetObjectsListCheck (ctx="{0}")]',
            $zkBox.utils.dump(ctx));

    // objects list by ids
    if (1 == ctx.requestType)
    {
        // we are checking to see which objects we already have in cache
        for (var i = 0, olength = ctx.objectsList.length; i < olength; i++)
        {
            var objectId = ctx.objectsList[i];
            var ob = $zkBox.utils.cache.getItem('object-' + objectId);
            if (null != ob)
            {
                ctx.existingObjectsList.push(ob);
            }
            else
            {
                ctx.newObjectsList.push(objectId);
            }
        }
        if ($zkBox.logging) $zkBox.utils.log("existingObjectsList: {0}", $zkBox.utils.dump(ctx.existingObjectsList));
        if ($zkBox.logging) $zkBox.utils.log("newObjectsList: {0}", $zkBox.utils.dump(ctx.newObjectsList));
        ctx.objectsList = ctx.newObjectsList;

        if (ctx.newObjectsList.length <= 0)
        {
            if ($zkBox.logging) $zkBox.utils.log('We already have all the objects in cache');
            // we already have the objects list
            ctx.ret = ctx.existingObjectsList;
            ctx.executeRequest = false;
            ctx.postprocessRequest = false;
        }
        else
        {
            if ($zkBox.logging) $zkBox.utils.log('We already have {0} objects in cache. We will request only {1} objects', ctx.existingObjectsList.length, ctx.newObjectsList.length);
        }
    }

    if ((2 == ctx.requestType)  // all objects by type
        ||
        (3 == ctx.requestType)) // all objects
    {
        var objectIdsOfType;
        if (2 == ctx.requestType)
        {
            // all objects by type
            objectIdsOfType = $zkBox.utils.cache.getItem('objects-type-' + ctx.type);
        }
        else
        {
            // all objects
            objectIdsOfType = $zkBox.utils.cache.getItem('objects-all');
        }

        if (null != objectIdsOfType)
        {
            if ($zkBox.logging) $zkBox.utils.log('We already have the list of objects (of this type): {0}', $zkBox.utils.dump(objectIdsOfType));

            ctx.ret = new Array();
            for (var i = 0, olength = objectIdsOfType.length; i < olength; i++)
            {
                var objectId = objectIdsOfType[i];
                var ob = $zkBox.utils.cache.getItem('object-' + objectId);
                if (null != ob)
                {
                    ctx.ret.push(ob);
                }
                else
                {
                    if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We have the object "{0}" listed in the objects (of this type) list, but the object was not found in the cache. The list is: {1}', objectId, objectIdsOfType);
                }
            }
            ctx.executeRequest = false;
            ctx.postprocessRequest = false;
        }
    }

    return true;
}

$zkBox.cache.core.GetObjectsListUpdate = function(ctx)
{
    // if we need postprocessing...
    if (true == ctx.postprocessRequest)
    {
        // objects list by ids
        if (1 == ctx.requestType)
        {
            if ($zkBox.logging) $zkBox.utils.log('Add the {0} returned objects to the cache', ctx.ret.length);
            for (var i = 0, rlength = ctx.ret.length; i < rlength; i++)
            {
                var ob = ctx.ret[i];
                var objectId = ob.objectId;
                $zkBox.utils.cache.setItem('object-' + objectId, ob);
            }

            if ($zkBox.logging) $zkBox.utils.log('Add {0} existing cached objects to the return list', ctx.existingObjectsList.length);
            for (var i = 0, elength = ctx.existingObjectsList.length; i < elength; i++)
            {
                ctx.ret.push(ctx.existingObjectsList[i]);
            }
        }

        if ((2 == ctx.requestType)  // all objects by type
            ||
            (3 == ctx.requestType)) // all objects
        {
            if ($zkBox.logging) $zkBox.utils.log('Add the {0} returned objects to the cache', ctx.ret.length);
            var objectIdsOfType = new Array();
            for (var i = 0, rlength = ctx.ret.length; i < rlength; i++)
            {
                var ob = ctx.ret[i];
                var objectId = ob.objectId;
                $zkBox.utils.cache.setItem('object-' + objectId, ob);
                objectIdsOfType.push(objectId);
            }

            // set the list of objects (of this type)
            if (2 == ctx.requestType)
            {
                // all objects by type
                $zkBox.utils.cache.setItem('objects-type-' + ctx.type, objectIdsOfType);
            }
            else
            {
                // all objects
                $zkBox.utils.cache.setItem('objects-all', objectIdsOfType);
            }
        }
    }
    return true;
}

