﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.cache.transactions.core.operations = new Array();

// queue an cache change request; return true if successful, null otherwise
$zkBox.cache.transactions.core.QueueOperation = function(transactionId, operation, fn, params)
{
    if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.Core.QueueOperation (transactionId="{0}" operation="{1}" fn="{2}" params="{3}")]',
            transactionId,
            operation,
            $zkBox.utils.dump(fn),
            $zkBox.utils.dump(params));

    if (!transactionId) throw "Cache.Transactions.Core.QueueOperation: transactionId was not provided";
    if (!operation) throw "Cache.Transactions.Core.QueueOperation: operation was not provided";

    if (!$zkBox.cache.transactions.core.operations[transactionId])
    {
        $zkBox.cache.transactions.core.operations[transactionId] = new Array();
    }

    $zkBox.cache.transactions.core.operations[transactionId].push(
    {
        fn: fn,
        operation: operation,
        params: params
    });

    return true;
}


// make the changes into the cache for a given transaction; return true if succeded, null otherwise
$zkBox.cache.transactions.core.CommitTransaction = function(transactionId)
{
    if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.Core.CommitTransaction (transactionId="{0}")]',
            transactionId);

    if (!transactionId) throw "Cache.Transactions.Core.CommitTransaction: transactionId was not provided";

    var ret = null;

    if ($zkBox.cache.transactions.core.operations[transactionId])
    {
        var tran = $zkBox.cache.transactions.core.operations[transactionId];
        if ($zkBox.logging) $zkBox.utils.log('Cache.Transactions.Core.CommitTransaction: We have {0} operations to commit', tran.length);

        ret = true;
        for (var i = 0; i < tran.length; i++)
        {
            var op = tran[i];
            if ($zkBox.logging) $zkBox.utils.log('Cache.Transactions.Core.CommitTransaction: commiting operation #{0} to the cache: "{1}"',
                i,
                $zkBox.utils.dump(op));

            tempret = op.fn(op.params);
            if (!tempret)
            {
                if ($zkBox.logging) $zkBox.utils.log('Cache.Transactions.Core.CommitTransaction: Failure while commiting to the cache an operation. Abort the cache updates.');
                ret = null;
                break;
            }
        }
    }
    else
    {
        // the transaction was not found
        if ($zkBox.logging) $zkBox.utils.log('Cache.Transactions.Core.CommitTransaction: transactionId was not found');
    }

    $zkBox.cache.transactions.core.operations[transactionId] = null;
    
    if ($zkBox.logging) $zkBox.utils.log('Cache.Transactions.Core.CommitTransaction will return {0}',
        ret);
    return ret;
}


// remove the queued cache changes for a given transaction; return true if succeded, null otherwise
$zkBox.cache.transactions.core.RollbackTransaction = function(transactionId)
{
    if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.Core.RollbackTransaction (transactionId="{0}")]',
            transactionId);

    if (!transactionId) throw "Cache.Transactions.Core.RollbackTransaction: transactionId was not provided";

    $zkBox.cache.transactions.core.operations[transactionId] = null;

    return true;
}
