﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns true if the operation succeeded, null otherwise
$zkBox.cache.transactions.AddObject = function(transactionId, ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.AddObject (transactionId="{0}" ob="{1}")]',
            transactionId,
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.Transactions.AddObject: ob.objectId was not provided";

        if (transactionId)
        {
            // we are in a transaction, so we'll queue the cache add request
            return $zkBox.cache.transactions.core.QueueOperation(
                transactionId,
                "AddObject",
                $zkBox.cache.core.AddObject,
                ob);
        }
        else
        {
            // the operation is not in a transaction, so make the change into the cache directly
            return $zkBox.cache.core.AddObject(ob);
        }
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.transactions.UpdateObject = function(transactionId, ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.UpdateObject (transactionId="{0}" ob="{1}")]',
            transactionId,
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.Transactions.UpdateObject: ob.objectId was not provided";

        if (transactionId)
        {
            // we are in a transaction, so we'll queue the cache update request
            return $zkBox.cache.transactions.core.QueueOperation(
                transactionId,
                "UpdateObject",
                $zkBox.cache.core.UpdateObject,
                ob);
        }
        else
        {
            // the operation is not in a transaction, so make the change into the cache directly
            return $zkBox.cache.core.UpdateObject(ob);
        }
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.transactions.GetObjectUpdate = function(transactionId, ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.GetObjectUpdate (transactionId="{0}" ob="{1}")]',
            transactionId,
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.Transactions.GetObjectUpdate: ob.objectId was not provided";

        if (transactionId)
        {
            // we are in a transaction, so we'll queue the cache update request
            return $zkBox.cache.transactions.core.QueueOperation(
                transactionId,
                "GetObjectUpdate",
                $zkBox.cache.core.GetObjectUpdate,
                ob);
        }
        else
        {
            // the operation is not in a transaction, so make the change into the cache directly
            return $zkBox.cache.core.GetObjectUpdate(ob);
        }
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.transactions.DeleteObject = function(transactionId, ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.DeleteObject (transactionId="{0}" ob="{1}")]',
            transactionId,
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.Transactions.DeleteObject: ob.objectId was not provided";

        if (transactionId)
        {
            // we are in a transaction, so we'll queue the cache delete request
            return $zkBox.cache.transactions.core.QueueOperation(
                transactionId,
                "DeleteObject",
                $zkBox.cache.core.DeleteObject,
                ob);
        }
        else
        {
            // the operation is not in a transaction, so make the change into the cache directly
            return $zkBox.cache.core.DeleteObject(ob);
        }
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.transactions.GetObjectsListUpdate = function(transactionId, ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.GetObjectsListUpdate (transactionId="{0}" ob="{1}")]',
            transactionId,
            $zkBox.utils.dump(ob));

        if (transactionId)
        {
            // we are in a transaction, so we'll queue the cache update request
            return $zkBox.cache.transactions.core.QueueOperation(
                transactionId,
                "GetObjectsListUpdate",
                $zkBox.cache.core.GetObjectsListUpdate,
                ob);
        }
        else
        {
            // the operation is not in a transaction, so make the change into the cache directly
            return $zkBox.cache.core.GetObjectsListUpdate(ob);
        }
    }

    return null;
}
