// srp4net 1.0
// SRP for .NET - A JavaScript/C# .NET library for implementing the SRP authentication protocol
// http://code.google.com/p/srp4net/
// Copyright 2010, Sorin Ostafiev (http://www.ostafiev.com/)
// License: GPL v3 (http://www.gnu.org/licenses/gpl-3.0.txt)


$zkBox.crypto =
{
    sha512: sha.create("SHA-512"),

    hash: function(message)
    {
        // hash a message
        // outputs a 128 hex chars string (512bits)
        return base16_encode(this.sha512.hash(str2utf8(message)));
    },

    hashHex: function(hexmessage)
    {
        // hash a hex string
        // outputs a 128 hex chars string (512bits)
        return base16_encode(this.sha512.hash(base16_decode(((0 == hexmessage.length % 2) ? '' : '0') + hexmessage)));
    },

    //encode: function(clearText, key)
    //{
    //    // encode a string using a key as string
    //    // outputs a base64 string
    //    var encoded = GibberishAES.enc(clearText, key);
    //    return encoded;
    //},
    //
    //decode: function(encodedText, key)
    //{
    //    // decode a string using a key as string
    //    // input is base64 and the output is the original string
    //    var decoded = GibberishAES.dec(encodedText, key);
    //    return decoded
    //},
    
    encodeRaw: function(clearText, keyRaw)
    {
        // encode a string using a key in raw format
        // outputs a base64 string
        var encoded = GibberishAES.encRaw(clearText, keyRaw);
        return encoded;
    },

    decodeRaw: function(encodedText, keyRaw)
    {
        // decode a string using a key in raw format
        // input is base64 and the output is the original string
        var decoded = GibberishAES.decRaw(encodedText, keyRaw);
        return decoded
    },

    cryptokey: function(pass)
    {
        // generate a crypto key from a pass (for inputing into the AES)
        // input is a 128 hex chars (512bits) string and the output is a 32 chars (256bits) string
        // pass is already a hash, so we'll just reduce from 512 bits to 256 bits
        if (128 != pass.length)
        {
            if ($zkBox.logging) $zkBox.utils.log("Unexpected pass.length '" + pass.length + "'");
            return pass;
        }

        // pass the password through a hmac hash 50 times using
        // the previous value as input key at the next iteration
        var pnext = this.hashHex(pass);
        var i;
        var ptmp;
        for (i = 0; i < 50; i++)
        {
            ptmp = this.hmachash(pass, pnext);
            pass = pnext;
            pnext = ptmp;
        }
        pass = pnext;

        var ar = base16_decode(pass);
        if (64 != ar.length)
        {
            if ($zkBox.logging) $zkBox.utils.log("Unexpected ar.length '" + ar.length + "'");
            return pass;
        }

        var ret = new Array();
        for (i = 0; i < 32; i++)
        {
            ret[i] = ar[i] ^ ar[32 + i];
        }

        var strret = char2str(ret)
        if (32 != strret.length)
        {
            if ($zkBox.logging) $zkBox.utils.log("Unexpected strret.length '" + strret.length + "'");
            return pass;
        }

        return strret;
    },

    hmachash: function(key, message)
    {
        return hex_hmac_sha512(key, message).toUpperCase();
    },

    getNonce: function()
    {
        return $zkBox.utils.getUnique().toUpperCase();
    }
};
