// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.data.server =
{
    // server data layer
    // persist the data on the server

    Call: function(
        isAsynchronous,
        userIsAuthenticated,
        action,
        data,
        fnSuccess,
        fnError)
    {
        if (isAsynchronous) // if is asynchronous
        {
            // put it in the execution queue
            return this.WSq(userIsAuthenticated, action, data, fnSuccess, fnError);
        }
        else
        {
            // if is synchronous, run it directly
            return this.WS(userIsAuthenticated, action, data, fnSuccess, fnError);
        }
    },


    WS: function(userIsAuthenticated, action, data, fnSuccess, fnError)
    {
        try
        {
            return $.ajax(this.WScoreGetOptions(userIsAuthenticated, action, data, fnSuccess, fnError));
        }
        catch (ex)
        {
            if ($zkBox.logging) $zkBox.utils.log("Exception occurred in WS: " + ex);
            fnError(1);
        }
    },

    WSq: function(userIsAuthenticated, action, data, fnSuccess, fnError)
    {
        try
        {
            // using action as queue's name
            return $zkBox.jquery.ajaxq(action, this.WScoreGetOptions(userIsAuthenticated, action, data, fnSuccess, fnError));
        }
        catch (ex)
        {
            if ($zkBox.logging) $zkBox.utils.log("Exception occurred in WSq: " + ex);
            fnError(1);
        }
    },

    isEmpty: function(obj)
    {
        for (var i in obj)
        {
            return false;
        }
        return true;
    },

    WScoreGetOptions: function(userIsAuthenticated, action, data, fnSuccess, fnError)
    {
        var method = "";
        var resource = "";
        var pos = action.indexOf(" ");

        if (pos <= 0)
        {
            // no action? this shouldn't happen
            method = action;
        }
        else
        {
            method = action.substr(0, pos);
            resource = escape("/" + action.substr(1 + pos));
        }

        //var method = action.split(" ", 2)[0];
        //var resource = "/" + action.split(" ", 2)[1];
        //if ($zkBox.logging) $zkBox.utils.log("action: '{0}'", action);
        //if ($zkBox.logging) $zkBox.utils.log("method: '{0}'", method);
        //if ($zkBox.logging) $zkBox.utils.log("resource: '{0}'", resource);

        if (!(("POST" == method) || ("PUT" == method) || ("DELETE" == method) || ("GET" == method)))
        {
            throw ("Method can be only GET, POST, PUT, DELETE, but '" + method + "' was received");
        }

        var fullresource = $zkBox.conf.ProxyUrl + resource;
        if ($zkBox.logging) $zkBox.utils.log("Ajax call: [" + method + "] " + fullresource);
        var errmsg;

        var hasData = (("POST" == method) || ("PUT" == method));
        var strdata = (this.isEmpty(data)) ? (hasData ? "{}" : null) : $zkBox.utils.toJSON(data);
        var contenttype = hasData ? "application/json; charset=utf-8" : "";
        var datatype = "json";


        if (true == userIsAuthenticated)
        {
            //sign the request

            if ('' != $zkBox.status.sessionKey)
            {
                var nonce = $zkBox.crypto.getNonce();
                if ($zkBox.logging) $zkBox.utils.log("method: " + method);

                var dataToSign = method + "\n" +
                         resource + "\n" +
                         nonce + "\n" +
                         contenttype + "\n" +
                         ((null == strdata) ? "" : strdata) + "\n";

                var signature = $zkBox.crypto.hmachash($zkBox.status.sessionKey, dataToSign)
                if ($zkBox.logging) $zkBox.utils.log("signing key: '" + $zkBox.status.sessionKey + "'");
                if ($zkBox.logging) $zkBox.utils.log("dataToSign length: " + dataToSign.length);
                if ($zkBox.logging) $zkBox.utils.log("dataToSign:\n" + dataToSign);
                if ($zkBox.logging) $zkBox.utils.log("nonce: '" + nonce + "'");
                if ($zkBox.logging) $zkBox.utils.log("signature: '" + signature + "'");
                var userAuth = nonce + signature;

                $zkBox.status.updateUserAuth(userAuth);
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log("An authenticated method was called, but the user is not authenticated");
            }
        }
        else
        {
            $zkBox.status.updateUserAuth(null);
        }


        return {
            type: method,
            contentType: contenttype,
            //contentType: "application/x-www-form-urlencoded",
            async: false,
            url: fullresource,
            data: strdata,
            dataType: datatype,

            success: function(msg)
            {
                if ((typeof (msg) !== 'undefined') && (msg) && (typeof (msg.error) !== 'undefined'))
                {
                    if (msg.error === 0) // success
                    {
                        //if ($zkBox.logging) $zkBox.utils.log("Success: " + $zkBox.utils.dump(msg.data));
                        fnSuccess(msg.data);
                    }
                    else // some logical error occured
                    {
                        if ($zkBox.logging) $zkBox.utils.log("Logical error occured: " + msg.error);
                        fnError(msg.error);
                    }
                }
                else
                {
                    errmsg = $zkBox.utils.msg(
                        'Failure when calling the "{0}" action. The server returned "' + $zkBox.utils.dump(msg) + '"',
                        action);
                    if ($zkBox.logging) $zkBox.utils.log(errmsg);
                    $zkBox.conf.ShowError(10);
                }

            },

            error: function(xhr, msg, error)
            {
                var errmsg = $zkBox.utils.msg(
                    'Failure when calling the "{0}" action',
                    action);
                if ($zkBox.logging) $zkBox.utils.log(errmsg);

                var errcode;
                try
                {
                    var err = eval("(" + xhr.responseText + ")");
                    errmsg = err.message;
                    errcode = err.error;
                    if ($zkBox.logging) $zkBox.utils.log('Error occurred. Server returned: code = "' + errcode + '", message = "' + errmsg + "'");
                }
                catch (ex)
                {
                    if ($zkBox.logging) $zkBox.utils.log('Exception occured while evaluating the response');
                }

                $zkBox.conf.ShowError(11); // Failure when calling method
                fnError(11);
            }
        };
    }
}
