﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


////////  combined operations  ////////

// set a single object; returns null if fail, true otherwise
$zkBox.objects.combined.SetSingleObject = function(
    ctx,
    params,
    fnGetObjectsList,
    fnUpdateObject, 
    fnDeleteObject)
{
    params = jQuery.extend(
    {
        transactionId: null,
        type: null,
        data: null
    }, params);

    var transactionId = params.transactionId;
    var type = params.type;
    var data = params.data;

    if (null == type)
    {
        //if ($zkBox.logging) $zkBox.utils.log('SetSingleObject was called, but the type is missing.');
        throw 'SetSingleObject was called, but the type is missing.';
        return null;
    }

    var ret = fnGetObjectsList(ctx, { transactionId: transactionId, type: type });

    if (null != ret)
    {
        if (1 == ret.length)
        {
            var objectId = ret[0].objectId;
            ret = fnUpdateObject(
            ctx,
            {
                transactionId: transactionId,
                objectId: objectId,
                data: data
            });
        }
        else
        {
            if (0 == ret.length)
            {
                if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We got zero objects of the requested type.');
                ret = null;
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We got more than one object. However, we will update the first one.');

                // delete the objects that are in excess
                try
                {
                    if ($zkBox.logging) $zkBox.utils.log('Trying to delete the excess objects');
                    var i;
                    for (i = 1; i < ret.length; i++)
                    {
                        var obId = ret[i].objectId;
                        if ($zkBox.logging) $zkBox.utils.log('Trying to delete the excess object id: "' + obId + '"');
                        if (fnDeleteObject(ctx, { transactionId: transactionId, objectId: obId }))
                        {
                            if ($zkBox.logging) $zkBox.utils.log('Excess object id: "' + obId + '" was successfully deleted');
                        }
                        else
                        {
                            if ($zkBox.logging) $zkBox.utils.log('Failure while trying to delete the excess object id: "' + obId + '"');
                        }
                    }
                }
                catch (ex)
                {
                    if ($zkBox.logging) $zkBox.utils.log("Exception occurred in SetSingleObject when deleting the excess objects: " + ex);
                }

                var objectId = ret[0].objectId;
                ret = fnUpdateObject(
                ctx,
                {
                    transactionId: transactionId,
                    objectId: objectId,
                    data: data
                });
            }
        }
    }

    return ret;
}

// get an single object or default; returns null if fail
$zkBox.objects.combined.GetSingleObject = function(
    ctx,
    params,
    fnGetObjectsList,
    fnGetObject,
    fnAddObject,
    fnDeleteObject)
{
    params = jQuery.extend(
    {
        transactionId: null,
        type: null,
        dataDefault: null
    }, params);

    var transactionId = params.transactionId;
    var type = params.type;
    var dataDefault = params.dataDefault;

    if (null == type)
    {
        //if ($zkBox.logging) $zkBox.utils.log('GetSingleObject was called, but the type is missing.');
        throw 'GetSingleObject was called, but the type is missing.';
        return null;
    }

    var ret = fnGetObjectsList(ctx, { transactionId: transactionId, type: type });

    if (null != ret)
    {
        if (1 == ret.length)
        {
            var objectId = ret[0].objectId;
            ret = fnGetObject(ctx, { transactionId: transactionId, objectId: objectId });

            if (null != ret)
            {
                // we've got the object id from the storage, so return the data
                ret = ret.data;
            }
        }
        else
        {
            if (0 == ret.length)
            {
                if ($zkBox.logging) $zkBox.utils.log('There is no object of this type. We will add it.');

                ret = fnAddObject(
                ctx,
                {
                    transactionId: transactionId,
                    type: type,
                    data: dataDefault
                });
                if (null != ret)
                {
                    if ($zkBox.logging) $zkBox.utils.log('We will now return the default value.');
                    ret = dataDefault;
                }
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We got more than one object. However, we will return the first one.');

                // delete the objects that are in excess
                try
                {
                    if ($zkBox.logging) $zkBox.utils.log('Trying to delete the excess objects');
                    var i;
                    for (i = 1; i < ret.length; i++)
                    {
                        var obId = ret[i].objectId;
                        if ($zkBox.logging) $zkBox.utils.log('Trying to delete the excess object id: "' + obId + '"');
                        if (fnDeleteObject(ctx, { transactionId: transactionId, objectId: obId }))
                        {
                            if ($zkBox.logging) $zkBox.utils.log('Excess object id: "' + obId + '" was successfully deleted');
                        }
                        else
                        {
                            if ($zkBox.logging) $zkBox.utils.log('Failure while trying to delete the excess object id: "' + obId + '"');
                        }
                    }
                }
                catch (ex)
                {
                    if ($zkBox.logging) $zkBox.utils.log("Exception occurred in GetSingleObject when deleting the excess objects: " + ex);
                }

                var objectId = ret[0].objectId;
                ret = fnGetObject(ctx, { transactionId: transactionId, objectId: objectId });

                if (null != ret)
                {
                    // we've got the object id from the storage, so return the data
                    ret = ret.data;
                }
            }
        }
    }

    return ret;
}
