// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// zkBox Objects
////////  core operations  ////////

$zkBox.objects.core.AddObject = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> AddObject] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        transactionId: null,
        type: null,
        data: null
    }, params);

    var transactionId = params.transactionId;
    var type = params.type;
    var data = params.data;

    $zkBox.objects.crypto.Encrypt(data,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(encdata)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Adding the object');

            var resource = ((null == transactionId) ? "" : "transaction/" + transactionId + "/") + ((null == type) ? "object" : ("object/type/" + type));
            $zkBox.model.Call(false, true, "POST " + resource, {
                data: encdata
            },
                function(data)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was added');
                    $zkBox.utils.runFunction("fnSuccess", fnSuccess)(data);
                },
                function(error)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while adding the object');
                    $zkBox.utils.runFunction("fnError", fnError)(error);
                });
        },
        $zkBox.utils.runFunction("fnError", fnError));
}

$zkBox.objects.core.UpdateObject = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> UpdateObject] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null,
        data: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;
    var data = params.data;

    $zkBox.objects.crypto.Encrypt(data,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(encdata)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Updating the object');

            var resource = ((null == transactionId) ? "" : "transaction/" + transactionId + "/") + "object/" + objectId;
            $zkBox.model.Call(false, true, "PUT " + resource, {
                data: encdata
            },
                function(data)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was updated');
                    $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
                },
                function(error)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while updating the object');
                    $zkBox.utils.runFunction("fnError", fnError)(error);
                });
        },
        $zkBox.utils.runFunction("fnError", fnError));
}

$zkBox.objects.core.SetObject = function(cryptoKeyRaw, params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> SetObject] ({0}, {1})',
        $zkBox.utils.dump(cryptoKeyRaw),
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        newUserVersion: null,
        objectId: null,
        type: null,
        data: null
    }, params);

    var newUserVersion = params.newUserVersion;
    var objectId = params.objectId;
    var type = params.type;
    var data = params.data;

    $zkBox.objects.crypto.EncryptCore(cryptoKeyRaw, data,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(encdata)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Setting the object');

            $zkBox.model.Call(false, true, "PUT " + "user/update/" + newUserVersion + "/object/" + objectId, 
                {
                    type: type,
                    data: encdata
                },
                function(data)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was set');
                    $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
                },
                function(error)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while setting the object');
                    $zkBox.utils.runFunction("fnError", fnError)(error);
                });
        },
        $zkBox.utils.runFunction("fnError", fnError));
}

$zkBox.objects.core.GetObject = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> GetObject] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;

    $zkBox.utils.runFunction("fnProgress", fnProgress)('Retrieving the object');
    $zkBox.model.Call(false, true, "GET object/" + objectId, {},
        function(data)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was retrieved');

            var encdata = data.data;
            $zkBox.objects.crypto.Decrypt(encdata,
                $zkBox.utils.runFunction("fnProgress", fnProgress),
                function(ob)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was decrypted');
                    $zkBox.utils.runFunction("fnSuccess", fnSuccess)({
                        objectId: data.objectId,
                        type: data.type,
                        data: ob
                    });
                },
                $zkBox.utils.runFunction("fnError", fnError));
        },
        function(error)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while retrieving the object');
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
}

$zkBox.objects.core.DeleteObject = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> DeleteObject] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;

    $zkBox.utils.runFunction("fnProgress", fnProgress)('Deleting the object');
    var resource = ((null == transactionId) ? "" : "transaction/" + transactionId + "/") + "object/" + objectId;
    $zkBox.model.Call(false, true, "DELETE " + resource, {},
        function(data)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was deleted');
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)({
                type: data.type
            });
        },
        function(error)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while deleting the object');
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
}

$zkBox.objects.core.GetObjectsList = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> GetObjectsList] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        type: null,
        objectsList: null
    }, params);

    var type = params.type;
    var objectsList = params.objectsList;

    $zkBox.utils.runFunction("fnProgress", fnProgress)('Retrieving the objects list');

    var action;
    var sendData = {};
    if (null != type)
    {
        // get objects of a given type
        action = "GET objects/type/" + type;
    }
    else
    {
        if ($zkBox.utils.isArray(objectsList))
        {
            action = "PUT objects/list";
            sendData = objectsList;
        }
        else
        {
            // get all objects
            action = "GET objects";
        }
    }

    $zkBox.model.Call(false, true, action, sendData,
        function(data)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('The objects list was retrieved');
            if ($zkBox.logging) $zkBox.utils.log("We've got the following list: {0}", $zkBox.utils.dump(data));

            var objects = data.objects;
            var cnt = objects.length;

            if (0 < cnt)
            {
                if ($zkBox.logging) $zkBox.utils.log("There are {0} item(s) to decrypt", cnt);
                $zkBox.utils.runFunction("fnProgress", fnProgress)("There are " + cnt + " item(s) to decrypt");

                var hasErrors = false;
                for (var i = 0; i < cnt; i++)
                {
                    var ob = objects[i];
                    var encdata = ob.data;
                    $zkBox.objects.crypto.Decrypt(encdata,
                        $zkBox.utils.runFunction("fnProgress", fnProgress),
                        function(data)
                        {
                            objects[i] = {
                                objectId: ob.objectId,
                                type: ob.type,
                                data: data
                            };
                            $zkBox.utils.runFunction("fnProgress", fnProgress)('The object #' + i + ' was decrypted');

                        },
                        function(err)
                        {
                            hasErrors = true;
                            if ($zkBox.logging) $zkBox.utils.log('Failure while decripting the object #' + i + ' (objectid="' + ob.objectId + '")');
                            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while decripting the object #' + i + '');
                            i = cnt;
                            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while decrypting the objects list');
                            $zkBox.utils.runFunction("fnError", fnError)(err);
                        });
                }

                if (false == hasErrors)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('The objects list was decrypted');
                    $zkBox.utils.runFunction("fnSuccess", fnSuccess)(objects);
                }
            }
            else
            {
                $zkBox.utils.runFunction("fnProgress", fnProgress)("There are no items to decrypt");
                if ($zkBox.logging) $zkBox.utils.log("There are no items to decrypt");
                $zkBox.utils.runFunction("fnSuccess", fnSuccess)(objects);
            }
        },
        function(error)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while retrieving the objects list');
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
}

$zkBox.objects.core.GetObjectsListIDs = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> GetObjectsListIDs] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        type: null
    }, params);

    var type = params.type;

    $zkBox.utils.runFunction("fnProgress", fnProgress)('Retrieving the objects list IDs');

    var action;
    if (null != type)
    {
        // get objects IDs of a given type
        action = "GET objects/ids/type/" + type;
    }
    else
    {
        // get all objects IDs
        action = "GET objects/ids";
    }

    $zkBox.model.Call(false, true, action, {},
        function(data)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('The objects list IDs was retrieved');
            if ($zkBox.logging) $zkBox.utils.log("We've got the following list IDs: {0}", $zkBox.utils.dump(data));

            var objectsIDs = data.objectsIDs;

            $zkBox.utils.runFunction("fnProgress", fnProgress)('The objects list IDs was received');
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)(objectsIDs);
        },
        function(error)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while retrieving the objects list IDs');
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
}
