﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.objects.crypto.EncryptCore = function(cryptoKeyRaw, ob, fnProgress, fnSuccess, fnError)
{
    // encrypts an object

    if (typeof cryptoKeyRaw === "undefined")
    {
        fnError("The encryption key is missing. We cannon encrypt without the key.");
        return;
    }

    // light check to see if we have the key (32 * 8 = 256 bits)
    if (32 != cryptoKeyRaw.length)
    {
        fnError("The encryption key is invalid.");
        return;
    }

    fnProgress('Encrypting the data');

    if (typeof ob !== "undefined")
    {
        //if (null != ob)
        {
            if ($zkBox.logging) $zkBox.utils.log("Encrypting object: " + $zkBox.utils.dump(ob));

            try
            {
                var datajson = $zkBox.utils.toJSON(ob);
                if ($zkBox.logging) $zkBox.utils.log("JSON-ized object is: " + datajson);

                var encdata = $zkBox.crypto.encodeRaw(
                    datajson,
                    cryptoKeyRaw);

                if ($zkBox.logging) $zkBox.utils.log("Encrypted object: " + encdata);

                fnProgress('The data was encrypted');

                fnSuccess(encdata);
            }
            catch (ex)
            {
                if ($zkBox.logging) $zkBox.utils.log('Failure while encrypting the object: ' + ex);
                fnProgress('Failure while encrypting the object');
                fnError('Failure while encrypting the object');
            }

        }
        //else
        //{
        //    fnError("Cannot encrypt a null object");
        //}
    }
    else
    {
        fnError("Cannot encrypt an undefined object");
    }
}

$zkBox.objects.crypto.Encrypt = function(ob, fnProgress, fnSuccess, fnError)
{
    return $zkBox.objects.crypto.EncryptCore(
        $zkBox.status.cryptoKeyRaw,
        ob,
        fnProgress,
        fnSuccess, 
        fnError);
}

$zkBox.objects.crypto.Decrypt = function(encdata, fnProgress, fnSuccess, fnError)
{
    // decrypts an object

    if (typeof $zkBox.status.cryptoKeyRaw === "undefined")
    {
        fnError("The decryption key is missing. We cannon decrypt without the key.");
        return;
    }

    // light check to see if we have the key (32 * 8 = 256 bits)
    if (32 != $zkBox.status.cryptoKeyRaw.length)
    {
        fnError("The decryption key is invalid.");
        return;
    }

    fnProgress('Decrypting the data');

    if (typeof encdata !== "undefined")
    {
        //if (null != ob)
        {
            if ($zkBox.logging) $zkBox.utils.log("Decrypting string: " + encdata);

            try
            {
                var decdata = $zkBox.crypto.decodeRaw(
                    encdata,
                    $zkBox.status.cryptoKeyRaw);
                if ($zkBox.logging) $zkBox.utils.log("Decrypted data: " + decdata);


                var ob = $zkBox.utils.evalJSON(decdata);
                if ($zkBox.logging) $zkBox.utils.log("Resulted object: " + $zkBox.utils.dump(ob));

                fnProgress('The data was decrypted');

                fnSuccess(ob);
            }
            catch (ex)
            {
                if ($zkBox.logging) $zkBox.utils.log('Failure while decrypting the data:' + ex);
                fnProgress('Failure while decrypting the data');
                fnError('Failure while decrypting the data');
            }
        }
        //else
        //{
        //    fnError("Cannot encrypt a null object");
        //}
    }
    else
    {
        fnError("Cannot decrypt an undefined string");
    }
}
