// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//////////////////////////////////
////////  api operations  ////////
//////////////////////////////////


/** caching

    we have the following cache entries:
    for objects:
        - object-[objectId]     e.g.:   object-0123456789ABCDEF

    for getobjectslist:
        - by type               e.g.:   objects-type-[some type goes here]
        - by list
        - all                   e.g.:   objects-all

**/


// returns the objectId if the operation succeeded, null otherwise
$zkBox.AddObject = function(params)
{
    var ret = null;

    params = jQuery.extend(
    {
        type: null,
        data: null
    }, params);

    var type = params.type;
    var data = params.data;

    if (!data) throw "Objects.AddObject: data was not provided";

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("AddObject");

    $zkBox.objects.core.AddObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = data.objectId;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("AddObject");

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then add the object to the cache
        $zkBox.cache.AddObject(
        {
            objectId: ret,
            type: type,
            data: data
        });
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Objects.AddObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// return true if the operation succeeded, null otherwise
$zkBox.UpdateObject = function(params)
{
    var ret = null;

    params = jQuery.extend(
    {
        objectId: null,
        data: null
    }, params);

    var objectId = params.objectId;
    var data = params.data;

    if (!objectId) throw "Objects.UpdateObject: objectId was not provided";
    if (!data) throw "Objects.UpdateObject: data was not provided";

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("UpdateObject");

    $zkBox.objects.core.UpdateObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("UpdateObject");

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then update the object into the cache
        $zkBox.cache.UpdateObject(
        {
            objectId: objectId,
            data: data
        });
    }
    ////////////

    ret = (null == ret) ? null : true;

    if ($zkBox.logging) $zkBox.utils.log("Objects.UpdateObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// returns the object if the operation succeeded, null otherwise
$zkBox.GetObject = function(params)
{
    var ret = null;

    params = jQuery.extend(
    {
        objectId: null
    }, params);

    var objectId = params.objectId;

    if (!objectId) throw "Objects.GetObject: objectId was not provided";

    //// caching
    var ob = $zkBox.cache.GetObjectCheck(objectId);
    if (null != ob)
    {
        return ob;
    }
    ////////////

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("GetObject");

    $zkBox.objects.core.GetObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(ob)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(ob);
            ret = ob;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("GetObject");

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then set the object into the cache
        $zkBox.cache.GetObjectUpdate(ret);
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Objects.GetObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// return data (object's type) if the operation succeeded, null otherwise
$zkBox.DeleteObject = function(params)
{
    var ret = null;

    params = jQuery.extend(
    {
        objectId: null
    }, params);

    var objectId = params.objectId;

    if (!objectId) throw "Objects.DeleteObject: objectId was not provided";

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("DeleteObject");

    $zkBox.objects.core.DeleteObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = data;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("DeleteObject");

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then delete the object from the cache
        $zkBox.cache.DeleteObject(
        {
            objectId: objectId,
            type: ret.type
        });
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Objects.DeleteObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// returns the objects list if the operation succeeded, null otherwise
$zkBox.GetObjectsList = function(params)
{
    params = jQuery.extend(
    {
        type: null,
        objectsList: null
    }, params);

    var ctx =
    {
        type: params.type,
        objectsList: params.objectsList,
        executeRequest: true,
        ret: null
    }

    //// caching
    $zkBox.cache.GetObjectsListCheck(ctx);
    ////////////

    if (ctx.executeRequest)
    {
        $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("GetObjectsList");

        $zkBox.objects.core.GetObjectsList(
            {
                type: ctx.type,
                objectsList: ctx.objectsList
            },
            function(msg)
            {
                $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
            },
            function(objects)
            {
                $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(objects);
                ctx.ret = objects;
            },
            function(error)
            {
                $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
            });

        $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("GetObjectsList");
    }
    else
    {
        if ($zkBox.logging) $zkBox.utils.log('There is no need to make the request');
    }

    //// caching
    if (null != ctx.ret)
    {
        // if the operation succeeded, do something according to the operation
        $zkBox.cache.GetObjectsListUpdate(ctx);
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Objects.GetObjectsList: will return {0}", $zkBox.utils.dump(ctx.ret));
    return ctx.ret;
}

// returns the objects list IDs if the operation succeeded, null otherwise
$zkBox.GetObjectsListIDs = function(params)
{
    var ret = null;

    params = jQuery.extend(
    {
        type: null
    }, params);

    //// caching
    // GetObjectsListIDs is not using caching (TODO_LATER: use caching?)
    ////////////

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("GetObjectsListIDs");

    $zkBox.objects.core.GetObjectsListIDs(
        {
            type: params.type
        },
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(objectsIDs)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(objectsIDs);
            ret = objectsIDs;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("GetObjectsListIDs");

    if ($zkBox.logging) $zkBox.utils.log("Objects.GetObjectsListIDs: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}



////////  combined operations  ////////

// set a single object
// returns null if fail, true otherwise
$zkBox.SetSingleObject = function(params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> SetSingleObject] ({0})',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.SetSingleObject(
        null,   // ctx
        params,
        function(ctx, params)
        {
            return $zkBox.GetObjectsList(params);
        },
        function(ctx, params)
        {
            return $zkBox.UpdateObject(params);
        },
        function(ctx, params)
        {
            return $zkBox.DeleteObject(params);
        });

    if ($zkBox.logging) $zkBox.utils.log("Objects.SetSingleObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}


// get an single object or default
// returns null if fail
$zkBox.GetSingleObject = function(params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> GetSingleObject] ({0})',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.GetSingleObject(
        null,   // ctx
        params,
        function(ctx, params)
        {
            return $zkBox.GetObjectsList(params);
        },
        function(ctx, params)
        {
            return $zkBox.GetObject(params);
        },
        function(ctx, params)
        {
            return $zkBox.AddObject(params);
        },
        function(ctx, params)
        {
            return $zkBox.DeleteObject(params);
        });

    if ($zkBox.logging) $zkBox.utils.log("Objects.GetSingleObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}
