﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//////////////////////////////////
////////  api operations  ////////
//////////////////////////////////

// returns the last operation's result if the transaction succeeded, null otherwise
$zkBox.RunTransaction = function()
{
    if ($zkBox.logging) $zkBox.utils.log('[TRANSACTIONS -> RunTransaction (arguments="{0}")]',
        $zkBox.utils.dump(arguments));

    var ret = null;
    var operations = arguments;

    if (operations.length < 1)
    {
        if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failure because there are no operations to run');
        return ret;
    }

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("RunTransaction");

    var ctx = null;
    var tranId = $zkBox.transactions.Begin(ctx);

    if (null != tranId)
    {
        var fnAddObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                type: null,
                data: null
            }, params);

            return $zkBox.transactions.AddObject(ctx, params);
        };
        var fnUpdateObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                objectId: null,
                data: null
            }, params);

            return $zkBox.transactions.UpdateObject(ctx, params);
        };
        var fnGetObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                objectId: null
            }, params);

            return $zkBox.transactions.GetObject(ctx, params);
        };
        var fnDeleteObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                objectId: null
            }, params);

            return $zkBox.transactions.DeleteObject(ctx, params);
        };
        var fnGetObjectsList = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                type: null,
                objectsList: null
            }, params);

            return $zkBox.transactions.GetObjectsList(ctx, params);
        };
        var fnGetObjectsListIDs = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                type: null
            }, params);

            return $zkBox.transactions.GetObjectsListIDs(ctx, params);
        };
        var fnSetSingleObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                type: null,
                data: null
            }, params);

            return $zkBox.transactions.SetSingleObject(ctx, params);
        };
        var fnGetSingleObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                type: null,
                dataDefault: null
            }, params);

            return $zkBox.transactions.GetSingleObject(ctx, params);
        };



        // the transaction was created
        var failed = false;
        var opidx = 0;
        var oplength = operations.length;
        var results = new Array();
        var prevResult;

        while ((!failed) && (opidx < oplength))
        {
            // if we didn't fail and there are still operations...
            // execute the operation

            // construct the transaction context object
            var tran =
            {
                tranId: tranId,
                results: results,
                prevResult: prevResult,
                AddObject: fnAddObject,
                UpdateObject: fnUpdateObject,
                GetObject: fnGetObject,
                DeleteObject: fnDeleteObject,
                GetObjectsList: fnGetObjectsList,
                GetObjectsListIDs: fnGetObjectsListIDs,
                SetSingleObject: fnSetSingleObject,
                GetSingleObject: fnGetSingleObject
            }

            if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Running operation #{0}...', opidx);
            var opret = operations[opidx](tran); // pass the transaction context as parameter

            if (opret)
            {
                // the operation succeeded, go to the next operation
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Operation #{0} finished with success', opidx);
                results.push(opret); // add this operation's result to the results list
                prevResult = opret; // set the previous operation result
                opidx++; // move to the next operation
            }
            else
            {
                // the operation failed; abort
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Operation #{0} failed', opidx);
                failed = true;
            }
        }

        if (!failed)
        {
            // the operations were successfull; try to commit the transaction
            if ($zkBox.logging) $zkBox.utils.log('RunTransaction: The operations were successful; we will commit the transaction...');

            var commit = $zkBox.transactions.Commit(ctx, tranId);
            if (true == commit)
            {
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: The transaction was committed');
                ret = prevResult; // everything went OK, return the last operation's result
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failure while committing the transaction');
            }
        }
        else
        {
            // there was an issue while executing an operation; try to rollback the transaction
            if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failure at operation #{0}; we will rollback the transaction...', opidx);

            var rollback = $zkBox.transactions.Rollback(ctx, tranId);

            if (true == rollback)
            {
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: The transaction was rolled back');
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failure while rolling back the transaction');
            }
        }
    }
    else
    {
        if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failure while creating the transaction');
    }

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("RunTransaction");

    if (ret)
    {
        if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Success. Will return "{0}"', $zkBox.utils.dump(ret));
    }
    else
    {
        if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failed');
    }

    return ret;
}
