﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns the objectId if the operation succeeded, null otherwise
$zkBox.transactions.AddObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
    {
        transactionId: null,
        type: null,
        data: null
    }, params);

    var transactionId = params.transactionId;
    var type = params.type;
    var data = params.data;

    if (!data) throw "Transactions.Objects.AddObject: data was not provided";

    $zkBox.objects.core.AddObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = data.objectId;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then add/queue the object to the cache
        $zkBox.cache.transactions.AddObject(
            transactionId,
            {
                objectId: ret,
                type: type,
                data: data
            });
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.AddObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// return true if the operation succeeded, null otherwise
$zkBox.transactions.UpdateObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null,
        data: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;
    var data = params.data;

    if (!objectId) throw "Transactions.Objects.UpdateObject: objectId was not provided";
    if (!data) throw "Transactions.Objects.UpdateObject: data was not provided";

    $zkBox.objects.core.UpdateObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then update/queue the object into the cache
        $zkBox.cache.transactions.UpdateObject(
            transactionId,
            {
                objectId: objectId,
                data: data
            });
        
    }
    ////////////

    ret = (null == ret) ? null : true;

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.UpdateObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// returns the object if the operation succeeded, null otherwise
$zkBox.transactions.GetObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;

    if (!objectId) throw "Transactions.Objects.GetObject: objectId was not provided";

    //// caching
    var ob = $zkBox.cache.GetObjectCheck(objectId); // GetObjectCheck is not transactional
    if (null != ob)
    {
        return ob;
    }
    ////////////

    $zkBox.objects.core.GetObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(ob)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(ob);
            ret = ob;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then set the object into the cache
        $zkBox.cache.transactions.GetObjectUpdate(
            transactionId,
            ret);
    }
    ////////////
    
    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.GetObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// return data if the operation succeeded, null otherwise
$zkBox.transactions.DeleteObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;

    if (!objectId) throw "Transactions.Objects.DeleteObject: objectId was not provided";

    $zkBox.objects.core.DeleteObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = data;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then delete the object from the cache
        $zkBox.cache.transactions.DeleteObject(
            transactionId,
            {
                objectId: objectId,
                type: ret.type
            });
        
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.DeleteObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// returns the objects list if the operation succeeded, null otherwise
$zkBox.transactions.GetObjectsList = function(ctx, params)
{
    params = jQuery.extend(
    {
        transactionId: null,
        type: null,
        objectsList: null
    }, params);

    var transactionId = params.transactionId;

    var ctx =
    {
        type: params.type,
        objectsList: params.objectsList,
        executeRequest: true,
        ret: null
    }

    //// caching
    $zkBox.cache.GetObjectsListCheck(ctx); // GetObjectsListCheck is not transactional
    ////////////

    if (ctx.executeRequest)
    {
        $zkBox.objects.core.GetObjectsList(
            {
                type: ctx.type,
                objectsList: ctx.objectsList
            },
            function(msg)
            {
                $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
            },
            function(objects)
            {
                $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(objects);
                ctx.ret = objects;
            },
            function(error)
            {
                $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
            });
    }
    else
    {
        if ($zkBox.logging) $zkBox.utils.log('There is no need to make the request');
    }

    //// caching
    if (null != ctx.ret)
    {
        // if the operation succeeded, do something according to the operation
        $zkBox.cache.transactions.GetObjectsListUpdate(
            transactionId,
            ctx);
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.GetObjectsList: will return {0}", $zkBox.utils.dump(ctx.ret));
    return ctx.ret;
}

// returns the objects list IDs if the operation succeeded, null otherwise
$zkBox.transactions.GetObjectsListIDs = function(ctx, params)
{
    var ret = null;
    
    params = jQuery.extend(
    {
        transactionId: null,
        type: null
    }, params);

    var transactionId = params.transactionId;

    //// caching
    // GetObjectsListIDs is not using caching (TODO_LATER: use caching?)
    ////////////

    $zkBox.objects.core.GetObjectsListIDs(
        {
            type: params.type
        },
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(objectsIDs)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(objectsIDs);
            ret = objectsIDs;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.GetObjectsListIDs: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}


////////  combined operations  ////////

// set a single object; returns null if fail, true otherwise
$zkBox.transactions.SetSingleObject = function(ctx, params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> TransactionSetSingleObject] ({0}")',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.SetSingleObject(
        ctx,
        params,
        $zkBox.transactions.GetObjectsList,
        $zkBox.transactions.UpdateObject,
        $zkBox.transactions.DeleteObject);

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.SetSingleObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// get a single object or default; returns null if fail
$zkBox.transactions.GetSingleObject = function(ctx, params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> TransactionGetSingleObject] ({0})',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.GetSingleObject(
        ctx,
        params,
        $zkBox.transactions.GetObjectsList,
        $zkBox.transactions.GetObject,
        $zkBox.transactions.AddObject,
        $zkBox.transactions.DeleteObject);

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.GetSingleObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}
