﻿// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// zkBox Users
////////  core operations  ////////

$zkBox.users.core.CheckUser = function(user, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> CheckUser] (user="{0}")',
        user);

    $zkBox.crypto.srp.CheckUser(user,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        $zkBox.utils.runFunction("fnSuccess", fnSuccess),
        $zkBox.utils.runFunction("fnError", fnError));
};

$zkBox.users.core.AddUser = function(user, password, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> AddUser] (user="{0}", password="{1}")',
        user,
        password);

    $zkBox.crypto.srp.AddUser(user, password,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(user, pass)
        {
            $zkBox.users.core.AuthenticateUserCore(true, user, pass, fnProgress, fnSuccess, fnError);
        },
        $zkBox.utils.runFunction("fnError", fnError));
};

$zkBox.users.core.LogoutUser = function(fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> LogoutUser]');
    $zkBox.utils.runFunction("fnProgress", fnProgress)("Logging out and clearing the current session");

    $zkBox.crypto.srp.LogoutUser(
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function()
        {
            $zkBox.status.clearAuth();
            $zkBox.utils.runFunction("fnProgress", fnProgress)("Logout finished");
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
        },
        $zkBox.utils.runFunction("fnError", fnError));
};

$zkBox.users.core.UpdateUser = function(user, oldpassword, newpassword, fnStep, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> UpdateUser] (user="{0}", oldpassword="{1}", newpassword="{2}")',
        user,
        oldpassword,
        newpassword);

    if ($zkBox.crypto.srp.HashPass(oldpassword) != $zkBox.status.pass)
    {
        $zkBox.utils.runFunction("fnError", fnError)(101); // Invalid username or password
        return;
    }
    oldpassword = '';

    if ($zkBox.crypto.srp.HashUser(user) != $zkBox.status.user)
    {
        $zkBox.utils.runFunction("fnError", fnError)(101); // Invalid username or password
        return;
    }
    user = '';


    // TODO_LATER: optimize the change password function: run batch operations
    // ...when requesting objects, do it in a batch; we have a method to get the objects by id
    // ...say something like "give these 10 objects: id1, ... , id10", then a batch call for the next 10.. and so on
    // ...implement also SetObject to work in batch (to get an array of objects to update)


    $zkBox.crypto.srp.BeginUpdateUser($zkBox.status.user, newpassword,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(user, newpass, newUserVersion)
        {
            var shouldCommit = false;
            var shouldRollback = true;

            $zkBox.objects.core.GetObjectsListIDs({},
                $zkBox.utils.runFunction("fnProgress", fnProgress),
                function(objectsIDs)
                {
                    // computer the new crypto key
                    var newckey = $zkBox.crypto.cryptokey(newpass);
                    var newCryptoKey = base16_encode(str2char(newckey));
                    newckey = '';
                    var newCryptoKeyRaw = base16_decode(newCryptoKey);
                    newCryptoKey = '';

                    var hasErrors = false;
                    for (var i = 0; i < objectsIDs.length; i++)
                    {
                        if (typeof (fnStep) === "function")
                        {
                            fnStep(i + 1, objectsIDs.length);
                        }

                        var objectId = objectsIDs[i];
                        if ($zkBox.logging) $zkBox.utils.log("Reencrypting object " + objectId);

                        var object;
                        $zkBox.objects.core.GetObject(
                            {
                                objectId: objectId
                            },
                            $zkBox.utils.runFunction("fnProgress", fnProgress),
                            function(ob)
                            {
                                object = ob;
                            },
                            $zkBox.utils.runFunction("fnError", fnError));

                        if (object)
                        {
                            if ($zkBox.logging) $zkBox.utils.log("Object {0} was successfully retrieved", objectId);

                            var objectset = false;
                            $zkBox.objects.core.SetObject(
                                newCryptoKeyRaw,
                                {
                                    newUserVersion: newUserVersion,
                                    objectId: objectId,
                                    type: object.type,
                                    data: object.data
                                },
                                $zkBox.utils.runFunction("fnProgress", fnProgress),
                                function()
                                {
                                    objectset = true;
                                },
                                $zkBox.utils.runFunction("fnError", fnError));

                            if (true == objectset)
                            {
                                // the object was set
                            }
                            else
                            {
                                hasErrors = true;
                                if ($zkBox.logging) $zkBox.utils.log("Failure while setting the object {0}", objectId);
                                break;
                            }
                        }
                        else
                        {
                            hasErrors = true;
                            if ($zkBox.logging) $zkBox.utils.log("Failure while retrieving the object {0}", objectId);
                            break;
                        }
                    }

                    newCryptoKeyRaw = [];

                    if (false == hasErrors)
                    {
                        shouldCommit = true;
                        shouldRollback = false;
                    }
                },
                $zkBox.utils.runFunction("fnError", fnError));

            if (true == shouldCommit)
            {
                var commited = false;
                $zkBox.crypto.srp.CommitUpdateUser(newUserVersion,
                    $zkBox.utils.runFunction("fnProgress", fnProgress),
                    function()
                    {
                        commited = true;
                    },
                    $zkBox.utils.runFunction("fnError", fnError));

                if (true == commited)
                {
                    if ($zkBox.logging) $zkBox.utils.log("The user was successfully updated");
                    $zkBox.status.pass = newpass;
                    $zkBox.status.updateAuth();
                    $zkBox.users.core.AuthenticateUserCore(true, user, newpass, fnProgress, fnSuccess, fnError);
                }
            }

            if (true == shouldRollback)
            {
                var rolledback = false;
                $zkBox.crypto.srp.RollbackUpdateUser(newUserVersion,
                    $zkBox.utils.runFunction("fnProgress", fnProgress),
                    function()
                    {
                        rolledback = true;
                    },
                    $zkBox.utils.runFunction("fnError", fnError));

                if (true == rolledback)
                {
                    if ($zkBox.logging) $zkBox.utils.log("The user update was rolledback");
                }
            }
        },
        $zkBox.utils.runFunction("fnError", fnError));
};

$zkBox.users.core.DeleteUser = function(user, password, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> DeleteUser] (user="{0}", password="{1}")',
        user,
        password);

    if ($zkBox.crypto.srp.HashPass(password) != $zkBox.status.pass)
    {
        $zkBox.utils.runFunction("fnError", fnError)(101); // Invalid username or password
        return;
    }
    password = '';

    if ($zkBox.crypto.srp.HashUser(user) != $zkBox.status.user)
    {
        $zkBox.utils.runFunction("fnError", fnError)(101); // Invalid username or password
        return;
    }
    user = '';

    $zkBox.crypto.srp.DeleteUser($zkBox.status.user,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function()
        {
            $zkBox.status.clearAuth();
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
        },
        $zkBox.utils.runFunction("fnError", fnError));
};


$zkBox.users.core.AuthenticateUser = function(user, password, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> AuthenticateUser] (user="{0}", password="{1}")',
        user,
        password);
    $zkBox.users.core.AuthenticateUserCore(false, user, password, fnProgress, fnSuccess, fnError);
}

$zkBox.users.core.AuthenticateUserCore = function(dataIsHashed, user, password, fnProgress, fnSuccess, fnError)
{
    $zkBox.crypto.srp.AuthenticateUser(dataIsHashed, user, password,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(user, pass, userId, userVersion, sessionId, sessionKey, cryptoKey)
        {
            $zkBox.status.user = user;
            $zkBox.status.pass = pass;
            $zkBox.status.userId = userId;
            $zkBox.status.userVersion = userVersion;
            $zkBox.status.sessionId = sessionId;
            $zkBox.status.sessionKey = sessionKey;
            $zkBox.status.cryptoKey = cryptoKey;
            $zkBox.status.updateAuth();
            $zkBox.utils.cache.clearAllItems();
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
        },
        function(error)
        {
            $zkBox.status.clearAuth();
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
};
