// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//////////////////////////////////
////////  api operations  ////////
//////////////////////////////////

// returns true if the user exists, false otherwise or if the operation failed
$zkBox.CheckUser = function(user)
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("CheckUser");

    $zkBox.users.core.CheckUser(user,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = data.exists;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("CheckUser");

    return ret;
}

// returns true if the operation succeded, false otherwise
$zkBox.AddUser = function(user, password)
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("AddUser");

    $zkBox.users.core.AddUser(user, password,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("AddUser");

    return ret;
}

// returns true if the authentication succeded, false otherwise or if the operation failed
$zkBox.AuthenticateUser = function(user, password)
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("AuthenticateUser");

    $zkBox.users.core.AuthenticateUser(user, password,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("AuthenticateUser");
    
    return ret;
}

// returns true if the operation succeded, false otherwise
$zkBox.LogoutUser = function()
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("LogoutUser");

    $zkBox.users.core.LogoutUser(
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("LogoutUser");
    
    return ret;
}

// returns true if the operation succeded, false otherwise
$zkBox.UpdateUser = function(user, oldpassword, newpassword, fnStep)
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("UpdateUser");

    $zkBox.users.core.UpdateUser(user, oldpassword, newpassword, fnStep,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("UpdateUser");
    
    return ret;
}

// returns true if the operation succeded, false otherwise
$zkBox.DeleteUser = function(user, password)
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("DeleteUser");

    $zkBox.users.core.DeleteUser(user, password,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("DeleteUser");
    
    return ret;
}

// returns true if an user is successfully authenticated; false otherwise
$zkBox.IsAuthenticated = function()
{
    return $zkBox.status.isAuthenticated;
}
