// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.utils =
{
    log: function(msg, params)
    {
        var message = "";
        if (arguments.length <= 1)
        {
            message = msg;
        }
        else
        {
            params = $.makeArray(arguments).slice(1);
            message = this.format(msg, params);
        }

        toLocalFormat = function(date, theFormat)
        {
            var aVar = {
                MM: date.getMonth() + 1,
                DD: date.getDate(),
                YYYY: date.getFullYear(),
                hh: date.getHours(),
                mm: date.getMinutes(),
                ss: date.getSeconds(),
                ap: "AM"
            }
            if (theFormat.match(/\bap\b/i))
            {
                if (aVar.hh > 12)
                {
                    aVar.hh -= 12;
                    aVar.ap = "PM";
                }
            }
            for (v in aVar)
            {
                theFormat = theFormat.replace(v, f2(aVar[v]));
            }
            function f2(theVal)
            {
                if (theVal < 10) return "0" + theVal;
                return theVal;
            }
            return theFormat;
        }


        var currentTime = new Date()

        return $zkBox.jquery.logs(
            "[" + toLocalFormat(currentTime, "YYYY-MM-DD hh:mm:ss") + "] ZKBOX " +
            message);
    },

    dump: function(object)
    {
        return $zkBox.jquery.dump(object);
    },

    msg: function(source, params)
    {

        if (arguments.length == 1)
        {
            return this.format(source, "");
        }
        else
        {
            return this.format(source, params);
        }
    },

    format: function(source, params)
    {
        if (arguments.length == 1)
            return function()
            {
                var args = $.makeArray(arguments);
                args.unshift(source);
                return $.format.apply(this, args);
            };
        if (arguments.length > 2 && params.constructor != Array)
        {
            params = $.makeArray(arguments).slice(1);
        }
        if (params.constructor != Array)
        {
            params = [params];
        }
        $.each(params, function(i, n)
        {
            source = source.replace(new RegExp("\\{" + i + "\\}", "g"), n);
        });
        return source;
    },

    toJSON: function(object)
    {
        return JSON.stringify(object);
    },

    evalJSON: function(text)
    {
        return JSON.parse(text);
    },

    clone: function(obj)
    {
        // TODO_LATER: perhaps improve this
        return this.evalJSON(this.toJSON(obj));
    },

    isArray: function(obj)
    {
        // taken from jQuery library
        return Object.prototype.toString.call(obj) === "[object Array]";
    },

    runFunction: function(fnName, fn)
    {
        return function(msg)
        {
            var text = "[none]";

            if (typeof (msg) != "undefined")
            {
                text = $zkBox.utils.dump(msg);
            }

            if (typeof (fn) === "function")
            {
                if ($zkBox.logging) $zkBox.utils.log(fnName + ": " + text);
                fn(msg);
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log(fnName + " (default): " + text);
            }
        };
    },

    runFunctionOnly: function(fnName, fn)
    {
        return function(msg)
        {
            if (typeof (fn) === "function")
            {
                fn(msg);
            }
            else
            {
            }
        };
    },

    uniqueCounter: 0,
    uniqueAlphabet: "abcdefghijklmnopqrstuvwxyz0123456789",

    getUnique: function()
    {
        var uniq = '';

        // 4 chars from random
        {
            var rnd = Math.floor(1679616 * Math.random()); // 1679616 = 36^4

            var i;
            for (i = 0; i < 4; i++)
            {
                uniq += this.uniqueAlphabet.charAt(rnd % 36);
                rnd = Math.floor(rnd / 36);
            }
        }

        // 8 chars from ticks
        {
            var ticks = new Date().getTime();

            var i;
            for (i = 0; i < 8; i++)
            {
                uniq += this.uniqueAlphabet.charAt(ticks % 36);
                ticks = Math.floor(ticks / 36);
            }
        }

        // 4 chars from counter
        {
            var cnt = this.uniqueCounter;
            this.uniqueCounter++;

            var i;
            for (i = 0; i < 4; i++)
            {
                uniq += this.uniqueAlphabet.charAt(cnt % 36);
                cnt = Math.floor(cnt / 36);
            }
        }

        return uniq;
    },

    removeItemFromArray: function(ar, item)
    {
        var found = false;
        var j = 0;
        while (j < ar.length)
        {
            if (ar[j] == item)
            {
                found = true;
                ar.splice(j, 1);
            }
            else
            {
                j++;
            }
        }
        return found;
    }
};
