﻿// zkBox ASP.NET web proxy
// http://www.zkbox.com/developers/client/javascript/aspnet
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
// 
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Bitground.zkBox.Client.JS.WebProxy
{
    public class WSRet<W, U> 
        where W : new()
        where U : new()
    {
        private int _error;

        public int error
        {
            set
            {
                _error = value;

                switch (value)
                {
                    case 0:
                        _message = "Success";
                        break;

                    case 1:
                        _message = "System error";
                        break;
                    case 2:
                        _message = "Not authenticated";
                        break;

                    default:
                        _message = (0 == value) ? "" : "Server error: " + value.ToString();
                        break;
                }
            }
            get
            {
                return _error;
            }
        }

        private string _message;

        public string message
        {
            get
            {
                return _message;
            }
            set
            {
                _message = value;
            }
        }

        public U data;

        public string reference { get; set; }
        public string runtime { get; set; }

        public T ErrorOnly<T, D>()
            where T : WSRet<W, D>, new()
            where D : new()
        {
            return new T()
            {
                error = error,
                message = message
            };
        }

        private WSRet()
        {
        }

        public WSRet(CS.WSRet<W> wsRet)
        {
            if (null != wsRet)
            {
                this._error = wsRet.error;
                this._message = wsRet.message;
                this.reference = wsRet.reference;
                this.runtime = wsRet.runtime;
            }
        }
    }
}
