﻿<%--
zkBox JS + ASP.NET client - web sample
http://www.zkbox.com/developers/client/javascript/aspnet

Copyright 2010, Sorin Ostafiev
http://www.ostafiev.com/

Released under GPL version 3 license
http://www.gnu.org/licenses/gpl-3.0.txt
--%><%@ Page Language="C#" AutoEventWireup="true" Inherits="System.Web.UI.Page" %>
<%@ Import Namespace="Bitground.Common.Helpers" %>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" >
<head runat="server">
    <title>zkBox web client sample</title>
    <script type="text/javascript" src="https://api.zkbox.com/s/zkboxandjquery-1.0.js"></script>
    <style type="text/css">
        code 
        { 
            font-size: 8pt; 
            font-family: monaco, "Lucida Console", "Courier New", Courier, Monospace;
            background-color:#FFFFCC;
            border: 1px solid #DDD;
            padding: 2px 3px;
            vertical-align: 1px;
        }
        #progress
        {
            border: 1px solid red; 
            width: 800px; 
            margin-top: 10pt; 
            font-family: monaco, "Lucida Console", "Courier New", Courier, Monospace;
            padding: 2px 2px;
        }
        #progress div
        {
            padding: 0 5px;
            margin-bottom: 1px;
        }
    </style>
</head>
<body>
    <script runat="server">
        protected string ProxyUrl
        {
            get
            {
                return HttpContext.Current.Request.Url.AbsoluteUri
                    .Replace("http://", "https://")
                    .Replace("Default.aspx", "zkboxproxy"); // [TODO] if on IIS6, change "zkboxproxy" to "zkboxproxy.asmx"
            }
        }
    </script>
    
    This is the "zkBox web sample<br />
    <br />
    
    Using the following zkBox service: <code><%=ConfigurationManager.AppSettings["zkBox.apiUrl"]%></code><br />
    The proxy is here: <code><%= ProxyUrl %></code><br />
    <br />
    
    Get app info: <code><%=Params.GetObjectAsString(Bitground.zkBox.Client.JS.WebProxy.Handler.Application.GetInfo())%></code><br />
    <br />
    
    <input type="button" value="Run the demo!" id="runit" />
    <div id="progress"></div>
    
    
    <script type="text/javascript">
        progress(0, 'gray', 'transparent', 'Logger was started...');

        $('#runit').click(function()
        {
            initIt();
            runIt();
        });

        function progress(indent, foreColor, bkColor, s)
        {
            var ind = "";
            for (var i = 0; i < indent; i++)
            {
                ind += "&nbsp;";
            };

            $('#progress').append('<div>' + ind + s + '</div>');
            $('#progress div:last').css('background-color', bkColor).css('color', foreColor);
        }

        function info(msg)
        {
            progress(0, 'black', '#BE9', msg);
        }

        function errorOccurred(err)
        {
            var msg;
            if (err)
            {
                msg = "ABORT: " + err;
            }
            else
            {
                msg = "ABORT";
            }
            
            progress(0, 'white', 'red', msg);
        }
        
        function initIt()
        {
            // initialize zkBox
            $zkBox.Init({
                Log: false,     // set here to true for more detailed logging; the output will be the debug console (e.g FireBug)
                Cache: false,   // normally you should not do this and leave this option to its default value, which is true
                                //      but here, for demo purposes, in order to see some action for the GetObject operation
                                //      we're just disabling the local cache
                ProxyUrl: "<%=ProxyUrl%>"
            });

            // set the zkBox callbacks (methods that will be called in given moments when processing a request)
            $zkBox.SetCallbacks(
                function(fn)
                {
                    // start
                    progress(0, '#117', '#CCC', "ZKBOX: " + fn);
                },
                function(fn)
                {
                    // stop
                    progress(4, '#aaa', 'transparent', fn + " finished");
                },
                function(msg)
                {
                    // progress
                    progress(4, '#447', 'transparent', msg);
                },
                function(data)
                {
                    // success
                    progress(4, 'blue', 'transparent', 'RETURN: ' + $zkBox.utils.dump(data));
                },
                function(err)
                {
                    // error
                    progress(4, 'yellow', 'red', 'ERROR: ' + err);
                });
        }
        
        function runIt()
        {
            ////////////////////////////////////////////////////////////////////////
            // demo of the JavaScript zkBox client
            ////////////////////////////////////////////////////////////////////////

            
            var ret;
            var u = "john_" + "<%=UniqueId.GetPseudo()%>";
            var p = "sEcReTp@sS\/\/0rd";

            info("Using user='" + u + "' and password='" + p + "'");


            ////////////////////////////////////////////////////////////////////////
            info("Check the user");
            ret = $zkBox.CheckUser(u);
            if (ret)
            {
                return errorOccurred("User already exists??? This is strange...");
            }


            ////////////////////////////////////////////////////////////////////////
            info("Create the user");
            ret = $zkBox.AddUser(u, p);
            if (!ret)
            {
                return errorOccurred("Couldn't create the user");
            }


            ////////////////////////////////////////////////////////////////////////
            // AddUser is also authenticating the user it into the system
            // ...but for the demo, we'll also call the AuthenticateUser method
            info("Authenticate the user");
            ret = $zkBox.AuthenticateUser(u, p);
            if (!ret)
            {
                return errorOccurred("Couldn't authenticate the user");
            }


            ////////////////////////////////////////////////////////////////////////
            info("Add an object");
            ret = $zkBox.AddObject(
                {
                    type: "some-type",
                    data:
                    {
                        store: "Kjell",
                        amount: 46.5,
                        currency: "EUR"
                    }
                });
            if (!ret)
            {
                return errorOccurred("Couldn't add the object");
            }
            
            // we've got the objectId as return
            var objectId = ret;


            ////////////////////////////////////////////////////////////////////////
            info("Get object details");
            ret = $zkBox.GetObject(
                {
                    objectId: objectId
                });
            if (!ret)
            {
                return errorOccurred("Couldn't get the object");
            }
            info("We've got the following: " + $zkBox.utils.dump(ret));


            ////////////////////////////////////////////////////////////////////////
            info("Delete the object");
            ret = $zkBox.DeleteObject(
                {
                    objectId: objectId
                });
            if (!ret)
            {
                return errorOccurred("Couldn't delete the object");
            }


            ////////////////////////////////////////////////////////////////////////
            info("Delete the user");
            ret = $zkBox.DeleteUser(u, p);
            if (!ret)
            {
                return errorOccurred("Couldn't delete the user");
            }


            info("The demo finished without errors");
            return;
        }
    </script>
</body>
</html>
