﻿/*!*************************************************** 
 *   zkBox JavaScript client v1.0 + jQuery v1.3.2   * 
 ****************************************************/ 
 
/************************************************ 
**** jquery-1.3.2.js ***** 
************************************************/ 
/*!
 * jQuery JavaScript Library v1.3.2
 * http://jquery.com/
 *
 * Copyright (c) 2009 John Resig
 * Dual licensed under the MIT and GPL licenses.
 * http://docs.jquery.com/License
 *
 * Date: 2009-02-19 17:34:21 -0500 (Thu, 19 Feb 2009)
 * Revision: 6246
 */
(function(){

var 
	// Will speed up references to window, and allows munging its name.
	window = this,
	// Will speed up references to undefined, and allows munging its name.
	undefined,
	// Map over jQuery in case of overwrite
	_jQuery = window.jQuery,
	// Map over the $ in case of overwrite
	_$ = window.$,

	jQuery = window.jQuery = window.$ = function( selector, context ) {
		// The jQuery object is actually just the init constructor 'enhanced'
		return new jQuery.fn.init( selector, context );
	},

	// A simple way to check for HTML strings or ID strings
	// (both of which we optimize for)
	quickExpr = /^[^<]*(<(.|\s)+>)[^>]*$|^#([\w-]+)$/,
	// Is it a simple selector
	isSimple = /^.[^:#\[\.,]*$/;

jQuery.fn = jQuery.prototype = {
	init: function( selector, context ) {
		// Make sure that a selection was provided
		selector = selector || document;

		// Handle $(DOMElement)
		if ( selector.nodeType ) {
			this[0] = selector;
			this.length = 1;
			this.context = selector;
			return this;
		}
		// Handle HTML strings
		if ( typeof selector === "string" ) {
			// Are we dealing with HTML string or an ID?
			var match = quickExpr.exec( selector );

			// Verify a match, and that no context was specified for #id
			if ( match && (match[1] || !context) ) {

				// HANDLE: $(html) -> $(array)
				if ( match[1] )
					selector = jQuery.clean( [ match[1] ], context );

				// HANDLE: $("#id")
				else {
					var elem = document.getElementById( match[3] );

					// Handle the case where IE and Opera return items
					// by name instead of ID
					if ( elem && elem.id != match[3] )
						return jQuery().find( selector );

					// Otherwise, we inject the element directly into the jQuery object
					var ret = jQuery( elem || [] );
					ret.context = document;
					ret.selector = selector;
					return ret;
				}

			// HANDLE: $(expr, [context])
			// (which is just equivalent to: $(content).find(expr)
			} else
				return jQuery( context ).find( selector );

		// HANDLE: $(function)
		// Shortcut for document ready
		} else if ( jQuery.isFunction( selector ) )
			return jQuery( document ).ready( selector );

		// Make sure that old selector state is passed along
		if ( selector.selector && selector.context ) {
			this.selector = selector.selector;
			this.context = selector.context;
		}

		return this.setArray(jQuery.isArray( selector ) ?
			selector :
			jQuery.makeArray(selector));
	},

	// Start with an empty selector
	selector: "",

	// The current version of jQuery being used
	jquery: "1.3.2",

	// The number of elements contained in the matched element set
	size: function() {
		return this.length;
	},

	// Get the Nth element in the matched element set OR
	// Get the whole matched element set as a clean array
	get: function( num ) {
		return num === undefined ?

			// Return a 'clean' array
			Array.prototype.slice.call( this ) :

			// Return just the object
			this[ num ];
	},

	// Take an array of elements and push it onto the stack
	// (returning the new matched element set)
	pushStack: function( elems, name, selector ) {
		// Build a new jQuery matched element set
		var ret = jQuery( elems );

		// Add the old object onto the stack (as a reference)
		ret.prevObject = this;

		ret.context = this.context;

		if ( name === "find" )
			ret.selector = this.selector + (this.selector ? " " : "") + selector;
		else if ( name )
			ret.selector = this.selector + "." + name + "(" + selector + ")";

		// Return the newly-formed element set
		return ret;
	},

	// Force the current matched set of elements to become
	// the specified array of elements (destroying the stack in the process)
	// You should use pushStack() in order to do this, but maintain the stack
	setArray: function( elems ) {
		// Resetting the length to 0, then using the native Array push
		// is a super-fast way to populate an object with array-like properties
		this.length = 0;
		Array.prototype.push.apply( this, elems );

		return this;
	},

	// Execute a callback for every element in the matched set.
	// (You can seed the arguments with an array of args, but this is
	// only used internally.)
	each: function( callback, args ) {
		return jQuery.each( this, callback, args );
	},

	// Determine the position of an element within
	// the matched set of elements
	index: function( elem ) {
		// Locate the position of the desired element
		return jQuery.inArray(
			// If it receives a jQuery object, the first element is used
			elem && elem.jquery ? elem[0] : elem
		, this );
	},

	attr: function( name, value, type ) {
		var options = name;

		// Look for the case where we're accessing a style value
		if ( typeof name === "string" )
			if ( value === undefined )
				return this[0] && jQuery[ type || "attr" ]( this[0], name );

			else {
				options = {};
				options[ name ] = value;
			}

		// Check to see if we're setting style values
		return this.each(function(i){
			// Set all the styles
			for ( name in options )
				jQuery.attr(
					type ?
						this.style :
						this,
					name, jQuery.prop( this, options[ name ], type, i, name )
				);
		});
	},

	css: function( key, value ) {
		// ignore negative width and height values
		if ( (key == 'width' || key == 'height') && parseFloat(value) < 0 )
			value = undefined;
		return this.attr( key, value, "curCSS" );
	},

	text: function( text ) {
		if ( typeof text !== "object" && text != null )
			return this.empty().append( (this[0] && this[0].ownerDocument || document).createTextNode( text ) );

		var ret = "";

		jQuery.each( text || this, function(){
			jQuery.each( this.childNodes, function(){
				if ( this.nodeType != 8 )
					ret += this.nodeType != 1 ?
						this.nodeValue :
						jQuery.fn.text( [ this ] );
			});
		});

		return ret;
	},

	wrapAll: function( html ) {
		if ( this[0] ) {
			// The elements to wrap the target around
			var wrap = jQuery( html, this[0].ownerDocument ).clone();

			if ( this[0].parentNode )
				wrap.insertBefore( this[0] );

			wrap.map(function(){
				var elem = this;

				while ( elem.firstChild )
					elem = elem.firstChild;

				return elem;
			}).append(this);
		}

		return this;
	},

	wrapInner: function( html ) {
		return this.each(function(){
			jQuery( this ).contents().wrapAll( html );
		});
	},

	wrap: function( html ) {
		return this.each(function(){
			jQuery( this ).wrapAll( html );
		});
	},

	append: function() {
		return this.domManip(arguments, true, function(elem){
			if (this.nodeType == 1)
				this.appendChild( elem );
		});
	},

	prepend: function() {
		return this.domManip(arguments, true, function(elem){
			if (this.nodeType == 1)
				this.insertBefore( elem, this.firstChild );
		});
	},

	before: function() {
		return this.domManip(arguments, false, function(elem){
			this.parentNode.insertBefore( elem, this );
		});
	},

	after: function() {
		return this.domManip(arguments, false, function(elem){
			this.parentNode.insertBefore( elem, this.nextSibling );
		});
	},

	end: function() {
		return this.prevObject || jQuery( [] );
	},

	// For internal use only.
	// Behaves like an Array's method, not like a jQuery method.
	push: [].push,
	sort: [].sort,
	splice: [].splice,

	find: function( selector ) {
		if ( this.length === 1 ) {
			var ret = this.pushStack( [], "find", selector );
			ret.length = 0;
			jQuery.find( selector, this[0], ret );
			return ret;
		} else {
			return this.pushStack( jQuery.unique(jQuery.map(this, function(elem){
				return jQuery.find( selector, elem );
			})), "find", selector );
		}
	},

	clone: function( events ) {
		// Do the clone
		var ret = this.map(function(){
			if ( !jQuery.support.noCloneEvent && !jQuery.isXMLDoc(this) ) {
				// IE copies events bound via attachEvent when
				// using cloneNode. Calling detachEvent on the
				// clone will also remove the events from the orignal
				// In order to get around this, we use innerHTML.
				// Unfortunately, this means some modifications to
				// attributes in IE that are actually only stored
				// as properties will not be copied (such as the
				// the name attribute on an input).
				var html = this.outerHTML;
				if ( !html ) {
					var div = this.ownerDocument.createElement("div");
					div.appendChild( this.cloneNode(true) );
					html = div.innerHTML;
				}

				return jQuery.clean([html.replace(/ jQuery\d+="(?:\d+|null)"/g, "").replace(/^\s*/, "")])[0];
			} else
				return this.cloneNode(true);
		});

		// Copy the events from the original to the clone
		if ( events === true ) {
			var orig = this.find("*").andSelf(), i = 0;

			ret.find("*").andSelf().each(function(){
				if ( this.nodeName !== orig[i].nodeName )
					return;

				var events = jQuery.data( orig[i], "events" );

				for ( var type in events ) {
					for ( var handler in events[ type ] ) {
						jQuery.event.add( this, type, events[ type ][ handler ], events[ type ][ handler ].data );
					}
				}

				i++;
			});
		}

		// Return the cloned set
		return ret;
	},

	filter: function( selector ) {
		return this.pushStack(
			jQuery.isFunction( selector ) &&
			jQuery.grep(this, function(elem, i){
				return selector.call( elem, i );
			}) ||

			jQuery.multiFilter( selector, jQuery.grep(this, function(elem){
				return elem.nodeType === 1;
			}) ), "filter", selector );
	},

	closest: function( selector ) {
		var pos = jQuery.expr.match.POS.test( selector ) ? jQuery(selector) : null,
			closer = 0;

		return this.map(function(){
			var cur = this;
			while ( cur && cur.ownerDocument ) {
				if ( pos ? pos.index(cur) > -1 : jQuery(cur).is(selector) ) {
					jQuery.data(cur, "closest", closer);
					return cur;
				}
				cur = cur.parentNode;
				closer++;
			}
		});
	},

	not: function( selector ) {
		if ( typeof selector === "string" )
			// test special case where just one selector is passed in
			if ( isSimple.test( selector ) )
				return this.pushStack( jQuery.multiFilter( selector, this, true ), "not", selector );
			else
				selector = jQuery.multiFilter( selector, this );

		var isArrayLike = selector.length && selector[selector.length - 1] !== undefined && !selector.nodeType;
		return this.filter(function() {
			return isArrayLike ? jQuery.inArray( this, selector ) < 0 : this != selector;
		});
	},

	add: function( selector ) {
		return this.pushStack( jQuery.unique( jQuery.merge(
			this.get(),
			typeof selector === "string" ?
				jQuery( selector ) :
				jQuery.makeArray( selector )
		)));
	},

	is: function( selector ) {
		return !!selector && jQuery.multiFilter( selector, this ).length > 0;
	},

	hasClass: function( selector ) {
		return !!selector && this.is( "." + selector );
	},

	val: function( value ) {
		if ( value === undefined ) {			
			var elem = this[0];

			if ( elem ) {
				if( jQuery.nodeName( elem, 'option' ) )
					return (elem.attributes.value || {}).specified ? elem.value : elem.text;
				
				// We need to handle select boxes special
				if ( jQuery.nodeName( elem, "select" ) ) {
					var index = elem.selectedIndex,
						values = [],
						options = elem.options,
						one = elem.type == "select-one";

					// Nothing was selected
					if ( index < 0 )
						return null;

					// Loop through all the selected options
					for ( var i = one ? index : 0, max = one ? index + 1 : options.length; i < max; i++ ) {
						var option = options[ i ];

						if ( option.selected ) {
							// Get the specifc value for the option
							value = jQuery(option).val();

							// We don't need an array for one selects
							if ( one )
								return value;

							// Multi-Selects return an array
							values.push( value );
						}
					}

					return values;				
				}

				// Everything else, we just grab the value
				return (elem.value || "").replace(/\r/g, "");

			}

			return undefined;
		}

		if ( typeof value === "number" )
			value += '';

		return this.each(function(){
			if ( this.nodeType != 1 )
				return;

			if ( jQuery.isArray(value) && /radio|checkbox/.test( this.type ) )
				this.checked = (jQuery.inArray(this.value, value) >= 0 ||
					jQuery.inArray(this.name, value) >= 0);

			else if ( jQuery.nodeName( this, "select" ) ) {
				var values = jQuery.makeArray(value);

				jQuery( "option", this ).each(function(){
					this.selected = (jQuery.inArray( this.value, values ) >= 0 ||
						jQuery.inArray( this.text, values ) >= 0);
				});

				if ( !values.length )
					this.selectedIndex = -1;

			} else
				this.value = value;
		});
	},

	html: function( value ) {
		return value === undefined ?
			(this[0] ?
				this[0].innerHTML.replace(/ jQuery\d+="(?:\d+|null)"/g, "") :
				null) :
			this.empty().append( value );
	},

	replaceWith: function( value ) {
		return this.after( value ).remove();
	},

	eq: function( i ) {
		return this.slice( i, +i + 1 );
	},

	slice: function() {
		return this.pushStack( Array.prototype.slice.apply( this, arguments ),
			"slice", Array.prototype.slice.call(arguments).join(",") );
	},

	map: function( callback ) {
		return this.pushStack( jQuery.map(this, function(elem, i){
			return callback.call( elem, i, elem );
		}));
	},

	andSelf: function() {
		return this.add( this.prevObject );
	},

	domManip: function( args, table, callback ) {
		if ( this[0] ) {
			var fragment = (this[0].ownerDocument || this[0]).createDocumentFragment(),
				scripts = jQuery.clean( args, (this[0].ownerDocument || this[0]), fragment ),
				first = fragment.firstChild;

			if ( first )
				for ( var i = 0, l = this.length; i < l; i++ )
					callback.call( root(this[i], first), this.length > 1 || i > 0 ?
							fragment.cloneNode(true) : fragment );
		
			if ( scripts )
				jQuery.each( scripts, evalScript );
		}

		return this;
		
		function root( elem, cur ) {
			return table && jQuery.nodeName(elem, "table") && jQuery.nodeName(cur, "tr") ?
				(elem.getElementsByTagName("tbody")[0] ||
				elem.appendChild(elem.ownerDocument.createElement("tbody"))) :
				elem;
		}
	}
};

// Give the init function the jQuery prototype for later instantiation
jQuery.fn.init.prototype = jQuery.fn;

function evalScript( i, elem ) {
	if ( elem.src )
		jQuery.ajax({
			url: elem.src,
			async: false,
			dataType: "script"
		});

	else
		jQuery.globalEval( elem.text || elem.textContent || elem.innerHTML || "" );

	if ( elem.parentNode )
		elem.parentNode.removeChild( elem );
}

function now(){
	return +new Date;
}

jQuery.extend = jQuery.fn.extend = function() {
	// copy reference to target object
	var target = arguments[0] || {}, i = 1, length = arguments.length, deep = false, options;

	// Handle a deep copy situation
	if ( typeof target === "boolean" ) {
		deep = target;
		target = arguments[1] || {};
		// skip the boolean and the target
		i = 2;
	}

	// Handle case when target is a string or something (possible in deep copy)
	if ( typeof target !== "object" && !jQuery.isFunction(target) )
		target = {};

	// extend jQuery itself if only one argument is passed
	if ( length == i ) {
		target = this;
		--i;
	}

	for ( ; i < length; i++ )
		// Only deal with non-null/undefined values
		if ( (options = arguments[ i ]) != null )
			// Extend the base object
			for ( var name in options ) {
				var src = target[ name ], copy = options[ name ];

				// Prevent never-ending loop
				if ( target === copy )
					continue;

				// Recurse if we're merging object values
				if ( deep && copy && typeof copy === "object" && !copy.nodeType )
					target[ name ] = jQuery.extend( deep, 
						// Never move original objects, clone them
						src || ( copy.length != null ? [ ] : { } )
					, copy );

				// Don't bring in undefined values
				else if ( copy !== undefined )
					target[ name ] = copy;

			}

	// Return the modified object
	return target;
};

// exclude the following css properties to add px
var	exclude = /z-?index|font-?weight|opacity|zoom|line-?height/i,
	// cache defaultView
	defaultView = document.defaultView || {},
	toString = Object.prototype.toString;

jQuery.extend({
	noConflict: function( deep ) {
		window.$ = _$;

		if ( deep )
			window.jQuery = _jQuery;

		return jQuery;
	},

	// See test/unit/core.js for details concerning isFunction.
	// Since version 1.3, DOM methods and functions like alert
	// aren't supported. They return false on IE (#2968).
	isFunction: function( obj ) {
		return toString.call(obj) === "[object Function]";
	},

	isArray: function( obj ) {
		return toString.call(obj) === "[object Array]";
	},

	// check if an element is in a (or is an) XML document
	isXMLDoc: function( elem ) {
		return elem.nodeType === 9 && elem.documentElement.nodeName !== "HTML" ||
			!!elem.ownerDocument && jQuery.isXMLDoc( elem.ownerDocument );
	},

	// Evalulates a script in a global context
	globalEval: function( data ) {
		if ( data && /\S/.test(data) ) {
			// Inspired by code by Andrea Giammarchi
			// http://webreflection.blogspot.com/2007/08/global-scope-evaluation-and-dom.html
			var head = document.getElementsByTagName("head")[0] || document.documentElement,
				script = document.createElement("script");

			script.type = "text/javascript";
			if ( jQuery.support.scriptEval )
				script.appendChild( document.createTextNode( data ) );
			else
				script.text = data;

			// Use insertBefore instead of appendChild  to circumvent an IE6 bug.
			// This arises when a base node is used (#2709).
			head.insertBefore( script, head.firstChild );
			head.removeChild( script );
		}
	},

	nodeName: function( elem, name ) {
		return elem.nodeName && elem.nodeName.toUpperCase() == name.toUpperCase();
	},

	// args is for internal usage only
	each: function( object, callback, args ) {
		var name, i = 0, length = object.length;

		if ( args ) {
			if ( length === undefined ) {
				for ( name in object )
					if ( callback.apply( object[ name ], args ) === false )
						break;
			} else
				for ( ; i < length; )
					if ( callback.apply( object[ i++ ], args ) === false )
						break;

		// A special, fast, case for the most common use of each
		} else {
			if ( length === undefined ) {
				for ( name in object )
					if ( callback.call( object[ name ], name, object[ name ] ) === false )
						break;
			} else
				for ( var value = object[0];
					i < length && callback.call( value, i, value ) !== false; value = object[++i] ){}
		}

		return object;
	},

	prop: function( elem, value, type, i, name ) {
		// Handle executable functions
		if ( jQuery.isFunction( value ) )
			value = value.call( elem, i );

		// Handle passing in a number to a CSS property
		return typeof value === "number" && type == "curCSS" && !exclude.test( name ) ?
			value + "px" :
			value;
	},

	className: {
		// internal only, use addClass("class")
		add: function( elem, classNames ) {
			jQuery.each((classNames || "").split(/\s+/), function(i, className){
				if ( elem.nodeType == 1 && !jQuery.className.has( elem.className, className ) )
					elem.className += (elem.className ? " " : "") + className;
			});
		},

		// internal only, use removeClass("class")
		remove: function( elem, classNames ) {
			if (elem.nodeType == 1)
				elem.className = classNames !== undefined ?
					jQuery.grep(elem.className.split(/\s+/), function(className){
						return !jQuery.className.has( classNames, className );
					}).join(" ") :
					"";
		},

		// internal only, use hasClass("class")
		has: function( elem, className ) {
			return elem && jQuery.inArray( className, (elem.className || elem).toString().split(/\s+/) ) > -1;
		}
	},

	// A method for quickly swapping in/out CSS properties to get correct calculations
	swap: function( elem, options, callback ) {
		var old = {};
		// Remember the old values, and insert the new ones
		for ( var name in options ) {
			old[ name ] = elem.style[ name ];
			elem.style[ name ] = options[ name ];
		}

		callback.call( elem );

		// Revert the old values
		for ( var name in options )
			elem.style[ name ] = old[ name ];
	},

	css: function( elem, name, force, extra ) {
		if ( name == "width" || name == "height" ) {
			var val, props = { position: "absolute", visibility: "hidden", display:"block" }, which = name == "width" ? [ "Left", "Right" ] : [ "Top", "Bottom" ];

			function getWH() {
				val = name == "width" ? elem.offsetWidth : elem.offsetHeight;

				if ( extra === "border" )
					return;

				jQuery.each( which, function() {
					if ( !extra )
						val -= parseFloat(jQuery.curCSS( elem, "padding" + this, true)) || 0;
					if ( extra === "margin" )
						val += parseFloat(jQuery.curCSS( elem, "margin" + this, true)) || 0;
					else
						val -= parseFloat(jQuery.curCSS( elem, "border" + this + "Width", true)) || 0;
				});
			}

			if ( elem.offsetWidth !== 0 )
				getWH();
			else
				jQuery.swap( elem, props, getWH );

			return Math.max(0, Math.round(val));
		}

		return jQuery.curCSS( elem, name, force );
	},

	curCSS: function( elem, name, force ) {
		var ret, style = elem.style;

		// We need to handle opacity special in IE
		if ( name == "opacity" && !jQuery.support.opacity ) {
			ret = jQuery.attr( style, "opacity" );

			return ret == "" ?
				"1" :
				ret;
		}

		// Make sure we're using the right name for getting the float value
		if ( name.match( /float/i ) )
			name = styleFloat;

		if ( !force && style && style[ name ] )
			ret = style[ name ];

		else if ( defaultView.getComputedStyle ) {

			// Only "float" is needed here
			if ( name.match( /float/i ) )
				name = "float";

			name = name.replace( /([A-Z])/g, "-$1" ).toLowerCase();

			var computedStyle = defaultView.getComputedStyle( elem, null );

			if ( computedStyle )
				ret = computedStyle.getPropertyValue( name );

			// We should always get a number back from opacity
			if ( name == "opacity" && ret == "" )
				ret = "1";

		} else if ( elem.currentStyle ) {
			var camelCase = name.replace(/\-(\w)/g, function(all, letter){
				return letter.toUpperCase();
			});

			ret = elem.currentStyle[ name ] || elem.currentStyle[ camelCase ];

			// From the awesome hack by Dean Edwards
			// http://erik.eae.net/archives/2007/07/27/18.54.15/#comment-102291

			// If we're not dealing with a regular pixel number
			// but a number that has a weird ending, we need to convert it to pixels
			if ( !/^\d+(px)?$/i.test( ret ) && /^\d/.test( ret ) ) {
				// Remember the original values
				var left = style.left, rsLeft = elem.runtimeStyle.left;

				// Put in the new values to get a computed value out
				elem.runtimeStyle.left = elem.currentStyle.left;
				style.left = ret || 0;
				ret = style.pixelLeft + "px";

				// Revert the changed values
				style.left = left;
				elem.runtimeStyle.left = rsLeft;
			}
		}

		return ret;
	},

	clean: function( elems, context, fragment ) {
		context = context || document;

		// !context.createElement fails in IE with an error but returns typeof 'object'
		if ( typeof context.createElement === "undefined" )
			context = context.ownerDocument || context[0] && context[0].ownerDocument || document;

		// If a single string is passed in and it's a single tag
		// just do a createElement and skip the rest
		if ( !fragment && elems.length === 1 && typeof elems[0] === "string" ) {
			var match = /^<(\w+)\s*\/?>$/.exec(elems[0]);
			if ( match )
				return [ context.createElement( match[1] ) ];
		}

		var ret = [], scripts = [], div = context.createElement("div");

		jQuery.each(elems, function(i, elem){
			if ( typeof elem === "number" )
				elem += '';

			if ( !elem )
				return;

			// Convert html string into DOM nodes
			if ( typeof elem === "string" ) {
				// Fix "XHTML"-style tags in all browsers
				elem = elem.replace(/(<(\w+)[^>]*?)\/>/g, function(all, front, tag){
					return tag.match(/^(abbr|br|col|img|input|link|meta|param|hr|area|embed)$/i) ?
						all :
						front + "></" + tag + ">";
				});

				// Trim whitespace, otherwise indexOf won't work as expected
				var tags = elem.replace(/^\s+/, "").substring(0, 10).toLowerCase();

				var wrap =
					// option or optgroup
					!tags.indexOf("<opt") &&
					[ 1, "<select multiple='multiple'>", "</select>" ] ||

					!tags.indexOf("<leg") &&
					[ 1, "<fieldset>", "</fieldset>" ] ||

					tags.match(/^<(thead|tbody|tfoot|colg|cap)/) &&
					[ 1, "<table>", "</table>" ] ||

					!tags.indexOf("<tr") &&
					[ 2, "<table><tbody>", "</tbody></table>" ] ||

				 	// <thead> matched above
					(!tags.indexOf("<td") || !tags.indexOf("<th")) &&
					[ 3, "<table><tbody><tr>", "</tr></tbody></table>" ] ||

					!tags.indexOf("<col") &&
					[ 2, "<table><tbody></tbody><colgroup>", "</colgroup></table>" ] ||

					// IE can't serialize <link> and <script> tags normally
					!jQuery.support.htmlSerialize &&
					[ 1, "div<div>", "</div>" ] ||

					[ 0, "", "" ];

				// Go to html and back, then peel off extra wrappers
				div.innerHTML = wrap[1] + elem + wrap[2];

				// Move to the right depth
				while ( wrap[0]-- )
					div = div.lastChild;

				// Remove IE's autoinserted <tbody> from table fragments
				if ( !jQuery.support.tbody ) {

					// String was a <table>, *may* have spurious <tbody>
					var hasBody = /<tbody/i.test(elem),
						tbody = !tags.indexOf("<table") && !hasBody ?
							div.firstChild && div.firstChild.childNodes :

						// String was a bare <thead> or <tfoot>
						wrap[1] == "<table>" && !hasBody ?
							div.childNodes :
							[];

					for ( var j = tbody.length - 1; j >= 0 ; --j )
						if ( jQuery.nodeName( tbody[ j ], "tbody" ) && !tbody[ j ].childNodes.length )
							tbody[ j ].parentNode.removeChild( tbody[ j ] );

					}

				// IE completely kills leading whitespace when innerHTML is used
				if ( !jQuery.support.leadingWhitespace && /^\s/.test( elem ) )
					div.insertBefore( context.createTextNode( elem.match(/^\s*/)[0] ), div.firstChild );
				
				elem = jQuery.makeArray( div.childNodes );
			}

			if ( elem.nodeType )
				ret.push( elem );
			else
				ret = jQuery.merge( ret, elem );

		});

		if ( fragment ) {
			for ( var i = 0; ret[i]; i++ ) {
				if ( jQuery.nodeName( ret[i], "script" ) && (!ret[i].type || ret[i].type.toLowerCase() === "text/javascript") ) {
					scripts.push( ret[i].parentNode ? ret[i].parentNode.removeChild( ret[i] ) : ret[i] );
				} else {
					if ( ret[i].nodeType === 1 )
						ret.splice.apply( ret, [i + 1, 0].concat(jQuery.makeArray(ret[i].getElementsByTagName("script"))) );
					fragment.appendChild( ret[i] );
				}
			}
			
			return scripts;
		}

		return ret;
	},

	attr: function( elem, name, value ) {
		// don't set attributes on text and comment nodes
		if (!elem || elem.nodeType == 3 || elem.nodeType == 8)
			return undefined;

		var notxml = !jQuery.isXMLDoc( elem ),
			// Whether we are setting (or getting)
			set = value !== undefined;

		// Try to normalize/fix the name
		name = notxml && jQuery.props[ name ] || name;

		// Only do all the following if this is a node (faster for style)
		// IE elem.getAttribute passes even for style
		if ( elem.tagName ) {

			// These attributes require special treatment
			var special = /href|src|style/.test( name );

			// Safari mis-reports the default selected property of a hidden option
			// Accessing the parent's selectedIndex property fixes it
			if ( name == "selected" && elem.parentNode )
				elem.parentNode.selectedIndex;

			// If applicable, access the attribute via the DOM 0 way
			if ( name in elem && notxml && !special ) {
				if ( set ){
					// We can't allow the type property to be changed (since it causes problems in IE)
					if ( name == "type" && jQuery.nodeName( elem, "input" ) && elem.parentNode )
						throw "type property can't be changed";

					elem[ name ] = value;
				}

				// browsers index elements by id/name on forms, give priority to attributes.
				if( jQuery.nodeName( elem, "form" ) && elem.getAttributeNode(name) )
					return elem.getAttributeNode( name ).nodeValue;

				// elem.tabIndex doesn't always return the correct value when it hasn't been explicitly set
				// http://fluidproject.org/blog/2008/01/09/getting-setting-and-removing-tabindex-values-with-javascript/
				if ( name == "tabIndex" ) {
					var attributeNode = elem.getAttributeNode( "tabIndex" );
					return attributeNode && attributeNode.specified
						? attributeNode.value
						: elem.nodeName.match(/(button|input|object|select|textarea)/i)
							? 0
							: elem.nodeName.match(/^(a|area)$/i) && elem.href
								? 0
								: undefined;
				}

				return elem[ name ];
			}

			if ( !jQuery.support.style && notxml &&  name == "style" )
				return jQuery.attr( elem.style, "cssText", value );

			if ( set )
				// convert the value to a string (all browsers do this but IE) see #1070
				elem.setAttribute( name, "" + value );

			var attr = !jQuery.support.hrefNormalized && notxml && special
					// Some attributes require a special call on IE
					? elem.getAttribute( name, 2 )
					: elem.getAttribute( name );

			// Non-existent attributes return null, we normalize to undefined
			return attr === null ? undefined : attr;
		}

		// elem is actually elem.style ... set the style

		// IE uses filters for opacity
		if ( !jQuery.support.opacity && name == "opacity" ) {
			if ( set ) {
				// IE has trouble with opacity if it does not have layout
				// Force it by setting the zoom level
				elem.zoom = 1;

				// Set the alpha filter to set the opacity
				elem.filter = (elem.filter || "").replace( /alpha\([^)]*\)/, "" ) +
					(parseInt( value ) + '' == "NaN" ? "" : "alpha(opacity=" + value * 100 + ")");
			}

			return elem.filter && elem.filter.indexOf("opacity=") >= 0 ?
				(parseFloat( elem.filter.match(/opacity=([^)]*)/)[1] ) / 100) + '':
				"";
		}

		name = name.replace(/-([a-z])/ig, function(all, letter){
			return letter.toUpperCase();
		});

		if ( set )
			elem[ name ] = value;

		return elem[ name ];
	},

	trim: function( text ) {
		return (text || "").replace( /^\s+|\s+$/g, "" );
	},

	makeArray: function( array ) {
		var ret = [];

		if( array != null ){
			var i = array.length;
			// The window, strings (and functions) also have 'length'
			if( i == null || typeof array === "string" || jQuery.isFunction(array) || array.setInterval )
				ret[0] = array;
			else
				while( i )
					ret[--i] = array[i];
		}

		return ret;
	},

	inArray: function( elem, array ) {
		for ( var i = 0, length = array.length; i < length; i++ )
		// Use === because on IE, window == document
			if ( array[ i ] === elem )
				return i;

		return -1;
	},

	merge: function( first, second ) {
		// We have to loop this way because IE & Opera overwrite the length
		// expando of getElementsByTagName
		var i = 0, elem, pos = first.length;
		// Also, we need to make sure that the correct elements are being returned
		// (IE returns comment nodes in a '*' query)
		if ( !jQuery.support.getAll ) {
			while ( (elem = second[ i++ ]) != null )
				if ( elem.nodeType != 8 )
					first[ pos++ ] = elem;

		} else
			while ( (elem = second[ i++ ]) != null )
				first[ pos++ ] = elem;

		return first;
	},

	unique: function( array ) {
		var ret = [], done = {};

		try {

			for ( var i = 0, length = array.length; i < length; i++ ) {
				var id = jQuery.data( array[ i ] );

				if ( !done[ id ] ) {
					done[ id ] = true;
					ret.push( array[ i ] );
				}
			}

		} catch( e ) {
			ret = array;
		}

		return ret;
	},

	grep: function( elems, callback, inv ) {
		var ret = [];

		// Go through the array, only saving the items
		// that pass the validator function
		for ( var i = 0, length = elems.length; i < length; i++ )
			if ( !inv != !callback( elems[ i ], i ) )
				ret.push( elems[ i ] );

		return ret;
	},

	map: function( elems, callback ) {
		var ret = [];

		// Go through the array, translating each of the items to their
		// new value (or values).
		for ( var i = 0, length = elems.length; i < length; i++ ) {
			var value = callback( elems[ i ], i );

			if ( value != null )
				ret[ ret.length ] = value;
		}

		return ret.concat.apply( [], ret );
	}
});

// Use of jQuery.browser is deprecated.
// It's included for backwards compatibility and plugins,
// although they should work to migrate away.

var userAgent = navigator.userAgent.toLowerCase();

// Figure out what browser is being used
jQuery.browser = {
	version: (userAgent.match( /.+(?:rv|it|ra|ie)[\/: ]([\d.]+)/ ) || [0,'0'])[1],
	safari: /webkit/.test( userAgent ),
	opera: /opera/.test( userAgent ),
	msie: /msie/.test( userAgent ) && !/opera/.test( userAgent ),
	mozilla: /mozilla/.test( userAgent ) && !/(compatible|webkit)/.test( userAgent )
};

jQuery.each({
	parent: function(elem){return elem.parentNode;},
	parents: function(elem){return jQuery.dir(elem,"parentNode");},
	next: function(elem){return jQuery.nth(elem,2,"nextSibling");},
	prev: function(elem){return jQuery.nth(elem,2,"previousSibling");},
	nextAll: function(elem){return jQuery.dir(elem,"nextSibling");},
	prevAll: function(elem){return jQuery.dir(elem,"previousSibling");},
	siblings: function(elem){return jQuery.sibling(elem.parentNode.firstChild,elem);},
	children: function(elem){return jQuery.sibling(elem.firstChild);},
	contents: function(elem){return jQuery.nodeName(elem,"iframe")?elem.contentDocument||elem.contentWindow.document:jQuery.makeArray(elem.childNodes);}
}, function(name, fn){
	jQuery.fn[ name ] = function( selector ) {
		var ret = jQuery.map( this, fn );

		if ( selector && typeof selector == "string" )
			ret = jQuery.multiFilter( selector, ret );

		return this.pushStack( jQuery.unique( ret ), name, selector );
	};
});

jQuery.each({
	appendTo: "append",
	prependTo: "prepend",
	insertBefore: "before",
	insertAfter: "after",
	replaceAll: "replaceWith"
}, function(name, original){
	jQuery.fn[ name ] = function( selector ) {
		var ret = [], insert = jQuery( selector );

		for ( var i = 0, l = insert.length; i < l; i++ ) {
			var elems = (i > 0 ? this.clone(true) : this).get();
			jQuery.fn[ original ].apply( jQuery(insert[i]), elems );
			ret = ret.concat( elems );
		}

		return this.pushStack( ret, name, selector );
	};
});

jQuery.each({
	removeAttr: function( name ) {
		jQuery.attr( this, name, "" );
		if (this.nodeType == 1)
			this.removeAttribute( name );
	},

	addClass: function( classNames ) {
		jQuery.className.add( this, classNames );
	},

	removeClass: function( classNames ) {
		jQuery.className.remove( this, classNames );
	},

	toggleClass: function( classNames, state ) {
		if( typeof state !== "boolean" )
			state = !jQuery.className.has( this, classNames );
		jQuery.className[ state ? "add" : "remove" ]( this, classNames );
	},

	remove: function( selector ) {
		if ( !selector || jQuery.filter( selector, [ this ] ).length ) {
			// Prevent memory leaks
			jQuery( "*", this ).add([this]).each(function(){
				jQuery.event.remove(this);
				jQuery.removeData(this);
			});
			if (this.parentNode)
				this.parentNode.removeChild( this );
		}
	},

	empty: function() {
		// Remove element nodes and prevent memory leaks
		jQuery(this).children().remove();

		// Remove any remaining nodes
		while ( this.firstChild )
			this.removeChild( this.firstChild );
	}
}, function(name, fn){
	jQuery.fn[ name ] = function(){
		return this.each( fn, arguments );
	};
});

// Helper function used by the dimensions and offset modules
function num(elem, prop) {
	return elem[0] && parseInt( jQuery.curCSS(elem[0], prop, true), 10 ) || 0;
}
var expando = "jQuery" + now(), uuid = 0, windowData = {};

jQuery.extend({
	cache: {},

	data: function( elem, name, data ) {
		elem = elem == window ?
			windowData :
			elem;

		var id = elem[ expando ];

		// Compute a unique ID for the element
		if ( !id )
			id = elem[ expando ] = ++uuid;

		// Only generate the data cache if we're
		// trying to access or manipulate it
		if ( name && !jQuery.cache[ id ] )
			jQuery.cache[ id ] = {};

		// Prevent overriding the named cache with undefined values
		if ( data !== undefined )
			jQuery.cache[ id ][ name ] = data;

		// Return the named cache data, or the ID for the element
		return name ?
			jQuery.cache[ id ][ name ] :
			id;
	},

	removeData: function( elem, name ) {
		elem = elem == window ?
			windowData :
			elem;

		var id = elem[ expando ];

		// If we want to remove a specific section of the element's data
		if ( name ) {
			if ( jQuery.cache[ id ] ) {
				// Remove the section of cache data
				delete jQuery.cache[ id ][ name ];

				// If we've removed all the data, remove the element's cache
				name = "";

				for ( name in jQuery.cache[ id ] )
					break;

				if ( !name )
					jQuery.removeData( elem );
			}

		// Otherwise, we want to remove all of the element's data
		} else {
			// Clean up the element expando
			try {
				delete elem[ expando ];
			} catch(e){
				// IE has trouble directly removing the expando
				// but it's ok with using removeAttribute
				if ( elem.removeAttribute )
					elem.removeAttribute( expando );
			}

			// Completely remove the data cache
			delete jQuery.cache[ id ];
		}
	},
	queue: function( elem, type, data ) {
		if ( elem ){
	
			type = (type || "fx") + "queue";
	
			var q = jQuery.data( elem, type );
	
			if ( !q || jQuery.isArray(data) )
				q = jQuery.data( elem, type, jQuery.makeArray(data) );
			else if( data )
				q.push( data );
	
		}
		return q;
	},

	dequeue: function( elem, type ){
		var queue = jQuery.queue( elem, type ),
			fn = queue.shift();
		
		if( !type || type === "fx" )
			fn = queue[0];
			
		if( fn !== undefined )
			fn.call(elem);
	}
});

jQuery.fn.extend({
	data: function( key, value ){
		var parts = key.split(".");
		parts[1] = parts[1] ? "." + parts[1] : "";

		if ( value === undefined ) {
			var data = this.triggerHandler("getData" + parts[1] + "!", [parts[0]]);

			if ( data === undefined && this.length )
				data = jQuery.data( this[0], key );

			return data === undefined && parts[1] ?
				this.data( parts[0] ) :
				data;
		} else
			return this.trigger("setData" + parts[1] + "!", [parts[0], value]).each(function(){
				jQuery.data( this, key, value );
			});
	},

	removeData: function( key ){
		return this.each(function(){
			jQuery.removeData( this, key );
		});
	},
	queue: function(type, data){
		if ( typeof type !== "string" ) {
			data = type;
			type = "fx";
		}

		if ( data === undefined )
			return jQuery.queue( this[0], type );

		return this.each(function(){
			var queue = jQuery.queue( this, type, data );
			
			 if( type == "fx" && queue.length == 1 )
				queue[0].call(this);
		});
	},
	dequeue: function(type){
		return this.each(function(){
			jQuery.dequeue( this, type );
		});
	}
});/*!
 * Sizzle CSS Selector Engine - v0.9.3
 *  Copyright 2009, The Dojo Foundation
 *  Released under the MIT, BSD, and GPL Licenses.
 *  More information: http://sizzlejs.com/
 */
(function(){

var chunker = /((?:\((?:\([^()]+\)|[^()]+)+\)|\[(?:\[[^[\]]*\]|['"][^'"]*['"]|[^[\]'"]+)+\]|\\.|[^ >+~,(\[\\]+)+|[>+~])(\s*,\s*)?/g,
	done = 0,
	toString = Object.prototype.toString;

var Sizzle = function(selector, context, results, seed) {
	results = results || [];
	context = context || document;

	if ( context.nodeType !== 1 && context.nodeType !== 9 )
		return [];
	
	if ( !selector || typeof selector !== "string" ) {
		return results;
	}

	var parts = [], m, set, checkSet, check, mode, extra, prune = true;
	
	// Reset the position of the chunker regexp (start from head)
	chunker.lastIndex = 0;
	
	while ( (m = chunker.exec(selector)) !== null ) {
		parts.push( m[1] );
		
		if ( m[2] ) {
			extra = RegExp.rightContext;
			break;
		}
	}

	if ( parts.length > 1 && origPOS.exec( selector ) ) {
		if ( parts.length === 2 && Expr.relative[ parts[0] ] ) {
			set = posProcess( parts[0] + parts[1], context );
		} else {
			set = Expr.relative[ parts[0] ] ?
				[ context ] :
				Sizzle( parts.shift(), context );

			while ( parts.length ) {
				selector = parts.shift();

				if ( Expr.relative[ selector ] )
					selector += parts.shift();

				set = posProcess( selector, set );
			}
		}
	} else {
		var ret = seed ?
			{ expr: parts.pop(), set: makeArray(seed) } :
			Sizzle.find( parts.pop(), parts.length === 1 && context.parentNode ? context.parentNode : context, isXML(context) );
		set = Sizzle.filter( ret.expr, ret.set );

		if ( parts.length > 0 ) {
			checkSet = makeArray(set);
		} else {
			prune = false;
		}

		while ( parts.length ) {
			var cur = parts.pop(), pop = cur;

			if ( !Expr.relative[ cur ] ) {
				cur = "";
			} else {
				pop = parts.pop();
			}

			if ( pop == null ) {
				pop = context;
			}

			Expr.relative[ cur ]( checkSet, pop, isXML(context) );
		}
	}

	if ( !checkSet ) {
		checkSet = set;
	}

	if ( !checkSet ) {
		throw "Syntax error, unrecognized expression: " + (cur || selector);
	}

	if ( toString.call(checkSet) === "[object Array]" ) {
		if ( !prune ) {
			results.push.apply( results, checkSet );
		} else if ( context.nodeType === 1 ) {
			for ( var i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && (checkSet[i] === true || checkSet[i].nodeType === 1 && contains(context, checkSet[i])) ) {
					results.push( set[i] );
				}
			}
		} else {
			for ( var i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && checkSet[i].nodeType === 1 ) {
					results.push( set[i] );
				}
			}
		}
	} else {
		makeArray( checkSet, results );
	}

	if ( extra ) {
		Sizzle( extra, context, results, seed );

		if ( sortOrder ) {
			hasDuplicate = false;
			results.sort(sortOrder);

			if ( hasDuplicate ) {
				for ( var i = 1; i < results.length; i++ ) {
					if ( results[i] === results[i-1] ) {
						results.splice(i--, 1);
					}
				}
			}
		}
	}

	return results;
};

Sizzle.matches = function(expr, set){
	return Sizzle(expr, null, null, set);
};

Sizzle.find = function(expr, context, isXML){
	var set, match;

	if ( !expr ) {
		return [];
	}

	for ( var i = 0, l = Expr.order.length; i < l; i++ ) {
		var type = Expr.order[i], match;
		
		if ( (match = Expr.match[ type ].exec( expr )) ) {
			var left = RegExp.leftContext;

			if ( left.substr( left.length - 1 ) !== "\\" ) {
				match[1] = (match[1] || "").replace(/\\/g, "");
				set = Expr.find[ type ]( match, context, isXML );
				if ( set != null ) {
					expr = expr.replace( Expr.match[ type ], "" );
					break;
				}
			}
		}
	}

	if ( !set ) {
		set = context.getElementsByTagName("*");
	}

	return {set: set, expr: expr};
};

Sizzle.filter = function(expr, set, inplace, not){
	var old = expr, result = [], curLoop = set, match, anyFound,
		isXMLFilter = set && set[0] && isXML(set[0]);

	while ( expr && set.length ) {
		for ( var type in Expr.filter ) {
			if ( (match = Expr.match[ type ].exec( expr )) != null ) {
				var filter = Expr.filter[ type ], found, item;
				anyFound = false;

				if ( curLoop == result ) {
					result = [];
				}

				if ( Expr.preFilter[ type ] ) {
					match = Expr.preFilter[ type ]( match, curLoop, inplace, result, not, isXMLFilter );

					if ( !match ) {
						anyFound = found = true;
					} else if ( match === true ) {
						continue;
					}
				}

				if ( match ) {
					for ( var i = 0; (item = curLoop[i]) != null; i++ ) {
						if ( item ) {
							found = filter( item, match, i, curLoop );
							var pass = not ^ !!found;

							if ( inplace && found != null ) {
								if ( pass ) {
									anyFound = true;
								} else {
									curLoop[i] = false;
								}
							} else if ( pass ) {
								result.push( item );
								anyFound = true;
							}
						}
					}
				}

				if ( found !== undefined ) {
					if ( !inplace ) {
						curLoop = result;
					}

					expr = expr.replace( Expr.match[ type ], "" );

					if ( !anyFound ) {
						return [];
					}

					break;
				}
			}
		}

		// Improper expression
		if ( expr == old ) {
			if ( anyFound == null ) {
				throw "Syntax error, unrecognized expression: " + expr;
			} else {
				break;
			}
		}

		old = expr;
	}

	return curLoop;
};

var Expr = Sizzle.selectors = {
	order: [ "ID", "NAME", "TAG" ],
	match: {
		ID: /#((?:[\w\u00c0-\uFFFF_-]|\\.)+)/,
		CLASS: /\.((?:[\w\u00c0-\uFFFF_-]|\\.)+)/,
		NAME: /\[name=['"]*((?:[\w\u00c0-\uFFFF_-]|\\.)+)['"]*\]/,
		ATTR: /\[\s*((?:[\w\u00c0-\uFFFF_-]|\\.)+)\s*(?:(\S?=)\s*(['"]*)(.*?)\3|)\s*\]/,
		TAG: /^((?:[\w\u00c0-\uFFFF\*_-]|\\.)+)/,
		CHILD: /:(only|nth|last|first)-child(?:\((even|odd|[\dn+-]*)\))?/,
		POS: /:(nth|eq|gt|lt|first|last|even|odd)(?:\((\d*)\))?(?=[^-]|$)/,
		PSEUDO: /:((?:[\w\u00c0-\uFFFF_-]|\\.)+)(?:\((['"]*)((?:\([^\)]+\)|[^\2\(\)]*)+)\2\))?/
	},
	attrMap: {
		"class": "className",
		"for": "htmlFor"
	},
	attrHandle: {
		href: function(elem){
			return elem.getAttribute("href");
		}
	},
	relative: {
		"+": function(checkSet, part, isXML){
			var isPartStr = typeof part === "string",
				isTag = isPartStr && !/\W/.test(part),
				isPartStrNotTag = isPartStr && !isTag;

			if ( isTag && !isXML ) {
				part = part.toUpperCase();
			}

			for ( var i = 0, l = checkSet.length, elem; i < l; i++ ) {
				if ( (elem = checkSet[i]) ) {
					while ( (elem = elem.previousSibling) && elem.nodeType !== 1 ) {}

					checkSet[i] = isPartStrNotTag || elem && elem.nodeName === part ?
						elem || false :
						elem === part;
				}
			}

			if ( isPartStrNotTag ) {
				Sizzle.filter( part, checkSet, true );
			}
		},
		">": function(checkSet, part, isXML){
			var isPartStr = typeof part === "string";

			if ( isPartStr && !/\W/.test(part) ) {
				part = isXML ? part : part.toUpperCase();

				for ( var i = 0, l = checkSet.length; i < l; i++ ) {
					var elem = checkSet[i];
					if ( elem ) {
						var parent = elem.parentNode;
						checkSet[i] = parent.nodeName === part ? parent : false;
					}
				}
			} else {
				for ( var i = 0, l = checkSet.length; i < l; i++ ) {
					var elem = checkSet[i];
					if ( elem ) {
						checkSet[i] = isPartStr ?
							elem.parentNode :
							elem.parentNode === part;
					}
				}

				if ( isPartStr ) {
					Sizzle.filter( part, checkSet, true );
				}
			}
		},
		"": function(checkSet, part, isXML){
			var doneName = done++, checkFn = dirCheck;

			if ( !part.match(/\W/) ) {
				var nodeCheck = part = isXML ? part : part.toUpperCase();
				checkFn = dirNodeCheck;
			}

			checkFn("parentNode", part, doneName, checkSet, nodeCheck, isXML);
		},
		"~": function(checkSet, part, isXML){
			var doneName = done++, checkFn = dirCheck;

			if ( typeof part === "string" && !part.match(/\W/) ) {
				var nodeCheck = part = isXML ? part : part.toUpperCase();
				checkFn = dirNodeCheck;
			}

			checkFn("previousSibling", part, doneName, checkSet, nodeCheck, isXML);
		}
	},
	find: {
		ID: function(match, context, isXML){
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				return m ? [m] : [];
			}
		},
		NAME: function(match, context, isXML){
			if ( typeof context.getElementsByName !== "undefined" ) {
				var ret = [], results = context.getElementsByName(match[1]);

				for ( var i = 0, l = results.length; i < l; i++ ) {
					if ( results[i].getAttribute("name") === match[1] ) {
						ret.push( results[i] );
					}
				}

				return ret.length === 0 ? null : ret;
			}
		},
		TAG: function(match, context){
			return context.getElementsByTagName(match[1]);
		}
	},
	preFilter: {
		CLASS: function(match, curLoop, inplace, result, not, isXML){
			match = " " + match[1].replace(/\\/g, "") + " ";

			if ( isXML ) {
				return match;
			}

			for ( var i = 0, elem; (elem = curLoop[i]) != null; i++ ) {
				if ( elem ) {
					if ( not ^ (elem.className && (" " + elem.className + " ").indexOf(match) >= 0) ) {
						if ( !inplace )
							result.push( elem );
					} else if ( inplace ) {
						curLoop[i] = false;
					}
				}
			}

			return false;
		},
		ID: function(match){
			return match[1].replace(/\\/g, "");
		},
		TAG: function(match, curLoop){
			for ( var i = 0; curLoop[i] === false; i++ ){}
			return curLoop[i] && isXML(curLoop[i]) ? match[1] : match[1].toUpperCase();
		},
		CHILD: function(match){
			if ( match[1] == "nth" ) {
				// parse equations like 'even', 'odd', '5', '2n', '3n+2', '4n-1', '-n+6'
				var test = /(-?)(\d*)n((?:\+|-)?\d*)/.exec(
					match[2] == "even" && "2n" || match[2] == "odd" && "2n+1" ||
					!/\D/.test( match[2] ) && "0n+" + match[2] || match[2]);

				// calculate the numbers (first)n+(last) including if they are negative
				match[2] = (test[1] + (test[2] || 1)) - 0;
				match[3] = test[3] - 0;
			}

			// [TODO] Move to normal caching system
			match[0] = done++;

			return match;
		},
		ATTR: function(match, curLoop, inplace, result, not, isXML){
			var name = match[1].replace(/\\/g, "");
			
			if ( !isXML && Expr.attrMap[name] ) {
				match[1] = Expr.attrMap[name];
			}

			if ( match[2] === "~=" ) {
				match[4] = " " + match[4] + " ";
			}

			return match;
		},
		PSEUDO: function(match, curLoop, inplace, result, not){
			if ( match[1] === "not" ) {
				// If we're dealing with a complex expression, or a simple one
				if ( match[3].match(chunker).length > 1 || /^\w/.test(match[3]) ) {
					match[3] = Sizzle(match[3], null, null, curLoop);
				} else {
					var ret = Sizzle.filter(match[3], curLoop, inplace, true ^ not);
					if ( !inplace ) {
						result.push.apply( result, ret );
					}
					return false;
				}
			} else if ( Expr.match.POS.test( match[0] ) || Expr.match.CHILD.test( match[0] ) ) {
				return true;
			}
			
			return match;
		},
		POS: function(match){
			match.unshift( true );
			return match;
		}
	},
	filters: {
		enabled: function(elem){
			return elem.disabled === false && elem.type !== "hidden";
		},
		disabled: function(elem){
			return elem.disabled === true;
		},
		checked: function(elem){
			return elem.checked === true;
		},
		selected: function(elem){
			// Accessing this property makes selected-by-default
			// options in Safari work properly
			elem.parentNode.selectedIndex;
			return elem.selected === true;
		},
		parent: function(elem){
			return !!elem.firstChild;
		},
		empty: function(elem){
			return !elem.firstChild;
		},
		has: function(elem, i, match){
			return !!Sizzle( match[3], elem ).length;
		},
		header: function(elem){
			return /h\d/i.test( elem.nodeName );
		},
		text: function(elem){
			return "text" === elem.type;
		},
		radio: function(elem){
			return "radio" === elem.type;
		},
		checkbox: function(elem){
			return "checkbox" === elem.type;
		},
		file: function(elem){
			return "file" === elem.type;
		},
		password: function(elem){
			return "password" === elem.type;
		},
		submit: function(elem){
			return "submit" === elem.type;
		},
		image: function(elem){
			return "image" === elem.type;
		},
		reset: function(elem){
			return "reset" === elem.type;
		},
		button: function(elem){
			return "button" === elem.type || elem.nodeName.toUpperCase() === "BUTTON";
		},
		input: function(elem){
			return /input|select|textarea|button/i.test(elem.nodeName);
		}
	},
	setFilters: {
		first: function(elem, i){
			return i === 0;
		},
		last: function(elem, i, match, array){
			return i === array.length - 1;
		},
		even: function(elem, i){
			return i % 2 === 0;
		},
		odd: function(elem, i){
			return i % 2 === 1;
		},
		lt: function(elem, i, match){
			return i < match[3] - 0;
		},
		gt: function(elem, i, match){
			return i > match[3] - 0;
		},
		nth: function(elem, i, match){
			return match[3] - 0 == i;
		},
		eq: function(elem, i, match){
			return match[3] - 0 == i;
		}
	},
	filter: {
		PSEUDO: function(elem, match, i, array){
			var name = match[1], filter = Expr.filters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			} else if ( name === "contains" ) {
				return (elem.textContent || elem.innerText || "").indexOf(match[3]) >= 0;
			} else if ( name === "not" ) {
				var not = match[3];

				for ( var i = 0, l = not.length; i < l; i++ ) {
					if ( not[i] === elem ) {
						return false;
					}
				}

				return true;
			}
		},
		CHILD: function(elem, match){
			var type = match[1], node = elem;
			switch (type) {
				case 'only':
				case 'first':
					while (node = node.previousSibling)  {
						if ( node.nodeType === 1 ) return false;
					}
					if ( type == 'first') return true;
					node = elem;
				case 'last':
					while (node = node.nextSibling)  {
						if ( node.nodeType === 1 ) return false;
					}
					return true;
				case 'nth':
					var first = match[2], last = match[3];

					if ( first == 1 && last == 0 ) {
						return true;
					}
					
					var doneName = match[0],
						parent = elem.parentNode;
	
					if ( parent && (parent.sizcache !== doneName || !elem.nodeIndex) ) {
						var count = 0;
						for ( node = parent.firstChild; node; node = node.nextSibling ) {
							if ( node.nodeType === 1 ) {
								node.nodeIndex = ++count;
							}
						} 
						parent.sizcache = doneName;
					}
					
					var diff = elem.nodeIndex - last;
					if ( first == 0 ) {
						return diff == 0;
					} else {
						return ( diff % first == 0 && diff / first >= 0 );
					}
			}
		},
		ID: function(elem, match){
			return elem.nodeType === 1 && elem.getAttribute("id") === match;
		},
		TAG: function(elem, match){
			return (match === "*" && elem.nodeType === 1) || elem.nodeName === match;
		},
		CLASS: function(elem, match){
			return (" " + (elem.className || elem.getAttribute("class")) + " ")
				.indexOf( match ) > -1;
		},
		ATTR: function(elem, match){
			var name = match[1],
				result = Expr.attrHandle[ name ] ?
					Expr.attrHandle[ name ]( elem ) :
					elem[ name ] != null ?
						elem[ name ] :
						elem.getAttribute( name ),
				value = result + "",
				type = match[2],
				check = match[4];

			return result == null ?
				type === "!=" :
				type === "=" ?
				value === check :
				type === "*=" ?
				value.indexOf(check) >= 0 :
				type === "~=" ?
				(" " + value + " ").indexOf(check) >= 0 :
				!check ?
				value && result !== false :
				type === "!=" ?
				value != check :
				type === "^=" ?
				value.indexOf(check) === 0 :
				type === "$=" ?
				value.substr(value.length - check.length) === check :
				type === "|=" ?
				value === check || value.substr(0, check.length + 1) === check + "-" :
				false;
		},
		POS: function(elem, match, i, array){
			var name = match[2], filter = Expr.setFilters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			}
		}
	}
};

var origPOS = Expr.match.POS;

for ( var type in Expr.match ) {
	Expr.match[ type ] = RegExp( Expr.match[ type ].source + /(?![^\[]*\])(?![^\(]*\))/.source );
}

var makeArray = function(array, results) {
	array = Array.prototype.slice.call( array );

	if ( results ) {
		results.push.apply( results, array );
		return results;
	}
	
	return array;
};

// Perform a simple check to determine if the browser is capable of
// converting a NodeList to an array using builtin methods.
try {
	Array.prototype.slice.call( document.documentElement.childNodes );

// Provide a fallback method if it does not work
} catch(e){
	makeArray = function(array, results) {
		var ret = results || [];

		if ( toString.call(array) === "[object Array]" ) {
			Array.prototype.push.apply( ret, array );
		} else {
			if ( typeof array.length === "number" ) {
				for ( var i = 0, l = array.length; i < l; i++ ) {
					ret.push( array[i] );
				}
			} else {
				for ( var i = 0; array[i]; i++ ) {
					ret.push( array[i] );
				}
			}
		}

		return ret;
	};
}

var sortOrder;

if ( document.documentElement.compareDocumentPosition ) {
	sortOrder = function( a, b ) {
		var ret = a.compareDocumentPosition(b) & 4 ? -1 : a === b ? 0 : 1;
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
} else if ( "sourceIndex" in document.documentElement ) {
	sortOrder = function( a, b ) {
		var ret = a.sourceIndex - b.sourceIndex;
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
} else if ( document.createRange ) {
	sortOrder = function( a, b ) {
		var aRange = a.ownerDocument.createRange(), bRange = b.ownerDocument.createRange();
		aRange.selectNode(a);
		aRange.collapse(true);
		bRange.selectNode(b);
		bRange.collapse(true);
		var ret = aRange.compareBoundaryPoints(Range.START_TO_END, bRange);
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
}

// Check to see if the browser returns elements by name when
// querying by getElementById (and provide a workaround)
(function(){
	// We're going to inject a fake input element with a specified name
	var form = document.createElement("form"),
		id = "script" + (new Date).getTime();
	form.innerHTML = "<input name='" + id + "'/>";

	// Inject it into the root element, check its status, and remove it quickly
	var root = document.documentElement;
	root.insertBefore( form, root.firstChild );

	// The workaround has to do additional checks after a getElementById
	// Which slows things down for other browsers (hence the branching)
	if ( !!document.getElementById( id ) ) {
		Expr.find.ID = function(match, context, isXML){
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				return m ? m.id === match[1] || typeof m.getAttributeNode !== "undefined" && m.getAttributeNode("id").nodeValue === match[1] ? [m] : undefined : [];
			}
		};

		Expr.filter.ID = function(elem, match){
			var node = typeof elem.getAttributeNode !== "undefined" && elem.getAttributeNode("id");
			return elem.nodeType === 1 && node && node.nodeValue === match;
		};
	}

	root.removeChild( form );
})();

(function(){
	// Check to see if the browser returns only elements
	// when doing getElementsByTagName("*")

	// Create a fake element
	var div = document.createElement("div");
	div.appendChild( document.createComment("") );

	// Make sure no comments are found
	if ( div.getElementsByTagName("*").length > 0 ) {
		Expr.find.TAG = function(match, context){
			var results = context.getElementsByTagName(match[1]);

			// Filter out possible comments
			if ( match[1] === "*" ) {
				var tmp = [];

				for ( var i = 0; results[i]; i++ ) {
					if ( results[i].nodeType === 1 ) {
						tmp.push( results[i] );
					}
				}

				results = tmp;
			}

			return results;
		};
	}

	// Check to see if an attribute returns normalized href attributes
	div.innerHTML = "<a href='#'></a>";
	if ( div.firstChild && typeof div.firstChild.getAttribute !== "undefined" &&
			div.firstChild.getAttribute("href") !== "#" ) {
		Expr.attrHandle.href = function(elem){
			return elem.getAttribute("href", 2);
		};
	}
})();

if ( document.querySelectorAll ) (function(){
	var oldSizzle = Sizzle, div = document.createElement("div");
	div.innerHTML = "<p class='TEST'></p>";

	// Safari can't handle uppercase or unicode characters when
	// in quirks mode.
	if ( div.querySelectorAll && div.querySelectorAll(".TEST").length === 0 ) {
		return;
	}
	
	Sizzle = function(query, context, extra, seed){
		context = context || document;

		// Only use querySelectorAll on non-XML documents
		// (ID selectors don't work in non-HTML documents)
		if ( !seed && context.nodeType === 9 && !isXML(context) ) {
			try {
				return makeArray( context.querySelectorAll(query), extra );
			} catch(e){}
		}
		
		return oldSizzle(query, context, extra, seed);
	};

	Sizzle.find = oldSizzle.find;
	Sizzle.filter = oldSizzle.filter;
	Sizzle.selectors = oldSizzle.selectors;
	Sizzle.matches = oldSizzle.matches;
})();

if ( document.getElementsByClassName && document.documentElement.getElementsByClassName ) (function(){
	var div = document.createElement("div");
	div.innerHTML = "<div class='test e'></div><div class='test'></div>";

	// Opera can't find a second classname (in 9.6)
	if ( div.getElementsByClassName("e").length === 0 )
		return;

	// Safari caches class attributes, doesn't catch changes (in 3.2)
	div.lastChild.className = "e";

	if ( div.getElementsByClassName("e").length === 1 )
		return;

	Expr.order.splice(1, 0, "CLASS");
	Expr.find.CLASS = function(match, context, isXML) {
		if ( typeof context.getElementsByClassName !== "undefined" && !isXML ) {
			return context.getElementsByClassName(match[1]);
		}
	};
})();

function dirNodeCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	var sibDir = dir == "previousSibling" && !isXML;
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];
		if ( elem ) {
			if ( sibDir && elem.nodeType === 1 ){
				elem.sizcache = doneName;
				elem.sizset = i;
			}
			elem = elem[dir];
			var match = false;

			while ( elem ) {
				if ( elem.sizcache === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 && !isXML ){
					elem.sizcache = doneName;
					elem.sizset = i;
				}

				if ( elem.nodeName === cur ) {
					match = elem;
					break;
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

function dirCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	var sibDir = dir == "previousSibling" && !isXML;
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];
		if ( elem ) {
			if ( sibDir && elem.nodeType === 1 ) {
				elem.sizcache = doneName;
				elem.sizset = i;
			}
			elem = elem[dir];
			var match = false;

			while ( elem ) {
				if ( elem.sizcache === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 ) {
					if ( !isXML ) {
						elem.sizcache = doneName;
						elem.sizset = i;
					}
					if ( typeof cur !== "string" ) {
						if ( elem === cur ) {
							match = true;
							break;
						}

					} else if ( Sizzle.filter( cur, [elem] ).length > 0 ) {
						match = elem;
						break;
					}
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

var contains = document.compareDocumentPosition ?  function(a, b){
	return a.compareDocumentPosition(b) & 16;
} : function(a, b){
	return a !== b && (a.contains ? a.contains(b) : true);
};

var isXML = function(elem){
	return elem.nodeType === 9 && elem.documentElement.nodeName !== "HTML" ||
		!!elem.ownerDocument && isXML( elem.ownerDocument );
};

var posProcess = function(selector, context){
	var tmpSet = [], later = "", match,
		root = context.nodeType ? [context] : context;

	// Position selectors must be done after the filter
	// And so must :not(positional) so we move all PSEUDOs to the end
	while ( (match = Expr.match.PSEUDO.exec( selector )) ) {
		later += match[0];
		selector = selector.replace( Expr.match.PSEUDO, "" );
	}

	selector = Expr.relative[selector] ? selector + "*" : selector;

	for ( var i = 0, l = root.length; i < l; i++ ) {
		Sizzle( selector, root[i], tmpSet );
	}

	return Sizzle.filter( later, tmpSet );
};

// EXPOSE
jQuery.find = Sizzle;
jQuery.filter = Sizzle.filter;
jQuery.expr = Sizzle.selectors;
jQuery.expr[":"] = jQuery.expr.filters;

Sizzle.selectors.filters.hidden = function(elem){
	return elem.offsetWidth === 0 || elem.offsetHeight === 0;
};

Sizzle.selectors.filters.visible = function(elem){
	return elem.offsetWidth > 0 || elem.offsetHeight > 0;
};

Sizzle.selectors.filters.animated = function(elem){
	return jQuery.grep(jQuery.timers, function(fn){
		return elem === fn.elem;
	}).length;
};

jQuery.multiFilter = function( expr, elems, not ) {
	if ( not ) {
		expr = ":not(" + expr + ")";
	}

	return Sizzle.matches(expr, elems);
};

jQuery.dir = function( elem, dir ){
	var matched = [], cur = elem[dir];
	while ( cur && cur != document ) {
		if ( cur.nodeType == 1 )
			matched.push( cur );
		cur = cur[dir];
	}
	return matched;
};

jQuery.nth = function(cur, result, dir, elem){
	result = result || 1;
	var num = 0;

	for ( ; cur; cur = cur[dir] )
		if ( cur.nodeType == 1 && ++num == result )
			break;

	return cur;
};

jQuery.sibling = function(n, elem){
	var r = [];

	for ( ; n; n = n.nextSibling ) {
		if ( n.nodeType == 1 && n != elem )
			r.push( n );
	}

	return r;
};

return;

window.Sizzle = Sizzle;

})();
/*
 * A number of helper functions used for managing events.
 * Many of the ideas behind this code originated from
 * Dean Edwards' addEvent library.
 */
jQuery.event = {

	// Bind an event to an element
	// Original by Dean Edwards
	add: function(elem, types, handler, data) {
		if ( elem.nodeType == 3 || elem.nodeType == 8 )
			return;

		// For whatever reason, IE has trouble passing the window object
		// around, causing it to be cloned in the process
		if ( elem.setInterval && elem != window )
			elem = window;

		// Make sure that the function being executed has a unique ID
		if ( !handler.guid )
			handler.guid = this.guid++;

		// if data is passed, bind to handler
		if ( data !== undefined ) {
			// Create temporary function pointer to original handler
			var fn = handler;

			// Create unique handler function, wrapped around original handler
			handler = this.proxy( fn );

			// Store data in unique handler
			handler.data = data;
		}

		// Init the element's event structure
		var events = jQuery.data(elem, "events") || jQuery.data(elem, "events", {}),
			handle = jQuery.data(elem, "handle") || jQuery.data(elem, "handle", function(){
				// Handle the second event of a trigger and when
				// an event is called after a page has unloaded
				return typeof jQuery !== "undefined" && !jQuery.event.triggered ?
					jQuery.event.handle.apply(arguments.callee.elem, arguments) :
					undefined;
			});
		// Add elem as a property of the handle function
		// This is to prevent a memory leak with non-native
		// event in IE.
		handle.elem = elem;

		// Handle multiple events separated by a space
		// jQuery(...).bind("mouseover mouseout", fn);
		jQuery.each(types.split(/\s+/), function(index, type) {
			// Namespaced event handlers
			var namespaces = type.split(".");
			type = namespaces.shift();
			handler.type = namespaces.slice().sort().join(".");

			// Get the current list of functions bound to this event
			var handlers = events[type];
			
			if ( jQuery.event.specialAll[type] )
				jQuery.event.specialAll[type].setup.call(elem, data, namespaces);

			// Init the event handler queue
			if (!handlers) {
				handlers = events[type] = {};

				// Check for a special event handler
				// Only use addEventListener/attachEvent if the special
				// events handler returns false
				if ( !jQuery.event.special[type] || jQuery.event.special[type].setup.call(elem, data, namespaces) === false ) {
					// Bind the global event handler to the element
					if (elem.addEventListener)
						elem.addEventListener(type, handle, false);
					else if (elem.attachEvent)
						elem.attachEvent("on" + type, handle);
				}
			}

			// Add the function to the element's handler list
			handlers[handler.guid] = handler;

			// Keep track of which events have been used, for global triggering
			jQuery.event.global[type] = true;
		});

		// Nullify elem to prevent memory leaks in IE
		elem = null;
	},

	guid: 1,
	global: {},

	// Detach an event or set of events from an element
	remove: function(elem, types, handler) {
		// don't do events on text and comment nodes
		if ( elem.nodeType == 3 || elem.nodeType == 8 )
			return;

		var events = jQuery.data(elem, "events"), ret, index;

		if ( events ) {
			// Unbind all events for the element
			if ( types === undefined || (typeof types === "string" && types.charAt(0) == ".") )
				for ( var type in events )
					this.remove( elem, type + (types || "") );
			else {
				// types is actually an event object here
				if ( types.type ) {
					handler = types.handler;
					types = types.type;
				}

				// Handle multiple events seperated by a space
				// jQuery(...).unbind("mouseover mouseout", fn);
				jQuery.each(types.split(/\s+/), function(index, type){
					// Namespaced event handlers
					var namespaces = type.split(".");
					type = namespaces.shift();
					var namespace = RegExp("(^|\\.)" + namespaces.slice().sort().join(".*\\.") + "(\\.|$)");

					if ( events[type] ) {
						// remove the given handler for the given type
						if ( handler )
							delete events[type][handler.guid];

						// remove all handlers for the given type
						else
							for ( var handle in events[type] )
								// Handle the removal of namespaced events
								if ( namespace.test(events[type][handle].type) )
									delete events[type][handle];
									
						if ( jQuery.event.specialAll[type] )
							jQuery.event.specialAll[type].teardown.call(elem, namespaces);

						// remove generic event handler if no more handlers exist
						for ( ret in events[type] ) break;
						if ( !ret ) {
							if ( !jQuery.event.special[type] || jQuery.event.special[type].teardown.call(elem, namespaces) === false ) {
								if (elem.removeEventListener)
									elem.removeEventListener(type, jQuery.data(elem, "handle"), false);
								else if (elem.detachEvent)
									elem.detachEvent("on" + type, jQuery.data(elem, "handle"));
							}
							ret = null;
							delete events[type];
						}
					}
				});
			}

			// Remove the expando if it's no longer used
			for ( ret in events ) break;
			if ( !ret ) {
				var handle = jQuery.data( elem, "handle" );
				if ( handle ) handle.elem = null;
				jQuery.removeData( elem, "events" );
				jQuery.removeData( elem, "handle" );
			}
		}
	},

	// bubbling is internal
	trigger: function( event, data, elem, bubbling ) {
		// Event object or event type
		var type = event.type || event;

		if( !bubbling ){
			event = typeof event === "object" ?
				// jQuery.Event object
				event[expando] ? event :
				// Object literal
				jQuery.extend( jQuery.Event(type), event ) :
				// Just the event type (string)
				jQuery.Event(type);

			if ( type.indexOf("!") >= 0 ) {
				event.type = type = type.slice(0, -1);
				event.exclusive = true;
			}

			// Handle a global trigger
			if ( !elem ) {
				// Don't bubble custom events when global (to avoid too much overhead)
				event.stopPropagation();
				// Only trigger if we've ever bound an event for it
				if ( this.global[type] )
					jQuery.each( jQuery.cache, function(){
						if ( this.events && this.events[type] )
							jQuery.event.trigger( event, data, this.handle.elem );
					});
			}

			// Handle triggering a single element

			// don't do events on text and comment nodes
			if ( !elem || elem.nodeType == 3 || elem.nodeType == 8 )
				return undefined;
			
			// Clean up in case it is reused
			event.result = undefined;
			event.target = elem;
			
			// Clone the incoming data, if any
			data = jQuery.makeArray(data);
			data.unshift( event );
		}

		event.currentTarget = elem;

		// Trigger the event, it is assumed that "handle" is a function
		var handle = jQuery.data(elem, "handle");
		if ( handle )
			handle.apply( elem, data );

		// Handle triggering native .onfoo handlers (and on links since we don't call .click() for links)
		if ( (!elem[type] || (jQuery.nodeName(elem, 'a') && type == "click")) && elem["on"+type] && elem["on"+type].apply( elem, data ) === false )
			event.result = false;

		// Trigger the native events (except for clicks on links)
		if ( !bubbling && elem[type] && !event.isDefaultPrevented() && !(jQuery.nodeName(elem, 'a') && type == "click") ) {
			this.triggered = true;
			try {
				elem[ type ]();
			// prevent IE from throwing an error for some hidden elements
			} catch (e) {}
		}

		this.triggered = false;

		if ( !event.isPropagationStopped() ) {
			var parent = elem.parentNode || elem.ownerDocument;
			if ( parent )
				jQuery.event.trigger(event, data, parent, true);
		}
	},

	handle: function(event) {
		// returned undefined or false
		var all, handlers;

		event = arguments[0] = jQuery.event.fix( event || window.event );
		event.currentTarget = this;
		
		// Namespaced event handlers
		var namespaces = event.type.split(".");
		event.type = namespaces.shift();

		// Cache this now, all = true means, any handler
		all = !namespaces.length && !event.exclusive;
		
		var namespace = RegExp("(^|\\.)" + namespaces.slice().sort().join(".*\\.") + "(\\.|$)");

		handlers = ( jQuery.data(this, "events") || {} )[event.type];

		for ( var j in handlers ) {
			var handler = handlers[j];

			// Filter the functions by class
			if ( all || namespace.test(handler.type) ) {
				// Pass in a reference to the handler function itself
				// So that we can later remove it
				event.handler = handler;
				event.data = handler.data;

				var ret = handler.apply(this, arguments);

				if( ret !== undefined ){
					event.result = ret;
					if ( ret === false ) {
						event.preventDefault();
						event.stopPropagation();
					}
				}

				if( event.isImmediatePropagationStopped() )
					break;

			}
		}
	},

	props: "altKey attrChange attrName bubbles button cancelable charCode clientX clientY ctrlKey currentTarget data detail eventPhase fromElement handler keyCode metaKey newValue originalTarget pageX pageY prevValue relatedNode relatedTarget screenX screenY shiftKey srcElement target toElement view wheelDelta which".split(" "),

	fix: function(event) {
		if ( event[expando] )
			return event;

		// store a copy of the original event object
		// and "clone" to set read-only properties
		var originalEvent = event;
		event = jQuery.Event( originalEvent );

		for ( var i = this.props.length, prop; i; ){
			prop = this.props[ --i ];
			event[ prop ] = originalEvent[ prop ];
		}

		// Fix target property, if necessary
		if ( !event.target )
			event.target = event.srcElement || document; // Fixes #1925 where srcElement might not be defined either

		// check if target is a textnode (safari)
		if ( event.target.nodeType == 3 )
			event.target = event.target.parentNode;

		// Add relatedTarget, if necessary
		if ( !event.relatedTarget && event.fromElement )
			event.relatedTarget = event.fromElement == event.target ? event.toElement : event.fromElement;

		// Calculate pageX/Y if missing and clientX/Y available
		if ( event.pageX == null && event.clientX != null ) {
			var doc = document.documentElement, body = document.body;
			event.pageX = event.clientX + (doc && doc.scrollLeft || body && body.scrollLeft || 0) - (doc.clientLeft || 0);
			event.pageY = event.clientY + (doc && doc.scrollTop || body && body.scrollTop || 0) - (doc.clientTop || 0);
		}

		// Add which for key events
		if ( !event.which && ((event.charCode || event.charCode === 0) ? event.charCode : event.keyCode) )
			event.which = event.charCode || event.keyCode;

		// Add metaKey to non-Mac browsers (use ctrl for PC's and Meta for Macs)
		if ( !event.metaKey && event.ctrlKey )
			event.metaKey = event.ctrlKey;

		// Add which for click: 1 == left; 2 == middle; 3 == right
		// Note: button is not normalized, so don't use it
		if ( !event.which && event.button )
			event.which = (event.button & 1 ? 1 : ( event.button & 2 ? 3 : ( event.button & 4 ? 2 : 0 ) ));

		return event;
	},

	proxy: function( fn, proxy ){
		proxy = proxy || function(){ return fn.apply(this, arguments); };
		// Set the guid of unique handler to the same of original handler, so it can be removed
		proxy.guid = fn.guid = fn.guid || proxy.guid || this.guid++;
		// So proxy can be declared as an argument
		return proxy;
	},

	special: {
		ready: {
			// Make sure the ready event is setup
			setup: bindReady,
			teardown: function() {}
		}
	},
	
	specialAll: {
		live: {
			setup: function( selector, namespaces ){
				jQuery.event.add( this, namespaces[0], liveHandler );
			},
			teardown:  function( namespaces ){
				if ( namespaces.length ) {
					var remove = 0, name = RegExp("(^|\\.)" + namespaces[0] + "(\\.|$)");
					
					jQuery.each( (jQuery.data(this, "events").live || {}), function(){
						if ( name.test(this.type) )
							remove++;
					});
					
					if ( remove < 1 )
						jQuery.event.remove( this, namespaces[0], liveHandler );
				}
			}
		}
	}
};

jQuery.Event = function( src ){
	// Allow instantiation without the 'new' keyword
	if( !this.preventDefault )
		return new jQuery.Event(src);
	
	// Event object
	if( src && src.type ){
		this.originalEvent = src;
		this.type = src.type;
	// Event type
	}else
		this.type = src;

	// timeStamp is buggy for some events on Firefox(#3843)
	// So we won't rely on the native value
	this.timeStamp = now();
	
	// Mark it as fixed
	this[expando] = true;
};

function returnFalse(){
	return false;
}
function returnTrue(){
	return true;
}

// jQuery.Event is based on DOM3 Events as specified by the ECMAScript Language Binding
// http://www.w3.org/TR/2003/WD-DOM-Level-3-Events-20030331/ecma-script-binding.html
jQuery.Event.prototype = {
	preventDefault: function() {
		this.isDefaultPrevented = returnTrue;

		var e = this.originalEvent;
		if( !e )
			return;
		// if preventDefault exists run it on the original event
		if (e.preventDefault)
			e.preventDefault();
		// otherwise set the returnValue property of the original event to false (IE)
		e.returnValue = false;
	},
	stopPropagation: function() {
		this.isPropagationStopped = returnTrue;

		var e = this.originalEvent;
		if( !e )
			return;
		// if stopPropagation exists run it on the original event
		if (e.stopPropagation)
			e.stopPropagation();
		// otherwise set the cancelBubble property of the original event to true (IE)
		e.cancelBubble = true;
	},
	stopImmediatePropagation:function(){
		this.isImmediatePropagationStopped = returnTrue;
		this.stopPropagation();
	},
	isDefaultPrevented: returnFalse,
	isPropagationStopped: returnFalse,
	isImmediatePropagationStopped: returnFalse
};
// Checks if an event happened on an element within another element
// Used in jQuery.event.special.mouseenter and mouseleave handlers
var withinElement = function(event) {
	// Check if mouse(over|out) are still within the same parent element
	var parent = event.relatedTarget;
	// Traverse up the tree
	while ( parent && parent != this )
		try { parent = parent.parentNode; }
		catch(e) { parent = this; }
	
	if( parent != this ){
		// set the correct event type
		event.type = event.data;
		// handle event if we actually just moused on to a non sub-element
		jQuery.event.handle.apply( this, arguments );
	}
};
	
jQuery.each({ 
	mouseover: 'mouseenter', 
	mouseout: 'mouseleave'
}, function( orig, fix ){
	jQuery.event.special[ fix ] = {
		setup: function(){
			jQuery.event.add( this, orig, withinElement, fix );
		},
		teardown: function(){
			jQuery.event.remove( this, orig, withinElement );
		}
	};			   
});

jQuery.fn.extend({
	bind: function( type, data, fn ) {
		return type == "unload" ? this.one(type, data, fn) : this.each(function(){
			jQuery.event.add( this, type, fn || data, fn && data );
		});
	},

	one: function( type, data, fn ) {
		var one = jQuery.event.proxy( fn || data, function(event) {
			jQuery(this).unbind(event, one);
			return (fn || data).apply( this, arguments );
		});
		return this.each(function(){
			jQuery.event.add( this, type, one, fn && data);
		});
	},

	unbind: function( type, fn ) {
		return this.each(function(){
			jQuery.event.remove( this, type, fn );
		});
	},

	trigger: function( type, data ) {
		return this.each(function(){
			jQuery.event.trigger( type, data, this );
		});
	},

	triggerHandler: function( type, data ) {
		if( this[0] ){
			var event = jQuery.Event(type);
			event.preventDefault();
			event.stopPropagation();
			jQuery.event.trigger( event, data, this[0] );
			return event.result;
		}		
	},

	toggle: function( fn ) {
		// Save reference to arguments for access in closure
		var args = arguments, i = 1;

		// link all the functions, so any of them can unbind this click handler
		while( i < args.length )
			jQuery.event.proxy( fn, args[i++] );

		return this.click( jQuery.event.proxy( fn, function(event) {
			// Figure out which function to execute
			this.lastToggle = ( this.lastToggle || 0 ) % i;

			// Make sure that clicks stop
			event.preventDefault();

			// and execute the function
			return args[ this.lastToggle++ ].apply( this, arguments ) || false;
		}));
	},

	hover: function(fnOver, fnOut) {
		return this.mouseenter(fnOver).mouseleave(fnOut);
	},

	ready: function(fn) {
		// Attach the listeners
		bindReady();

		// If the DOM is already ready
		if ( jQuery.isReady )
			// Execute the function immediately
			fn.call( document, jQuery );

		// Otherwise, remember the function for later
		else
			// Add the function to the wait list
			jQuery.readyList.push( fn );

		return this;
	},
	
	live: function( type, fn ){
		var proxy = jQuery.event.proxy( fn );
		proxy.guid += this.selector + type;

		jQuery(document).bind( liveConvert(type, this.selector), this.selector, proxy );

		return this;
	},
	
	die: function( type, fn ){
		jQuery(document).unbind( liveConvert(type, this.selector), fn ? { guid: fn.guid + this.selector + type } : null );
		return this;
	}
});

function liveHandler( event ){
	var check = RegExp("(^|\\.)" + event.type + "(\\.|$)"),
		stop = true,
		elems = [];

	jQuery.each(jQuery.data(this, "events").live || [], function(i, fn){
		if ( check.test(fn.type) ) {
			var elem = jQuery(event.target).closest(fn.data)[0];
			if ( elem )
				elems.push({ elem: elem, fn: fn });
		}
	});

	elems.sort(function(a,b) {
		return jQuery.data(a.elem, "closest") - jQuery.data(b.elem, "closest");
	});
	
	jQuery.each(elems, function(){
		if ( this.fn.call(this.elem, event, this.fn.data) === false )
			return (stop = false);
	});

	return stop;
}

function liveConvert(type, selector){
	return ["live", type, selector.replace(/\./g, "`").replace(/ /g, "|")].join(".");
}

jQuery.extend({
	isReady: false,
	readyList: [],
	// Handle when the DOM is ready
	ready: function() {
		// Make sure that the DOM is not already loaded
		if ( !jQuery.isReady ) {
			// Remember that the DOM is ready
			jQuery.isReady = true;

			// If there are functions bound, to execute
			if ( jQuery.readyList ) {
				// Execute all of them
				jQuery.each( jQuery.readyList, function(){
					this.call( document, jQuery );
				});

				// Reset the list of functions
				jQuery.readyList = null;
			}

			// Trigger any bound ready events
			jQuery(document).triggerHandler("ready");
		}
	}
});

var readyBound = false;

function bindReady(){
	if ( readyBound ) return;
	readyBound = true;

	// Mozilla, Opera and webkit nightlies currently support this event
	if ( document.addEventListener ) {
		// Use the handy event callback
		document.addEventListener( "DOMContentLoaded", function(){
			document.removeEventListener( "DOMContentLoaded", arguments.callee, false );
			jQuery.ready();
		}, false );

	// If IE event model is used
	} else if ( document.attachEvent ) {
		// ensure firing before onload,
		// maybe late but safe also for iframes
		document.attachEvent("onreadystatechange", function(){
			if ( document.readyState === "complete" ) {
				document.detachEvent( "onreadystatechange", arguments.callee );
				jQuery.ready();
			}
		});

		// If IE and not an iframe
		// continually check to see if the document is ready
		if ( document.documentElement.doScroll && window == window.top ) (function(){
			if ( jQuery.isReady ) return;

			try {
				// If IE is used, use the trick by Diego Perini
				// http://javascript.nwbox.com/IEContentLoaded/
				document.documentElement.doScroll("left");
			} catch( error ) {
				setTimeout( arguments.callee, 0 );
				return;
			}

			// and execute any waiting functions
			jQuery.ready();
		})();
	}

	// A fallback to window.onload, that will always work
	jQuery.event.add( window, "load", jQuery.ready );
}

jQuery.each( ("blur,focus,load,resize,scroll,unload,click,dblclick," +
	"mousedown,mouseup,mousemove,mouseover,mouseout,mouseenter,mouseleave," +
	"change,select,submit,keydown,keypress,keyup,error").split(","), function(i, name){

	// Handle event binding
	jQuery.fn[name] = function(fn){
		return fn ? this.bind(name, fn) : this.trigger(name);
	};
});

// Prevent memory leaks in IE
// And prevent errors on refresh with events like mouseover in other browsers
// Window isn't included so as not to unbind existing unload events
jQuery( window ).bind( 'unload', function(){ 
	for ( var id in jQuery.cache )
		// Skip the window
		if ( id != 1 && jQuery.cache[ id ].handle )
			jQuery.event.remove( jQuery.cache[ id ].handle.elem );
}); 
(function(){

	jQuery.support = {};

	var root = document.documentElement,
		script = document.createElement("script"),
		div = document.createElement("div"),
		id = "script" + (new Date).getTime();

	div.style.display = "none";
	div.innerHTML = '   <link/><table></table><a href="/a" style="color:red;float:left;opacity:.5;">a</a><select><option>text</option></select><object><param/></object>';

	var all = div.getElementsByTagName("*"),
		a = div.getElementsByTagName("a")[0];

	// Can't get basic test support
	if ( !all || !all.length || !a ) {
		return;
	}

	jQuery.support = {
		// IE strips leading whitespace when .innerHTML is used
		leadingWhitespace: div.firstChild.nodeType == 3,
		
		// Make sure that tbody elements aren't automatically inserted
		// IE will insert them into empty tables
		tbody: !div.getElementsByTagName("tbody").length,
		
		// Make sure that you can get all elements in an <object> element
		// IE 7 always returns no results
		objectAll: !!div.getElementsByTagName("object")[0]
			.getElementsByTagName("*").length,
		
		// Make sure that link elements get serialized correctly by innerHTML
		// This requires a wrapper element in IE
		htmlSerialize: !!div.getElementsByTagName("link").length,
		
		// Get the style information from getAttribute
		// (IE uses .cssText insted)
		style: /red/.test( a.getAttribute("style") ),
		
		// Make sure that URLs aren't manipulated
		// (IE normalizes it by default)
		hrefNormalized: a.getAttribute("href") === "/a",
		
		// Make sure that element opacity exists
		// (IE uses filter instead)
		opacity: a.style.opacity === "0.5",
		
		// Verify style float existence
		// (IE uses styleFloat instead of cssFloat)
		cssFloat: !!a.style.cssFloat,

		// Will be defined later
		scriptEval: false,
		noCloneEvent: true,
		boxModel: null
	};
	
	script.type = "text/javascript";
	try {
		script.appendChild( document.createTextNode( "window." + id + "=1;" ) );
	} catch(e){}

	root.insertBefore( script, root.firstChild );
	
	// Make sure that the execution of code works by injecting a script
	// tag with appendChild/createTextNode
	// (IE doesn't support this, fails, and uses .text instead)
	if ( window[ id ] ) {
		jQuery.support.scriptEval = true;
		delete window[ id ];
	}

	root.removeChild( script );

	if ( div.attachEvent && div.fireEvent ) {
		div.attachEvent("onclick", function(){
			// Cloning a node shouldn't copy over any
			// bound event handlers (IE does this)
			jQuery.support.noCloneEvent = false;
			div.detachEvent("onclick", arguments.callee);
		});
		div.cloneNode(true).fireEvent("onclick");
	}

	// Figure out if the W3C box model works as expected
	// document.body must exist before we can do this
	jQuery(function(){
		var div = document.createElement("div");
		div.style.width = div.style.paddingLeft = "1px";

		document.body.appendChild( div );
		jQuery.boxModel = jQuery.support.boxModel = div.offsetWidth === 2;
		document.body.removeChild( div ).style.display = 'none';
	});
})();

var styleFloat = jQuery.support.cssFloat ? "cssFloat" : "styleFloat";

jQuery.props = {
	"for": "htmlFor",
	"class": "className",
	"float": styleFloat,
	cssFloat: styleFloat,
	styleFloat: styleFloat,
	readonly: "readOnly",
	maxlength: "maxLength",
	cellspacing: "cellSpacing",
	rowspan: "rowSpan",
	tabindex: "tabIndex"
};
jQuery.fn.extend({
	// Keep a copy of the old load
	_load: jQuery.fn.load,

	load: function( url, params, callback ) {
		if ( typeof url !== "string" )
			return this._load( url );

		var off = url.indexOf(" ");
		if ( off >= 0 ) {
			var selector = url.slice(off, url.length);
			url = url.slice(0, off);
		}

		// Default to a GET request
		var type = "GET";

		// If the second parameter was provided
		if ( params )
			// If it's a function
			if ( jQuery.isFunction( params ) ) {
				// We assume that it's the callback
				callback = params;
				params = null;

			// Otherwise, build a param string
			} else if( typeof params === "object" ) {
				params = jQuery.param( params );
				type = "POST";
			}

		var self = this;

		// Request the remote document
		jQuery.ajax({
			url: url,
			type: type,
			dataType: "html",
			data: params,
			complete: function(res, status){
				// If successful, inject the HTML into all the matched elements
				if ( status == "success" || status == "notmodified" )
					// See if a selector was specified
					self.html( selector ?
						// Create a dummy div to hold the results
						jQuery("<div/>")
							// inject the contents of the document in, removing the scripts
							// to avoid any 'Permission Denied' errors in IE
							.append(res.responseText.replace(/<script(.|\s)*?\/script>/g, ""))

							// Locate the specified elements
							.find(selector) :

						// If not, just inject the full result
						res.responseText );

				if( callback )
					self.each( callback, [res.responseText, status, res] );
			}
		});
		return this;
	},

	serialize: function() {
		return jQuery.param(this.serializeArray());
	},
	serializeArray: function() {
		return this.map(function(){
			return this.elements ? jQuery.makeArray(this.elements) : this;
		})
		.filter(function(){
			return this.name && !this.disabled &&
				(this.checked || /select|textarea/i.test(this.nodeName) ||
					/text|hidden|password|search/i.test(this.type));
		})
		.map(function(i, elem){
			var val = jQuery(this).val();
			return val == null ? null :
				jQuery.isArray(val) ?
					jQuery.map( val, function(val, i){
						return {name: elem.name, value: val};
					}) :
					{name: elem.name, value: val};
		}).get();
	}
});

// Attach a bunch of functions for handling common AJAX events
jQuery.each( "ajaxStart,ajaxStop,ajaxComplete,ajaxError,ajaxSuccess,ajaxSend".split(","), function(i,o){
	jQuery.fn[o] = function(f){
		return this.bind(o, f);
	};
});

var jsc = now();

jQuery.extend({
  
	get: function( url, data, callback, type ) {
		// shift arguments if data argument was ommited
		if ( jQuery.isFunction( data ) ) {
			callback = data;
			data = null;
		}

		return jQuery.ajax({
			type: "GET",
			url: url,
			data: data,
			success: callback,
			dataType: type
		});
	},

	getScript: function( url, callback ) {
		return jQuery.get(url, null, callback, "script");
	},

	getJSON: function( url, data, callback ) {
		return jQuery.get(url, data, callback, "json");
	},

	post: function( url, data, callback, type ) {
		if ( jQuery.isFunction( data ) ) {
			callback = data;
			data = {};
		}

		return jQuery.ajax({
			type: "POST",
			url: url,
			data: data,
			success: callback,
			dataType: type
		});
	},

	ajaxSetup: function( settings ) {
		jQuery.extend( jQuery.ajaxSettings, settings );
	},

	ajaxSettings: {
		url: location.href,
		global: true,
		type: "GET",
		contentType: "application/x-www-form-urlencoded",
		processData: true,
		async: true,
		/*
		timeout: 0,
		data: null,
		username: null,
		password: null,
		*/
		// Create the request object; Microsoft failed to properly
		// implement the XMLHttpRequest in IE7, so we use the ActiveXObject when it is available
		// This function can be overriden by calling jQuery.ajaxSetup
		xhr:function(){
			return window.ActiveXObject ? new ActiveXObject("Microsoft.XMLHTTP") : new XMLHttpRequest();
		},
		accepts: {
			xml: "application/xml, text/xml",
			html: "text/html",
			script: "text/javascript, application/javascript",
			json: "application/json, text/javascript",
			text: "text/plain",
			_default: "*/*"
		}
	},

	// Last-Modified header cache for next request
	lastModified: {},

	ajax: function( s ) {
		// Extend the settings, but re-extend 's' so that it can be
		// checked again later (in the test suite, specifically)
		s = jQuery.extend(true, s, jQuery.extend(true, {}, jQuery.ajaxSettings, s));

		var jsonp, jsre = /=\?(&|$)/g, status, data,
			type = s.type.toUpperCase();

		// convert data if not already a string
		if ( s.data && s.processData && typeof s.data !== "string" )
			s.data = jQuery.param(s.data);

		// Handle JSONP Parameter Callbacks
		if ( s.dataType == "jsonp" ) {
			if ( type == "GET" ) {
				if ( !s.url.match(jsre) )
					s.url += (s.url.match(/\?/) ? "&" : "?") + (s.jsonp || "callback") + "=?";
			} else if ( !s.data || !s.data.match(jsre) )
				s.data = (s.data ? s.data + "&" : "") + (s.jsonp || "callback") + "=?";
			s.dataType = "json";
		}

		// Build temporary JSONP function
		if ( s.dataType == "json" && (s.data && s.data.match(jsre) || s.url.match(jsre)) ) {
			jsonp = "jsonp" + jsc++;

			// Replace the =? sequence both in the query string and the data
			if ( s.data )
				s.data = (s.data + "").replace(jsre, "=" + jsonp + "$1");
			s.url = s.url.replace(jsre, "=" + jsonp + "$1");

			// We need to make sure
			// that a JSONP style response is executed properly
			s.dataType = "script";

			// Handle JSONP-style loading
			window[ jsonp ] = function(tmp){
				data = tmp;
				success();
				complete();
				// Garbage collect
				window[ jsonp ] = undefined;
				try{ delete window[ jsonp ]; } catch(e){}
				if ( head )
					head.removeChild( script );
			};
		}

		if ( s.dataType == "script" && s.cache == null )
			s.cache = false;

		if ( s.cache === false && type == "GET" ) {
			var ts = now();
			// try replacing _= if it is there
			var ret = s.url.replace(/(\?|&)_=.*?(&|$)/, "$1_=" + ts + "$2");
			// if nothing was replaced, add timestamp to the end
			s.url = ret + ((ret == s.url) ? (s.url.match(/\?/) ? "&" : "?") + "_=" + ts : "");
		}

		// If data is available, append data to url for get requests
		if ( s.data && type == "GET" ) {
			s.url += (s.url.match(/\?/) ? "&" : "?") + s.data;

			// IE likes to send both get and post data, prevent this
			s.data = null;
		}

		// Watch for a new set of requests
		if ( s.global && ! jQuery.active++ )
			jQuery.event.trigger( "ajaxStart" );

		// Matches an absolute URL, and saves the domain
		var parts = /^(\w+:)?\/\/([^\/?#]+)/.exec( s.url );

		// If we're requesting a remote document
		// and trying to load JSON or Script with a GET
		if ( s.dataType == "script" && type == "GET" && parts
			&& ( parts[1] && parts[1] != location.protocol || parts[2] != location.host )){

			var head = document.getElementsByTagName("head")[0];
			var script = document.createElement("script");
			script.src = s.url;
			if (s.scriptCharset)
				script.charset = s.scriptCharset;

			// Handle Script loading
			if ( !jsonp ) {
				var done = false;

				// Attach handlers for all browsers
				script.onload = script.onreadystatechange = function(){
					if ( !done && (!this.readyState ||
							this.readyState == "loaded" || this.readyState == "complete") ) {
						done = true;
						success();
						complete();

						// Handle memory leak in IE
						script.onload = script.onreadystatechange = null;
						head.removeChild( script );
					}
				};
			}

			head.appendChild(script);

			// We handle everything using the script element injection
			return undefined;
		}

		var requestDone = false;

		// Create the request object
		var xhr = s.xhr();

		// Open the socket
		// Passing null username, generates a login popup on Opera (#2865)
		if( s.username )
			xhr.open(type, s.url, s.async, s.username, s.password);
		else
			xhr.open(type, s.url, s.async);

		// Need an extra try/catch for cross domain requests in Firefox 3
		try {
			// Set the correct header, if data is being sent
			if ( s.data )
				xhr.setRequestHeader("Content-Type", s.contentType);

			// Set the If-Modified-Since header, if ifModified mode.
			if ( s.ifModified )
				xhr.setRequestHeader("If-Modified-Since",
					jQuery.lastModified[s.url] || "Thu, 01 Jan 1970 00:00:00 GMT" );

			// Set header so the called script knows that it's an XMLHttpRequest
			xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");

			// Set the Accepts header for the server, depending on the dataType
			xhr.setRequestHeader("Accept", s.dataType && s.accepts[ s.dataType ] ?
				s.accepts[ s.dataType ] + ", */*" :
				s.accepts._default );
		} catch(e){}

		// Allow custom headers/mimetypes and early abort
		if ( s.beforeSend && s.beforeSend(xhr, s) === false ) {
			// Handle the global AJAX counter
			if ( s.global && ! --jQuery.active )
				jQuery.event.trigger( "ajaxStop" );
			// close opended socket
			xhr.abort();
			return false;
		}

		if ( s.global )
			jQuery.event.trigger("ajaxSend", [xhr, s]);

		// Wait for a response to come back
		var onreadystatechange = function(isTimeout){
			// The request was aborted, clear the interval and decrement jQuery.active
			if (xhr.readyState == 0) {
				if (ival) {
					// clear poll interval
					clearInterval(ival);
					ival = null;
					// Handle the global AJAX counter
					if ( s.global && ! --jQuery.active )
						jQuery.event.trigger( "ajaxStop" );
				}
			// The transfer is complete and the data is available, or the request timed out
			} else if ( !requestDone && xhr && (xhr.readyState == 4 || isTimeout == "timeout") ) {
				requestDone = true;

				// clear poll interval
				if (ival) {
					clearInterval(ival);
					ival = null;
				}

				status = isTimeout == "timeout" ? "timeout" :
					!jQuery.httpSuccess( xhr ) ? "error" :
					s.ifModified && jQuery.httpNotModified( xhr, s.url ) ? "notmodified" :
					"success";

				if ( status == "success" ) {
					// Watch for, and catch, XML document parse errors
					try {
						// process the data (runs the xml through httpData regardless of callback)
						data = jQuery.httpData( xhr, s.dataType, s );
					} catch(e) {
						status = "parsererror";
					}
				}

				// Make sure that the request was successful or notmodified
				if ( status == "success" ) {
					// Cache Last-Modified header, if ifModified mode.
					var modRes;
					try {
						modRes = xhr.getResponseHeader("Last-Modified");
					} catch(e) {} // swallow exception thrown by FF if header is not available

					if ( s.ifModified && modRes )
						jQuery.lastModified[s.url] = modRes;

					// JSONP handles its own success callback
					if ( !jsonp )
						success();
				} else
					jQuery.handleError(s, xhr, status);

				// Fire the complete handlers
				complete();

				if ( isTimeout )
					xhr.abort();

				// Stop memory leaks
				if ( s.async )
					xhr = null;
			}
		};

		if ( s.async ) {
			// don't attach the handler to the request, just poll it instead
			var ival = setInterval(onreadystatechange, 13);

			// Timeout checker
			if ( s.timeout > 0 )
				setTimeout(function(){
					// Check to see if the request is still happening
					if ( xhr && !requestDone )
						onreadystatechange( "timeout" );
				}, s.timeout);
		}

		// Send the data
		try {
			xhr.send(s.data);
		} catch(e) {
			jQuery.handleError(s, xhr, null, e);
		}

		// firefox 1.5 doesn't fire statechange for sync requests
		if ( !s.async )
			onreadystatechange();

		function success(){
			// If a local callback was specified, fire it and pass it the data
			if ( s.success )
				s.success( data, status );

			// Fire the global callback
			if ( s.global )
				jQuery.event.trigger( "ajaxSuccess", [xhr, s] );
		}

		function complete(){
			// Process result
			if ( s.complete )
				s.complete(xhr, status);

			// The request was completed
			if ( s.global )
				jQuery.event.trigger( "ajaxComplete", [xhr, s] );

			// Handle the global AJAX counter
			if ( s.global && ! --jQuery.active )
				jQuery.event.trigger( "ajaxStop" );
		}

		// return XMLHttpRequest to allow aborting the request etc.
		return xhr;
	},

	handleError: function( s, xhr, status, e ) {
		// If a local callback was specified, fire it
		if ( s.error ) s.error( xhr, status, e );

		// Fire the global callback
		if ( s.global )
			jQuery.event.trigger( "ajaxError", [xhr, s, e] );
	},

	// Counter for holding the number of active queries
	active: 0,

	// Determines if an XMLHttpRequest was successful or not
	httpSuccess: function( xhr ) {
		try {
			// IE error sometimes returns 1223 when it should be 204 so treat it as success, see #1450
			return !xhr.status && location.protocol == "file:" ||
				( xhr.status >= 200 && xhr.status < 300 ) || xhr.status == 304 || xhr.status == 1223;
		} catch(e){}
		return false;
	},

	// Determines if an XMLHttpRequest returns NotModified
	httpNotModified: function( xhr, url ) {
		try {
			var xhrRes = xhr.getResponseHeader("Last-Modified");

			// Firefox always returns 200. check Last-Modified date
			return xhr.status == 304 || xhrRes == jQuery.lastModified[url];
		} catch(e){}
		return false;
	},

	httpData: function( xhr, type, s ) {
		var ct = xhr.getResponseHeader("content-type"),
			xml = type == "xml" || !type && ct && ct.indexOf("xml") >= 0,
			data = xml ? xhr.responseXML : xhr.responseText;

		if ( xml && data.documentElement.tagName == "parsererror" )
			throw "parsererror";
			
		// Allow a pre-filtering function to sanitize the response
		// s != null is checked to keep backwards compatibility
		if( s && s.dataFilter )
			data = s.dataFilter( data, type );

		// The filter can actually parse the response
		if( typeof data === "string" ){

			// If the type is "script", eval it in global context
			if ( type == "script" )
				jQuery.globalEval( data );

			// Get the JavaScript object, if JSON is used.
			if ( type == "json" )
				data = window["eval"]("(" + data + ")");
		}
		
		return data;
	},

	// Serialize an array of form elements or a set of
	// key/values into a query string
	param: function( a ) {
		var s = [ ];

		function add( key, value ){
			s[ s.length ] = encodeURIComponent(key) + '=' + encodeURIComponent(value);
		};

		// If an array was passed in, assume that it is an array
		// of form elements
		if ( jQuery.isArray(a) || a.jquery )
			// Serialize the form elements
			jQuery.each( a, function(){
				add( this.name, this.value );
			});

		// Otherwise, assume that it's an object of key/value pairs
		else
			// Serialize the key/values
			for ( var j in a )
				// If the value is an array then the key names need to be repeated
				if ( jQuery.isArray(a[j]) )
					jQuery.each( a[j], function(){
						add( j, this );
					});
				else
					add( j, jQuery.isFunction(a[j]) ? a[j]() : a[j] );

		// Return the resulting serialization
		return s.join("&").replace(/%20/g, "+");
	}

});
var elemdisplay = {},
	timerId,
	fxAttrs = [
		// height animations
		[ "height", "marginTop", "marginBottom", "paddingTop", "paddingBottom" ],
		// width animations
		[ "width", "marginLeft", "marginRight", "paddingLeft", "paddingRight" ],
		// opacity animations
		[ "opacity" ]
	];

function genFx( type, num ){
	var obj = {};
	jQuery.each( fxAttrs.concat.apply([], fxAttrs.slice(0,num)), function(){
		obj[ this ] = type;
	});
	return obj;
}

jQuery.fn.extend({
	show: function(speed,callback){
		if ( speed ) {
			return this.animate( genFx("show", 3), speed, callback);
		} else {
			for ( var i = 0, l = this.length; i < l; i++ ){
				var old = jQuery.data(this[i], "olddisplay");
				
				this[i].style.display = old || "";
				
				if ( jQuery.css(this[i], "display") === "none" ) {
					var tagName = this[i].tagName, display;
					
					if ( elemdisplay[ tagName ] ) {
						display = elemdisplay[ tagName ];
					} else {
						var elem = jQuery("<" + tagName + " />").appendTo("body");
						
						display = elem.css("display");
						if ( display === "none" )
							display = "block";
						
						elem.remove();
						
						elemdisplay[ tagName ] = display;
					}
					
					jQuery.data(this[i], "olddisplay", display);
				}
			}

			// Set the display of the elements in a second loop
			// to avoid the constant reflow
			for ( var i = 0, l = this.length; i < l; i++ ){
				this[i].style.display = jQuery.data(this[i], "olddisplay") || "";
			}
			
			return this;
		}
	},

	hide: function(speed,callback){
		if ( speed ) {
			return this.animate( genFx("hide", 3), speed, callback);
		} else {
			for ( var i = 0, l = this.length; i < l; i++ ){
				var old = jQuery.data(this[i], "olddisplay");
				if ( !old && old !== "none" )
					jQuery.data(this[i], "olddisplay", jQuery.css(this[i], "display"));
			}

			// Set the display of the elements in a second loop
			// to avoid the constant reflow
			for ( var i = 0, l = this.length; i < l; i++ ){
				this[i].style.display = "none";
			}

			return this;
		}
	},

	// Save the old toggle function
	_toggle: jQuery.fn.toggle,

	toggle: function( fn, fn2 ){
		var bool = typeof fn === "boolean";

		return jQuery.isFunction(fn) && jQuery.isFunction(fn2) ?
			this._toggle.apply( this, arguments ) :
			fn == null || bool ?
				this.each(function(){
					var state = bool ? fn : jQuery(this).is(":hidden");
					jQuery(this)[ state ? "show" : "hide" ]();
				}) :
				this.animate(genFx("toggle", 3), fn, fn2);
	},

	fadeTo: function(speed,to,callback){
		return this.animate({opacity: to}, speed, callback);
	},

	animate: function( prop, speed, easing, callback ) {
		var optall = jQuery.speed(speed, easing, callback);

		return this[ optall.queue === false ? "each" : "queue" ](function(){
		
			var opt = jQuery.extend({}, optall), p,
				hidden = this.nodeType == 1 && jQuery(this).is(":hidden"),
				self = this;
	
			for ( p in prop ) {
				if ( prop[p] == "hide" && hidden || prop[p] == "show" && !hidden )
					return opt.complete.call(this);

				if ( ( p == "height" || p == "width" ) && this.style ) {
					// Store display property
					opt.display = jQuery.css(this, "display");

					// Make sure that nothing sneaks out
					opt.overflow = this.style.overflow;
				}
			}

			if ( opt.overflow != null )
				this.style.overflow = "hidden";

			opt.curAnim = jQuery.extend({}, prop);

			jQuery.each( prop, function(name, val){
				var e = new jQuery.fx( self, opt, name );

				if ( /toggle|show|hide/.test(val) )
					e[ val == "toggle" ? hidden ? "show" : "hide" : val ]( prop );
				else {
					var parts = val.toString().match(/^([+-]=)?([\d+-.]+)(.*)$/),
						start = e.cur(true) || 0;

					if ( parts ) {
						var end = parseFloat(parts[2]),
							unit = parts[3] || "px";

						// We need to compute starting value
						if ( unit != "px" ) {
							self.style[ name ] = (end || 1) + unit;
							start = ((end || 1) / e.cur(true)) * start;
							self.style[ name ] = start + unit;
						}

						// If a +=/-= token was provided, we're doing a relative animation
						if ( parts[1] )
							end = ((parts[1] == "-=" ? -1 : 1) * end) + start;

						e.custom( start, end, unit );
					} else
						e.custom( start, val, "" );
				}
			});

			// For JS strict compliance
			return true;
		});
	},

	stop: function(clearQueue, gotoEnd){
		var timers = jQuery.timers;

		if (clearQueue)
			this.queue([]);

		this.each(function(){
			// go in reverse order so anything added to the queue during the loop is ignored
			for ( var i = timers.length - 1; i >= 0; i-- )
				if ( timers[i].elem == this ) {
					if (gotoEnd)
						// force the next step to be the last
						timers[i](true);
					timers.splice(i, 1);
				}
		});

		// start the next in the queue if the last step wasn't forced
		if (!gotoEnd)
			this.dequeue();

		return this;
	}

});

// Generate shortcuts for custom animations
jQuery.each({
	slideDown: genFx("show", 1),
	slideUp: genFx("hide", 1),
	slideToggle: genFx("toggle", 1),
	fadeIn: { opacity: "show" },
	fadeOut: { opacity: "hide" }
}, function( name, props ){
	jQuery.fn[ name ] = function( speed, callback ){
		return this.animate( props, speed, callback );
	};
});

jQuery.extend({

	speed: function(speed, easing, fn) {
		var opt = typeof speed === "object" ? speed : {
			complete: fn || !fn && easing ||
				jQuery.isFunction( speed ) && speed,
			duration: speed,
			easing: fn && easing || easing && !jQuery.isFunction(easing) && easing
		};

		opt.duration = jQuery.fx.off ? 0 : typeof opt.duration === "number" ? opt.duration :
			jQuery.fx.speeds[opt.duration] || jQuery.fx.speeds._default;

		// Queueing
		opt.old = opt.complete;
		opt.complete = function(){
			if ( opt.queue !== false )
				jQuery(this).dequeue();
			if ( jQuery.isFunction( opt.old ) )
				opt.old.call( this );
		};

		return opt;
	},

	easing: {
		linear: function( p, n, firstNum, diff ) {
			return firstNum + diff * p;
		},
		swing: function( p, n, firstNum, diff ) {
			return ((-Math.cos(p*Math.PI)/2) + 0.5) * diff + firstNum;
		}
	},

	timers: [],

	fx: function( elem, options, prop ){
		this.options = options;
		this.elem = elem;
		this.prop = prop;

		if ( !options.orig )
			options.orig = {};
	}

});

jQuery.fx.prototype = {

	// Simple function for setting a style value
	update: function(){
		if ( this.options.step )
			this.options.step.call( this.elem, this.now, this );

		(jQuery.fx.step[this.prop] || jQuery.fx.step._default)( this );

		// Set display property to block for height/width animations
		if ( ( this.prop == "height" || this.prop == "width" ) && this.elem.style )
			this.elem.style.display = "block";
	},

	// Get the current size
	cur: function(force){
		if ( this.elem[this.prop] != null && (!this.elem.style || this.elem.style[this.prop] == null) )
			return this.elem[ this.prop ];

		var r = parseFloat(jQuery.css(this.elem, this.prop, force));
		return r && r > -10000 ? r : parseFloat(jQuery.curCSS(this.elem, this.prop)) || 0;
	},

	// Start an animation from one number to another
	custom: function(from, to, unit){
		this.startTime = now();
		this.start = from;
		this.end = to;
		this.unit = unit || this.unit || "px";
		this.now = this.start;
		this.pos = this.state = 0;

		var self = this;
		function t(gotoEnd){
			return self.step(gotoEnd);
		}

		t.elem = this.elem;

		if ( t() && jQuery.timers.push(t) && !timerId ) {
			timerId = setInterval(function(){
				var timers = jQuery.timers;

				for ( var i = 0; i < timers.length; i++ )
					if ( !timers[i]() )
						timers.splice(i--, 1);

				if ( !timers.length ) {
					clearInterval( timerId );
					timerId = undefined;
				}
			}, 13);
		}
	},

	// Simple 'show' function
	show: function(){
		// Remember where we started, so that we can go back to it later
		this.options.orig[this.prop] = jQuery.attr( this.elem.style, this.prop );
		this.options.show = true;

		// Begin the animation
		// Make sure that we start at a small width/height to avoid any
		// flash of content
		this.custom(this.prop == "width" || this.prop == "height" ? 1 : 0, this.cur());

		// Start by showing the element
		jQuery(this.elem).show();
	},

	// Simple 'hide' function
	hide: function(){
		// Remember where we started, so that we can go back to it later
		this.options.orig[this.prop] = jQuery.attr( this.elem.style, this.prop );
		this.options.hide = true;

		// Begin the animation
		this.custom(this.cur(), 0);
	},

	// Each step of an animation
	step: function(gotoEnd){
		var t = now();

		if ( gotoEnd || t >= this.options.duration + this.startTime ) {
			this.now = this.end;
			this.pos = this.state = 1;
			this.update();

			this.options.curAnim[ this.prop ] = true;

			var done = true;
			for ( var i in this.options.curAnim )
				if ( this.options.curAnim[i] !== true )
					done = false;

			if ( done ) {
				if ( this.options.display != null ) {
					// Reset the overflow
					this.elem.style.overflow = this.options.overflow;

					// Reset the display
					this.elem.style.display = this.options.display;
					if ( jQuery.css(this.elem, "display") == "none" )
						this.elem.style.display = "block";
				}

				// Hide the element if the "hide" operation was done
				if ( this.options.hide )
					jQuery(this.elem).hide();

				// Reset the properties, if the item has been hidden or shown
				if ( this.options.hide || this.options.show )
					for ( var p in this.options.curAnim )
						jQuery.attr(this.elem.style, p, this.options.orig[p]);
					
				// Execute the complete function
				this.options.complete.call( this.elem );
			}

			return false;
		} else {
			var n = t - this.startTime;
			this.state = n / this.options.duration;

			// Perform the easing function, defaults to swing
			this.pos = jQuery.easing[this.options.easing || (jQuery.easing.swing ? "swing" : "linear")](this.state, n, 0, 1, this.options.duration);
			this.now = this.start + ((this.end - this.start) * this.pos);

			// Perform the next step of the animation
			this.update();
		}

		return true;
	}

};

jQuery.extend( jQuery.fx, {
	speeds:{
		slow: 600,
 		fast: 200,
 		// Default speed
 		_default: 400
	},
	step: {

		opacity: function(fx){
			jQuery.attr(fx.elem.style, "opacity", fx.now);
		},

		_default: function(fx){
			if ( fx.elem.style && fx.elem.style[ fx.prop ] != null )
				fx.elem.style[ fx.prop ] = fx.now + fx.unit;
			else
				fx.elem[ fx.prop ] = fx.now;
		}
	}
});
if ( document.documentElement["getBoundingClientRect"] )
	jQuery.fn.offset = function() {
		if ( !this[0] ) return { top: 0, left: 0 };
		if ( this[0] === this[0].ownerDocument.body ) return jQuery.offset.bodyOffset( this[0] );
		var box  = this[0].getBoundingClientRect(), doc = this[0].ownerDocument, body = doc.body, docElem = doc.documentElement,
			clientTop = docElem.clientTop || body.clientTop || 0, clientLeft = docElem.clientLeft || body.clientLeft || 0,
			top  = box.top  + (self.pageYOffset || jQuery.boxModel && docElem.scrollTop  || body.scrollTop ) - clientTop,
			left = box.left + (self.pageXOffset || jQuery.boxModel && docElem.scrollLeft || body.scrollLeft) - clientLeft;
		return { top: top, left: left };
	};
else 
	jQuery.fn.offset = function() {
		if ( !this[0] ) return { top: 0, left: 0 };
		if ( this[0] === this[0].ownerDocument.body ) return jQuery.offset.bodyOffset( this[0] );
		jQuery.offset.initialized || jQuery.offset.initialize();

		var elem = this[0], offsetParent = elem.offsetParent, prevOffsetParent = elem,
			doc = elem.ownerDocument, computedStyle, docElem = doc.documentElement,
			body = doc.body, defaultView = doc.defaultView,
			prevComputedStyle = defaultView.getComputedStyle(elem, null),
			top = elem.offsetTop, left = elem.offsetLeft;

		while ( (elem = elem.parentNode) && elem !== body && elem !== docElem ) {
			computedStyle = defaultView.getComputedStyle(elem, null);
			top -= elem.scrollTop, left -= elem.scrollLeft;
			if ( elem === offsetParent ) {
				top += elem.offsetTop, left += elem.offsetLeft;
				if ( jQuery.offset.doesNotAddBorder && !(jQuery.offset.doesAddBorderForTableAndCells && /^t(able|d|h)$/i.test(elem.tagName)) )
					top  += parseInt( computedStyle.borderTopWidth,  10) || 0,
					left += parseInt( computedStyle.borderLeftWidth, 10) || 0;
				prevOffsetParent = offsetParent, offsetParent = elem.offsetParent;
			}
			if ( jQuery.offset.subtractsBorderForOverflowNotVisible && computedStyle.overflow !== "visible" )
				top  += parseInt( computedStyle.borderTopWidth,  10) || 0,
				left += parseInt( computedStyle.borderLeftWidth, 10) || 0;
			prevComputedStyle = computedStyle;
		}

		if ( prevComputedStyle.position === "relative" || prevComputedStyle.position === "static" )
			top  += body.offsetTop,
			left += body.offsetLeft;

		if ( prevComputedStyle.position === "fixed" )
			top  += Math.max(docElem.scrollTop, body.scrollTop),
			left += Math.max(docElem.scrollLeft, body.scrollLeft);

		return { top: top, left: left };
	};

jQuery.offset = {
	initialize: function() {
		if ( this.initialized ) return;
		var body = document.body, container = document.createElement('div'), innerDiv, checkDiv, table, td, rules, prop, bodyMarginTop = body.style.marginTop,
			html = '<div style="position:absolute;top:0;left:0;margin:0;border:5px solid #000;padding:0;width:1px;height:1px;"><div></div></div><table style="position:absolute;top:0;left:0;margin:0;border:5px solid #000;padding:0;width:1px;height:1px;" cellpadding="0" cellspacing="0"><tr><td></td></tr></table>';

		rules = { position: 'absolute', top: 0, left: 0, margin: 0, border: 0, width: '1px', height: '1px', visibility: 'hidden' };
		for ( prop in rules ) container.style[prop] = rules[prop];

		container.innerHTML = html;
		body.insertBefore(container, body.firstChild);
		innerDiv = container.firstChild, checkDiv = innerDiv.firstChild, td = innerDiv.nextSibling.firstChild.firstChild;

		this.doesNotAddBorder = (checkDiv.offsetTop !== 5);
		this.doesAddBorderForTableAndCells = (td.offsetTop === 5);

		innerDiv.style.overflow = 'hidden', innerDiv.style.position = 'relative';
		this.subtractsBorderForOverflowNotVisible = (checkDiv.offsetTop === -5);

		body.style.marginTop = '1px';
		this.doesNotIncludeMarginInBodyOffset = (body.offsetTop === 0);
		body.style.marginTop = bodyMarginTop;

		body.removeChild(container);
		this.initialized = true;
	},

	bodyOffset: function(body) {
		jQuery.offset.initialized || jQuery.offset.initialize();
		var top = body.offsetTop, left = body.offsetLeft;
		if ( jQuery.offset.doesNotIncludeMarginInBodyOffset )
			top  += parseInt( jQuery.curCSS(body, 'marginTop',  true), 10 ) || 0,
			left += parseInt( jQuery.curCSS(body, 'marginLeft', true), 10 ) || 0;
		return { top: top, left: left };
	}
};


jQuery.fn.extend({
	position: function() {
		var left = 0, top = 0, results;

		if ( this[0] ) {
			// Get *real* offsetParent
			var offsetParent = this.offsetParent(),

			// Get correct offsets
			offset       = this.offset(),
			parentOffset = /^body|html$/i.test(offsetParent[0].tagName) ? { top: 0, left: 0 } : offsetParent.offset();

			// Subtract element margins
			// note: when an element has margin: auto the offsetLeft and marginLeft 
			// are the same in Safari causing offset.left to incorrectly be 0
			offset.top  -= num( this, 'marginTop'  );
			offset.left -= num( this, 'marginLeft' );

			// Add offsetParent borders
			parentOffset.top  += num( offsetParent, 'borderTopWidth'  );
			parentOffset.left += num( offsetParent, 'borderLeftWidth' );

			// Subtract the two offsets
			results = {
				top:  offset.top  - parentOffset.top,
				left: offset.left - parentOffset.left
			};
		}

		return results;
	},

	offsetParent: function() {
		var offsetParent = this[0].offsetParent || document.body;
		while ( offsetParent && (!/^body|html$/i.test(offsetParent.tagName) && jQuery.css(offsetParent, 'position') == 'static') )
			offsetParent = offsetParent.offsetParent;
		return jQuery(offsetParent);
	}
});


// Create scrollLeft and scrollTop methods
jQuery.each( ['Left', 'Top'], function(i, name) {
	var method = 'scroll' + name;
	
	jQuery.fn[ method ] = function(val) {
		if (!this[0]) return null;

		return val !== undefined ?

			// Set the scroll offset
			this.each(function() {
				this == window || this == document ?
					window.scrollTo(
						!i ? val : jQuery(window).scrollLeft(),
						 i ? val : jQuery(window).scrollTop()
					) :
					this[ method ] = val;
			}) :

			// Return the scroll offset
			this[0] == window || this[0] == document ?
				self[ i ? 'pageYOffset' : 'pageXOffset' ] ||
					jQuery.boxModel && document.documentElement[ method ] ||
					document.body[ method ] :
				this[0][ method ];
	};
});
// Create innerHeight, innerWidth, outerHeight and outerWidth methods
jQuery.each([ "Height", "Width" ], function(i, name){

	var tl = i ? "Left"  : "Top",  // top or left
		br = i ? "Right" : "Bottom", // bottom or right
		lower = name.toLowerCase();

	// innerHeight and innerWidth
	jQuery.fn["inner" + name] = function(){
		return this[0] ?
			jQuery.css( this[0], lower, false, "padding" ) :
			null;
	};

	// outerHeight and outerWidth
	jQuery.fn["outer" + name] = function(margin) {
		return this[0] ?
			jQuery.css( this[0], lower, false, margin ? "margin" : "border" ) :
			null;
	};
	
	var type = name.toLowerCase();

	jQuery.fn[ type ] = function( size ) {
		// Get window width or height
		return this[0] == window ?
			// Everyone else use document.documentElement or document.body depending on Quirks vs Standards mode
			document.compatMode == "CSS1Compat" && document.documentElement[ "client" + name ] ||
			document.body[ "client" + name ] :

			// Get document width or height
			this[0] == document ?
				// Either scroll[Width/Height] or offset[Width/Height], whichever is greater
				Math.max(
					document.documentElement["client" + name],
					document.body["scroll" + name], document.documentElement["scroll" + name],
					document.body["offset" + name], document.documentElement["offset" + name]
				) :

				// Get or set width or height on the element
				size === undefined ?
					// Get width or height on the element
					(this.length ? jQuery.css( this[0], type ) : null) :

					// Set the width or height on the element (default to pixels if value is unitless)
					this.css( type, typeof size === "string" ? size : size + "px" );
	};

});
})();
; 
 
 
 
/************************************************ 
**** ..\api.uncompressed.js ***** 
************************************************/ 
/*!*********************************** 
 *   zkBox JavaScript client v1.0   * 
 ************************************/ 
 
/************************************************ 
**** zkBox.core.header.js ***** 
************************************************/ 
/*!
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt
*/
; 
 
 
 
/************************************************ 
**** json2.js ***** 
************************************************/ 
/*
    http://www.JSON.org/json2.js
    2008-11-19

    Public Domain.

    NO WARRANTY EXPRESSED OR IMPLIED. USE AT YOUR OWN RISK.

    See http://www.JSON.org/js.html

    This file creates a global JSON object containing two methods: stringify
    and parse.

        JSON.stringify(value, replacer, space)
            value       any JavaScript value, usually an object or array.

            replacer    an optional parameter that determines how object
                        values are stringified for objects. It can be a
                        function or an array of strings.

            space       an optional parameter that specifies the indentation
                        of nested structures. If it is omitted, the text will
                        be packed without extra whitespace. If it is a number,
                        it will specify the number of spaces to indent at each
                        level. If it is a string (such as '\t' or '&nbsp;'),
                        it contains the characters used to indent at each level.

            This method produces a JSON text from a JavaScript value.

            When an object value is found, if the object contains a toJSON
            method, its toJSON method will be called and the result will be
            stringified. A toJSON method does not serialize: it returns the
            value represented by the name/value pair that should be serialized,
            or undefined if nothing should be serialized. The toJSON method
            will be passed the key associated with the value, and this will be
            bound to the object holding the key.

            For example, this would serialize Dates as ISO strings.

                Date.prototype.toJSON = function (key) {
                    function f(n) {
                        // Format integers to have at least two digits.
                        return n < 10 ? '0' + n : n;
                    }

                    return this.getUTCFullYear()   + '-' +
                         f(this.getUTCMonth() + 1) + '-' +
                         f(this.getUTCDate())      + 'T' +
                         f(this.getUTCHours())     + ':' +
                         f(this.getUTCMinutes())   + ':' +
                         f(this.getUTCSeconds())   + 'Z';
                };

            You can provide an optional replacer method. It will be passed the
            key and value of each member, with this bound to the containing
            object. The value that is returned from your method will be
            serialized. If your method returns undefined, then the member will
            be excluded from the serialization.

            If the replacer parameter is an array of strings, then it will be
            used to select the members to be serialized. It filters the results
            such that only members with keys listed in the replacer array are
            stringified.

            Values that do not have JSON representations, such as undefined or
            functions, will not be serialized. Such values in objects will be
            dropped; in arrays they will be replaced with null. You can use
            a replacer function to replace those with JSON values.
            JSON.stringify(undefined) returns undefined.

            The optional space parameter produces a stringification of the
            value that is filled with line breaks and indentation to make it
            easier to read.

            If the space parameter is a non-empty string, then that string will
            be used for indentation. If the space parameter is a number, then
            the indentation will be that many spaces.

            Example:

            text = JSON.stringify(['e', {pluribus: 'unum'}]);
            // text is '["e",{"pluribus":"unum"}]'


            text = JSON.stringify(['e', {pluribus: 'unum'}], null, '\t');
            // text is '[\n\t"e",\n\t{\n\t\t"pluribus": "unum"\n\t}\n]'

            text = JSON.stringify([new Date()], function (key, value) {
                return this[key] instanceof Date ?
                    'Date(' + this[key] + ')' : value;
            });
            // text is '["Date(---current time---)"]'


        JSON.parse(text, reviver)
            This method parses a JSON text to produce an object or array.
            It can throw a SyntaxError exception.

            The optional reviver parameter is a function that can filter and
            transform the results. It receives each of the keys and values,
            and its return value is used instead of the original value.
            If it returns what it received, then the structure is not modified.
            If it returns undefined then the member is deleted.

            Example:

            // Parse the text. Values that look like ISO date strings will
            // be converted to Date objects.

            myData = JSON.parse(text, function (key, value) {
                var a;
                if (typeof value === 'string') {
                    a =
/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)Z$/.exec(value);
                    if (a) {
                        return new Date(Date.UTC(+a[1], +a[2] - 1, +a[3], +a[4],
                            +a[5], +a[6]));
                    }
                }
                return value;
            });

            myData = JSON.parse('["Date(09/09/2001)"]', function (key, value) {
                var d;
                if (typeof value === 'string' &&
                        value.slice(0, 5) === 'Date(' &&
                        value.slice(-1) === ')') {
                    d = new Date(value.slice(5, -1));
                    if (d) {
                        return d;
                    }
                }
                return value;
            });


    This is a reference implementation. You are free to copy, modify, or
    redistribute.

    This code should be minified before deployment.
    See http://javascript.crockford.com/jsmin.html

    USE YOUR OWN COPY. IT IS EXTREMELY UNWISE TO LOAD CODE FROM SERVERS YOU DO
    NOT CONTROL.
*/

/*jslint evil: true */

/*global JSON */

/*members "", "\b", "\t", "\n", "\f", "\r", "\"", JSON, "\\", apply,
    call, charCodeAt, getUTCDate, getUTCFullYear, getUTCHours,
    getUTCMinutes, getUTCMonth, getUTCSeconds, hasOwnProperty, join,
    lastIndex, length, parse, prototype, push, replace, slice, stringify,
    test, toJSON, toString, valueOf
*/

// Create a JSON object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.JSON) {
    JSON = {};
}
(function () {

    function f(n) {
        // Format integers to have at least two digits.
        return n < 10 ? '0' + n : n;
    }

    if (typeof Date.prototype.toJSON !== 'function') {

        Date.prototype.toJSON = function (key) {

            return this.getUTCFullYear()   + '-' +
                 f(this.getUTCMonth() + 1) + '-' +
                 f(this.getUTCDate())      + 'T' +
                 f(this.getUTCHours())     + ':' +
                 f(this.getUTCMinutes())   + ':' +
                 f(this.getUTCSeconds())   + 'Z';
        };

        String.prototype.toJSON =
        Number.prototype.toJSON =
        Boolean.prototype.toJSON = function (key) {
            return this.valueOf();
        };
    }

    var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        escapable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        gap,
        indent,
        meta = {    // table of character substitutions
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
        },
        rep;


    function quote(string) {

// If the string contains no control characters, no quote characters, and no
// backslash characters, then we can safely slap some quotes around it.
// Otherwise we must also replace the offending characters with safe escape
// sequences.

        escapable.lastIndex = 0;
        return escapable.test(string) ?
            '"' + string.replace(escapable, function (a) {
                var c = meta[a];
                return typeof c === 'string' ? c :
                    '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
            }) + '"' :
            '"' + string + '"';
    }


    function str(key, holder) {

// Produce a string from holder[key].

        var i,          // The loop counter.
            k,          // The member key.
            v,          // The member value.
            length,
            mind = gap,
            partial,
            value = holder[key];

// If the value has a toJSON method, call it to obtain a replacement value.

        if (value && typeof value === 'object' &&
                typeof value.toJSON === 'function') {
            value = value.toJSON(key);
        }

// If we were called with a replacer function, then call the replacer to
// obtain a replacement value.

        if (typeof rep === 'function') {
            value = rep.call(holder, key, value);
        }

// What happens next depends on the value's type.

        switch (typeof value) {
        case 'string':
            return quote(value);

        case 'number':

// JSON numbers must be finite. Encode non-finite numbers as null.

            return isFinite(value) ? String(value) : 'null';

        case 'boolean':
        case 'null':

// If the value is a boolean or null, convert it to a string. Note:
// typeof null does not produce 'null'. The case is included here in
// the remote chance that this gets fixed someday.

            return String(value);

// If the type is 'object', we might be dealing with an object or an array or
// null.

        case 'object':

// Due to a specification blunder in ECMAScript, typeof null is 'object',
// so watch out for that case.

            if (!value) {
                return 'null';
            }

// Make an array to hold the partial results of stringifying this object value.

            gap += indent;
            partial = [];

// Is the value an array?

            if (Object.prototype.toString.apply(value) === '[object Array]') {

// The value is an array. Stringify every element. Use null as a placeholder
// for non-JSON values.

                length = value.length;
                for (i = 0; i < length; i += 1) {
                    partial[i] = str(i, value) || 'null';
                }

// Join all of the elements together, separated with commas, and wrap them in
// brackets.

                v = partial.length === 0 ? '[]' :
                    gap ? '[\n' + gap +
                            partial.join(',\n' + gap) + '\n' +
                                mind + ']' :
                          '[' + partial.join(',') + ']';
                gap = mind;
                return v;
            }

// If the replacer is an array, use it to select the members to be stringified.

            if (rep && typeof rep === 'object') {
                length = rep.length;
                for (i = 0; i < length; i += 1) {
                    k = rep[i];
                    if (typeof k === 'string') {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            } else {

// Otherwise, iterate through all of the keys in the object.

                for (k in value) {
                    if (Object.hasOwnProperty.call(value, k)) {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            }

// Join all of the member texts together, separated with commas,
// and wrap them in braces.

            v = partial.length === 0 ? '{}' :
                gap ? '{\n' + gap + partial.join(',\n' + gap) + '\n' +
                        mind + '}' : '{' + partial.join(',') + '}';
            gap = mind;
            return v;
        }
    }

// If the JSON object does not yet have a stringify method, give it one.

    if (typeof JSON.stringify !== 'function') {
        JSON.stringify = function (value, replacer, space) {

// The stringify method takes a value and an optional replacer, and an optional
// space parameter, and returns a JSON text. The replacer can be a function
// that can replace values, or an array of strings that will select the keys.
// A default replacer method can be provided. Use of the space parameter can
// produce text that is more easily readable.

            var i;
            gap = '';
            indent = '';

// If the space parameter is a number, make an indent string containing that
// many spaces.

            if (typeof space === 'number') {
                for (i = 0; i < space; i += 1) {
                    indent += ' ';
                }

// If the space parameter is a string, it will be used as the indent string.

            } else if (typeof space === 'string') {
                indent = space;
            }

// If there is a replacer, it must be a function or an array.
// Otherwise, throw an error.

            rep = replacer;
            if (replacer && typeof replacer !== 'function' &&
                    (typeof replacer !== 'object' ||
                     typeof replacer.length !== 'number')) {
                throw new Error('JSON.stringify');
            }

// Make a fake root object containing our value under the key of ''.
// Return the result of stringifying the value.

            return str('', {'': value});
        };
    }


// If the JSON object does not yet have a parse method, give it one.

    if (typeof JSON.parse !== 'function') {
        JSON.parse = function (text, reviver) {

// The parse method takes a text and an optional reviver function, and returns
// a JavaScript value if the text is a valid JSON text.

            var j;

            function walk(holder, key) {

// The walk method is used to recursively walk the resulting structure so
// that modifications can be made.

                var k, v, value = holder[key];
                if (value && typeof value === 'object') {
                    for (k in value) {
                        if (Object.hasOwnProperty.call(value, k)) {
                            v = walk(value, k);
                            if (v !== undefined) {
                                value[k] = v;
                            } else {
                                delete value[k];
                            }
                        }
                    }
                }
                return reviver.call(holder, key, value);
            }


// Parsing happens in four stages. In the first stage, we replace certain
// Unicode characters with escape sequences. JavaScript handles many characters
// incorrectly, either silently deleting them, or treating them as line endings.

            cx.lastIndex = 0;
            if (cx.test(text)) {
                text = text.replace(cx, function (a) {
                    return '\\u' +
                        ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
                });
            }

// In the second stage, we run the text against regular expressions that look
// for non-JSON patterns. We are especially concerned with '()' and 'new'
// because they can cause invocation, and '=' because it can cause mutation.
// But just to be safe, we want to reject all unexpected forms.

// We split the second stage into 4 regexp operations in order to work around
// crippling inefficiencies in IE's and Safari's regexp engines. First we
// replace the JSON backslash pairs with '@' (a non-JSON character). Second, we
// replace all simple value tokens with ']' characters. Third, we delete all
// open brackets that follow a colon or comma or that begin the text. Finally,
// we look to see that the remaining characters are only whitespace or ']' or
// ',' or ':' or '{' or '}'. If that is so, then the text is safe for eval.

            if (/^[\],:{}\s]*$/.
test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@').
replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']').
replace(/(?:^|:|,)(?:\s*\[)+/g, ''))) {

// In the third stage we use the eval function to compile the text into a
// JavaScript structure. The '{' operator is subject to a syntactic ambiguity
// in JavaScript: it can begin a block or an object literal. We wrap the text
// in parens to eliminate the ambiguity.

                j = eval('(' + text + ')');

// In the optional fourth stage, we recursively walk the new structure, passing
// each name/value pair to a reviver function for possible transformation.

                return typeof reviver === 'function' ?
                    walk({'': j}, '') : j;
            }

// If the text is not JSON parseable, then a SyntaxError is thrown.

            throw new SyntaxError('JSON.parse');
        };
    }
})();
; 
 
 
 
/************************************************ 
**** binary.js ***** 
************************************************/ 
/*
 * binary.js
 * Tools for creating, modifying binary data
 * including base64-encoding, base64-decoding , utf8-encoding and utf8-decoding
 * See binary.readme.txt for further information.
 *
 * Copyright(c) 2009 Atsushi Oka [ http://oka.nu/ ]
 * This script file is distributed under the LGPL
 */
function initBinary( packageRoot ) {
    if ( packageRoot.__PACKAGE_ENABLED ) {
	__unit( "binary.js" );
    }

var i2a  = [
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
    'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
    'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/'
];

function base64_encode( s ) {
    var length = s.length;
    var groupCount = Math.floor( length / 3 );
    var remaining = length - 3 * groupCount;
    var result = "";

    var idx = 0;
    for (var i=0; i<groupCount; i++) {
	var b0 = s[idx++] & 0xff;
	var b1 = s[idx++] & 0xff;
	var b2 = s[idx++] & 0xff;
	result += (i2a[ b0 >> 2]);
	result += (i2a[(b0 << 4) &0x3f | (b1 >> 4)]);
	result += (i2a[(b1 << 2) &0x3f | (b2 >> 6)]);
	result += (i2a[ b2 & 0x3f]);
    }

    if ( remaining == 0 ) {
    } else if ( remaining == 1 ) {
	var b0 = s[idx++] & 0xff;
	result += ( i2a[ b0 >> 2 ] );
	result += ( i2a[ (b0 << 4) & 0x3f] );
	result += ( "==" );
    } else if ( remaining == 2 ) {
	var b0 = s[idx++] & 0xff;
	var b1 = s[idx++] & 0xff;
	result += ( i2a[ b0 >> 2 ] );
	result += ( i2a[(b0 << 4) & 0x3f | (b1 >> 4)]);
	result += ( i2a[(b1 << 2) & 0x3f ] );
	result += ('=');
    } else {
	throw "never happen";
    }
    return result;
}

var a2i = [
    -1,   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, -1,
    -1,   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, -1,
    -1,   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  62,  -1,  -1,  -1, 63,
    52,   53,  54,  55,  56,  57,  58,  59,  60,  61,  -1,  -1,  -1,  -1,  -1, -1,
    -1,    0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13, 14,
    15,   16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  -1,  -1,  -1,  -1, -1,
    -1,   26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39, 40,
    41,   42,  43,  44,  45,  46,  47,  48,  49,  50,  51
];

function get_a2i( c ) {
    var result = (0<=c) && (c<a2i.length) ? a2i[ c ] : -1;
    if (result < 0) throw "Illegal character " + c;
    return result;
}

function base64_decode(s) {
    var length = s.length;
    var groupCount = Math.floor( length/4 );
    if ( 4 * groupCount != length )
	throw "String length must be a multiple of four.";

    var missing = 0;
    if (length != 0) {
	if ( s.charAt( length - 1 ) == '=' ) {
	    missing++;
	    groupCount--;
	}
	if ( s.charAt( length - 2 ) == '=' )
	    missing++;
    }

    var len = ( 3 * groupCount - missing );
    if ( len < 0 ) {
	len=0;
    }
    var result = new Array( len );
    // var result = new Array( 3 * groupCount - missing );
    // var result = new Array( 3 * ( groupCount +1 ) - missing );
    var idx_in = 0;
    var idx_out = 0;
    for ( var i=0; i<groupCount; i++ ) {
	var c0 = get_a2i( s.charCodeAt( idx_in++ ) );
	var c1 = get_a2i( s.charCodeAt( idx_in++ ) );
	var c2 = get_a2i( s.charCodeAt( idx_in++ ) );
	var c3 = get_a2i( s.charCodeAt( idx_in++ ) );
	result[ idx_out++ ] = 0xFF & ( (c0 << 2) | (c1 >> 4) );
	result[ idx_out++ ] = 0xFF & ( (c1 << 4) | (c2 >> 2) );
	result[ idx_out++ ] = 0xFF & ( (c2 << 6) | c3 );
    }

    if ( missing == 0 ) {
    } else if ( missing == 1 ) {
	var c0 = get_a2i( s.charCodeAt( idx_in++ ) );
	var c1 = get_a2i( s.charCodeAt( idx_in++ ) );
	var c2 = get_a2i( s.charCodeAt( idx_in++ ) );
	result[ idx_out++ ] = 0xFF & ( (c0 << 2) | (c1 >> 4) );
	result[ idx_out++ ] = 0xFF & ( (c1 << 4) | (c2 >> 2) );

    } else if ( missing == 2 ) {
	var c0 = get_a2i( s.charCodeAt( idx_in++ ) );
	var c1 = get_a2i( s.charCodeAt( idx_in++ ) );
	result[ idx_out++ ] = 0xFF & ( ( c0 << 2 ) | ( c1 >> 4 ) );
    } else {
	throw "never happen";
    }
    return result;
}

function base64x_encode( s ) {
    return base64x_pre_encode( base64_encode(s)  );
}
function base64x_decode( s ) {
    return base64_decode( base64x_pre_decode(s) );
}

var base64x_pre_encode_map = {};
base64x_pre_encode_map["x"] = "xx";
base64x_pre_encode_map["+"] = "xa";
base64x_pre_encode_map["/"] = "xb";
base64x_pre_encode_map["="] = "";


function base64x_pre_encode( s ) {
    var ss = "";
    for ( var i=0; i<s.length; i++ ) {
	var c = s.charAt(i);
	var cc = base64x_pre_encode_map[ c ]; 
	if ( cc != null ) {
	    ss = ss + cc;
	} else {
	    ss = ss + c;
	}
    }
    return ss;
}

var base64x_pre_decode_map = {};
base64x_pre_decode_map['x'] = 'x';
base64x_pre_decode_map['a'] = '+';
base64x_pre_decode_map['b'] = '/';

function base64x_pre_decode( s ) {
    var ss = "";
    for ( var i=0; i<s.length; i++ ) {
	var c = s.charAt(i);
	if ( c == 'x' ) {
	    c = s.charAt(++i);
	    var cc = base64x_pre_decode_map[ c ];
	    if ( cc != null ) {
		ss = ss + cc;
		// ss = ss + '/';
	    } else {
		// throw "invalid character was found. ("+cc+")"; // ignore.
	    }
	} else {
	    ss = ss + c;
	}
    }
    while ( ss.length % 4 != 0 ) {
	ss += "=";
    }
    return ss;
}

function equals( a, b ){
    if ( a.length != b.length )
	return false;
    var size=a.length;
    for ( var i=0;i<size;i++ ){
	// trace( a[i] + "/" + b[i] );
	if ( a[i] != b[i] )
	    return false;
    }
    return true;
}


function hex( i ){
    if ( i == null ) 
	return "??";
    //if ( i < 0 ) i+=256;
    i&=0xff;
    var result = i.toString(16);
    return ( result.length<2 ) ? "0" +result : result;
}

function base16( data, columns, delim ) {
    return base16_encode( data,columns,delim );
}
function base16_encode( data, columns, delim ) {
    if ( delim == null ){
	delim="";
    }
    if ( columns == null ) {
	columns = 256;
    }
    var result ="";
    for ( var i=0; i<data.length; i++ ) {
	if ( ( i % columns == 0 ) && ( 0<i ) )
	    result += "\n";
	result += hex( data[i] ) + delim;
    }
    return result.toUpperCase();
}

var amap = {};
 amap['0'] =   0; amap['1'] =   1; amap['2'] =   2; amap['3'] =   3;
 amap['4'] =   4; amap['5'] =   5; amap['6'] =   6; amap['7'] =   7;
 amap['8'] =   8; amap['9'] =   9; amap['A'] =  10; amap['B'] =  11;
 amap['C'] =  12; amap['D'] =  13; amap['E'] =  14; amap['F'] =  15;
                                   amap['a'] =  10; amap['b'] =  11; 
 amap['c'] =  12; amap['d'] =  13; amap['e'] =  14; amap['f'] =  15;

function get_amap( c ) {
    var cc = amap[c];
    //trace(c + "=>" + cc );
    if ( cc == null ) 
	throw "found an invalid character.";
    return cc;
}

function base16_decode( data ) {
    var ca = [];
    for ( var i=0,j=0; i<data.length; i++ ) {
	var c = data.charAt( i );
	if ( c == "\s" ) {
	    continue;
	} else {
	    ca[j++] = c;
	}
    }
    if ( ca.length % 2 != 0 ) {
	throw "data must be a multiple of two.";
    }

    var result = new Array( ca.length >> 1 );
    for ( var i=0; i<ca.length; i+=2 ) {
	var v = 0xff & ( ( get_amap( ca[i] ) <<4 ) | ( get_amap( ca[i+1] ) ) )  ;
	result[i>>1] = v;
	// trace(  get_amap( ca[i+1] ) )
	// result[i>>1] =  get_amap( ca[i+1] );
    }
    return result;
}
// trace( base16_encode([0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,128,255 ] ) );
// trace( base16_encode( base16_decode("000102030405060708090A0B0C0D0E0F1080FF") ) );
// trace( base16_encode( base16_decode( "000102030405060708090A0B0C0D0E0F102030405060708090A0B0C0D0E0F0FF" ) ) );
//                                       000102030405060708090A0B0C0D0E0F102030405060708090A0B0C0D0E0F0FF


/////////////////////////////////////////////////////////////////////////////////////////////

var B10000000 = 0x80;
var B11000000 = 0xC0;
var B11100000 = 0xE0;
var B11110000 = 0xF0;
var B11111000 = 0xF8;
var B11111100 = 0xFC;
var B11111110 = 0xFE;
var B01111111 = 0x7F;
var B00111111 = 0x3F;
var B00011111 = 0x1F;
var B00001111 = 0x0F;
var B00000111 = 0x07;
var B00000011 = 0x03;
var B00000001 = 0x01;

function str2utf8( str ){
    var result = [];
    var length = str.length;
    var idx=0;
    for ( var i=0; i<length; i++ ){
	var c = str.charCodeAt( i );
	if ( c <= 0x7f ) {
	    result[idx++] = c;
	} else if ( c <= 0x7ff ) {
	    result[idx++] = B11000000 | ( B00011111 & ( c >>>  6 ) );
	    result[idx++] = B10000000 | ( B00111111 & ( c >>>  0 ) );
	} else if ( c <= 0xffff ) {
	    result[idx++] = B11100000 | ( B00001111 & ( c >>> 12 ) ) ;
	    result[idx++] = B10000000 | ( B00111111 & ( c >>>  6 ) ) ;
	    result[idx++] = B10000000 | ( B00111111 & ( c >>>  0 ) ) ;
	} else if ( c <= 0x10ffff ) {
	    result[idx++] = B11110000 | ( B00000111 & ( c >>> 18 ) ) ;
	    result[idx++] = B10000000 | ( B00111111 & ( c >>> 12 ) ) ;
	    result[idx++] = B10000000 | ( B00111111 & ( c >>>  6 ) ) ;
	    result[idx++] = B10000000 | ( B00111111 & ( c >>>  0 ) ) ;
	} else {
	    throw "error";
	}
    }
    return result;
}

function utf82str( data ) {
    var result = "";
    var length = data.length;

    for ( var i=0; i<length; ){
	var c = data[i++];
	if ( c < 0x80 ) {
	    result += String.fromCharCode( c );
	} else if ( ( c < B11100000 ) ) {
	    result += String.fromCharCode(
		( ( B00011111 & c         ) <<  6 ) |
		( ( B00111111 & data[i++] ) <<  0 )
	    );
	} else if ( ( c < B11110000 ) ) {
	    result += String.fromCharCode(
		( ( B00001111 & c         ) << 12 ) |
		( ( B00111111 & data[i++] ) <<  6 ) |
		( ( B00111111 & data[i++] ) <<  0 )
	    );
	} else if ( ( c < B11111000 ) ) {
	    result += String.fromCharCode(
		( ( B00000111 & c         ) << 18 ) |
		( ( B00111111 & data[i++] ) << 12 ) |
		( ( B00111111 & data[i++] ) <<  6 ) |
		( ( B00111111 & data[i++] ) <<  0 )
	    );
	} else if ( ( c < B11111100 ) ) {
	    result += String.fromCharCode(
		( ( B00000011 & c         ) << 24 ) |
		( ( B00111111 & data[i++] ) << 18 ) |
		( ( B00111111 & data[i++] ) << 12 ) |
		( ( B00111111 & data[i++] ) <<  6 ) |
		( ( B00111111 & data[i++] ) <<  0 )
	    );
	} else if ( ( c < B11111110 ) ) {
	    result += String.fromCharCode(
		( ( B00000001 & c         ) << 30 ) |
		( ( B00111111 & data[i++] ) << 24 ) |
		( ( B00111111 & data[i++] ) << 18 ) |
		( ( B00111111 & data[i++] ) << 12 ) |
		( ( B00111111 & data[i++] ) <<  6 ) |
		( ( B00111111 & data[i++] ) <<  0 )
	    );
	}
    }
    return result;
}

/////////////////////////////////////////////////////////////////////////////////////////////

// convert unicode character array to string
function char2str( ca ) {
    var result = "";
    for ( var i=0; i<ca.length; i++ ) {
	result += String.fromCharCode( ca[i] );
    }
    return result;
}

// convert string to unicode character array
function str2char( str ) {
    var result = new Array( str.length );
    for ( var i=0; i<str.length; i++ ) {
	result[i] = str.charCodeAt( i );
    }
    return result;
}

/////////////////////////////////////////////////////////////////////////////////////////////

// byte expressions (big endian)
function i2ba_be(i) {
    return [
	0xff & (i>>24),
	0xff & (i>>16),
	0xff & (i>> 8),
	0xff & (i>> 0)
    ];
}
function ba2i_be(bs) {
    return (
	  ( bs[0]<<24 )
	| ( bs[1]<<16 )
	| ( bs[2]<< 8 )
	| ( bs[3]<< 0 )
    );
}
function s2ba_be(i) {
    return [
	0xff & (i>> 8),
	0xff & (i>> 0)
    ];
}
function ba2s_be(bs) {
    return (
	0
	| ( bs[0]<< 8 )
	| ( bs[1]<< 0 )
    );
}

// byte expressions (little endian)
function i2ba_le(i) {
    return [
	0xff & (i>> 0),
	0xff & (i>> 8),
	0xff & (i>>16),
	0xff & (i>>24)
    ];
}
function ba2i_le(bs) {
    return (
	0
	| ( bs[3]<< 0 )
	| ( bs[2]<< 8 )
	| ( bs[1]<<16 )
	| ( bs[0]<<24 )
    );
}
function s2ba_le(i) {
    return [
	0xff & (i>> 0),
	0xff & (i>> 8)
    ];
}
function ba2s_le(bs) {
    return (
	0
	| ( bs[1]<< 0 )
	| ( bs[0]<< 8 )
    );
}

function ia2ba_be( ia ) {
    var length = ia.length <<2;
    var ba = new Array( length );
    for(var ii=0,bi=0;ii<ia.length&&bi<ba.length; ){
        ba[bi++] = 0xff & ( ia[ii] >> 24 );
        ba[bi++] = 0xff & ( ia[ii] >> 16 );
        ba[bi++] = 0xff & ( ia[ii] >>  8 );
        ba[bi++] = 0xff & ( ia[ii] >>  0 );
        ii++;
    }
    return ba;
}
function ba2ia_be( ba ) {
    var length = (ba.length+3)>>2;
    var ia = new Array( length );;
    for(var ii=0,bi=0; ii<ia.length && bi<ba.length; ){
        ia[ii++] = 
            ( bi < ba.length ? (ba[bi++]  << 24 ) : 0 ) |
            ( bi < ba.length ? (ba[bi++]  << 16 ) : 0 ) |
            ( bi < ba.length ? (ba[bi++]  <<  8 ) : 0 ) |
            ( bi < ba.length ? (ba[bi++]/*<< 0*/) : 0 ) ;
    }
    return ia;
}

function ia2ba_le( ia ) {
    var length = ia.length <<2;
    var ba = new Array( length );
    for(var ii=0,bi=0;ii<ia.length&&bi<ba.length; ){
        ba[bi++] = 0xff & ( ia[ii] >>  0 );
        ba[bi++] = 0xff & ( ia[ii] >>  8 );
        ba[bi++] = 0xff & ( ia[ii] >> 16 );
        ba[bi++] = 0xff & ( ia[ii] >> 24 );
        ii++;
    }
    return ba;
}
function ba2ia_le( ba ) {
    var length = (ba.length+3)>>2;
    var ia = new Array( length );;
    for(var ii=0,bi=0; ii<ia.length && bi<ba.length; ){
        ia[ii++] = 
            ( bi < ba.length ? (ba[bi++]/*<< 0*/) : 0 ) |
            ( bi < ba.length ? (ba[bi++]  <<  8 ) : 0 ) |
            ( bi < ba.length ? (ba[bi++]  << 16 ) : 0 ) |
            ( bi < ba.length ? (ba[bi++]  << 24 ) : 0 ) ;
    }
    return ia;
}

/////////////////////////////////////////////////////////////////////////////////////////////

function trim( s ){
    var result = "";
    for ( var idx=0; idx<s.length; idx++ ){
	var c = s.charAt( idx );
	if ( c == "\s" || c == "\t" || c == "\r" || c == "\n" ) {
	} else {
	    result += c;
	}
    }
    return result;
}

/////////////////////////////////////////////////////////////////////////////////////////////

function mktst( encode, decode ) {
    return function ( trial,from,to ) {
	var flg=true;
	for (var i=0; i<trial; i++) {
	    for (var j=from; j<to; j++) {
		var arr = new Array(j);
		for (var k=0; k<j; k++)
		    arr[k] = Math.floor( Math.random() * 256 );

		var s = encode(arr);
		var b = decode(s);

		// trace( "in:"+arr.length);
		// trace( "base64:"+s.length);
		// trace( "out:"+b.length);
		// trace( "in:"+arr);
		// trace( "base64:"+s );
		// trace( "out:"+b );
		trace( "in :"+arr.length + ":"+ base16_encode(arr) );
		trace( "b64:"+s.length+":"+s);
		trace( "out:"+b.length + ":"+ base16_encode(arr) );
		if ( equals( arr, b ) ) {
		    trace( "OK! ( " + i + "," + j + ")" );
		} else {
		    trace( "ERR ( " + i + "," + j + ")" );
		    flg=false;
		}
		trace( "-----------");
	    }
	}
	if ( flg ) {
	    trace( "ALL OK! " );
	} else {
	    trace( "FOUND ERROR!" );
	}
    };
}

// export

// base64
packageRoot.base64_encode = base64_encode;
packageRoot.base64_decode = base64_decode;
packageRoot.base64_test   = mktst( base64_encode, base64_decode );

// base64ex
packageRoot.base64x_encode = base64x_encode;
packageRoot.base64x_decode = base64x_decode;
packageRoot.base64x_test   = mktst( base64x_encode, base64x_decode );

packageRoot.base64x_pre_encode = base64x_pre_encode;
packageRoot.base64x_pre_decode = base64x_pre_decode;

// base16
packageRoot.base16_encode = base16_encode;
packageRoot.base16_decode = base16_decode;
packageRoot.base16        = base16;
packageRoot.hex           = base16;

// utf8
packageRoot.utf82str      = utf82str;
packageRoot.str2utf8      = str2utf8;
packageRoot.str2char      = str2char;
packageRoot.char2str      = char2str;

// byte expressions
packageRoot.i2ba    = i2ba_be;
packageRoot.ba2i    = ba2i_be;
packageRoot.i2ba_be = i2ba_be;
packageRoot.ba2i_be = ba2i_be;
packageRoot.i2ba_le = i2ba_le;
packageRoot.ba2i_le = ba2i_le;

packageRoot.s2ba    = s2ba_be;
packageRoot.ba2s    = ba2s_be;
packageRoot.s2ba_be = s2ba_be;
packageRoot.ba2s_be = ba2s_be;
packageRoot.s2ba_le = s2ba_le;
packageRoot.ba2s_le = ba2s_le;

packageRoot.ba2ia    = ba2ia_be;
packageRoot.ia2ba    = ia2ba_be;
packageRoot.ia2ba_be = ia2ba_be;
packageRoot.ba2ia_be = ba2ia_be;
packageRoot.ia2ba_le = ia2ba_le;
packageRoot.ba2ia_le = ba2ia_le;


// arrays
packageRoot.cmparr        = equals;
}

initBinary(this);


; 
 
 
 
/************************************************ 
**** cache.js ***** 
************************************************/ 
/*
MIT LICENSE
Copyright (c) 2007 Monsur Hossain (http://www.monsur.com)

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

// ****************************************************************************
// CachePriority ENUM
// An easier way to refer to the priority of a cache item
var zkBoxCachePriority = {
    Low: 1,
    Normal: 2,
    High: 4
}

// ****************************************************************************
// Cache constructor
// Creates a new cache object
// INPUT: maxSize (optional) - indicates how many items the cache can hold.
//                             default is -1, which means no limit on the 
//                             number of items.
function zkBoxCache(maxSize) {
    this.items = {};
    this.count = 0;
    if (maxSize == null)
        maxSize = -1;
    this.maxSize = maxSize;
    this.fillFactor = .75;
    this.purgeSize = Math.round(this.maxSize * this.fillFactor);
    
    this.stats = {}
    this.stats.hits = 0;
    this.stats.misses = 0;
}

// ****************************************************************************
// Cache.getItem
// retrieves an item from the cache, returns null if the item doesn't exist
// or it is expired.
// INPUT: key - the key to load from the cache
zkBoxCache.prototype.getItem = function(key) {

    // retrieve the item from the cache
    var item = this.items[key];
    
    if (item != null) {
        if (!this._isExpired(item)) {
            // if the item is not expired
            // update its last accessed date
            item.lastAccessed = new Date().getTime();
        } else {
            // if the item is expired, remove it from the cache
            this._removeItem(key);
            item = null;
        }
    }
    
    // return the item value (if it exists), or null
    var returnVal = null;
    if (item != null) {
        returnVal = item.value;
        this.stats.hits++;
    } else {
        this.stats.misses++;
    }
    return returnVal;
}

// ****************************************************************************
// Cache.setItem
// sets an item in the cache
// parameters: key - the key to refer to the object
//             value - the object to cache
//             options - an optional parameter described below
// the last parameter accepts an object which controls various caching options:
//      expirationAbsolute: the datetime when the item should expire
//      expirationSliding: an integer representing the seconds since
//                         the last cache access after which the item
//                         should expire
//      priority: How important it is to leave this item in the cache.
//                You can use the values CachePriority.Low, .Normal, or 
//                .High, or you can just use an integer.  Note that 
//                placing a priority on an item does not guarantee 
//                it will remain in cache.  It can still be purged if 
//                an expiration is hit, or if the cache is full.
//      callback: A function that gets called when the item is purged
//                from cache.  The key and value of the removed item
//                are passed as parameters to the callback function.
zkBoxCache.prototype.setItem = function(key, value, options) {

    function CacheItem(k, v, o) {
        if ((k == null) || (k == ''))
            throw new Error("key cannot be null or empty");
        this.key = k;
        this.value = v;
        if (o == null)
            o = {};
        if (o.expirationAbsolute != null)
            o.expirationAbsolute = o.expirationAbsolute.getTime();
        if (o.priority == null)
            o.priority = zkBoxCachePriority.Normal;
        this.options = o;
        this.lastAccessed = new Date().getTime();
    }

    // add a new cache item to the cache
    if (this.items[key] != null)
        this._removeItem(key);
    this._addItem(new CacheItem(key, value, options));
    
    // if the cache is full, purge it
    if ((this.maxSize > 0) && (this.count > this.maxSize)) {
        this._purge();
    }
}

// ****************************************************************************
// Cache.clear
// Remove all items from the cache
zkBoxCache.prototype.clear = function() {

    // loop through each item in the cache and remove it
    for (var key in this.items) {
      this._removeItem(key);
    }  
}

// ****************************************************************************
// Cache.remove
// Remove an item from the cache
zkBoxCache.prototype.removeItem = function(key) {

    // retrieve the item from the cache
    var item = this.items[key];
    
    if (item != null) {
        this._removeItem(key);
    }
}

// ****************************************************************************
// Cache._purge (PRIVATE FUNCTION)
// remove old elements from the cache
zkBoxCache.prototype._purge = function() {
    
    var tmparray = new Array();
    
    // loop through the cache, expire items that should be expired
    // otherwise, add the item to an array
    for (var key in this.items) {
        var item = this.items[key];
        if (this._isExpired(item)) {
            this._removeItem(key);
        } else {
            tmparray.push(item);
        }
    }
    
    if (tmparray.length > this.purgeSize) {

        // sort this array based on cache priority and the last accessed date
        tmparray = tmparray.sort(function(a, b) { 
            if (a.options.priority != b.options.priority) {
                return b.options.priority - a.options.priority;
            } else {
                return b.lastAccessed - a.lastAccessed;
            }
        });
        
        // remove items from the end of the array
        while (tmparray.length > this.purgeSize) {
            var ritem = tmparray.pop();
            this._removeItem(ritem.key);
        }
    }
}

// ****************************************************************************
// Cache._addItem (PRIVATE FUNCTION)
// add an item to the cache
zkBoxCache.prototype._addItem = function(item) {
    this.items[item.key] = item;
    this.count++;
}

// ****************************************************************************
// Cache._removeItem (PRIVATE FUNCTION)
// Remove an item from the cache, call the callback function (if necessary)
zkBoxCache.prototype._removeItem = function(key) {
    var item = this.items[key];
    delete this.items[key];
    this.count--;
    
    // if there is a callback function, call it at the end of execution
    if (item.options.callback != null) {
        var callback = function() {
            item.options.callback(item.key, item.value);
        }
        setTimeout(callback, 0);
    }
}

// ****************************************************************************
// Cache._isExpired (PRIVATE FUNCTION)
// Returns true if the item should be expired based on its expiration options
zkBoxCache.prototype._isExpired = function(item) {
    var now = new Date().getTime();
    var expired = false;
    if ((item.options.expirationAbsolute) && (item.options.expirationAbsolute < now)) {
        // if the absolute expiration has passed, expire the item
        expired = true;
    } 
    if ((expired == false) && (item.options.expirationSliding)) {
        // if the sliding expiration has passed, expire the item
        var lastAccess = item.lastAccessed + (item.options.expirationSliding * 1000);
        if (lastAccess < now) {
            expired = true;
        }
    }
    return expired;
}

zkBoxCache.prototype.toHtmlString = function() {
    var returnStr = this.count + " item(s) in cache<br /><ul>";
    for (var key in this.items) {
        var item = this.items[key];
        returnStr = returnStr + "<li>" + item.key.toString() + " = " + item.value.toString() + "</li>";
    }
    returnStr = returnStr + "</ul>";
    return returnStr;
}
; 
 
 
 
/************************************************ 
**** BigInt.js ***** 
************************************************/ 
////////////////////////////////////////////////////////////////////////////////////////
// Big Integer Library v. 5.1
// Created 2000, last modified 2007
// Leemon Baird
// www.leemon.com
//
// Version history:
//
// v 5.1  8 Oct 2007 
//   - renamed inverseModInt_ to inverseModInt since it doesn't change its parameters
//   - added functions GCD and randBigInt, which call GCD_ and randBigInt_
//   - fixed a bug found by Rob Visser (see comment with his name below)
//   - improved comments
//
// This file is public domain.   You can use it for any purpose without restriction.
// I do not guarantee that it is correct, so use it at your own risk.  If you use 
// it for something interesting, I'd appreciate hearing about it.  If you find 
// any bugs or make any improvements, I'd appreciate hearing about those too.
// It would also be nice if my name and address were left in the comments.
// But none of that is required.
//
// This code defines a bigInt library for arbitrary-precision integers.
// A bigInt is an array of integers storing the value in chunks of bpe bits, 
// little endian (buff[0] is the least significant word).
// Negative bigInts are stored two's complement.
// Some functions assume their parameters have at least one leading zero element.
// Functions with an underscore at the end of the name have unpredictable behavior in case of overflow, 
// so the caller must make sure the arrays must be big enough to hold the answer.
// For each function where a parameter is modified, that same 
// variable must not be used as another argument too.
// So, you cannot square x by doing multMod_(x,x,n).  
// You must use squareMod_(x,n) instead, or do y=dup(x); multMod_(x,y,n).
//
// These functions are designed to avoid frequent dynamic memory allocation in the inner loop.
// For most functions, if it needs a BigInt as a local variable it will actually use
// a global, and will only allocate to it only when it's not the right size.  This ensures
// that when a function is called repeatedly with same-sized parameters, it only allocates
// memory on the first call.
//
// Note that for cryptographic purposes, the calls to Math.random() must 
// be replaced with calls to a better pseudorandom number generator.
//
// In the following, "bigInt" means a bigInt with at least one leading zero element,
// and "integer" means a nonnegative integer less than radix.  In some cases, integer 
// can be negative.  Negative bigInts are 2s complement.
// 
// The following functions do not modify their inputs.
// Those returning a bigInt, string, or Array will dynamically allocate memory for that value.
// Those returning a boolean will return the integer 0 (false) or 1 (true).
// Those returning boolean or int will not allocate memory except possibly on the first time they're called with a given parameter size.
// 
// bigInt  add(x,y)               //return (x+y) for bigInts x and y.  
// bigInt  addInt(x,n)            //return (x+n) where x is a bigInt and n is an integer.
// string  bigInt2str(x,base)     //return a string form of bigInt x in a given base, with 2 <= base <= 95
// int     bitSize(x)             //return how many bits long the bigInt x is, not counting leading zeros
// bigInt  dup(x)                 //return a copy of bigInt x
// boolean equals(x,y)            //is the bigInt x equal to the bigint y?
// boolean equalsInt(x,y)         //is bigint x equal to integer y?
// bigInt  expand(x,n)            //return a copy of x with at least n elements, adding leading zeros if needed
// Array   findPrimes(n)          //return array of all primes less than integer n
// bigInt  GCD(x,y)               //return greatest common divisor of bigInts x and y (each with same number of elements).
// boolean greater(x,y)           //is x>y?  (x and y are nonnegative bigInts)
// boolean greaterShift(x,y,shift)//is (x <<(shift*bpe)) > y?
// bigInt  int2bigInt(t,n,m)      //return a bigInt equal to integer t, with at least n bits and m array elements
// bigInt  inverseMod(x,n)        //return (x**(-1) mod n) for bigInts x and n.  If no inverse exists, it returns null
// int     inverseModInt(x,n)     //return x**(-1) mod n, for integers x and n.  Return 0 if there is no inverse
// boolean isZero(x)              //is the bigInt x equal to zero?
// boolean millerRabin(x,b)       //does one round of Miller-Rabin base integer b say that bigInt x is possibly prime (as opposed to definitely composite)?
// bigInt  mod(x,n)               //return a new bigInt equal to (x mod n) for bigInts x and n.
// int     modInt(x,n)            //return x mod n for bigInt x and integer n.
// bigInt  mult(x,y)              //return x*y for bigInts x and y. This is faster when y<x.
// bigInt  multMod(x,y,n)         //return (x*y mod n) for bigInts x,y,n.  For greater speed, let y<x.
// boolean negative(x)            //is bigInt x negative?
// bigInt  powMod(x,y,n)          //return (x**y mod n) where x,y,n are bigInts and ** is exponentiation.  0**0=1. Faster for odd n.
// bigInt  randBigInt(n,s)        //return an n-bit random BigInt (n>=1).  If s=1, then the most significant of those n bits is set to 1.
// bigInt  randTruePrime(k)       //return a new, random, k-bit, true prime bigInt using Maurer's algorithm.
// bigInt  str2bigInt(s,b,n,m)    //return a bigInt for number represented in string s in base b with at least n bits and m array elements
// bigInt  sub(x,y)               //return (x-y) for bigInts x and y.  Negative answers will be 2s complement
// bigInt  trim(x,k)              //return a copy of x with exactly k leading zero elements
//
//
// The following functions each have a non-underscored version, which most users should call instead.
// These functions each write to a single parameter, and the caller is responsible for ensuring the array 
// passed in is large enough to hold the result. 
//
// void    addInt_(x,n)          //do x=x+n where x is a bigInt and n is an integer
// void    add_(x,y)             //do x=x+y for bigInts x and y
// void    copy_(x,y)            //do x=y on bigInts x and y
// void    copyInt_(x,n)         //do x=n on bigInt x and integer n
// void    GCD_(x,y)             //set x to the greatest common divisor of bigInts x and y, (y is destroyed).  (This never overflows its array).
// boolean inverseMod_(x,n)      //do x=x**(-1) mod n, for bigInts x and n. Returns 1 (0) if inverse does (doesn't) exist
// void    mod_(x,n)             //do x=x mod n for bigInts x and n. (This never overflows its array).
// void    mult_(x,y)            //do x=x*y for bigInts x and y.
// void    multMod_(x,y,n)       //do x=x*y  mod n for bigInts x,y,n.
// void    powMod_(x,y,n)        //do x=x**y mod n, where x,y,n are bigInts (n is odd) and ** is exponentiation.  0**0=1.
// void    randBigInt_(b,n,s)    //do b = an n-bit random BigInt. if s=1, then nth bit (most significant bit) is set to 1. n>=1.
// void    randTruePrime_(ans,k) //do ans = a random k-bit true random prime (not just probable prime) with 1 in the msb.
// void    sub_(x,y)             //do x=x-y for bigInts x and y. Negative answers will be 2s complement.
//
// The following functions do NOT have a non-underscored version. 
// They each write a bigInt result to one or more parameters.  The caller is responsible for
// ensuring the arrays passed in are large enough to hold the results. 
//
// void addShift_(x,y,ys)       //do x=x+(y<<(ys*bpe))
// void carry_(x)               //do carries and borrows so each element of the bigInt x fits in bpe bits.
// void divide_(x,y,q,r)        //divide x by y giving quotient q and remainder r
// int  divInt_(x,n)            //do x=floor(x/n) for bigInt x and integer n, and return the remainder. (This never overflows its array).
// int  eGCD_(x,y,d,a,b)        //sets a,b,d to positive bigInts such that d = GCD_(x,y) = a*x-b*y
// void halve_(x)               //do x=floor(|x|/2)*sgn(x) for bigInt x in 2's complement.  (This never overflows its array).
// void leftShift_(x,n)         //left shift bigInt x by n bits.  n<bpe.
// void linComb_(x,y,a,b)       //do x=a*x+b*y for bigInts x and y and integers a and b
// void linCombShift_(x,y,b,ys) //do x=x+b*(y<<(ys*bpe)) for bigInts x and y, and integers b and ys
// void mont_(x,y,n,np)         //Montgomery multiplication (see comments where the function is defined)
// void multInt_(x,n)           //do x=x*n where x is a bigInt and n is an integer.
// void rightShift_(x,n)        //right shift bigInt x by n bits.  0 <= n < bpe. (This never overflows its array).
// void squareMod_(x,n)         //do x=x*x  mod n for bigInts x,n
// void subShift_(x,y,ys)       //do x=x-(y<<(ys*bpe)). Negative answers will be 2s complement.
//
// The following functions are based on algorithms from the _Handbook of Applied Cryptography_
//    powMod_()           = algorithm 14.94, Montgomery exponentiation
//    eGCD_,inverseMod_() = algorithm 14.61, Binary extended GCD_
//    GCD_()              = algorothm 14.57, Lehmer's algorithm
//    mont_()             = algorithm 14.36, Montgomery multiplication
//    divide_()           = algorithm 14.20  Multiple-precision division
//    squareMod_()        = algorithm 14.16  Multiple-precision squaring
//    randTruePrime_()    = algorithm  4.62, Maurer's algorithm
//    millerRabin()       = algorithm  4.24, Miller-Rabin algorithm
//
// Profiling shows:
//     randTruePrime_() spends:
//         10% of its time in calls to powMod_()
//         85% of its time in calls to millerRabin()
//     millerRabin() spends:
//         99% of its time in calls to powMod_()   (always with a base of 2)
//     powMod_() spends:
//         94% of its time in calls to mont_()  (almost always with x==y)
//
// This suggests there are several ways to speed up this library slightly:
//     - convert powMod_ to use a Montgomery form of k-ary window (or maybe a Montgomery form of sliding window)
//         -- this should especially focus on being fast when raising 2 to a power mod n
//     - convert randTruePrime_() to use a minimum r of 1/3 instead of 1/2 with the appropriate change to the test
//     - tune the parameters in randTruePrime_(), including c, m, and recLimit
//     - speed up the single loop in mont_() that takes 95% of the runtime, perhaps by reducing checking
//       within the loop when all the parameters are the same length.
//
// There are several ideas that look like they wouldn't help much at all:
//     - replacing trial division in randTruePrime_() with a sieve (that speeds up something taking almost no time anyway)
//     - increase bpe from 15 to 30 (that would help if we had a 32*32->64 multiplier, but not with JavaScript's 32*32->32)
//     - speeding up mont_(x,y,n,np) when x==y by doing a non-modular, non-Montgomery square
//       followed by a Montgomery reduction.  The intermediate answer will be twice as long as x, so that
//       method would be slower.  This is unfortunate because the code currently spends almost all of its time
//       doing mont_(x,x,...), both for randTruePrime_() and powMod_().  A faster method for Montgomery squaring
//       would have a large impact on the speed of randTruePrime_() and powMod_().  HAC has a couple of poorly-worded
//       sentences that seem to imply it's faster to do a non-modular square followed by a single
//       Montgomery reduction, but that's obviously wrong.
////////////////////////////////////////////////////////////////////////////////////////

//globals
bpe=0;         //bits stored per array element
mask=0;        //AND this with an array element to chop it down to bpe bits
radix=mask+1;  //equals 2^bpe.  A single 1 bit to the left of the last bit of mask.

//the digits for converting to different bases
digitsStr='0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_=!@#$%^&*()[]{}|;:,.<>/?`~ \\\'\"+-';

//initialize the global variables
for (bpe=0; (1<<(bpe+1)) > (1<<bpe); bpe++);  //bpe=number of bits in the mantissa on this platform
bpe>>=1;                   //bpe=number of bits in one element of the array representing the bigInt
mask=(1<<bpe)-1;           //AND the mask with an integer to get its bpe least significant bits
radix=mask+1;              //2^bpe.  a single 1 bit to the left of the first bit of mask
one=int2bigInt(1,1,1);     //constant used in powMod_()

//the following global variables are scratchpad memory to 
//reduce dynamic memory allocation in the inner loop
t=new Array(0);
ss=t;       //used in mult_()
s0=t;       //used in multMod_(), squareMod_() 
s1=t;       //used in powMod_(), multMod_(), squareMod_() 
s2=t;       //used in powMod_(), multMod_()
s3=t;       //used in powMod_()
s4=t; s5=t; //used in mod_()
s6=t;       //used in bigInt2str()
s7=t;       //used in powMod_()
T=t;        //used in GCD_()
sa=t;       //used in mont_()
mr_x1=t; mr_r=t; mr_a=t;                                      //used in millerRabin()
eg_v=t; eg_u=t; eg_A=t; eg_B=t; eg_C=t; eg_D=t;               //used in eGCD_(), inverseMod_()
md_q1=t; md_q2=t; md_q3=t; md_r=t; md_r1=t; md_r2=t; md_tt=t; //used in mod_()

primes=t; pows=t; s_i=t; s_i2=t; s_R=t; s_rm=t; s_q=t; s_n1=t; 
  s_a=t; s_r2=t; s_n=t; s_b=t; s_d=t; s_x1=t; s_x2=t, s_aa=t; //used in randTruePrime_()

////////////////////////////////////////////////////////////////////////////////////////

//return array of all primes less than integer n
function findPrimes(n) {
  var i,s,p,ans;
  s=new Array(n);
  for (i=0;i<n;i++)
    s[i]=0;
  s[0]=2;
  p=0;    //first p elements of s are primes, the rest are a sieve
  for(;s[p]<n;) {                  //s[p] is the pth prime
    for(i=s[p]*s[p]; i<n; i+=s[p]) //mark multiples of s[p]
      s[i]=1;
    p++;
    s[p]=s[p-1]+1;
    for(; s[p]<n && s[s[p]]; s[p]++); //find next prime (where s[p]==0)
  }
  ans=new Array(p);
  for(i=0;i<p;i++)
    ans[i]=s[i];
  return ans;
}

//does a single round of Miller-Rabin base b consider x to be a possible prime?
//x is a bigInt, and b is an integer
function millerRabin(x,b) {
  var i,j,k,s;

  if (mr_x1.length!=x.length) {
    mr_x1=dup(x);
    mr_r=dup(x);
    mr_a=dup(x);
  }

  copyInt_(mr_a,b);
  copy_(mr_r,x);
  copy_(mr_x1,x);

  addInt_(mr_r,-1);
  addInt_(mr_x1,-1);

  //s=the highest power of two that divides mr_r
  k=0;
  for (i=0;i<mr_r.length;i++)
    for (j=1;j<mask;j<<=1)
      if (x[i] & j) {
        s=(k<mr_r.length+bpe ? k : 0); 
         i=mr_r.length;
         j=mask;
      } else
        k++;

  if (s)                
    rightShift_(mr_r,s);

  powMod_(mr_a,mr_r,x);

  if (!equalsInt(mr_a,1) && !equals(mr_a,mr_x1)) {
    j=1;
    while (j<=s-1 && !equals(mr_a,mr_x1)) {
      squareMod_(mr_a,x);
      if (equalsInt(mr_a,1)) {
        return 0;
      }
      j++;
    }
    if (!equals(mr_a,mr_x1)) {
      return 0;
    }
  }
  return 1;  
}

//returns how many bits long the bigInt is, not counting leading zeros.
function bitSize(x) {
  var j,z,w;
  for (j=x.length-1; (x[j]==0) && (j>0); j--);
  for (z=0,w=x[j]; w; (w>>=1),z++);
  z+=bpe*j;
  return z;
}

//return a copy of x with at least n elements, adding leading zeros if needed
function expand(x,n) {
  var ans=int2bigInt(0,(x.length>n ? x.length : n)*bpe,0);
  copy_(ans,x);
  return ans;
}

//return a k-bit true random prime using Maurer's algorithm.
function randTruePrime(k) {
  var ans=int2bigInt(0,k,0);
  randTruePrime_(ans,k);
  return trim(ans,1);
}

//return a new bigInt equal to (x mod n) for bigInts x and n.
function mod(x,n) {
  var ans=dup(x);
  mod_(ans,n);
  return trim(ans,1);
}

//return (x+n) where x is a bigInt and n is an integer.
function addInt(x,n) {
  var ans=expand(x,x.length+1);
  addInt_(ans,n);
  return trim(ans,1);
}

//return x*y for bigInts x and y. This is faster when y<x.
function mult(x,y) {
  var ans=expand(x,x.length+y.length);
  mult_(ans,y);
  return trim(ans,1);
}

//return (x**y mod n) where x,y,n are bigInts and ** is exponentiation.  0**0=1. Faster for odd n.
function powMod(x,y,n) {
  var ans=expand(x,n.length);  
  powMod_(ans,trim(y,2),trim(n,2),0);  //this should work without the trim, but doesn't
  return trim(ans,1);
}

//return (x-y) for bigInts x and y.  Negative answers will be 2s complement
function sub(x,y) {
  var ans=expand(x,(x.length>y.length ? x.length+1 : y.length+1)); 
  sub_(ans,y);
  return trim(ans,1);
}

//return (x+y) for bigInts x and y.  
function add(x,y) {
  var ans=expand(x,(x.length>y.length ? x.length+1 : y.length+1)); 
  add_(ans,y);
  return trim(ans,1);
}

//return (x**(-1) mod n) for bigInts x and n.  If no inverse exists, it returns null
function inverseMod(x,n) {
  var ans=expand(x,n.length); 
  var s;
  s=inverseMod_(ans,n);
  return s ? trim(ans,1) : null;
}

//return (x*y mod n) for bigInts x,y,n.  For greater speed, let y<x.
function multMod(x,y,n) {
  var ans=expand(x,n.length);
  multMod_(ans,y,n);
  return trim(ans,1);
}

//generate a k-bit true random prime using Maurer's algorithm,
//and put it into ans.  The bigInt ans must be large enough to hold it.
function randTruePrime_(ans,k) {
  var c,m,pm,dd,j,r,B,divisible,z,zz,recSize;

  if (primes.length==0)
    primes=findPrimes(30000);  //check for divisibility by primes <=30000

  if (pows.length==0) {
    pows=new Array(512);
    for (j=0;j<512;j++) {
      pows[j]=Math.pow(2,j/511.-1.);
    }
  }

  //c and m should be tuned for a particular machine and value of k, to maximize speed
  c=0.1;  //c=0.1 in HAC
  m=20;   //generate this k-bit number by first recursively generating a number that has between k/2 and k-m bits
  recLimit=20; //stop recursion when k <=recLimit.  Must have recLimit >= 2

  if (s_i2.length!=ans.length) {
    s_i2=dup(ans);
    s_R =dup(ans);
    s_n1=dup(ans);
    s_r2=dup(ans);
    s_d =dup(ans);
    s_x1=dup(ans);
    s_x2=dup(ans);
    s_b =dup(ans);
    s_n =dup(ans);
    s_i =dup(ans);
    s_rm=dup(ans);
    s_q =dup(ans);
    s_a =dup(ans);
    s_aa=dup(ans);
  }

  if (k <= recLimit) {  //generate small random primes by trial division up to its square root
    pm=(1<<((k+2)>>1))-1; //pm is binary number with all ones, just over sqrt(2^k)
    copyInt_(ans,0);
    for (dd=1;dd;) {
      dd=0;
      ans[0]= 1 | (1<<(k-1)) | Math.floor(Math.random()*(1<<k));  //random, k-bit, odd integer, with msb 1
      for (j=1;(j<primes.length) && ((primes[j]&pm)==primes[j]);j++) { //trial division by all primes 3...sqrt(2^k)
        if (0==(ans[0]%primes[j])) {
          dd=1;
          break;
        }
      }
    }
    carry_(ans);
    return;
  }

  B=c*k*k;    //try small primes up to B (or all the primes[] array if the largest is less than B).
  if (k>2*m)  //generate this k-bit number by first recursively generating a number that has between k/2 and k-m bits
    for (r=1; k-k*r<=m; )
      r=pows[Math.floor(Math.random()*512)];   //r=Math.pow(2,Math.random()-1);
  else
    r=.5;

  //simulation suggests the more complex algorithm using r=.333 is only slightly faster.

  recSize=Math.floor(r*k)+1;

  randTruePrime_(s_q,recSize);
  copyInt_(s_i2,0);
  s_i2[Math.floor((k-2)/bpe)] |= (1<<((k-2)%bpe));   //s_i2=2^(k-2)
  divide_(s_i2,s_q,s_i,s_rm);                        //s_i=floor((2^(k-1))/(2q))

  z=bitSize(s_i);

  for (;;) {
    for (;;) {  //generate z-bit numbers until one falls in the range [0,s_i-1]
      randBigInt_(s_R,z,0);
      if (greater(s_i,s_R))
        break;
    }                //now s_R is in the range [0,s_i-1]
    addInt_(s_R,1);  //now s_R is in the range [1,s_i]
    add_(s_R,s_i);   //now s_R is in the range [s_i+1,2*s_i]

    copy_(s_n,s_q);
    mult_(s_n,s_R); 
    multInt_(s_n,2);
    addInt_(s_n,1);    //s_n=2*s_R*s_q+1
    
    copy_(s_r2,s_R);
    multInt_(s_r2,2);  //s_r2=2*s_R

    //check s_n for divisibility by small primes up to B
    for (divisible=0,j=0; (j<primes.length) && (primes[j]<B); j++)
      if (modInt(s_n,primes[j])==0) {
        divisible=1;
        break;
      }      

    if (!divisible)    //if it passes small primes check, then try a single Miller-Rabin base 2
      if (!millerRabin(s_n,2)) //this line represents 75% of the total runtime for randTruePrime_ 
        divisible=1;

    if (!divisible) {  //if it passes that test, continue checking s_n
      addInt_(s_n,-3);
      for (j=s_n.length-1;(s_n[j]==0) && (j>0); j--);  //strip leading zeros
      for (zz=0,w=s_n[j]; w; (w>>=1),zz++);
      zz+=bpe*j;                             //zz=number of bits in s_n, ignoring leading zeros
      for (;;) {  //generate z-bit numbers until one falls in the range [0,s_n-1]
        randBigInt_(s_a,zz,0);
        if (greater(s_n,s_a))
          break;
      }                //now s_a is in the range [0,s_n-1]
      addInt_(s_n,3);  //now s_a is in the range [0,s_n-4]
      addInt_(s_a,2);  //now s_a is in the range [2,s_n-2]
      copy_(s_b,s_a);
      copy_(s_n1,s_n);
      addInt_(s_n1,-1);
      powMod_(s_b,s_n1,s_n);   //s_b=s_a^(s_n-1) modulo s_n
      addInt_(s_b,-1);
      if (isZero(s_b)) {
        copy_(s_b,s_a);
        powMod_(s_b,s_r2,s_n);
        addInt_(s_b,-1);
        copy_(s_aa,s_n);
        copy_(s_d,s_b);
        GCD_(s_d,s_n);  //if s_b and s_n are relatively prime, then s_n is a prime
        if (equalsInt(s_d,1)) {
          copy_(ans,s_aa);
          return;     //if we've made it this far, then s_n is absolutely guaranteed to be prime
        }
      }
    }
  }
}

//Return an n-bit random BigInt (n>=1).  If s=1, then the most significant of those n bits is set to 1.
function randBigInt(n,s) {
  var a,b;
  a=Math.floor((n-1)/bpe)+2; //# array elements to hold the BigInt with a leading 0 element
  b=int2bigInt(0,0,a);
  randBigInt_(b,n,s);
  return b;
}

//Set b to an n-bit random BigInt.  If s=1, then the most significant of those n bits is set to 1.
//Array b must be big enough to hold the result. Must have n>=1
function randBigInt_(b,n,s) {
  var i,a;
  for (i=0;i<b.length;i++)
    b[i]=0;
  a=Math.floor((n-1)/bpe)+1; //# array elements to hold the BigInt
  for (i=0;i<a;i++) {
    b[i]=Math.floor(Math.random()*(1<<(bpe-1)));
  }
  b[a-1] &= (2<<((n-1)%bpe))-1;
  if (s==1)
    b[a-1] |= (1<<((n-1)%bpe));
}

//Return the greatest common divisor of bigInts x and y (each with same number of elements).
function GCD(x,y) {
  var xc,yc;
  xc=dup(x);
  yc=dup(y);
  GCD_(xc,yc);
  return xc;
}

//set x to the greatest common divisor of bigInts x and y (each with same number of elements).
//y is destroyed.
function GCD_(x,y) {
  var i,xp,yp,A,B,C,D,q,sing;
  if (T.length!=x.length)
    T=dup(x);

  sing=1;
  while (sing) { //while y has nonzero elements other than y[0]
    sing=0;
    for (i=1;i<y.length;i++) //check if y has nonzero elements other than 0
      if (y[i]) {
        sing=1;
        break;
      }
    if (!sing) break; //quit when y all zero elements except possibly y[0]

    for (i=x.length;!x[i] && i>=0;i--);  //find most significant element of x
    xp=x[i];
    yp=y[i];
    A=1; B=0; C=0; D=1;
    while ((yp+C) && (yp+D)) {
      q =Math.floor((xp+A)/(yp+C));
      qp=Math.floor((xp+B)/(yp+D));
      if (q!=qp)
        break;
      t= A-q*C;   A=C;   C=t;    //  do (A,B,xp, C,D,yp) = (C,D,yp, A,B,xp) - q*(0,0,0, C,D,yp)      
      t= B-q*D;   B=D;   D=t;
      t=xp-q*yp; xp=yp; yp=t;
    }
    if (B) {
      copy_(T,x);
      linComb_(x,y,A,B); //x=A*x+B*y
      linComb_(y,T,D,C); //y=D*y+C*T
    } else {
      mod_(x,y);
      copy_(T,x);
      copy_(x,y);
      copy_(y,T);
    } 
  }
  if (y[0]==0)
    return;
  t=modInt(x,y[0]);
  copyInt_(x,y[0]);
  y[0]=t;
  while (y[0]) {
    x[0]%=y[0];
    t=x[0]; x[0]=y[0]; y[0]=t;
  }
}

//do x=x**(-1) mod n, for bigInts x and n.
//If no inverse exists, it sets x to zero and returns 0, else it returns 1.
//The x array must be at least as large as the n array.
function inverseMod_(x,n) {
  var k=1+2*Math.max(x.length,n.length);

  if(!(x[0]&1)  && !(n[0]&1)) {  //if both inputs are even, then inverse doesn't exist
    copyInt_(x,0);
    return 0;
  }

  if (eg_u.length!=k) {
    eg_u=new Array(k);
    eg_v=new Array(k);
    eg_A=new Array(k);
    eg_B=new Array(k);
    eg_C=new Array(k);
    eg_D=new Array(k);
  }

  copy_(eg_u,x);
  copy_(eg_v,n);
  copyInt_(eg_A,1);
  copyInt_(eg_B,0);
  copyInt_(eg_C,0);
  copyInt_(eg_D,1);
  for (;;) {
    while(!(eg_u[0]&1)) {  //while eg_u is even
      halve_(eg_u);
      if (!(eg_A[0]&1) && !(eg_B[0]&1)) { //if eg_A==eg_B==0 mod 2
        halve_(eg_A);
        halve_(eg_B);      
      } else {
        add_(eg_A,n);  halve_(eg_A);
        sub_(eg_B,x);  halve_(eg_B);
      }
    }

    while (!(eg_v[0]&1)) {  //while eg_v is even
      halve_(eg_v);
      if (!(eg_C[0]&1) && !(eg_D[0]&1)) { //if eg_C==eg_D==0 mod 2
        halve_(eg_C);
        halve_(eg_D);      
      } else {
        add_(eg_C,n);  halve_(eg_C);
        sub_(eg_D,x);  halve_(eg_D);
      }
    }

    if (!greater(eg_v,eg_u)) { //eg_v <= eg_u
      sub_(eg_u,eg_v);
      sub_(eg_A,eg_C);
      sub_(eg_B,eg_D);
    } else {                   //eg_v > eg_u
      sub_(eg_v,eg_u);
      sub_(eg_C,eg_A);
      sub_(eg_D,eg_B);
    }
  
    if (equalsInt(eg_u,0)) {
      if (negative(eg_C)) //make sure answer is nonnegative
        add_(eg_C,n);
      copy_(x,eg_C);

      if (!equalsInt(eg_v,1)) { //if GCD_(x,n)!=1, then there is no inverse
        copyInt_(x,0);
        return 0;
      }
      return 1;
    }
  }
}

//return x**(-1) mod n, for integers x and n.  Return 0 if there is no inverse
function inverseModInt(x,n) {
  var a=1,b=0,t;
  for (;;) {
    if (x==1) return a;
    if (x==0) return 0;
    b-=a*Math.floor(n/x);
    n%=x;

    if (n==1) return b; //to avoid negatives, change this b to n-b, and each -= to +=
    if (n==0) return 0;
    a-=b*Math.floor(x/n);
    x%=n;
  }
}

//this deprecated function is for backward compatibility only. 
function inverseModInt_(x,n) {
   return inverseModInt(x,n);
}


//Given positive bigInts x and y, change the bigints v, a, and b to positive bigInts such that:
//     v = GCD_(x,y) = a*x-b*y
//The bigInts v, a, b, must have exactly as many elements as the larger of x and y.
function eGCD_(x,y,v,a,b) {
  var g=0;
  var k=Math.max(x.length,y.length);
  if (eg_u.length!=k) {
    eg_u=new Array(k);
    eg_A=new Array(k);
    eg_B=new Array(k);
    eg_C=new Array(k);
    eg_D=new Array(k);
  }
  while(!(x[0]&1)  && !(y[0]&1)) {  //while x and y both even
    halve_(x);
    halve_(y);
    g++;
  }
  copy_(eg_u,x);
  copy_(v,y);
  copyInt_(eg_A,1);
  copyInt_(eg_B,0);
  copyInt_(eg_C,0);
  copyInt_(eg_D,1);
  for (;;) {
    while(!(eg_u[0]&1)) {  //while u is even
      halve_(eg_u);
      if (!(eg_A[0]&1) && !(eg_B[0]&1)) { //if A==B==0 mod 2
        halve_(eg_A);
        halve_(eg_B);      
      } else {
        add_(eg_A,y);  halve_(eg_A);
        sub_(eg_B,x);  halve_(eg_B);
      }
    }

    while (!(v[0]&1)) {  //while v is even
      halve_(v);
      if (!(eg_C[0]&1) && !(eg_D[0]&1)) { //if C==D==0 mod 2
        halve_(eg_C);
        halve_(eg_D);      
      } else {
        add_(eg_C,y);  halve_(eg_C);
        sub_(eg_D,x);  halve_(eg_D);
      }
    }

    if (!greater(v,eg_u)) { //v<=u
      sub_(eg_u,v);
      sub_(eg_A,eg_C);
      sub_(eg_B,eg_D);
    } else {                //v>u
      sub_(v,eg_u);
      sub_(eg_C,eg_A);
      sub_(eg_D,eg_B);
    }
    if (equalsInt(eg_u,0)) {
      if (negative(eg_C)) {   //make sure a (C)is nonnegative
        add_(eg_C,y);
        sub_(eg_D,x);
      }
      multInt_(eg_D,-1);  ///make sure b (D) is nonnegative
      copy_(a,eg_C);
      copy_(b,eg_D);
      leftShift_(v,g);
      return;
    }
  }
}


//is bigInt x negative?
function negative(x) {
  return ((x[x.length-1]>>(bpe-1))&1);
}


//is (x << (shift*bpe)) > y?
//x and y are nonnegative bigInts
//shift is a nonnegative integer
function greaterShift(x,y,shift) {
  var kx=x.length, ky=y.length;
  k=((kx+shift)<ky) ? (kx+shift) : ky;
  for (i=ky-1-shift; i<kx && i>=0; i++) 
    if (x[i]>0)
      return 1; //if there are nonzeros in x to the left of the first column of y, then x is bigger
  for (i=kx-1+shift; i<ky; i++)
    if (y[i]>0)
      return 0; //if there are nonzeros in y to the left of the first column of x, then x is not bigger
  for (i=k-1; i>=shift; i--)
    if      (x[i-shift]>y[i]) return 1;
    else if (x[i-shift]<y[i]) return 0;
  return 0;
}

//is x > y? (x and y both nonnegative)
function greater(x,y) {
  var i;
  var k=(x.length<y.length) ? x.length : y.length;

  for (i=x.length;i<y.length;i++)
    if (y[i])
      return 0;  //y has more digits

  for (i=y.length;i<x.length;i++)
    if (x[i])
      return 1;  //x has more digits

  for (i=k-1;i>=0;i--)
    if (x[i]>y[i])
      return 1;
    else if (x[i]<y[i])
      return 0;
  return 0;
}

//divide x by y giving quotient q and remainder r.  (q=floor(x/y),  r=x mod y).  All 4 are bigints.
//x must have at least one leading zero element.
//y must be nonzero.
//q and r must be arrays that are exactly the same length as x. (Or q can have more).
//Must have x.length >= y.length >= 2.
function divide_(x,y,q,r) {
  var kx, ky;
  var i,j,y1,y2,c,a,b;
  copy_(r,x);
  for (ky=y.length;y[ky-1]==0;ky--); //ky is number of elements in y, not including leading zeros

  //normalize: ensure the most significant element of y has its highest bit set  
  b=y[ky-1];
  for (a=0; b; a++)
    b>>=1;  
  a=bpe-a;  //a is how many bits to shift so that the high order bit of y is leftmost in its array element
  leftShift_(y,a);  //multiply both by 1<<a now, then divide both by that at the end
  leftShift_(r,a);

  //Rob Visser discovered a bug: the following line was originally just before the normalization.
  for (kx=r.length;r[kx-1]==0 && kx>ky;kx--); //kx is number of elements in normalized x, not including leading zeros

  copyInt_(q,0);                      // q=0
  while (!greaterShift(y,r,kx-ky)) {  // while (leftShift_(y,kx-ky) <= r) {
    subShift_(r,y,kx-ky);             //   r=r-leftShift_(y,kx-ky)
    q[kx-ky]++;                       //   q[kx-ky]++;
  }                                   // }

  for (i=kx-1; i>=ky; i--) {
    if (r[i]==y[ky-1])
      q[i-ky]=mask;
    else
      q[i-ky]=Math.floor((r[i]*radix+r[i-1])/y[ky-1]);	

    //The following for(;;) loop is equivalent to the commented while loop, 
    //except that the uncommented version avoids overflow.
    //The commented loop comes from HAC, which assumes r[-1]==y[-1]==0
    //  while (q[i-ky]*(y[ky-1]*radix+y[ky-2]) > r[i]*radix*radix+r[i-1]*radix+r[i-2])
    //    q[i-ky]--;    
    for (;;) {
      y2=(ky>1 ? y[ky-2] : 0)*q[i-ky];
      c=y2>>bpe;
      y2=y2 & mask;
      y1=c+q[i-ky]*y[ky-1];
      c=y1>>bpe;
      y1=y1 & mask;

      if (c==r[i] ? y1==r[i-1] ? y2>(i>1 ? r[i-2] : 0) : y1>r[i-1] : c>r[i]) 
        q[i-ky]--;
      else
        break;
    }

    linCombShift_(r,y,-q[i-ky],i-ky);    //r=r-q[i-ky]*leftShift_(y,i-ky)
    if (negative(r)) {
      addShift_(r,y,i-ky);         //r=r+leftShift_(y,i-ky)
      q[i-ky]--;
    }
  }

  rightShift_(y,a);  //undo the normalization step
  rightShift_(r,a);  //undo the normalization step
}

//do carries and borrows so each element of the bigInt x fits in bpe bits.
function carry_(x) {
  var i,k,c,b;
  k=x.length;
  c=0;
  for (i=0;i<k;i++) {
    c+=x[i];
    b=0;
    if (c<0) {
      b=-(c>>bpe);
      c+=b*radix;
    }
    x[i]=c & mask;
    c=(c>>bpe)-b;
  }
}

//return x mod n for bigInt x and integer n.
function modInt(x,n) {
  var i,c=0;
  for (i=x.length-1; i>=0; i--)
    c=(c*radix+x[i])%n;
  return c;
}

//convert the integer t into a bigInt with at least the given number of bits.
//the returned array stores the bigInt in bpe-bit chunks, little endian (buff[0] is least significant word)
//Pad the array with leading zeros so that it has at least minSize elements.
//There will always be at least one leading 0 element.
function int2bigInt(t,bits,minSize) {   
  var i,k;
  k=Math.ceil(bits/bpe)+1;
  k=minSize>k ? minSize : k;
  buff=new Array(k);
  copyInt_(buff,t);
  return buff;
}

//return the bigInt given a string representation in a given base.  
//Pad the array with leading zeros so that it has at least minSize elements.
//If base=-1, then it reads in a space-separated list of array elements in decimal.
//The array will always have at least one leading zero, unless base=-1.
function str2bigInt(s,base,minSize) {
  var d, i, j, x, y, kk;
  var k=s.length;
  if (base==-1) { //comma-separated list of array elements in decimal
    x=new Array(0);
    for (;;) {
      y=new Array(x.length+1);
      for (i=0;i<x.length;i++)
        y[i+1]=x[i];
      y[0]=parseInt(s,10);
      x=y;
      d=s.indexOf(',',0);
      if (d<1) 
        break;
      s=s.substring(d+1);
      if (s.length==0)
        break;
    }
    if (x.length<minSize) {
      y=new Array(minSize);
      copy_(y,x);
      return y;
    }
    return x;
  }

  x=int2bigInt(0,base*k,0);
  for (i=0;i<k;i++) {
    d=digitsStr.indexOf(s.substring(i,i+1),0);
    if (base<=36 && d>=36)  //convert lowercase to uppercase if base<=36
      d-=26;
    if (d<base && d>=0) {   //ignore illegal characters
      multInt_(x,base);
      addInt_(x,d);
    }
  }

  for (k=x.length;k>0 && !x[k-1];k--); //strip off leading zeros
  k=minSize>k+1 ? minSize : k+1;
  y=new Array(k);
  kk=k<x.length ? k : x.length;
  for (i=0;i<kk;i++)
    y[i]=x[i];
  for (;i<k;i++)
    y[i]=0;
  return y;
}

//is bigint x equal to integer y?
//y must have less than bpe bits
function equalsInt(x,y) {
  var i;
  if (x[0]!=y)
    return 0;
  for (i=1;i<x.length;i++)
    if (x[i])
      return 0;
  return 1;
}

//are bigints x and y equal?
//this works even if x and y are different lengths and have arbitrarily many leading zeros
function equals(x,y) {
  var i;
  var k=x.length<y.length ? x.length : y.length;
  for (i=0;i<k;i++)
    if (x[i]!=y[i])
      return 0;
  if (x.length>y.length) {
    for (;i<x.length;i++)
      if (x[i])
        return 0;
  } else {
    for (;i<y.length;i++)
      if (y[i])
        return 0;
  }
  return 1;
}

//is the bigInt x equal to zero?
function isZero(x) {
  var i;
  for (i=0;i<x.length;i++)
    if (x[i])
      return 0;
  return 1;
}

//convert a bigInt into a string in a given base, from base 2 up to base 95.
//Base -1 prints the contents of the array representing the number.
function bigInt2str(x,base) {
  var i,t,s="";

  if (s6.length!=x.length) 
    s6=dup(x);
  else
    copy_(s6,x);

  if (base==-1) { //return the list of array contents
    for (i=x.length-1;i>0;i--)
      s+=x[i]+',';
    s+=x[0];
  }
  else { //return it in the given base
    while (!isZero(s6)) {
      t=divInt_(s6,base);  //t=s6 % base; s6=floor(s6/base);
      s=digitsStr.substring(t,t+1)+s;
    }
  }
  if (s.length==0)
    s="0";
  return s;
}

//returns a duplicate of bigInt x
function dup(x) {
  var i;
  buff=new Array(x.length);
  copy_(buff,x);
  return buff;
}

//do x=y on bigInts x and y.  x must be an array at least as big as y (not counting the leading zeros in y).
function copy_(x,y) {
  var i;
  var k=x.length<y.length ? x.length : y.length;
  for (i=0;i<k;i++)
    x[i]=y[i];
  for (i=k;i<x.length;i++)
    x[i]=0;
}

//do x=y on bigInt x and integer y.  
function copyInt_(x,n) {
  var i,c;
  for (c=n,i=0;i<x.length;i++) {
    x[i]=c & mask;
    c>>=bpe;
  }
}

//do x=x+n where x is a bigInt and n is an integer.
//x must be large enough to hold the result.
function addInt_(x,n) {
  var i,k,c,b;
  x[0]+=n;
  k=x.length;
  c=0;
  for (i=0;i<k;i++) {
    c+=x[i];
    b=0;
    if (c<0) {
      b=-(c>>bpe);
      c+=b*radix;
    }
    x[i]=c & mask;
    c=(c>>bpe)-b;
    if (!c) return; //stop carrying as soon as the carry_ is zero
  }
}

//right shift bigInt x by n bits.  0 <= n < bpe.
function rightShift_(x,n) {
  var i;
  var k=Math.floor(n/bpe);
  if (k) {
    for (i=0;i<x.length-k;i++) //right shift x by k elements
      x[i]=x[i+k];
    for (;i<x.length;i++)
      x[i]=0;
    n%=bpe;
  }
  for (i=0;i<x.length-1;i++) {
    x[i]=mask & ((x[i+1]<<(bpe-n)) | (x[i]>>n));
  }
  x[i]>>=n;
}

//do x=floor(|x|/2)*sgn(x) for bigInt x in 2's complement
function halve_(x) {
  var i;
  for (i=0;i<x.length-1;i++) {
    x[i]=mask & ((x[i+1]<<(bpe-1)) | (x[i]>>1));
  }
  x[i]=(x[i]>>1) | (x[i] & (radix>>1));  //most significant bit stays the same
}

//left shift bigInt x by n bits.
function leftShift_(x,n) {
  var i;
  var k=Math.floor(n/bpe);
  if (k) {
    for (i=x.length; i>=k; i--) //left shift x by k elements
      x[i]=x[i-k];
    for (;i>=0;i--)
      x[i]=0;  
    n%=bpe;
  }
  if (!n)
    return;
  for (i=x.length-1;i>0;i--) {
    x[i]=mask & ((x[i]<<n) | (x[i-1]>>(bpe-n)));
  }
  x[i]=mask & (x[i]<<n);
}

//do x=x*n where x is a bigInt and n is an integer.
//x must be large enough to hold the result.
function multInt_(x,n) {
  var i,k,c,b;
  if (!n)
    return;
  k=x.length;
  c=0;
  for (i=0;i<k;i++) {
    c+=x[i]*n;
    b=0;
    if (c<0) {
      b=-(c>>bpe);
      c+=b*radix;
    }
    x[i]=c & mask;
    c=(c>>bpe)-b;
  }
}

//do x=floor(x/n) for bigInt x and integer n, and return the remainder
function divInt_(x,n) {
  var i,r=0,s;
  for (i=x.length-1;i>=0;i--) {
    s=r*radix+x[i];
    x[i]=Math.floor(s/n);
    r=s%n;
  }
  return r;
}

//do the linear combination x=a*x+b*y for bigInts x and y, and integers a and b.
//x must be large enough to hold the answer.
function linComb_(x,y,a,b) {
  var i,c,k,kk;
  k=x.length<y.length ? x.length : y.length;
  kk=x.length;
  for (c=0,i=0;i<k;i++) {
    c+=a*x[i]+b*y[i];
    x[i]=c & mask;
    c>>=bpe;
  }
  for (i=k;i<kk;i++) {
    c+=a*x[i];
    x[i]=c & mask;
    c>>=bpe;
  }
}

//do the linear combination x=a*x+b*(y<<(ys*bpe)) for bigInts x and y, and integers a, b and ys.
//x must be large enough to hold the answer.
function linCombShift_(x,y,b,ys) {
  var i,c,k,kk;
  k=x.length<ys+y.length ? x.length : ys+y.length;
  kk=x.length;
  for (c=0,i=ys;i<k;i++) {
    c+=x[i]+b*y[i-ys];
    x[i]=c & mask;
    c>>=bpe;
  }
  for (i=k;c && i<kk;i++) {
    c+=x[i];
    x[i]=c & mask;
    c>>=bpe;
  }
}

//do x=x+(y<<(ys*bpe)) for bigInts x and y, and integers a,b and ys.
//x must be large enough to hold the answer.
function addShift_(x,y,ys) {
  var i,c,k,kk;
  k=x.length<ys+y.length ? x.length : ys+y.length;
  kk=x.length;
  for (c=0,i=ys;i<k;i++) {
    c+=x[i]+y[i-ys];
    x[i]=c & mask;
    c>>=bpe;
  }
  for (i=k;c && i<kk;i++) {
    c+=x[i];
    x[i]=c & mask;
    c>>=bpe;
  }
}

//do x=x-(y<<(ys*bpe)) for bigInts x and y, and integers a,b and ys.
//x must be large enough to hold the answer.
function subShift_(x,y,ys) {
  var i,c,k,kk;
  k=x.length<ys+y.length ? x.length : ys+y.length;
  kk=x.length;
  for (c=0,i=ys;i<k;i++) {
    c+=x[i]-y[i-ys];
    x[i]=c & mask;
    c>>=bpe;
  }
  for (i=k;c && i<kk;i++) {
    c+=x[i];
    x[i]=c & mask;
    c>>=bpe;
  }
}

//do x=x-y for bigInts x and y.
//x must be large enough to hold the answer.
//negative answers will be 2s complement
function sub_(x,y) {
  var i,c,k,kk;
  k=x.length<y.length ? x.length : y.length;
  for (c=0,i=0;i<k;i++) {
    c+=x[i]-y[i];
    x[i]=c & mask;
    c>>=bpe;
  }
  for (i=k;c && i<x.length;i++) {
    c+=x[i];
    x[i]=c & mask;
    c>>=bpe;
  }
}

//do x=x+y for bigInts x and y.
//x must be large enough to hold the answer.
function add_(x,y) {
  var i,c,k,kk;
  k=x.length<y.length ? x.length : y.length;
  for (c=0,i=0;i<k;i++) {
    c+=x[i]+y[i];
    x[i]=c & mask;
    c>>=bpe;
  }
  for (i=k;c && i<x.length;i++) {
    c+=x[i];
    x[i]=c & mask;
    c>>=bpe;
  }
}

//do x=x*y for bigInts x and y.  This is faster when y<x.
function mult_(x,y) {
  var i;
  if (ss.length!=2*x.length)
    ss=new Array(2*x.length);
  copyInt_(ss,0);
  for (i=0;i<y.length;i++)
    if (y[i])
      linCombShift_(ss,x,y[i],i);   //ss=1*ss+y[i]*(x<<(i*bpe))
  copy_(x,ss);
}

//do x=x mod n for bigInts x and n.
function mod_(x,n) {
  if (s4.length!=x.length)
    s4=dup(x);
  else
    copy_(s4,x);
  if (s5.length!=x.length)
    s5=dup(x);  
  divide_(s4,n,s5,x);  //x = remainder of s4 / n
}

//do x=x*y mod n for bigInts x,y,n.
//for greater speed, let y<x.
function multMod_(x,y,n) {
  var i;
  if (s0.length!=2*x.length)
    s0=new Array(2*x.length);
  copyInt_(s0,0);
  for (i=0;i<y.length;i++)
    if (y[i])
      linCombShift_(s0,x,y[i],i);   //s0=1*s0+y[i]*(x<<(i*bpe))
  mod_(s0,n);
  copy_(x,s0);
}

//do x=x*x mod n for bigInts x,n.
function squareMod_(x,n) {
  var i,j,d,c,kx,kn,k;
  for (kx=x.length; kx>0 && !x[kx-1]; kx--);  //ignore leading zeros in x
  k=kx>n.length ? 2*kx : 2*n.length; //k=# elements in the product, which is twice the elements in the larger of x and n
  if (s0.length!=k) 
    s0=new Array(k);
  copyInt_(s0,0);
  for (i=0;i<kx;i++) {
    c=s0[2*i]+x[i]*x[i];
    s0[2*i]=c & mask;
    c>>=bpe;
    for (j=i+1;j<kx;j++) {
      c=s0[i+j]+2*x[i]*x[j]+c;
      s0[i+j]=(c & mask);
      c>>=bpe;
    }
    s0[i+kx]=c;
  }
  mod_(s0,n);
  copy_(x,s0);
}

//return x with exactly k leading zero elements
function trim(x,k) {
  var i,y;
  for (i=x.length; i>0 && !x[i-1]; i--);
  y=new Array(i+k);
  copy_(y,x);
  return y;
}

//do x=x**y mod n, where x,y,n are bigInts and ** is exponentiation.  0**0=1.
//this is faster when n is odd.  x usually needs to have as many elements as n.
function powMod_(x,y,n) {
  var k1,k2,kn,np;
  if(s7.length!=n.length)
    s7=dup(n);

  //for even modulus, use a simple square-and-multiply algorithm,
  //rather than using the more complex Montgomery algorithm.
  if ((n[0]&1)==0) {
    copy_(s7,x);
    copyInt_(x,1);
    while(!equalsInt(y,0)) {
      if (y[0]&1)
        multMod_(x,s7,n);
      divInt_(y,2);
      squareMod_(s7,n); 
    }
    return;
  }

  //calculate np from n for the Montgomery multiplications
  copyInt_(s7,0);
  for (kn=n.length;kn>0 && !n[kn-1];kn--);
  np=radix-inverseModInt(modInt(n,radix),radix);
  s7[kn]=1;
  multMod_(x ,s7,n);   // x = x * 2**(kn*bp) mod n

  if (s3.length!=x.length)
    s3=dup(x);
  else
    copy_(s3,x);

  for (k1=y.length-1;k1>0 & !y[k1]; k1--);  //k1=first nonzero element of y
  if (y[k1]==0) {  //anything to the 0th power is 1
    copyInt_(x,1);
    return;
  }
  for (k2=1<<(bpe-1);k2 && !(y[k1] & k2); k2>>=1);  //k2=position of first 1 bit in y[k1]
  for (;;) {
    if (!(k2>>=1)) {  //look at next bit of y
      k1--;
      if (k1<0) {
        mont_(x,one,n,np);
        return;
      }
      k2=1<<(bpe-1);
    }    
    mont_(x,x,n,np);

    if (k2 & y[k1]) //if next bit is a 1
      mont_(x,s3,n,np);
  }
}    

//do x=x*y*Ri mod n for bigInts x,y,n, 
//  where Ri = 2**(-kn*bpe) mod n, and kn is the 
//  number of elements in the n array, not 
//  counting leading zeros.  
//x must be large enough to hold the answer.
//It's OK if x and y are the same variable.
//must have:
//  x,y < n
//  n is odd
//  np = -(n^(-1)) mod radix
function mont_(x,y,n,np) {
  var i,j,c,ui,t;
  var kn=n.length;
  var ky=y.length;

  if (sa.length!=kn)
    sa=new Array(kn);

  for (;kn>0 && n[kn-1]==0;kn--); //ignore leading zeros of n
  //this function sometimes gives wrong answers when the next line is uncommented
  //for (;ky>0 && y[ky-1]==0;ky--); //ignore leading zeros of y

  copyInt_(sa,0);

  //the following loop consumes 95% of the runtime for randTruePrime_() and powMod_() for large keys
  for (i=0; i<kn; i++) {
    t=sa[0]+x[i]*y[0];
    ui=((t & mask) * np) & mask;  //the inner "& mask" is needed on Macintosh MSIE, but not windows MSIE
    c=(t+ui*n[0]) >> bpe;
    t=x[i];

    //do sa=(sa+x[i]*y+ui*n)/b   where b=2**bpe
    for (j=1;j<ky;j++) { 
      c+=sa[j]+t*y[j]+ui*n[j];
      sa[j-1]=c & mask;
      c>>=bpe;
    }    
    for (;j<kn;j++) { 
      c+=sa[j]+ui*n[j];
      sa[j-1]=c & mask;
      c>>=bpe;
    }    
    sa[j-1]=c & mask;
  }

  if (!greater(n,sa))
    sub_(sa,n);
  copy_(x,sa);
}


; 
 
 
 
/************************************************ 
**** jsSHA.js ***** 
************************************************/ 
/* 
 * A JavaScript implementation of the SHA family of hashes, as defined in FIPS PUB 180-2
 * Version 1.1 Copyright Brian Turek 2008
 * Distributed under the BSD License
 * See http://jssha.sourceforge.net/ for more information
 *
 * Several functions taken from Paul Johnson
 *
 * Modified on November 10,2008 by Atsushi Oka http://oka.nu/ 
 */

var digest = {};
function initializePackageSHA() {

/*
 * Int_64 is a object/container for 2 32-bit numbers emulating a 64-bit number
 *
 * @constructor
 * @param {Number} msint_32 The most significant 32-bits of a 64-bit number
 * @param {Number} lsint_32 The least significant 32-bits of a 64-bit number
 */
function Int_64(msint_32, lsint_32) {
    this.highOrder = msint_32;
    this.lowOrder = lsint_32;
}

/*
 * The 32-bit implementation of circular rotate left
 *
 * @private
 * @param {Number} x The 32-bit integer argument
 * @param {Number} n The number of bits to shift
 * @return The x shifted circularly by n bits
 */
function rotl_32(x, n) {
    if (n < 32) {
	return (x <<  n) | (x >>> (32 - n));
    } else {
	return x;
    }
}

/*
 * The 32-bit implementation of circular rotate right
 *
 * @private
 * @param {Number} x The 32-bit integer argument
 * @param {Number} n The number of bits to shift
 * @return The x shifted circularly by n bits
 */
function rotr_32(x, n) {
    if (n < 32) {
	return (x >>> n) | (x << (32 - n));
    } else {
	return x;
    }
}

/*
 * The 64-bit implementation of circular rotate right
 *
 * @private
 * @param {Int_64} x The 64-bit integer argument
 * @param {Number} n The number of bits to shift
 * @return The x shifted circularly by n bits
 */
function rotr_64(x, n) {
    if (n < 32) {
	return new Int_64(
		(x.highOrder >>> n) | (x.lowOrder << (32 - n)),
		(x.lowOrder >>> n) | (x.highOrder << (32 - n))
	    );
    } else if (n === 32) { // Apparently in JS, shifting a 32-bit value by 32 yields original value
	return new Int_64(x.lowOrder, x.highOrder);
    } else {
	return rotr_64(rotr_64(x, 32), n - 32);
    }
}

/*
 * The 32-bit implementation of shift right
 *
 * @private
 * @param {Number} x The 32-bit integer argument
 * @param {Number} n The number of bits to shift
 * @return The x shifted by n bits
 */
function shr_32(x, n) {
    if (n < 32) {
	return x >>> n;
    } else {
	return 0;
    }
}

/*
 * The 64-bit implementation of shift right
 *
 * @private
 * @param {Int_64} x The 64-bit integer argument
 * @param {Number} n The number of bits to shift
 * @return The x shifted by n bits
 */
function shr_64(x, n) {
    if (n < 32) {
	return new Int_64(
		x.highOrder >>> n,
		x.lowOrder >>> n | (x.highOrder << (32 - n))
	    );
    } else if (n === 32) { // Apparently in JS, shifting a 32-bit value by 32 yields original value
	return new Int_64(0, x.highOrder);
    } else {
	return shr_64(shr_64(x, 32), n - 32);
    }
}

/*
 * The 32-bit implementation of the NIST specified Parity function
 *
 * @private
 * @param {Number} x The first 32-bit integer argument
 * @param {Number} y The second 32-bit integer argument
 * @param {Number} z The third 32-bit integer argument
 * @return The NIST specified output of the function
 */
function parity_32(x, y, z) {
    return x ^ y ^ z;
}

/*
 * The 32-bit implementation of the NIST specified Ch function
 *
 * @private
 * @param {Number} x The first 32-bit integer argument
 * @param {Number} y The second 32-bit integer argument
 * @param {Number} z The third 32-bit integer argument
 * @return The NIST specified output of the function
 */
function ch_32(x, y, z) {
    return (x & y) ^ (~x & z);
}

/*
 * The 64-bit implementation of the NIST specified Ch function
 *
 * @private
 * @param {Int_64} x The first 64-bit integer argument
 * @param {Int_64} y The second 64-bit integer argument
 * @param {Int_64} z The third 64-bit integer argument
 * @return The NIST specified output of the function
 */
function ch_64(x, y, z) {
    return new Int_64(
	    (x.highOrder & y.highOrder) ^ (~x.highOrder & z.highOrder),
	    (x.lowOrder & y.lowOrder) ^ (~x.lowOrder & z.lowOrder)
	);
}

/*
 * The 32-bit implementation of the NIST specified Maj function
 *
 * @private
 * @param {Number} x The first 32-bit integer argument
 * @param {Number} y The second 32-bit integer argument
 * @param {Number} z The third 32-bit integer argument
 * @return The NIST specified output of the function
 */
function maj_32(x, y, z) {
    return (x & y) ^ (x & z) ^ (y & z);
}

/*
 * The 64-bit implementation of the NIST specified Maj function
 *
 * @private
 * @param {Int_64} x The first 64-bit integer argument
 * @param {Int_64} y The second 64-bit integer argument
 * @param {Int_64} z The third 64-bit integer argument
 * @return The NIST specified output of the function
 */
function maj_64(x, y, z) {
    return new Int_64(
	    (x.highOrder & y.highOrder) ^ (x.highOrder & z.highOrder) ^ (y.highOrder & z.highOrder),
	    (x.lowOrder & y.lowOrder) ^ (x.lowOrder & z.lowOrder) ^ (y.lowOrder & z.lowOrder)
	);
}

/*
 * The 32-bit implementation of the NIST specified Sigma0 function
 *
 * @private
 * @param {Number} x The 32-bit integer argument
 * @return The NIST specified output of the function
 */
function sigma0_32(x) {
    return rotr_32(x, 2) ^ rotr_32(x, 13) ^ rotr_32(x, 22);
}

/*
 * The 64-bit implementation of the NIST specified Sigma0 function
 *
 * @private
 * @param {Int_64} x The 64-bit integer argument
 * @return The NIST specified output of the function
 */
function sigma0_64(x) {
    var rotr28 = rotr_64(x, 28);
    var rotr34 = rotr_64(x, 34);
    var rotr39 = rotr_64(x, 39);

    return new Int_64(
	    rotr28.highOrder ^ rotr34.highOrder ^ rotr39.highOrder,
	    rotr28.lowOrder ^ rotr34.lowOrder ^ rotr39.lowOrder);
}

/*
 * The 32-bit implementation of the NIST specified Sigma1 function
 *
 * @private
 * @param {Number} x The 32-bit integer argument
 * @return The NIST specified output of the function
 */
function sigma1_32(x) {
    return rotr_32(x, 6) ^ rotr_32(x, 11) ^ rotr_32(x, 25);
}

/*
 * The 64-bit implementation of the NIST specified Sigma1 function
 *
 * @private
 * @param {Int_64} x The 64-bit integer argument
 * @return The NIST specified output of the function
 */
function sigma1_64(x) {
    var rotr14 = rotr_64(x, 14);
    var rotr18 = rotr_64(x, 18);
    var rotr41 = rotr_64(x, 41);

    return new Int_64(
	    rotr14.highOrder ^ rotr18.highOrder ^ rotr41.highOrder,
	    rotr14.lowOrder ^ rotr18.lowOrder ^ rotr41.lowOrder);
}

/*
 * The 32-bit implementation of the NIST specified Gamma0 function
 *
 * @private
 * @param {Number} x The 32-bit integer argument
 * @return The NIST specified output of the function
 */
function gamma0_32(x) {
    return rotr_32(x, 7) ^ rotr_32(x, 18) ^ shr_32(x, 3);
}

/*
 * The 64-bit implementation of the NIST specified Gamma0 function
 *
 * @private
 * @param {Int_64} x The 64-bit integer argument
 * @return The NIST specified output of the function
 */
function gamma0_64(x) {
    var rotr1 = rotr_64(x, 1);
    var rotr8 = rotr_64(x, 8);
    var shr7 = shr_64(x, 7);

    return new Int_64(
	    rotr1.highOrder ^ rotr8.highOrder ^ shr7.highOrder,
	    rotr1.lowOrder ^ rotr8.lowOrder ^ shr7.lowOrder);
}

/*
 * The 32-bit implementation of the NIST specified Gamma1 function
 *
 * @private
 * @param {Number} x The 32-bit integer argument
 * @return The NIST specified output of the function
 */
function gamma1_32(x) {
    return rotr_32(x, 17) ^ rotr_32(x, 19) ^ shr_32(x, 10);
}

/*
 * The 64-bit implementation of the NIST specified Gamma1 function
 *
 * @private
 * @param {Int_64} x The 64-bit integer argument
 * @return The NIST specified output of the function
 */
function gamma1_64(x) {
    var rotr19 = rotr_64(x, 19);
    var rotr61 = rotr_64(x, 61);
    var shr6 = shr_64(x, 6);

    return new Int_64(
	    rotr19.highOrder ^ rotr61.highOrder ^ shr6.highOrder,
	    rotr19.lowOrder ^ rotr61.lowOrder ^ shr6.lowOrder);
}

/*
 * Add 32-bit integers, wrapping at 2^32. This uses 16-bit operations internally
 * to work around bugs in some JS interpreters.
 *
 * @private
 * @param {Number} x The first 32-bit integer argument to be added
 * @param {Number} y The second 32-bit integer argument to be added
 * @return The sum of x + y
 */
function safeAdd_32(x, y) {
    var lsw = (x & 0xFFFF) + (y & 0xFFFF);
    var msw = (x >>> 16) + (y >>> 16) + (lsw >>> 16);

    return ((msw & 0xFFFF) << 16) | (lsw & 0xFFFF);
}

/*
 * Add 64-bit integers, wrapping at 2^64. This uses 16-bit operations internally
 * to work around bugs in some JS interpreters.
 *
 * @private
 * @param {Int_64} x The first 64-bit integer argument to be added
 * @param {Int_64} y The second 64-bit integer argument to be added
 * @return The sum of x + y
 */
function safeAdd_64(x, y) {
    var lsw = (x.lowOrder & 0xFFFF) + (y.lowOrder & 0xFFFF);
    var msw = (x.lowOrder >>> 16) + (y.lowOrder >>> 16) + (lsw >>> 16);
    var lowOrder = ((msw & 0xFFFF) << 16) | (lsw & 0xFFFF);

    lsw = (x.highOrder & 0xFFFF) + (y.highOrder & 0xFFFF) + (msw >>> 16);
    msw = (x.highOrder >>> 16) + (y.highOrder >>> 16) + (lsw >>> 16);
    var highOrder = ((msw & 0xFFFF) << 16) | (lsw & 0xFFFF);

    return new Int_64(highOrder, lowOrder);
}

/*
 * Calculates the SHA-1 hash of the string set at instantiation
 *
 * @private
 * @return The array of integers representing the SHA-1 hash of message
 */
// MODIFIED
// function coreSHA1() 
function coreSHA1(message,strBinLen) {
    var W = [];
    var a, b, c, d, e;
    var T;
    var ch = ch_32, parity = parity_32, maj = maj_32, rotl = rotl_32, safeAdd = safeAdd_32;
    var H = [
	0x67452301, 0xefcdab89, 0x98badcfe, 0x10325476,	0xc3d2e1f0
    ];
    var K = [
	0x5a827999, 0x5a827999, 0x5a827999, 0x5a827999,
	0x5a827999, 0x5a827999, 0x5a827999, 0x5a827999,
	0x5a827999, 0x5a827999, 0x5a827999, 0x5a827999,
	0x5a827999, 0x5a827999, 0x5a827999, 0x5a827999,
	0x5a827999, 0x5a827999, 0x5a827999, 0x5a827999,
	0x6ed9eba1, 0x6ed9eba1, 0x6ed9eba1, 0x6ed9eba1,
	0x6ed9eba1, 0x6ed9eba1, 0x6ed9eba1, 0x6ed9eba1,
	0x6ed9eba1, 0x6ed9eba1, 0x6ed9eba1, 0x6ed9eba1,
	0x6ed9eba1, 0x6ed9eba1, 0x6ed9eba1, 0x6ed9eba1,
	0x6ed9eba1, 0x6ed9eba1, 0x6ed9eba1, 0x6ed9eba1,
	0x8f1bbcdc, 0x8f1bbcdc, 0x8f1bbcdc, 0x8f1bbcdc,
	0x8f1bbcdc, 0x8f1bbcdc, 0x8f1bbcdc, 0x8f1bbcdc,
	0x8f1bbcdc, 0x8f1bbcdc, 0x8f1bbcdc, 0x8f1bbcdc,
	0x8f1bbcdc, 0x8f1bbcdc, 0x8f1bbcdc, 0x8f1bbcdc,
	0x8f1bbcdc, 0x8f1bbcdc, 0x8f1bbcdc, 0x8f1bbcdc,
	0xca62c1d6, 0xca62c1d6, 0xca62c1d6, 0xca62c1d6,
	0xca62c1d6, 0xca62c1d6, 0xca62c1d6, 0xca62c1d6,
	0xca62c1d6, 0xca62c1d6, 0xca62c1d6, 0xca62c1d6,
	0xca62c1d6, 0xca62c1d6, 0xca62c1d6, 0xca62c1d6,
	0xca62c1d6, 0xca62c1d6, 0xca62c1d6, 0xca62c1d6
    ];

    // var message = strToHash.slice();

    message[strBinLen >> 5] |= 0x80 << (24 - strBinLen % 32); // Append '1' at  the end of the binary string
    message[((strBinLen + 1 + 64 >> 9) << 4) + 15] = strBinLen; // Append length of binary string in the position such that the new length is a multiple of 512

    var appendedMessageLength = message.length;

    for (var i = 0; i < appendedMessageLength; i += 16) {
	a = H[0];
	b = H[1];
	c = H[2];
	d = H[3];
	e = H[4];

	for (var t = 0; t < 80; t++) {
	    if (t < 16) {
		W[t] = message[t + i];
	    } else {
		W[t] = rotl(W[t - 3] ^ W[t - 8] ^ W[t - 14] ^ W[t - 16], 1);
	    }

	    if (t < 20) {
		T = safeAdd(safeAdd(safeAdd(safeAdd(rotl(a, 5), ch(b, c, d)), e), K[t]), W[t]);
	    } else if (t < 40) {
		T = safeAdd(safeAdd(safeAdd(safeAdd(rotl(a, 5), parity(b, c, d)), e), K[t]), W[t]);
	    } else if (t < 60) {
		T = safeAdd(safeAdd(safeAdd(safeAdd(rotl(a, 5), maj(b, c, d)), e), K[t]), W[t]);
	    } else {
		T = safeAdd(safeAdd(safeAdd(safeAdd(rotl(a, 5), parity(b, c, d)), e), K[t]), W[t]);
	    }

	    e = d;
	    d = c;
	    c = rotl(b, 30);
	    b = a;
	    a = T;
	}

	H[0] = safeAdd(a, H[0]);
	H[1] = safeAdd(b, H[1]);
	H[2] = safeAdd(c, H[2]);
	H[3] = safeAdd(d, H[3]);
	H[4] = safeAdd(e, H[4]);
    }

    return H;
}

/*
 * Calculates the desired SHA-2 hash of the string set at instantiation
 *
 * @private
 * @param {String} variant The desired SHA-2 variant
 * @return The array of integers representing the SHA-2 hash of message
 */

// MODIFIED
// function coreSHA2(variant) 
function coreSHA2( message, strBinLen, variant ) {
    var W = [];
    var a, b, c, d, e, f, g, h;
    var T1, T2;
    var H;
    var numRounds, lengthPosition, binaryStringInc, binaryStringMult;
    var safeAdd, gamma0, gamma1, sigma0, sigma1, ch, maj, Int;
    var K;
    // MODIFIED
    // var message = strToHash.slice();

    // Set up the various function handles and variable for the specific variant
    if (variant === "SHA-224" || variant === "SHA-256") // 32-bit variant
    {
	numRounds = 64;
	lengthPosition = ((strBinLen + 1 + 64 >> 9) << 4) + 15;
	binaryStringInc = 16;
	binaryStringMult = 1;
	Int = Number;
	safeAdd = safeAdd_32;
	gamma0 = gamma0_32;
	gamma1 = gamma1_32;
	sigma0 = sigma0_32;
	sigma1 = sigma1_32;
	maj = maj_32;
	ch = ch_32;
	K = [
		0x428A2F98, 0x71374491, 0xB5C0FBCF, 0xE9B5DBA5,
		0x3956C25B, 0x59F111F1, 0x923F82A4, 0xAB1C5ED5,
		0xD807AA98, 0x12835B01, 0x243185BE, 0x550C7DC3,
		0x72BE5D74, 0x80DEB1FE, 0x9BDC06A7, 0xC19BF174,
		0xE49B69C1, 0xEFBE4786, 0x0FC19DC6, 0x240CA1CC,
		0x2DE92C6F, 0x4A7484AA, 0x5CB0A9DC, 0x76F988DA,
		0x983E5152, 0xA831C66D, 0xB00327C8, 0xBF597FC7,
		0xC6E00BF3, 0xD5A79147, 0x06CA6351, 0x14292967,
		0x27B70A85, 0x2E1B2138, 0x4D2C6DFC, 0x53380D13,
		0x650A7354, 0x766A0ABB, 0x81C2C92E, 0x92722C85,
		0xA2BFE8A1, 0xA81A664B, 0xC24B8B70, 0xC76C51A3,
		0xD192E819, 0xD6990624, 0xF40E3585, 0x106AA070,
		0x19A4C116, 0x1E376C08, 0x2748774C, 0x34B0BCB5,
		0x391C0CB3, 0x4ED8AA4A, 0x5B9CCA4F, 0x682E6FF3,
		0x748F82EE, 0x78A5636F, 0x84C87814, 0x8CC70208,
		0x90BEFFFA, 0xA4506CEB, 0xBEF9A3F7, 0xC67178F2
	    ];

	if (variant === "SHA-224") {
	    H = [
		    0xc1059ed8, 0x367cd507, 0x3070dd17, 0xf70e5939,
		    0xffc00b31, 0x68581511, 0x64f98fa7, 0xbefa4fa4
		];
	} else {
	    H = [
		    0x6A09E667, 0xBB67AE85, 0x3C6EF372, 0xA54FF53A,
		    0x510E527F, 0x9B05688C, 0x1F83D9AB, 0x5BE0CD19
		];
	}
    } else if (variant === "SHA-384" || variant === "SHA-512") {// 64-bit variant
	numRounds = 80;
	lengthPosition = ((strBinLen + 1 + 128 >> 10) << 5) + 31;
	binaryStringInc = 32;
	binaryStringMult = 2;
	Int = Int_64;
	safeAdd = safeAdd_64;
	gamma0 = gamma0_64;
	gamma1 = gamma1_64;
	sigma0 = sigma0_64;
	sigma1 = sigma1_64;
	maj = maj_64;
	ch = ch_64;

	K = [
		new Int_64(0x428a2f98, 0xd728ae22), new Int_64(0x71374491, 0x23ef65cd), new Int_64(0xb5c0fbcf, 0xec4d3b2f), new Int_64(0xe9b5dba5, 0x8189dbbc),
		new Int_64(0x3956c25b, 0xf348b538), new Int_64(0x59f111f1, 0xb605d019), new Int_64(0x923f82a4, 0xaf194f9b), new Int_64(0xab1c5ed5, 0xda6d8118),
		new Int_64(0xd807aa98, 0xa3030242), new Int_64(0x12835b01, 0x45706fbe), new Int_64(0x243185be, 0x4ee4b28c), new Int_64(0x550c7dc3, 0xd5ffb4e2),
		new Int_64(0x72be5d74, 0xf27b896f), new Int_64(0x80deb1fe, 0x3b1696b1), new Int_64(0x9bdc06a7, 0x25c71235), new Int_64(0xc19bf174, 0xcf692694),
		new Int_64(0xe49b69c1, 0x9ef14ad2), new Int_64(0xefbe4786, 0x384f25e3), new Int_64(0x0fc19dc6, 0x8b8cd5b5), new Int_64(0x240ca1cc, 0x77ac9c65),
		new Int_64(0x2de92c6f, 0x592b0275), new Int_64(0x4a7484aa, 0x6ea6e483), new Int_64(0x5cb0a9dc, 0xbd41fbd4), new Int_64(0x76f988da, 0x831153b5),
		new Int_64(0x983e5152, 0xee66dfab), new Int_64(0xa831c66d, 0x2db43210), new Int_64(0xb00327c8, 0x98fb213f), new Int_64(0xbf597fc7, 0xbeef0ee4),
		new Int_64(0xc6e00bf3, 0x3da88fc2), new Int_64(0xd5a79147, 0x930aa725), new Int_64(0x06ca6351, 0xe003826f), new Int_64(0x14292967, 0x0a0e6e70),
		new Int_64(0x27b70a85, 0x46d22ffc), new Int_64(0x2e1b2138, 0x5c26c926), new Int_64(0x4d2c6dfc, 0x5ac42aed), new Int_64(0x53380d13, 0x9d95b3df),
		new Int_64(0x650a7354, 0x8baf63de), new Int_64(0x766a0abb, 0x3c77b2a8), new Int_64(0x81c2c92e, 0x47edaee6), new Int_64(0x92722c85, 0x1482353b),
		new Int_64(0xa2bfe8a1, 0x4cf10364), new Int_64(0xa81a664b, 0xbc423001), new Int_64(0xc24b8b70, 0xd0f89791), new Int_64(0xc76c51a3, 0x0654be30),
		new Int_64(0xd192e819, 0xd6ef5218), new Int_64(0xd6990624, 0x5565a910), new Int_64(0xf40e3585, 0x5771202a), new Int_64(0x106aa070, 0x32bbd1b8),
		new Int_64(0x19a4c116, 0xb8d2d0c8), new Int_64(0x1e376c08, 0x5141ab53), new Int_64(0x2748774c, 0xdf8eeb99), new Int_64(0x34b0bcb5, 0xe19b48a8),
		new Int_64(0x391c0cb3, 0xc5c95a63), new Int_64(0x4ed8aa4a, 0xe3418acb), new Int_64(0x5b9cca4f, 0x7763e373), new Int_64(0x682e6ff3, 0xd6b2b8a3),
		new Int_64(0x748f82ee, 0x5defb2fc), new Int_64(0x78a5636f, 0x43172f60), new Int_64(0x84c87814, 0xa1f0ab72), new Int_64(0x8cc70208, 0x1a6439ec),
		new Int_64(0x90befffa, 0x23631e28), new Int_64(0xa4506ceb, 0xde82bde9), new Int_64(0xbef9a3f7, 0xb2c67915), new Int_64(0xc67178f2, 0xe372532b),
		new Int_64(0xca273ece, 0xea26619c), new Int_64(0xd186b8c7, 0x21c0c207), new Int_64(0xeada7dd6, 0xcde0eb1e), new Int_64(0xf57d4f7f, 0xee6ed178),
		new Int_64(0x06f067aa, 0x72176fba), new Int_64(0x0a637dc5, 0xa2c898a6), new Int_64(0x113f9804, 0xbef90dae), new Int_64(0x1b710b35, 0x131c471b),
		new Int_64(0x28db77f5, 0x23047d84), new Int_64(0x32caab7b, 0x40c72493), new Int_64(0x3c9ebe0a, 0x15c9bebc), new Int_64(0x431d67c4, 0x9c100d4c),
		new Int_64(0x4cc5d4be, 0xcb3e42b6), new Int_64(0x597f299c, 0xfc657e2a), new Int_64(0x5fcb6fab, 0x3ad6faec), new Int_64(0x6c44198c, 0x4a475817)
	    ];

	if (variant === "SHA-384") {
	    H = [
		    new Int_64(0xcbbb9d5d, 0xc1059ed8), new Int_64(0x0629a292a, 0x367cd507), new Int_64(0x9159015a, 0x3070dd17), new Int_64(0x152fecd8, 0xf70e5939),
		    new Int_64(0x67332667, 0xffc00b31), new Int_64(0x98eb44a87, 0x68581511), new Int_64(0xdb0c2e0d, 0x64f98fa7), new Int_64(0x47b5481d, 0xbefa4fa4)
		];
	} else {
	    H = [
		    new Int_64(0x6a09e667, 0xf3bcc908), new Int_64(0xbb67ae85, 0x84caa73b), new Int_64(0x3c6ef372, 0xfe94f82b), new Int_64(0xa54ff53a, 0x5f1d36f1),
		    new Int_64(0x510e527f, 0xade682d1), new Int_64(0x9b05688c, 0x2b3e6c1f), new Int_64(0x1f83d9ab, 0xfb41bd6b), new Int_64(0x5be0cd19, 0x137e2179)
		];
	}
    }

    message[strBinLen >> 5] |= 0x80 << (24 - strBinLen % 32); // Append '1' at  the end of the binary string
    message[lengthPosition] = strBinLen; // Append length of binary string in the position such that the new length is correct

    var appendedMessageLength = message.length;

    for (var i = 0; i < appendedMessageLength; i += binaryStringInc) {
	a = H[0];
	b = H[1];
	c = H[2];
	d = H[3];
	e = H[4];
	f = H[5];
	g = H[6];
	h = H[7];

	for (var t = 0; t < numRounds; t++) {
	    if (t < 16) {
		W[t] = new Int(message[t * binaryStringMult + i], message[t * binaryStringMult + i + 1]); // Bit of a hack - for 32-bit, the second term is ignored
	    } else {
		W[t] = safeAdd(safeAdd(safeAdd(gamma1(W[t - 2]), W[t - 7]), gamma0(W[t - 15])), W[t - 16]);
	    }

	    T1 = safeAdd(safeAdd(safeAdd(safeAdd(h, sigma1(e)), ch(e, f, g)), K[t]), W[t]);
	    T2 = safeAdd(sigma0(a), maj(a, b, c));
	    h = g;
	    g = f;
	    f = e;
	    e = safeAdd(d, T1);
	    d = c;
	    c = b;
	    b = a;
	    a = safeAdd(T1, T2);
	}

	H[0] = safeAdd(a, H[0]);
	H[1] = safeAdd(b, H[1]);
	H[2] = safeAdd(c, H[2]);
	H[3] = safeAdd(d, H[3]);
	H[4] = safeAdd(e, H[4]);
	H[5] = safeAdd(f, H[5]);
	H[6] = safeAdd(g, H[6]);
	H[7] = safeAdd(h, H[7]);
    }

    switch (variant) {
    case "SHA-224":
	return	[
	    H[0], H[1],	H[2], H[3],
	    H[4], H[5],	H[6]
	];
    case "SHA-256":
	return H;
    case "SHA-384":
	return	[
	    H[0].highOrder, H[0].lowOrder,
	    H[1].highOrder, H[1].lowOrder,
	    H[2].highOrder, H[2].lowOrder,
	    H[3].highOrder, H[3].lowOrder,
	    H[4].highOrder, H[4].lowOrder,
	    H[5].highOrder, H[5].lowOrder
	];
    case "SHA-512":
	return	[
	    H[0].highOrder, H[0].lowOrder,
	    H[1].highOrder, H[1].lowOrder,
	    H[2].highOrder, H[2].lowOrder,
	    H[3].highOrder, H[3].lowOrder,
	    H[4].highOrder, H[4].lowOrder,
	    H[5].highOrder, H[5].lowOrder,
	    H[6].highOrder, H[6].lowOrder,
	    H[7].highOrder, H[7].lowOrder
	];
    default:
	return []; // This should near be reached
    }
}

// CREATING PACKAGE

// PUBLISH CORE FUNCTIONS
digest.coreSHA1 = coreSHA1;
digest.coreSHA2 = coreSHA2;

} // of function initializePackageSHA()

initializePackageSHA();

// vim:ts=8 sw=4:noexpandtab:
; 
 
 
 
/************************************************ 
**** SHA.js ***** 
************************************************/ 
/*
 * SHA.js
 * See SHA.readme.txt for further information.
 *
 * Copyright(c) 2009 Atsushi Oka [ http://oka.nu/ ]
 * This script file is distributed under the LGPL
 */
function sha( ) {
    
    // import
    // .. nothing
    
    /*
     * main class
     */
    function sha( variant, shaFunc, inputFunc, outputFunc ) {
	this.variant = variant;
	this.shaFunc = shaFunc;
	this.inputFunc = inputFunc;
	this.outputFunc = outputFunc;
    }

    /*
     * a default convert function. do nothing.
     * @param	anything.
     * @return  anything.
     */
    function nop(value){
	return value;
    }


    /*
     * convert a 32bit-integer array to a byte array. ( as big-endian )
     * @param{Array<Number>}  32bit-integer array
     * @return  byte array
     */
    // function ia2ba( ia ) {
    //     var length = ia.length << 2 ;
    //     var ba = new Array( length );
    //     for ( var i=0; i< length; i++ ) {
    //         ba[i] = 0xFF & ( ia[ i >> 2 ] << ( 3 - ( i % 4 ) ) * 8 );
    //     }
    //     return ba;
    // }


    /*
     * convert a byte array to a 32bit-integer array. ( as big-endian )
     * @param  {Array<Number>} byte array
     * @return 32bit-integer array
     */
    // function ba2ia( ba ) {
    //     var ia = new Array( ( ba.length + 3 ) >> 2 ); // ceil( ba.length / 4 )
    //     for ( var i=0; i<ia.length; i++ ) {
    //         ia[i]=0;
    //     }
    //     for ( var i=0; i<ba.length; i++ ) {
    //         ia[ i>>2 ] |= ( ba[i] << ( 8 * ( 3 - ( i % 4 )  ) ) );
    //     }
    //     return ia;
    // }

    function ia2ba( ia ) {
	var length = ia.length <<2;
	var ba = new Array( length );
	for(var ii=0,bi=0;ii<ia.length&&bi<ba.length; ){
	    ba[bi++] = 0xff & ( ia[ii] >> 24 );
	    ba[bi++] = 0xff & ( ia[ii] >> 16 );
	    ba[bi++] = 0xff & ( ia[ii] >>  8 );
	    ba[bi++] = 0xff & ( ia[ii] >>  0 );
	    ii++;
	}
	return ba;
    }
    function ba2ia( ba ) {
	var length = (ba.length+3)>>2;
	var ia = new Array( length );;
	for(var ii=0,bi=0; ii<ia.length && bi<ba.length; ){
	    ia[ii++] = 
		( bi < ba.length ? (ba[bi++]  << 24 ) : 0 ) |
		( bi < ba.length ? (ba[bi++]  << 16 ) : 0 ) |
		( bi < ba.length ? (ba[bi++]  <<  8 ) : 0 ) |
		( bi < ba.length ? (ba[bi++]/*<< 0*/) : 0 ) ;
	}
	return ia;
    }

    function str2ba( str ) {
	return str2utf8( str );
    };

    function hex2ba( hexstr ) {
	return base16_decode( hexstr );
    };

    function b642ba( ba ) {
	return base64_encode( ba );
    };

    /*
     * convert an integer array to hexadecimal string.
     * @param  {Array<Integer>} an integer array
     * @return a hexadecimal string
     */
    var arr_upper = ["0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F" ];
    var arr_lower = ["0","1","2","3","4","5","6","7","8","9","a","b","c","d","e","f" ];
    function ia2hex(ia,arr) {
	var result ="";
	for ( var i=0; i<ia.length; i++ ) {
	    var ii=ia[i];
	    for ( var j=28;0<=j; j-=4 ) {
		result += arr[( 0xF & ( ii >> j ) )];
	    }
	}
	return result;
    }
    function ia2hex_lower(ia) {
	return ia2hex(ia,arr_lower );
    }
    function ia2hex_upper(ia) {
	return ia2hex(ia,arr_upper );
    }


    function ia2b64( ia ) {
	return base64_encode( ia2ba(ia) );
    }
    function ia2str( ia ) {
	return utf82str( ia2ba(ia) );
    };

    // trace( ( 0xFFFFFFF5 ).toString(16) );
    // trace( ia2hex( [ 0xFFFFFFF5, 0xFFFFFFFF ] ) );

    /*
     * 
     */
    function create( variant, inputFormat, outputFormat ) {
	var inputFunc  = null;
	var outputFunc = null;

	if ( inputFormat == null ) {
	    inputFunc = nop;
	} else if ( typeof( inputFormat ) == 'function' ) {
	    inputFunc = inputFormat;
	} else {
	    switch ( inputFormat ) {
	    case "BIN":
	    case "binary":
		inputFunc = nop;
		break;
	    case "STR":
	    case "STR(UTF8)":
	    case "string":
		inputFunc = str2ba;
		break;
	    case "HEX":
	    case "hex":
		inputFunc = hex2ba;
		break;
	    case "B64":
	    case "base64":
		inputFunc = b642ba;
		break;
	    default :
		throw "INPUT FORMAT NOT RECOGNIZED";
	    }
	}

	if ( outputFormat == null ) {
	    outputFunc = ia2ba;
	} else if ( typeof( outputFormat ) == 'function' ) {
	    outputFunc = outputFormat;
	} else {
	    switch ( outputFormat ) {
	    case "binary":
	    case "BIN":
		outputFunc = ia2ba;
		break;
	    case "STR":
	    case "STR(UTF8)":
	    case "string":
		inputFunc = ia2str;
	    case "HEX":
		outputFunc = ia2hex_upper;
		break;
	    case "hex":
		outputFunc = ia2hex_lower;
		break;
	    case "B64":
	    case "base64":
		outputFunc = ia2b64;
		break;
	    default:
		throw "OUTPUT FORMAT NOT RECOGNIZED";
	    }
	}

	var shaFunc = null;
	switch (variant) {
	case "SHA-1":
	    shaFunc = digest.coreSHA1
	    break;
	case "SHA-224":
	    shaFunc = digest.coreSHA2
	    break;
	case "SHA-256":
	    shaFunc = digest.coreSHA2
	    break;
	case "SHA-384":
	    shaFunc = digest.coreSHA2
	    break;
	case "SHA-512":
	    shaFunc = digest.coreSHA2
	    break;
	default:
	    throw "HASH NOT RECOGNIZED";
	}

	return new sha( variant, shaFunc, inputFunc, outputFunc );
    }

    function hash( input ) {
	var bytes         = this.inputFunc( input );
	// trace( "input bytes : " + base16( bytes ) );
	var messageLength = bytes.length * 8;
	var message = ba2ia(bytes);
	// trace( "input integer[0] : " + message[0].toString(16) );
	var output        = this.outputFunc( this.shaFunc( message , messageLength, this.variant ) );
	return output;
    }

    sha.prototype.hash = hash;
    sha.create = create;

    this.sha = sha;
} // of function initializeInterfaceSHA()


sha();


// vim:ts=8 sw=4:noexpandtab:
; 
 
 
 
/************************************************ 
**** sha512.js ***** 
************************************************/ 
/*
* A JavaScript implementation of the Secure Hash Algorithm, SHA-512, as defined
* in FIPS 180-2
* Version 2.2-beta Copyright Anonymous Contributor, Paul Johnston 2000 - 2009.
* Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
* Distributed under the BSD License
* See http://pajhome.org.uk/crypt/md5 for details.
*/

/*
* Configurable variables. You may need to tweak these to be compatible with
* the server-side, but the defaults work in most cases.
*/
var hexcase = 0;  /* hex output format. 0 - lowercase; 1 - uppercase        */
var b64pad = ""; /* base-64 pad character. "=" for strict RFC compliance   */

/*
* These are the functions you'll usually want to call
* They take string arguments and return either hex or base-64 encoded strings
*/
function hex_sha512(s) { return rstr2hex(rstr_sha512(str2rstr_utf8(s))); }
function b64_sha512(s) { return rstr2b64(rstr_sha512(str2rstr_utf8(s))); }
function any_sha512(s, e) { return rstr2any(rstr_sha512(str2rstr_utf8(s)), e); }
function hex_hmac_sha512(k, d)
{ return rstr2hex(rstr_hmac_sha512(str2rstr_utf8(k), str2rstr_utf8(d))); }
function b64_hmac_sha512(k, d)
{ return rstr2b64(rstr_hmac_sha512(str2rstr_utf8(k), str2rstr_utf8(d))); }
function any_hmac_sha512(k, d, e)
{ return rstr2any(rstr_hmac_sha512(str2rstr_utf8(k), str2rstr_utf8(d)), e); }

/*
* Perform a simple self-test to see if the VM is working
*/
function sha512_vm_test()
{
    return hex_sha512("abc").toLowerCase() ==
    "ddaf35a193617abacc417349ae20413112e6fa4e89a97ea20a9eeee64b55d39a" +
    "2192992a274fc1a836ba3c23a3feebbd454d4423643ce80e2a9ac94fa54ca49f";
}

/*
* Calculate the SHA-512 of a raw string
*/
function rstr_sha512(s)
{
    return binb2rstr(binb_sha512(rstr2binb(s), s.length * 8));
}

/*
* Calculate the HMAC-SHA-512 of a key and some data (raw strings)
*/
function rstr_hmac_sha512(key, data)
{
    var bkey = rstr2binb(key);
    if (bkey.length > 32) bkey = binb_sha512(bkey, key.length * 8);

    var ipad = Array(32), opad = Array(32);
    for (var i = 0; i < 32; i++)
    {
        ipad[i] = bkey[i] ^ 0x36363636;
        opad[i] = bkey[i] ^ 0x5C5C5C5C;
    }

    var hash = binb_sha512(ipad.concat(rstr2binb(data)), 1024 + data.length * 8);
    return binb2rstr(binb_sha512(opad.concat(hash), 1024 + 512));
}

/*
* Convert a raw string to a hex string
*/
function rstr2hex(input)
{
    try { hexcase } catch (e) { hexcase = 0; }
    var hex_tab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
    var output = "";
    var x;
    for (var i = 0; i < input.length; i++)
    {
        x = input.charCodeAt(i);
        output += hex_tab.charAt((x >>> 4) & 0x0F)
           + hex_tab.charAt(x & 0x0F);
    }
    return output;
}

/*
* Convert a raw string to a base-64 string
*/
function rstr2b64(input)
{
    try { b64pad } catch (e) { b64pad = ''; }
    var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var output = "";
    var len = input.length;
    for (var i = 0; i < len; i += 3)
    {
        var triplet = (input.charCodeAt(i) << 16)
                | (i + 1 < len ? input.charCodeAt(i + 1) << 8 : 0)
                | (i + 2 < len ? input.charCodeAt(i + 2) : 0);
        for (var j = 0; j < 4; j++)
        {
            if (i * 8 + j * 6 > input.length * 8) output += b64pad;
            else output += tab.charAt((triplet >>> 6 * (3 - j)) & 0x3F);
        }
    }
    return output;
}

/*
* Convert a raw string to an arbitrary string encoding
*/
function rstr2any(input, encoding)
{
    var divisor = encoding.length;
    var i, j, q, x, quotient;

    /* Convert to an array of 16-bit big-endian values, forming the dividend */
    var dividend = Array(Math.ceil(input.length / 2));
    for (i = 0; i < dividend.length; i++)
    {
        dividend[i] = (input.charCodeAt(i * 2) << 8) | input.charCodeAt(i * 2 + 1);
    }

    /*
    * Repeatedly perform a long division. The binary array forms the dividend,
    * the length of the encoding is the divisor. Once computed, the quotient
    * forms the dividend for the next step. All remainders are stored for later
    * use.
    */
    var full_length = Math.ceil(input.length * 8 /
                                    (Math.log(encoding.length) / Math.log(2)));
    var remainders = Array(full_length);
    for (j = 0; j < full_length; j++)
    {
        quotient = Array();
        x = 0;
        for (i = 0; i < dividend.length; i++)
        {
            x = (x << 16) + dividend[i];
            q = Math.floor(x / divisor);
            x -= q * divisor;
            if (quotient.length > 0 || q > 0)
                quotient[quotient.length] = q;
        }
        remainders[j] = x;
        dividend = quotient;
    }

    /* Convert the remainders to the output string */
    var output = "";
    for (i = remainders.length - 1; i >= 0; i--)
        output += encoding.charAt(remainders[i]);

    return output;
}

/*
* Encode a string as utf-8.
* For efficiency, this assumes the input is valid utf-16.
*/
function str2rstr_utf8(input)
{
    var output = "";
    var i = -1;
    var x, y;

    while (++i < input.length)
    {
        /* Decode utf-16 surrogate pairs */
        x = input.charCodeAt(i);
        y = i + 1 < input.length ? input.charCodeAt(i + 1) : 0;
        if (0xD800 <= x && x <= 0xDBFF && 0xDC00 <= y && y <= 0xDFFF)
        {
            x = 0x10000 + ((x & 0x03FF) << 10) + (y & 0x03FF);
            i++;
        }

        /* Encode output as utf-8 */
        if (x <= 0x7F)
            output += String.fromCharCode(x);
        else if (x <= 0x7FF)
            output += String.fromCharCode(0xC0 | ((x >>> 6) & 0x1F),
                                    0x80 | (x & 0x3F));
        else if (x <= 0xFFFF)
            output += String.fromCharCode(0xE0 | ((x >>> 12) & 0x0F),
                                    0x80 | ((x >>> 6) & 0x3F),
                                    0x80 | (x & 0x3F));
        else if (x <= 0x1FFFFF)
            output += String.fromCharCode(0xF0 | ((x >>> 18) & 0x07),
                                    0x80 | ((x >>> 12) & 0x3F),
                                    0x80 | ((x >>> 6) & 0x3F),
                                    0x80 | (x & 0x3F));
    }
    return output;
}

/*
* Encode a string as utf-16
*/
function str2rstr_utf16le(input)
{
    var output = "";
    for (var i = 0; i < input.length; i++)
        output += String.fromCharCode(input.charCodeAt(i) & 0xFF,
                                  (input.charCodeAt(i) >>> 8) & 0xFF);
    return output;
}

function str2rstr_utf16be(input)
{
    var output = "";
    for (var i = 0; i < input.length; i++)
        output += String.fromCharCode((input.charCodeAt(i) >>> 8) & 0xFF,
                                   input.charCodeAt(i) & 0xFF);
    return output;
}

/*
* Convert a raw string to an array of big-endian words
* Characters >255 have their high-byte silently ignored.
*/
function rstr2binb(input)
{
    var output = Array(input.length >> 2);
    for (var i = 0; i < output.length; i++)
        output[i] = 0;
    for (var i = 0; i < input.length * 8; i += 8)
        output[i >> 5] |= (input.charCodeAt(i / 8) & 0xFF) << (24 - i % 32);
    return output;
}

/*
* Convert an array of big-endian words to a string
*/
function binb2rstr(input)
{
    var output = "";
    for (var i = 0; i < input.length * 32; i += 8)
        output += String.fromCharCode((input[i >> 5] >>> (24 - i % 32)) & 0xFF);
    return output;
}

/*
* Calculate the SHA-512 of an array of big-endian dwords, and a bit length
*/
var sha512_k;
function binb_sha512(x, len)
{
    if (sha512_k == undefined)
    {
        //SHA512 constants
        sha512_k = new Array(
new int64(0x428a2f98, -685199838), new int64(0x71374491, 0x23ef65cd),
new int64(-1245643825, -330482897), new int64(-373957723, -2121671748),
new int64(0x3956c25b, -213338824), new int64(0x59f111f1, -1241133031),
new int64(-1841331548, -1357295717), new int64(-1424204075, -630357736),
new int64(-670586216, -1560083902), new int64(0x12835b01, 0x45706fbe),
new int64(0x243185be, 0x4ee4b28c), new int64(0x550c7dc3, -704662302),
new int64(0x72be5d74, -226784913), new int64(-2132889090, 0x3b1696b1),
new int64(-1680079193, 0x25c71235), new int64(-1046744716, -815192428),
new int64(-459576895, -1628353838), new int64(-272742522, 0x384f25e3),
new int64(0xfc19dc6, -1953704523), new int64(0x240ca1cc, 0x77ac9c65),
new int64(0x2de92c6f, 0x592b0275), new int64(0x4a7484aa, 0x6ea6e483),
new int64(0x5cb0a9dc, -1119749164), new int64(0x76f988da, -2096016459),
new int64(-1740746414, -295247957), new int64(-1473132947, 0x2db43210),
new int64(-1341970488, -1728372417), new int64(-1084653625, -1091629340),
new int64(-958395405, 0x3da88fc2), new int64(-710438585, -1828018395),
new int64(0x6ca6351, -536640913), new int64(0x14292967, 0xa0e6e70),
new int64(0x27b70a85, 0x46d22ffc), new int64(0x2e1b2138, 0x5c26c926),
new int64(0x4d2c6dfc, 0x5ac42aed), new int64(0x53380d13, -1651133473),
new int64(0x650a7354, -1951439906), new int64(0x766a0abb, 0x3c77b2a8),
new int64(-2117940946, 0x47edaee6), new int64(-1838011259, 0x1482353b),
new int64(-1564481375, 0x4cf10364), new int64(-1474664885, -1136513023),
new int64(-1035236496, -789014639), new int64(-949202525, 0x654be30),
new int64(-778901479, -688958952), new int64(-694614492, 0x5565a910),
new int64(-200395387, 0x5771202a), new int64(0x106aa070, 0x32bbd1b8),
new int64(0x19a4c116, -1194143544), new int64(0x1e376c08, 0x5141ab53),
new int64(0x2748774c, -544281703), new int64(0x34b0bcb5, -509917016),
new int64(0x391c0cb3, -976659869), new int64(0x4ed8aa4a, -482243893),
new int64(0x5b9cca4f, 0x7763e373), new int64(0x682e6ff3, -692930397),
new int64(0x748f82ee, 0x5defb2fc), new int64(0x78a5636f, 0x43172f60),
new int64(-2067236844, -1578062990), new int64(-1933114872, 0x1a6439ec),
new int64(-1866530822, 0x23631e28), new int64(-1538233109, -561857047),
new int64(-1090935817, -1295615723), new int64(-965641998, -479046869),
new int64(-903397682, -366583396), new int64(-779700025, 0x21c0c207),
new int64(-354779690, -840897762), new int64(-176337025, -294727304),
new int64(0x6f067aa, 0x72176fba), new int64(0xa637dc5, -1563912026),
new int64(0x113f9804, -1090974290), new int64(0x1b710b35, 0x131c471b),
new int64(0x28db77f5, 0x23047d84), new int64(0x32caab7b, 0x40c72493),
new int64(0x3c9ebe0a, 0x15c9bebc), new int64(0x431d67c4, -1676669620),
new int64(0x4cc5d4be, -885112138), new int64(0x597f299c, -60457430),
new int64(0x5fcb6fab, 0x3ad6faec), new int64(0x6c44198c, 0x4a475817));
    }

    //Initial hash values
    var H = new Array(
new int64(0x6a09e667, -205731576),
new int64(-1150833019, -2067093701),
new int64(0x3c6ef372, -23791573),
new int64(-1521486534, 0x5f1d36f1),
new int64(0x510e527f, -1377402159),
new int64(-1694144372, 0x2b3e6c1f),
new int64(0x1f83d9ab, -79577749),
new int64(0x5be0cd19, 0x137e2179));

    var T1 = new int64(0, 0),
    T2 = new int64(0, 0),
    a = new int64(0, 0),
    b = new int64(0, 0),
    c = new int64(0, 0),
    d = new int64(0, 0),
    e = new int64(0, 0),
    f = new int64(0, 0),
    g = new int64(0, 0),
    h = new int64(0, 0),
    //Temporary variables not specified by the document
    s0 = new int64(0, 0),
    s1 = new int64(0, 0),
    Ch = new int64(0, 0),
    Maj = new int64(0, 0),
    r1 = new int64(0, 0),
    r2 = new int64(0, 0),
    r3 = new int64(0, 0);
    var j, i;
    var W = new Array(80);
    for (i = 0; i < 80; i++)
        W[i] = new int64(0, 0);

    // append padding to the source string. The format is described in the FIPS.
    x[len >> 5] |= 0x80 << (24 - (len & 0x1f));
    x[((len + 128 >> 10) << 5) + 31] = len;

    for (i = 0; i < x.length; i += 32) //32 dwords is the block size
    {
        int64copy(a, H[0]);
        int64copy(b, H[1]);
        int64copy(c, H[2]);
        int64copy(d, H[3]);
        int64copy(e, H[4]);
        int64copy(f, H[5]);
        int64copy(g, H[6]);
        int64copy(h, H[7]);

        for (j = 0; j < 16; j++)
        {
            W[j].h = x[i + 2 * j];
            W[j].l = x[i + 2 * j + 1];
        }

        for (j = 16; j < 80; j++)
        {
            //sigma1
            int64rrot(r1, W[j - 2], 19);
            int64revrrot(r2, W[j - 2], 29);
            int64shr(r3, W[j - 2], 6);
            s1.l = r1.l ^ r2.l ^ r3.l;
            s1.h = r1.h ^ r2.h ^ r3.h;
            //sigma0
            int64rrot(r1, W[j - 15], 1);
            int64rrot(r2, W[j - 15], 8);
            int64shr(r3, W[j - 15], 7);
            s0.l = r1.l ^ r2.l ^ r3.l;
            s0.h = r1.h ^ r2.h ^ r3.h;

            int64add4(W[j], s1, W[j - 7], s0, W[j - 16]);
        }

        for (j = 0; j < 80; j++)
        {
            //Ch
            Ch.l = (e.l & f.l) ^ (~e.l & g.l);
            Ch.h = (e.h & f.h) ^ (~e.h & g.h);

            //Sigma1
            int64rrot(r1, e, 14);
            int64rrot(r2, e, 18);
            int64revrrot(r3, e, 9);
            s1.l = r1.l ^ r2.l ^ r3.l;
            s1.h = r1.h ^ r2.h ^ r3.h;

            //Sigma0
            int64rrot(r1, a, 28);
            int64revrrot(r2, a, 2);
            int64revrrot(r3, a, 7);
            s0.l = r1.l ^ r2.l ^ r3.l;
            s0.h = r1.h ^ r2.h ^ r3.h;

            //Maj
            Maj.l = (a.l & b.l) ^ (a.l & c.l) ^ (b.l & c.l);
            Maj.h = (a.h & b.h) ^ (a.h & c.h) ^ (b.h & c.h);

            int64add5(T1, h, s1, Ch, sha512_k[j], W[j]);
            int64add(T2, s0, Maj);

            int64copy(h, g);
            int64copy(g, f);
            int64copy(f, e);
            int64add(e, d, T1);
            int64copy(d, c);
            int64copy(c, b);
            int64copy(b, a);
            int64add(a, T1, T2);
        }
        int64add(H[0], H[0], a);
        int64add(H[1], H[1], b);
        int64add(H[2], H[2], c);
        int64add(H[3], H[3], d);
        int64add(H[4], H[4], e);
        int64add(H[5], H[5], f);
        int64add(H[6], H[6], g);
        int64add(H[7], H[7], h);
    }

    //represent the hash as an array of 32-bit dwords
    var hash = new Array(16);
    for (i = 0; i < 8; i++)
    {
        hash[2 * i] = H[i].h;
        hash[2 * i + 1] = H[i].l;
    }
    return hash;
}

//A constructor for 64-bit numbers
function int64(h, l)
{
    this.h = h;
    this.l = l;
    //this.toString = int64toString;
}

//Copies src into dst, assuming both are 64-bit numbers
function int64copy(dst, src)
{
    dst.h = src.h;
    dst.l = src.l;
}

//Right-rotates a 64-bit number by shift
//Won't handle cases of shift>=32
//The function revrrot() is for that
function int64rrot(dst, x, shift)
{
    dst.l = (x.l >>> shift) | (x.h << (32 - shift));
    dst.h = (x.h >>> shift) | (x.l << (32 - shift));
}

//Reverses the dwords of the source and then rotates right by shift.
//This is equivalent to rotation by 32+shift
function int64revrrot(dst, x, shift)
{
    dst.l = (x.h >>> shift) | (x.l << (32 - shift));
    dst.h = (x.l >>> shift) | (x.h << (32 - shift));
}

//Bitwise-shifts right a 64-bit number by shift
//Won't handle shift>=32, but it's never needed in SHA512
function int64shr(dst, x, shift)
{
    dst.l = (x.l >>> shift) | (x.h << (32 - shift));
    dst.h = (x.h >>> shift);
}

//Adds two 64-bit numbers
//Like the original implementation, does not rely on 32-bit operations
function int64add(dst, x, y)
{
    var w0 = (x.l & 0xffff) + (y.l & 0xffff);
    var w1 = (x.l >>> 16) + (y.l >>> 16) + (w0 >>> 16);
    var w2 = (x.h & 0xffff) + (y.h & 0xffff) + (w1 >>> 16);
    var w3 = (x.h >>> 16) + (y.h >>> 16) + (w2 >>> 16);
    dst.l = (w0 & 0xffff) | (w1 << 16);
    dst.h = (w2 & 0xffff) | (w3 << 16);
}

//Same, except with 4 addends. Works faster than adding them one by one.
function int64add4(dst, a, b, c, d)
{
    var w0 = (a.l & 0xffff) + (b.l & 0xffff) + (c.l & 0xffff) + (d.l & 0xffff);
    var w1 = (a.l >>> 16) + (b.l >>> 16) + (c.l >>> 16) + (d.l >>> 16) + (w0 >>> 16);
    var w2 = (a.h & 0xffff) + (b.h & 0xffff) + (c.h & 0xffff) + (d.h & 0xffff) + (w1 >>> 16);
    var w3 = (a.h >>> 16) + (b.h >>> 16) + (c.h >>> 16) + (d.h >>> 16) + (w2 >>> 16);
    dst.l = (w0 & 0xffff) | (w1 << 16);
    dst.h = (w2 & 0xffff) | (w3 << 16);
}

//Same, except with 5 addends
function int64add5(dst, a, b, c, d, e)
{
    var w0 = (a.l & 0xffff) + (b.l & 0xffff) + (c.l & 0xffff) + (d.l & 0xffff) + (e.l & 0xffff);
    var w1 = (a.l >>> 16) + (b.l >>> 16) + (c.l >>> 16) + (d.l >>> 16) + (e.l >>> 16) + (w0 >>> 16);
    var w2 = (a.h & 0xffff) + (b.h & 0xffff) + (c.h & 0xffff) + (d.h & 0xffff) + (e.h & 0xffff) + (w1 >>> 16);
    var w3 = (a.h >>> 16) + (b.h >>> 16) + (c.h >>> 16) + (d.h >>> 16) + (e.h >>> 16) + (w2 >>> 16);
    dst.l = (w0 & 0xffff) | (w1 << 16);
    dst.h = (w2 & 0xffff) | (w3 << 16);
}
; 
 
 
 
/************************************************ 
**** gibberish-aes.js ***** 
************************************************/ 
/*! Gibberish-AES 
* A lightweight JavaScript Libray for OpenSSL compatible AES CBC encryption.
*
* Author: Mark Percival
* Email: mark@mpercival.com
* Copyright: Mark Percival - http://mpercival.com 2008
* Josh Davis - http://www.josh-davis.org/ecmaScrypt 2007
* Chris Veness - http://www.movable-type.co.uk/scripts/aes.html 2007
* Michel I. Gallant - http://www.jensign.com/
*
* License: MIT
* Usage: Gibberish.encrypt("secret", "password", 256)
* Outputs: AES Encrypted text encoded in Base64
*/


var GibberishAES = (function(){
    var Nr = 14,
    /* Default to 256 Bit Encryption */
    Nk = 8,
    Decrypt = false,

    enc_utf8 = function(s)
    {
        try {
            return unescape(encodeURIComponent(s));
        }
        catch(e) {
            throw 'Error on UTF-8 encode';
        }
    },

    dec_utf8 = function(s)
    {
        try {
            return decodeURIComponent(escape(s));
        }
        catch(e) {
            throw ('Bad Key');
        }
    },

    padBlock = function(byteArr)
    {
        var array = [], cpad, i;
        if (byteArr.length < 16) {
            cpad = 16 - byteArr.length;
            array = [cpad, cpad, cpad, cpad, cpad, cpad, cpad, cpad, cpad, cpad, cpad, cpad, cpad, cpad, cpad, cpad];
        }
        for (i = 0; i < byteArr.length; i++)
        {
            array[i] = byteArr[i];
        }
        return array;
    },

    block2s = function(block, lastBlock)
    {
        var string = '', padding, i;
        if (lastBlock) {
            padding = block[15];
            if (padding > 16) {
                throw ('Decryption error: Maybe bad key');
            }
            if (padding == 16) {
                return '';
            }
            for (i = 0; i < 16 - padding; i++) {
                string += String.fromCharCode(block[i]);
            }
        } else {
            for (i = 0; i < 16; i++) {
                string += String.fromCharCode(block[i]);
            }
        }
        return string;
    },

    a2h = function(numArr)
    {
        var string = '', i;
        for (i = 0; i < numArr.length; i++) {
            string += (numArr[i] < 16 ? '0': '') + numArr[i].toString(16);
        }
        return string;
    },

    h2a = function(s)
    {
        var ret = [];
        s.replace(/(..)/g,
        function(s) {
            ret.push(parseInt(s, 16));
        });
        return ret;
    },

    s2a = function(string) {
        string = enc_utf8(string);
        var array = [], i;
        for (i = 0; i < string.length; i++)
        {
            array[i] = string.charCodeAt(i);
        }
        return array;
    },

    size = function(newsize)
    {
        switch (newsize)
        {
        case 128:
            Nr = 10;
            Nk = 4;
            break;
        case 192:
            Nr = 12;
            Nk = 6;
            break;
        case 256:
            Nr = 14;
            Nk = 8;
            break;
        default:
            throw ('Invalid Key Size Specified:' + newsize);
        }
    },

    randArr = function(num) {
        var result = [], i;
        for (i = 0; i < num; i++) {
            result = result.concat(Math.floor(Math.random() * 256));
        }
        return result;
    },

    openSSLKey = function(passwordArr, saltArr) {
        // Number of rounds depends on the size of the AES in use
        // 3 rounds for 256
        //        2 rounds for the key, 1 for the IV
        // 2 rounds for 128
        //        1 round for the key, 1 round for the IV
        // 3 rounds for 192 since it's not evenly divided by 128 bits
        var rounds = Nr >= 12 ? 3: 2,
        key = [],
        iv = [],
        md5_hash = [],
        result = [],
        data00 = passwordArr.concat(saltArr),
        i;
        md5_hash[0] = GibberishAES.Hash.MD5(data00);
        result = md5_hash[0];
        for (i = 1; i < rounds; i++) {
            md5_hash[i] = GibberishAES.Hash.MD5(md5_hash[i - 1].concat(data00));
            result = result.concat(md5_hash[i]);
        }
        key = result.slice(0, 4 * Nk);
        iv = result.slice(4 * Nk, 4 * Nk + 16);
        return {
            key: key,
            iv: iv
        };
    },

    rawEncrypt = function(plaintext, key, iv) {
        // plaintext, key and iv as byte arrays
        key = expandKey(key);
        var numBlocks = Math.ceil(plaintext.length / 16),
        blocks = [],
        i,
        cipherBlocks = [];
        for (i = 0; i < numBlocks; i++) {
            blocks[i] = padBlock(plaintext.slice(i * 16, i * 16 + 16));
        }
        if (plaintext.length % 16 === 0) {
            blocks.push([16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16]);
            // CBC OpenSSL padding scheme
            numBlocks++;
        }
        for (i = 0; i < blocks.length; i++) {
            blocks[i] = (i === 0) ? xorBlocks(blocks[i], iv) : xorBlocks(blocks[i], cipherBlocks[i - 1]);
            cipherBlocks[i] = encryptBlock(blocks[i], key);
        }
        return cipherBlocks;
    },

    rawDecrypt = function(cryptArr, key, iv) {
        //  cryptArr, key and iv as byte arrays
        key = expandKey(key);
        var numBlocks = cryptArr.length / 16,
        cipherBlocks = [],
        i,
        plainBlocks = [],
        string = '';
        for (i = 0; i < numBlocks; i++) {
            cipherBlocks.push(cryptArr.slice(i * 16, (i + 1) * 16));
        }
        for (i = cipherBlocks.length - 1; i >= 0; i--) {
            plainBlocks[i] = decryptBlock(cipherBlocks[i], key);
            plainBlocks[i] = (i === 0) ? xorBlocks(plainBlocks[i], iv) : xorBlocks(plainBlocks[i], cipherBlocks[i - 1]);
        }
        for (i = 0; i < numBlocks - 1; i++) {
            string += block2s(plainBlocks[i]);
        }
        string += block2s(plainBlocks[i], true);
        return dec_utf8(string);
    },

    encryptBlock = function(block, words) {
        Decrypt = false;
        var state = addRoundKey(block, words, 0),
        round;
        for (round = 1; round < (Nr + 1); round++) {
            state = subBytes(state);
            state = shiftRows(state);
            if (round < Nr) {
                state = mixColumns(state);
            }
            //last round? don't mixColumns
            state = addRoundKey(state, words, round);
        }

        return state;
    },

    decryptBlock = function(block, words) {
        Decrypt = true;
        var state = addRoundKey(block, words, Nr),
        round;
        for (round = Nr - 1; round > -1; round--) {
            state = shiftRows(state);
            state = subBytes(state);
            state = addRoundKey(state, words, round);
            if (round > 0) {
                state = mixColumns(state);
            }
            //last round? don't mixColumns
        }

        return state;
    },

    subBytes = function(state) {
        var S = Decrypt ? SBoxInv: SBox,
        temp = [],
        i;
        for (i = 0; i < 16; i++) {
            temp[i] = S[state[i]];
        }
        return temp;
    },

    shiftRows = function(state) {
        var temp = [],
        shiftBy = Decrypt ? [0, 13, 10, 7, 4, 1, 14, 11, 8, 5, 2, 15, 12, 9, 6, 3] : [0, 5, 10, 15, 4, 9, 14, 3, 8, 13, 2, 7, 12, 1, 6, 11],
        i;
        for (i = 0; i < 16; i++) {
            temp[i] = state[shiftBy[i]];
        }
        return temp;
    },

    mixColumns = function(state) {
        var t = [],
        c;
        if (!Decrypt) {
            for (c = 0; c < 4; c++) {
                t[c * 4] = G2X[state[c * 4]] ^ G3X[state[1 + c * 4]] ^ state[2 + c * 4] ^ state[3 + c * 4];
                t[1 + c * 4] = state[c * 4] ^ G2X[state[1 + c * 4]] ^ G3X[state[2 + c * 4]] ^ state[3 + c * 4];
                t[2 + c * 4] = state[c * 4] ^ state[1 + c * 4] ^ G2X[state[2 + c * 4]] ^ G3X[state[3 + c * 4]];
                t[3 + c * 4] = G3X[state[c * 4]] ^ state[1 + c * 4] ^ state[2 + c * 4] ^ G2X[state[3 + c * 4]];
            }
        }else {
            for (c = 0; c < 4; c++) {
                t[c*4] = GEX[state[c*4]] ^ GBX[state[1+c*4]] ^ GDX[state[2+c*4]] ^ G9X[state[3+c*4]];
                t[1+c*4] = G9X[state[c*4]] ^ GEX[state[1+c*4]] ^ GBX[state[2+c*4]] ^ GDX[state[3+c*4]];
                t[2+c*4] = GDX[state[c*4]] ^ G9X[state[1+c*4]] ^ GEX[state[2+c*4]] ^ GBX[state[3+c*4]];
                t[3+c*4] = GBX[state[c*4]] ^ GDX[state[1+c*4]] ^ G9X[state[2+c*4]] ^ GEX[state[3+c*4]];
            }
        }
        
        return t;
    },

    addRoundKey = function(state, words, round) {
        var temp = [],
        i;
        for (i = 0; i < 16; i++) {
            temp[i] = state[i] ^ words[round][i];
        }
        return temp;
    },

    xorBlocks = function(block1, block2) {
        var temp = [],
        i;
        for (i = 0; i < 16; i++) {
            temp[i] = block1[i] ^ block2[i];
        }
        return temp;
    },

    expandKey = function(key) {
        // Expects a 1d number array
        var w = [],
        temp = [],
        i,
        r,
        t,
        flat = [],
        j;

        for (i = 0; i < Nk; i++) {
            r = [key[4 * i], key[4 * i + 1], key[4 * i + 2], key[4 * i + 3]];
            w[i] = r;
        }

        for (i = Nk; i < (4 * (Nr + 1)); i++) {
            w[i] = [];
            for (t = 0; t < 4; t++) {
                temp[t] = w[i - 1][t];
            }
            if (i % Nk === 0) {
                temp = subWord(rotWord(temp));
                temp[0] ^= Rcon[i / Nk - 1];
            } else if (Nk > 6 && i % Nk == 4) {
                temp = subWord(temp);
            }
            for (t = 0; t < 4; t++) {
                w[i][t] = w[i - Nk][t] ^ temp[t];
            }
        }
        for (i = 0; i < (Nr + 1); i++) {
            flat[i] = [];
            for (j = 0; j < 4; j++) {
                flat[i].push(w[i * 4 + j][0], w[i * 4 + j][1], w[i * 4 + j][2], w[i * 4 + j][3]);
            }
        }
        return flat;
    },

    subWord = function(w) {
        // apply SBox to 4-byte word w
        for (var i = 0; i < 4; i++) {
            w[i] = SBox[w[i]];
        }
        return w;
    },

    rotWord = function(w) {
        // rotate 4-byte word w left by one byte
        var tmp = w[0],
        i;
        for (i = 0; i < 4; i++) {
            w[i] = w[i + 1];
        }
        w[3] = tmp;
        return w;
    },


    // S-box
    SBox = [
    99, 124, 119, 123, 242, 107, 111, 197, 48, 1, 103, 43, 254, 215, 171,
    118, 202, 130, 201, 125, 250, 89, 71, 240, 173, 212, 162, 175, 156, 164,
    114, 192, 183, 253, 147, 38, 54, 63, 247, 204, 52, 165, 229, 241, 113,
    216, 49, 21, 4, 199, 35, 195, 24, 150, 5, 154, 7, 18, 128, 226,
    235, 39, 178, 117, 9, 131, 44, 26, 27, 110, 90, 160, 82, 59, 214,
    179, 41, 227, 47, 132, 83, 209, 0, 237, 32, 252, 177, 91, 106, 203,
    190, 57, 74, 76, 88, 207, 208, 239, 170, 251, 67, 77, 51, 133, 69,
    249, 2, 127, 80, 60, 159, 168, 81, 163, 64, 143, 146, 157, 56, 245,
    188, 182, 218, 33, 16, 255, 243, 210, 205, 12, 19, 236, 95, 151, 68,
    23, 196, 167, 126, 61, 100, 93, 25, 115, 96, 129, 79, 220, 34, 42,
    144, 136, 70, 238, 184, 20, 222, 94, 11, 219, 224, 50, 58, 10, 73,
    6, 36, 92, 194, 211, 172, 98, 145, 149, 228, 121, 231, 200, 55, 109,
    141, 213, 78, 169, 108, 86, 244, 234, 101, 122, 174, 8, 186, 120, 37,
    46, 28, 166, 180, 198, 232, 221, 116, 31, 75, 189, 139, 138, 112, 62,
    181, 102, 72, 3, 246, 14, 97, 53, 87, 185, 134, 193, 29, 158, 225,
    248, 152, 17, 105, 217, 142, 148, 155, 30, 135, 233, 206, 85, 40, 223,
    140, 161, 137, 13, 191, 230, 66, 104, 65, 153, 45, 15, 176, 84, 187,
    22],

    // Precomputed lookup table for the inverse SBox
    SBoxInv = [
    82, 9, 106, 213, 48, 54, 165, 56, 191, 64, 163, 158, 129, 243, 215,
    251, 124, 227, 57, 130, 155, 47, 255, 135, 52, 142, 67, 68, 196, 222,
    233, 203, 84, 123, 148, 50, 166, 194, 35, 61, 238, 76, 149, 11, 66,
    250, 195, 78, 8, 46, 161, 102, 40, 217, 36, 178, 118, 91, 162, 73,
    109, 139, 209, 37, 114, 248, 246, 100, 134, 104, 152, 22, 212, 164, 92,
    204, 93, 101, 182, 146, 108, 112, 72, 80, 253, 237, 185, 218, 94, 21,
    70, 87, 167, 141, 157, 132, 144, 216, 171, 0, 140, 188, 211, 10, 247,
    228, 88, 5, 184, 179, 69, 6, 208, 44, 30, 143, 202, 63, 15, 2,
    193, 175, 189, 3, 1, 19, 138, 107, 58, 145, 17, 65, 79, 103, 220,
    234, 151, 242, 207, 206, 240, 180, 230, 115, 150, 172, 116, 34, 231, 173,
    53, 133, 226, 249, 55, 232, 28, 117, 223, 110, 71, 241, 26, 113, 29,
    41, 197, 137, 111, 183, 98, 14, 170, 24, 190, 27, 252, 86, 62, 75,
    198, 210, 121, 32, 154, 219, 192, 254, 120, 205, 90, 244, 31, 221, 168,
    51, 136, 7, 199, 49, 177, 18, 16, 89, 39, 128, 236, 95, 96, 81,
    127, 169, 25, 181, 74, 13, 45, 229, 122, 159, 147, 201, 156, 239, 160,
    224, 59, 77, 174, 42, 245, 176, 200, 235, 187, 60, 131, 83, 153, 97,
    23, 43, 4, 126, 186, 119, 214, 38, 225, 105, 20, 99, 85, 33, 12,
    125],
    // Rijndael Rcon
    Rcon = [1, 2, 4, 8, 16, 32, 64, 128, 27, 54, 108, 216, 171, 77, 154, 47, 94,
    188, 99, 198, 151, 53, 106, 212, 179, 125, 250, 239, 197, 145],

    G2X = [
    0x00, 0x02, 0x04, 0x06, 0x08, 0x0a, 0x0c, 0x0e, 0x10, 0x12, 0x14, 0x16,
    0x18, 0x1a, 0x1c, 0x1e, 0x20, 0x22, 0x24, 0x26, 0x28, 0x2a, 0x2c, 0x2e,
    0x30, 0x32, 0x34, 0x36, 0x38, 0x3a, 0x3c, 0x3e, 0x40, 0x42, 0x44, 0x46,
    0x48, 0x4a, 0x4c, 0x4e, 0x50, 0x52, 0x54, 0x56, 0x58, 0x5a, 0x5c, 0x5e,
    0x60, 0x62, 0x64, 0x66, 0x68, 0x6a, 0x6c, 0x6e, 0x70, 0x72, 0x74, 0x76,
    0x78, 0x7a, 0x7c, 0x7e, 0x80, 0x82, 0x84, 0x86, 0x88, 0x8a, 0x8c, 0x8e,
    0x90, 0x92, 0x94, 0x96, 0x98, 0x9a, 0x9c, 0x9e, 0xa0, 0xa2, 0xa4, 0xa6,
    0xa8, 0xaa, 0xac, 0xae, 0xb0, 0xb2, 0xb4, 0xb6, 0xb8, 0xba, 0xbc, 0xbe,
    0xc0, 0xc2, 0xc4, 0xc6, 0xc8, 0xca, 0xcc, 0xce, 0xd0, 0xd2, 0xd4, 0xd6,
    0xd8, 0xda, 0xdc, 0xde, 0xe0, 0xe2, 0xe4, 0xe6, 0xe8, 0xea, 0xec, 0xee,
    0xf0, 0xf2, 0xf4, 0xf6, 0xf8, 0xfa, 0xfc, 0xfe, 0x1b, 0x19, 0x1f, 0x1d,
    0x13, 0x11, 0x17, 0x15, 0x0b, 0x09, 0x0f, 0x0d, 0x03, 0x01, 0x07, 0x05,
    0x3b, 0x39, 0x3f, 0x3d, 0x33, 0x31, 0x37, 0x35, 0x2b, 0x29, 0x2f, 0x2d,
    0x23, 0x21, 0x27, 0x25, 0x5b, 0x59, 0x5f, 0x5d, 0x53, 0x51, 0x57, 0x55,
    0x4b, 0x49, 0x4f, 0x4d, 0x43, 0x41, 0x47, 0x45, 0x7b, 0x79, 0x7f, 0x7d,
    0x73, 0x71, 0x77, 0x75, 0x6b, 0x69, 0x6f, 0x6d, 0x63, 0x61, 0x67, 0x65,
    0x9b, 0x99, 0x9f, 0x9d, 0x93, 0x91, 0x97, 0x95, 0x8b, 0x89, 0x8f, 0x8d,
    0x83, 0x81, 0x87, 0x85, 0xbb, 0xb9, 0xbf, 0xbd, 0xb3, 0xb1, 0xb7, 0xb5,
    0xab, 0xa9, 0xaf, 0xad, 0xa3, 0xa1, 0xa7, 0xa5, 0xdb, 0xd9, 0xdf, 0xdd,
    0xd3, 0xd1, 0xd7, 0xd5, 0xcb, 0xc9, 0xcf, 0xcd, 0xc3, 0xc1, 0xc7, 0xc5,
    0xfb, 0xf9, 0xff, 0xfd, 0xf3, 0xf1, 0xf7, 0xf5, 0xeb, 0xe9, 0xef, 0xed,
    0xe3, 0xe1, 0xe7, 0xe5
    ],

    G3X = [
    0x00, 0x03, 0x06, 0x05, 0x0c, 0x0f, 0x0a, 0x09, 0x18, 0x1b, 0x1e, 0x1d,
    0x14, 0x17, 0x12, 0x11, 0x30, 0x33, 0x36, 0x35, 0x3c, 0x3f, 0x3a, 0x39,
    0x28, 0x2b, 0x2e, 0x2d, 0x24, 0x27, 0x22, 0x21, 0x60, 0x63, 0x66, 0x65,
    0x6c, 0x6f, 0x6a, 0x69, 0x78, 0x7b, 0x7e, 0x7d, 0x74, 0x77, 0x72, 0x71,
    0x50, 0x53, 0x56, 0x55, 0x5c, 0x5f, 0x5a, 0x59, 0x48, 0x4b, 0x4e, 0x4d,
    0x44, 0x47, 0x42, 0x41, 0xc0, 0xc3, 0xc6, 0xc5, 0xcc, 0xcf, 0xca, 0xc9,
    0xd8, 0xdb, 0xde, 0xdd, 0xd4, 0xd7, 0xd2, 0xd1, 0xf0, 0xf3, 0xf6, 0xf5,
    0xfc, 0xff, 0xfa, 0xf9, 0xe8, 0xeb, 0xee, 0xed, 0xe4, 0xe7, 0xe2, 0xe1,
    0xa0, 0xa3, 0xa6, 0xa5, 0xac, 0xaf, 0xaa, 0xa9, 0xb8, 0xbb, 0xbe, 0xbd,
    0xb4, 0xb7, 0xb2, 0xb1, 0x90, 0x93, 0x96, 0x95, 0x9c, 0x9f, 0x9a, 0x99,
    0x88, 0x8b, 0x8e, 0x8d, 0x84, 0x87, 0x82, 0x81, 0x9b, 0x98, 0x9d, 0x9e,
    0x97, 0x94, 0x91, 0x92, 0x83, 0x80, 0x85, 0x86, 0x8f, 0x8c, 0x89, 0x8a,
    0xab, 0xa8, 0xad, 0xae, 0xa7, 0xa4, 0xa1, 0xa2, 0xb3, 0xb0, 0xb5, 0xb6,
    0xbf, 0xbc, 0xb9, 0xba, 0xfb, 0xf8, 0xfd, 0xfe, 0xf7, 0xf4, 0xf1, 0xf2,
    0xe3, 0xe0, 0xe5, 0xe6, 0xef, 0xec, 0xe9, 0xea, 0xcb, 0xc8, 0xcd, 0xce,
    0xc7, 0xc4, 0xc1, 0xc2, 0xd3, 0xd0, 0xd5, 0xd6, 0xdf, 0xdc, 0xd9, 0xda,
    0x5b, 0x58, 0x5d, 0x5e, 0x57, 0x54, 0x51, 0x52, 0x43, 0x40, 0x45, 0x46,
    0x4f, 0x4c, 0x49, 0x4a, 0x6b, 0x68, 0x6d, 0x6e, 0x67, 0x64, 0x61, 0x62,
    0x73, 0x70, 0x75, 0x76, 0x7f, 0x7c, 0x79, 0x7a, 0x3b, 0x38, 0x3d, 0x3e,
    0x37, 0x34, 0x31, 0x32, 0x23, 0x20, 0x25, 0x26, 0x2f, 0x2c, 0x29, 0x2a,
    0x0b, 0x08, 0x0d, 0x0e, 0x07, 0x04, 0x01, 0x02, 0x13, 0x10, 0x15, 0x16,
    0x1f, 0x1c, 0x19, 0x1a
    ],

    G9X = [
    0x00, 0x09, 0x12, 0x1b, 0x24, 0x2d, 0x36, 0x3f, 0x48, 0x41, 0x5a, 0x53,
    0x6c, 0x65, 0x7e, 0x77, 0x90, 0x99, 0x82, 0x8b, 0xb4, 0xbd, 0xa6, 0xaf,
    0xd8, 0xd1, 0xca, 0xc3, 0xfc, 0xf5, 0xee, 0xe7, 0x3b, 0x32, 0x29, 0x20,
    0x1f, 0x16, 0x0d, 0x04, 0x73, 0x7a, 0x61, 0x68, 0x57, 0x5e, 0x45, 0x4c,
    0xab, 0xa2, 0xb9, 0xb0, 0x8f, 0x86, 0x9d, 0x94, 0xe3, 0xea, 0xf1, 0xf8,
    0xc7, 0xce, 0xd5, 0xdc, 0x76, 0x7f, 0x64, 0x6d, 0x52, 0x5b, 0x40, 0x49,
    0x3e, 0x37, 0x2c, 0x25, 0x1a, 0x13, 0x08, 0x01, 0xe6, 0xef, 0xf4, 0xfd,
    0xc2, 0xcb, 0xd0, 0xd9, 0xae, 0xa7, 0xbc, 0xb5, 0x8a, 0x83, 0x98, 0x91,
    0x4d, 0x44, 0x5f, 0x56, 0x69, 0x60, 0x7b, 0x72, 0x05, 0x0c, 0x17, 0x1e,
    0x21, 0x28, 0x33, 0x3a, 0xdd, 0xd4, 0xcf, 0xc6, 0xf9, 0xf0, 0xeb, 0xe2,
    0x95, 0x9c, 0x87, 0x8e, 0xb1, 0xb8, 0xa3, 0xaa, 0xec, 0xe5, 0xfe, 0xf7,
    0xc8, 0xc1, 0xda, 0xd3, 0xa4, 0xad, 0xb6, 0xbf, 0x80, 0x89, 0x92, 0x9b,
    0x7c, 0x75, 0x6e, 0x67, 0x58, 0x51, 0x4a, 0x43, 0x34, 0x3d, 0x26, 0x2f,
    0x10, 0x19, 0x02, 0x0b, 0xd7, 0xde, 0xc5, 0xcc, 0xf3, 0xfa, 0xe1, 0xe8,
    0x9f, 0x96, 0x8d, 0x84, 0xbb, 0xb2, 0xa9, 0xa0, 0x47, 0x4e, 0x55, 0x5c,
    0x63, 0x6a, 0x71, 0x78, 0x0f, 0x06, 0x1d, 0x14, 0x2b, 0x22, 0x39, 0x30,
    0x9a, 0x93, 0x88, 0x81, 0xbe, 0xb7, 0xac, 0xa5, 0xd2, 0xdb, 0xc0, 0xc9,
    0xf6, 0xff, 0xe4, 0xed, 0x0a, 0x03, 0x18, 0x11, 0x2e, 0x27, 0x3c, 0x35,
    0x42, 0x4b, 0x50, 0x59, 0x66, 0x6f, 0x74, 0x7d, 0xa1, 0xa8, 0xb3, 0xba,
    0x85, 0x8c, 0x97, 0x9e, 0xe9, 0xe0, 0xfb, 0xf2, 0xcd, 0xc4, 0xdf, 0xd6,
    0x31, 0x38, 0x23, 0x2a, 0x15, 0x1c, 0x07, 0x0e, 0x79, 0x70, 0x6b, 0x62,
    0x5d, 0x54, 0x4f, 0x46
    ],

    GBX = [
    0x00, 0x0b, 0x16, 0x1d, 0x2c, 0x27, 0x3a, 0x31, 0x58, 0x53, 0x4e, 0x45,
    0x74, 0x7f, 0x62, 0x69, 0xb0, 0xbb, 0xa6, 0xad, 0x9c, 0x97, 0x8a, 0x81,
    0xe8, 0xe3, 0xfe, 0xf5, 0xc4, 0xcf, 0xd2, 0xd9, 0x7b, 0x70, 0x6d, 0x66,
    0x57, 0x5c, 0x41, 0x4a, 0x23, 0x28, 0x35, 0x3e, 0x0f, 0x04, 0x19, 0x12,
    0xcb, 0xc0, 0xdd, 0xd6, 0xe7, 0xec, 0xf1, 0xfa, 0x93, 0x98, 0x85, 0x8e,
    0xbf, 0xb4, 0xa9, 0xa2, 0xf6, 0xfd, 0xe0, 0xeb, 0xda, 0xd1, 0xcc, 0xc7,
    0xae, 0xa5, 0xb8, 0xb3, 0x82, 0x89, 0x94, 0x9f, 0x46, 0x4d, 0x50, 0x5b,
    0x6a, 0x61, 0x7c, 0x77, 0x1e, 0x15, 0x08, 0x03, 0x32, 0x39, 0x24, 0x2f,
    0x8d, 0x86, 0x9b, 0x90, 0xa1, 0xaa, 0xb7, 0xbc, 0xd5, 0xde, 0xc3, 0xc8,
    0xf9, 0xf2, 0xef, 0xe4, 0x3d, 0x36, 0x2b, 0x20, 0x11, 0x1a, 0x07, 0x0c,
    0x65, 0x6e, 0x73, 0x78, 0x49, 0x42, 0x5f, 0x54, 0xf7, 0xfc, 0xe1, 0xea,
    0xdb, 0xd0, 0xcd, 0xc6, 0xaf, 0xa4, 0xb9, 0xb2, 0x83, 0x88, 0x95, 0x9e,
    0x47, 0x4c, 0x51, 0x5a, 0x6b, 0x60, 0x7d, 0x76, 0x1f, 0x14, 0x09, 0x02,
    0x33, 0x38, 0x25, 0x2e, 0x8c, 0x87, 0x9a, 0x91, 0xa0, 0xab, 0xb6, 0xbd,
    0xd4, 0xdf, 0xc2, 0xc9, 0xf8, 0xf3, 0xee, 0xe5, 0x3c, 0x37, 0x2a, 0x21,
    0x10, 0x1b, 0x06, 0x0d, 0x64, 0x6f, 0x72, 0x79, 0x48, 0x43, 0x5e, 0x55,
    0x01, 0x0a, 0x17, 0x1c, 0x2d, 0x26, 0x3b, 0x30, 0x59, 0x52, 0x4f, 0x44,
    0x75, 0x7e, 0x63, 0x68, 0xb1, 0xba, 0xa7, 0xac, 0x9d, 0x96, 0x8b, 0x80,
    0xe9, 0xe2, 0xff, 0xf4, 0xc5, 0xce, 0xd3, 0xd8, 0x7a, 0x71, 0x6c, 0x67,
    0x56, 0x5d, 0x40, 0x4b, 0x22, 0x29, 0x34, 0x3f, 0x0e, 0x05, 0x18, 0x13,
    0xca, 0xc1, 0xdc, 0xd7, 0xe6, 0xed, 0xf0, 0xfb, 0x92, 0x99, 0x84, 0x8f,
    0xbe, 0xb5, 0xa8, 0xa3
    ],

    GDX = [
    0x00, 0x0d, 0x1a, 0x17, 0x34, 0x39, 0x2e, 0x23, 0x68, 0x65, 0x72, 0x7f,
    0x5c, 0x51, 0x46, 0x4b, 0xd0, 0xdd, 0xca, 0xc7, 0xe4, 0xe9, 0xfe, 0xf3,
    0xb8, 0xb5, 0xa2, 0xaf, 0x8c, 0x81, 0x96, 0x9b, 0xbb, 0xb6, 0xa1, 0xac,
    0x8f, 0x82, 0x95, 0x98, 0xd3, 0xde, 0xc9, 0xc4, 0xe7, 0xea, 0xfd, 0xf0,
    0x6b, 0x66, 0x71, 0x7c, 0x5f, 0x52, 0x45, 0x48, 0x03, 0x0e, 0x19, 0x14,
    0x37, 0x3a, 0x2d, 0x20, 0x6d, 0x60, 0x77, 0x7a, 0x59, 0x54, 0x43, 0x4e,
    0x05, 0x08, 0x1f, 0x12, 0x31, 0x3c, 0x2b, 0x26, 0xbd, 0xb0, 0xa7, 0xaa,
    0x89, 0x84, 0x93, 0x9e, 0xd5, 0xd8, 0xcf, 0xc2, 0xe1, 0xec, 0xfb, 0xf6,
    0xd6, 0xdb, 0xcc, 0xc1, 0xe2, 0xef, 0xf8, 0xf5, 0xbe, 0xb3, 0xa4, 0xa9,
    0x8a, 0x87, 0x90, 0x9d, 0x06, 0x0b, 0x1c, 0x11, 0x32, 0x3f, 0x28, 0x25,
    0x6e, 0x63, 0x74, 0x79, 0x5a, 0x57, 0x40, 0x4d, 0xda, 0xd7, 0xc0, 0xcd,
    0xee, 0xe3, 0xf4, 0xf9, 0xb2, 0xbf, 0xa8, 0xa5, 0x86, 0x8b, 0x9c, 0x91,
    0x0a, 0x07, 0x10, 0x1d, 0x3e, 0x33, 0x24, 0x29, 0x62, 0x6f, 0x78, 0x75,
    0x56, 0x5b, 0x4c, 0x41, 0x61, 0x6c, 0x7b, 0x76, 0x55, 0x58, 0x4f, 0x42,
    0x09, 0x04, 0x13, 0x1e, 0x3d, 0x30, 0x27, 0x2a, 0xb1, 0xbc, 0xab, 0xa6,
    0x85, 0x88, 0x9f, 0x92, 0xd9, 0xd4, 0xc3, 0xce, 0xed, 0xe0, 0xf7, 0xfa,
    0xb7, 0xba, 0xad, 0xa0, 0x83, 0x8e, 0x99, 0x94, 0xdf, 0xd2, 0xc5, 0xc8,
    0xeb, 0xe6, 0xf1, 0xfc, 0x67, 0x6a, 0x7d, 0x70, 0x53, 0x5e, 0x49, 0x44,
    0x0f, 0x02, 0x15, 0x18, 0x3b, 0x36, 0x21, 0x2c, 0x0c, 0x01, 0x16, 0x1b,
    0x38, 0x35, 0x22, 0x2f, 0x64, 0x69, 0x7e, 0x73, 0x50, 0x5d, 0x4a, 0x47,
    0xdc, 0xd1, 0xc6, 0xcb, 0xe8, 0xe5, 0xf2, 0xff, 0xb4, 0xb9, 0xae, 0xa3,
    0x80, 0x8d, 0x9a, 0x97
    ],

    GEX = [
    0x00, 0x0e, 0x1c, 0x12, 0x38, 0x36, 0x24, 0x2a, 0x70, 0x7e, 0x6c, 0x62,
    0x48, 0x46, 0x54, 0x5a, 0xe0, 0xee, 0xfc, 0xf2, 0xd8, 0xd6, 0xc4, 0xca,
    0x90, 0x9e, 0x8c, 0x82, 0xa8, 0xa6, 0xb4, 0xba, 0xdb, 0xd5, 0xc7, 0xc9,
    0xe3, 0xed, 0xff, 0xf1, 0xab, 0xa5, 0xb7, 0xb9, 0x93, 0x9d, 0x8f, 0x81,
    0x3b, 0x35, 0x27, 0x29, 0x03, 0x0d, 0x1f, 0x11, 0x4b, 0x45, 0x57, 0x59,
    0x73, 0x7d, 0x6f, 0x61, 0xad, 0xa3, 0xb1, 0xbf, 0x95, 0x9b, 0x89, 0x87,
    0xdd, 0xd3, 0xc1, 0xcf, 0xe5, 0xeb, 0xf9, 0xf7, 0x4d, 0x43, 0x51, 0x5f,
    0x75, 0x7b, 0x69, 0x67, 0x3d, 0x33, 0x21, 0x2f, 0x05, 0x0b, 0x19, 0x17,
    0x76, 0x78, 0x6a, 0x64, 0x4e, 0x40, 0x52, 0x5c, 0x06, 0x08, 0x1a, 0x14,
    0x3e, 0x30, 0x22, 0x2c, 0x96, 0x98, 0x8a, 0x84, 0xae, 0xa0, 0xb2, 0xbc,
    0xe6, 0xe8, 0xfa, 0xf4, 0xde, 0xd0, 0xc2, 0xcc, 0x41, 0x4f, 0x5d, 0x53,
    0x79, 0x77, 0x65, 0x6b, 0x31, 0x3f, 0x2d, 0x23, 0x09, 0x07, 0x15, 0x1b,
    0xa1, 0xaf, 0xbd, 0xb3, 0x99, 0x97, 0x85, 0x8b, 0xd1, 0xdf, 0xcd, 0xc3,
    0xe9, 0xe7, 0xf5, 0xfb, 0x9a, 0x94, 0x86, 0x88, 0xa2, 0xac, 0xbe, 0xb0,
    0xea, 0xe4, 0xf6, 0xf8, 0xd2, 0xdc, 0xce, 0xc0, 0x7a, 0x74, 0x66, 0x68,
    0x42, 0x4c, 0x5e, 0x50, 0x0a, 0x04, 0x16, 0x18, 0x32, 0x3c, 0x2e, 0x20,
    0xec, 0xe2, 0xf0, 0xfe, 0xd4, 0xda, 0xc8, 0xc6, 0x9c, 0x92, 0x80, 0x8e,
    0xa4, 0xaa, 0xb8, 0xb6, 0x0c, 0x02, 0x10, 0x1e, 0x34, 0x3a, 0x28, 0x26,
    0x7c, 0x72, 0x60, 0x6e, 0x44, 0x4a, 0x58, 0x56, 0x37, 0x39, 0x2b, 0x25,
    0x0f, 0x01, 0x13, 0x1d, 0x47, 0x49, 0x5b, 0x55, 0x7f, 0x71, 0x63, 0x6d,
    0xd7, 0xd9, 0xcb, 0xc5, 0xef, 0xe1, 0xf3, 0xfd, 0xa7, 0xa9, 0xbb, 0xb5,
    0x9f, 0x91, 0x83, 0x8d
    ],

    enc = function(string, pass) {
        // string, password in plaintext
        var salt = randArr(8),
        pbe = openSSLKey(s2a(pass), salt),
        key = pbe.key,
        iv = pbe.iv,
        cipherBlocks,
        saltBlock = [[83, 97, 108, 116, 101, 100, 95, 95].concat(salt)];
        string = s2a(string);
        cipherBlocks = rawEncrypt(string, key, iv);
        // Spells out 'Salted__'
        cipherBlocks = saltBlock.concat(cipherBlocks);
        return Base64.encode(cipherBlocks);
    },

    encRaw = function(string, pass)
    {
        // string in plaintext, password is raw
        var salt = randArr(8),
        pbe = openSSLKey(pass, salt),
        key = pbe.key,
        iv = pbe.iv,
        cipherBlocks,
        saltBlock = [[83, 97, 108, 116, 101, 100, 95, 95].concat(salt)];
        string = s2a(string);
        cipherBlocks = rawEncrypt(string, key, iv);
        // Spells out 'Salted__'
        cipherBlocks = saltBlock.concat(cipherBlocks);
        return Base64.encode(cipherBlocks);
    },

    dec = function(string, pass) {
        // string, password in plaintext
        var cryptArr = Base64.decode(string),
        salt = cryptArr.slice(8, 16),
        pbe = openSSLKey(s2a(pass), salt),
        key = pbe.key,
        iv = pbe.iv;
        cryptArr = cryptArr.slice(16, cryptArr.length);
        // Take off the Salted__ffeeddcc
        string = rawDecrypt(cryptArr, key, iv);
        return string;
    },

    decRaw = function(string, pass)
    {
        // string in plaintext, password is raw
        var cryptArr = Base64.decode(string),
        salt = cryptArr.slice(8, 16),
        pbe = openSSLKey(pass, salt),
        key = pbe.key,
        iv = pbe.iv;
        cryptArr = cryptArr.slice(16, cryptArr.length);
        // Take off the Salted__ffeeddcc
        string = rawDecrypt(cryptArr, key, iv);
        return string;
    },
    
    MD5 = function(numArr) {

        function rotateLeft(lValue, iShiftBits) {
            return (lValue << iShiftBits) | (lValue >>> (32 - iShiftBits));
        }

        function addUnsigned(lX, lY) {
            var lX4,
            lY4,
            lX8,
            lY8,
            lResult;
            lX8 = (lX & 0x80000000);
            lY8 = (lY & 0x80000000);
            lX4 = (lX & 0x40000000);
            lY4 = (lY & 0x40000000);
            lResult = (lX & 0x3FFFFFFF) + (lY & 0x3FFFFFFF);
            if (lX4 & lY4) {
                return (lResult ^ 0x80000000 ^ lX8 ^ lY8);
            }
            if (lX4 | lY4) {
                if (lResult & 0x40000000) {
                    return (lResult ^ 0xC0000000 ^ lX8 ^ lY8);
                } else {
                    return (lResult ^ 0x40000000 ^ lX8 ^ lY8);
                }
            } else {
                return (lResult ^ lX8 ^ lY8);
            }
        }

        function f(x, y, z) {
            return (x & y) | ((~x) & z);
        }
        function g(x, y, z) {
            return (x & z) | (y & (~z));
        }
        function h(x, y, z) {
            return (x ^ y ^ z);
        }
        function funcI(x, y, z) {
            return (y ^ (x | (~z)));
        }

        function ff(a, b, c, d, x, s, ac) {
            a = addUnsigned(a, addUnsigned(addUnsigned(f(b, c, d), x), ac));
            return addUnsigned(rotateLeft(a, s), b);
        }

        function gg(a, b, c, d, x, s, ac) {
            a = addUnsigned(a, addUnsigned(addUnsigned(g(b, c, d), x), ac));
            return addUnsigned(rotateLeft(a, s), b);
        }

        function hh(a, b, c, d, x, s, ac) {
            a = addUnsigned(a, addUnsigned(addUnsigned(h(b, c, d), x), ac));
            return addUnsigned(rotateLeft(a, s), b);
        }

        function ii(a, b, c, d, x, s, ac) {
            a = addUnsigned(a, addUnsigned(addUnsigned(funcI(b, c, d), x), ac));
            return addUnsigned(rotateLeft(a, s), b);
        }

        function convertToWordArray(numArr) {
            var lWordCount,
            lMessageLength = numArr.length,
            lNumberOfWords_temp1 = lMessageLength + 8,
            lNumberOfWords_temp2 = (lNumberOfWords_temp1 - (lNumberOfWords_temp1 % 64)) / 64,
            lNumberOfWords = (lNumberOfWords_temp2 + 1) * 16,
            lWordArray = [],
            lBytePosition = 0,
            lByteCount = 0;
            while (lByteCount < lMessageLength) {
                lWordCount = (lByteCount - (lByteCount % 4)) / 4;
                lBytePosition = (lByteCount % 4) * 8;
                lWordArray[lWordCount] = (lWordArray[lWordCount] | (numArr[lByteCount] << lBytePosition));
                lByteCount++;
            }
            lWordCount = (lByteCount - (lByteCount % 4)) / 4;
            lBytePosition = (lByteCount % 4) * 8;
            lWordArray[lWordCount] = lWordArray[lWordCount] | (0x80 << lBytePosition);
            lWordArray[lNumberOfWords - 2] = lMessageLength << 3;
            lWordArray[lNumberOfWords - 1] = lMessageLength >>> 29;
            return lWordArray;
        }

        function wordToHex(lValue) {
            var lByte,
            lCount,
            wordToHexArr = [];
            for (lCount = 0; lCount <= 3; lCount++) {
                lByte = (lValue >>> (lCount * 8)) & 255;
                wordToHexArr = wordToHexArr.concat(lByte);
             }
            return wordToHexArr;
        }

        /*function utf8Encode(string) {
            string = string.replace(/\r\n/g, "\n");
            var utftext = "",
            n,
            c;

            for (n = 0; n < string.length; n++) {

                c = string.charCodeAt(n);

                if (c < 128) {
                    utftext += String.fromCharCode(c);
                }
                else if ((c > 127) && (c < 2048)) {
                    utftext += String.fromCharCode((c >> 6) | 192);
                    utftext += String.fromCharCode((c & 63) | 128);
                }
                else {
                    utftext += String.fromCharCode((c >> 12) | 224);
                    utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                    utftext += String.fromCharCode((c & 63) | 128);
                }

            }

            return utftext;
        }*/

        var x = [],
        k,
        AA,
        BB,
        CC,
        DD,
        a,
        b,
        c,
        d,
        S11 = 7,
        S12 = 12,
        S13 = 17,
        S14 = 22,
        S21 = 5,
        S22 = 9,
        S23 = 14,
        S24 = 20,
        S31 = 4,
        S32 = 11,
        S33 = 16,
        S34 = 23,
        S41 = 6,
        S42 = 10,
        S43 = 15,
        S44 = 21;

        x = convertToWordArray(numArr);

        a = 0x67452301;
        b = 0xEFCDAB89;
        c = 0x98BADCFE;
        d = 0x10325476;

        for (k = 0; k < x.length; k += 16) {
            AA = a;
            BB = b;
            CC = c;
            DD = d;
            a = ff(a, b, c, d, x[k + 0], S11, 0xD76AA478);
            d = ff(d, a, b, c, x[k + 1], S12, 0xE8C7B756);
            c = ff(c, d, a, b, x[k + 2], S13, 0x242070DB);
            b = ff(b, c, d, a, x[k + 3], S14, 0xC1BDCEEE);
            a = ff(a, b, c, d, x[k + 4], S11, 0xF57C0FAF);
            d = ff(d, a, b, c, x[k + 5], S12, 0x4787C62A);
            c = ff(c, d, a, b, x[k + 6], S13, 0xA8304613);
            b = ff(b, c, d, a, x[k + 7], S14, 0xFD469501);
            a = ff(a, b, c, d, x[k + 8], S11, 0x698098D8);
            d = ff(d, a, b, c, x[k + 9], S12, 0x8B44F7AF);
            c = ff(c, d, a, b, x[k + 10], S13, 0xFFFF5BB1);
            b = ff(b, c, d, a, x[k + 11], S14, 0x895CD7BE);
            a = ff(a, b, c, d, x[k + 12], S11, 0x6B901122);
            d = ff(d, a, b, c, x[k + 13], S12, 0xFD987193);
            c = ff(c, d, a, b, x[k + 14], S13, 0xA679438E);
            b = ff(b, c, d, a, x[k + 15], S14, 0x49B40821);
            a = gg(a, b, c, d, x[k + 1], S21, 0xF61E2562);
            d = gg(d, a, b, c, x[k + 6], S22, 0xC040B340);
            c = gg(c, d, a, b, x[k + 11], S23, 0x265E5A51);
            b = gg(b, c, d, a, x[k + 0], S24, 0xE9B6C7AA);
            a = gg(a, b, c, d, x[k + 5], S21, 0xD62F105D);
            d = gg(d, a, b, c, x[k + 10], S22, 0x2441453);
            c = gg(c, d, a, b, x[k + 15], S23, 0xD8A1E681);
            b = gg(b, c, d, a, x[k + 4], S24, 0xE7D3FBC8);
            a = gg(a, b, c, d, x[k + 9], S21, 0x21E1CDE6);
            d = gg(d, a, b, c, x[k + 14], S22, 0xC33707D6);
            c = gg(c, d, a, b, x[k + 3], S23, 0xF4D50D87);
            b = gg(b, c, d, a, x[k + 8], S24, 0x455A14ED);
            a = gg(a, b, c, d, x[k + 13], S21, 0xA9E3E905);
            d = gg(d, a, b, c, x[k + 2], S22, 0xFCEFA3F8);
            c = gg(c, d, a, b, x[k + 7], S23, 0x676F02D9);
            b = gg(b, c, d, a, x[k + 12], S24, 0x8D2A4C8A);
            a = hh(a, b, c, d, x[k + 5], S31, 0xFFFA3942);
            d = hh(d, a, b, c, x[k + 8], S32, 0x8771F681);
            c = hh(c, d, a, b, x[k + 11], S33, 0x6D9D6122);
            b = hh(b, c, d, a, x[k + 14], S34, 0xFDE5380C);
            a = hh(a, b, c, d, x[k + 1], S31, 0xA4BEEA44);
            d = hh(d, a, b, c, x[k + 4], S32, 0x4BDECFA9);
            c = hh(c, d, a, b, x[k + 7], S33, 0xF6BB4B60);
            b = hh(b, c, d, a, x[k + 10], S34, 0xBEBFBC70);
            a = hh(a, b, c, d, x[k + 13], S31, 0x289B7EC6);
            d = hh(d, a, b, c, x[k + 0], S32, 0xEAA127FA);
            c = hh(c, d, a, b, x[k + 3], S33, 0xD4EF3085);
            b = hh(b, c, d, a, x[k + 6], S34, 0x4881D05);
            a = hh(a, b, c, d, x[k + 9], S31, 0xD9D4D039);
            d = hh(d, a, b, c, x[k + 12], S32, 0xE6DB99E5);
            c = hh(c, d, a, b, x[k + 15], S33, 0x1FA27CF8);
            b = hh(b, c, d, a, x[k + 2], S34, 0xC4AC5665);
            a = ii(a, b, c, d, x[k + 0], S41, 0xF4292244);
            d = ii(d, a, b, c, x[k + 7], S42, 0x432AFF97);
            c = ii(c, d, a, b, x[k + 14], S43, 0xAB9423A7);
            b = ii(b, c, d, a, x[k + 5], S44, 0xFC93A039);
            a = ii(a, b, c, d, x[k + 12], S41, 0x655B59C3);
            d = ii(d, a, b, c, x[k + 3], S42, 0x8F0CCC92);
            c = ii(c, d, a, b, x[k + 10], S43, 0xFFEFF47D);
            b = ii(b, c, d, a, x[k + 1], S44, 0x85845DD1);
            a = ii(a, b, c, d, x[k + 8], S41, 0x6FA87E4F);
            d = ii(d, a, b, c, x[k + 15], S42, 0xFE2CE6E0);
            c = ii(c, d, a, b, x[k + 6], S43, 0xA3014314);
            b = ii(b, c, d, a, x[k + 13], S44, 0x4E0811A1);
            a = ii(a, b, c, d, x[k + 4], S41, 0xF7537E82);
            d = ii(d, a, b, c, x[k + 11], S42, 0xBD3AF235);
            c = ii(c, d, a, b, x[k + 2], S43, 0x2AD7D2BB);
            b = ii(b, c, d, a, x[k + 9], S44, 0xEB86D391);
            a = addUnsigned(a, AA);
            b = addUnsigned(b, BB);
            c = addUnsigned(c, CC);
            d = addUnsigned(d, DD);
        }

        return wordToHex(a).concat(wordToHex(b), wordToHex(c), wordToHex(d));
    },
    

    Base64 = (function(){
        // Takes a Nx16x1 byte array and converts it to Base64
        var _chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/',
        chars = _chars.split(''),
        
        encode = function(b, withBreaks) {
            var flatArr = [],
            b64 = '',
            i,
            broken_b64;
            totalChunks = Math.floor(b.length * 16 / 3);
            for (i = 0; i < b.length * 16; i++) {
                flatArr.push(b[Math.floor(i / 16)][i % 16]);
            }
            for (i = 0; i < flatArr.length; i = i + 3) {
                b64 += chars[flatArr[i] >> 2];
                b64 += chars[((flatArr[i] & 3) << 4) | (flatArr[i + 1] >> 4)];
                if (! (flatArr[i + 1] === undefined)) {
                    b64 += chars[((flatArr[i + 1] & 15) << 2) | (flatArr[i + 2] >> 6)];
                } else {
                    b64 += '=';
                }
                if (! (flatArr[i + 2] === undefined)) {
                    b64 += chars[flatArr[i + 2] & 63];
                } else {
                    b64 += '=';
                }
            }
            // OpenSSL is super particular about line breaks
            broken_b64 = b64.slice(0, 64) + '\n';
            for (i = 1; i < (Math.ceil(b64.length / 64)); i++) {
                broken_b64 += b64.slice(i * 64, i * 64 + 64) + (Math.ceil(b64.length / 64) == i + 1 ? '': '\n');
            }
            return broken_b64;
        },
        
        decode = function(string) {
            string = string.replace(/\n/g, '');
            var flatArr = [],
            c = [],
            b = [],
            i;
            for (i = 0; i < string.length; i = i + 4) {
                c[0] = _chars.indexOf(string.charAt(i));
                c[1] = _chars.indexOf(string.charAt(i + 1));
                c[2] = _chars.indexOf(string.charAt(i + 2));
                c[3] = _chars.indexOf(string.charAt(i + 3));

                b[0] = (c[0] << 2) | (c[1] >> 4);
                b[1] = ((c[1] & 15) << 4) | (c[2] >> 2);
                b[2] = ((c[2] & 3) << 6) | c[3];
                flatArr.push(b[0], b[1], b[2]);
            }
            flatArr = flatArr.slice(0, flatArr.length - (flatArr.length % 16));
            return flatArr;
        };
        
        //internet explorer
        if(typeof Array.indexOf === "function") {
            _chars = chars;
        }
        
        /*
        //other way to solve internet explorer problem
        if(!Array.indexOf){
            Array.prototype.indexOf = function(obj){
                for(var i=0; i<this.length; i++){
                    if(this[i]===obj){
                        return i;
                    }
                }
                return -1;
            }
        }
        */
        
        
        return {
            "encode": encode,
            "decode": decode
        };
    })();

    return {
        "size": size,
        "h2a":h2a,
        "expandKey":expandKey,
        "encryptBlock":encryptBlock,
        "decryptBlock":decryptBlock,
        "Decrypt":Decrypt,
        "s2a":s2a,
        "rawEncrypt":rawEncrypt,
        "dec":dec,
        "decRaw": decRaw,
        "openSSLKey": openSSLKey,
        "a2h":a2h,
        "enc":enc,
        "encRaw": encRaw,
        "Hash": { "MD5": MD5 },
        "Base64":Base64
    };

})();
; 
 
 
 
/************************************************ 
**** zkBox.core.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


var $zkBox =
{
    logging: false,

    Init: function(options)
    {
        if (typeof options == 'object')
        {
            if (typeof options.ProxyUrl === 'string') { $zkBox.conf.ProxyUrl = options.ProxyUrl; }

            if ("" === $zkBox.conf.ProxyUrl)
            {
                throw "zkBox: ProxyUrl was not specified";
            }


            if (typeof options.ShowError === 'function') { $zkBox.conf.ShowError = options.ShowError; }
            if (typeof options.Log === 'boolean') $zkBox.conf.Log = options.Log;
            $zkBox.logging = $zkBox.conf.Log;

            if (typeof options.Cache === 'boolean') $zkBox.conf.Cache = options.Cache;

            // allow changing the SRP parameters?
            //if (typeof options.SRP == 'object')
            //{
            //    $zkBox.conf.SRP = new Object();
            //    $zkBox.conf.SRP.N = options.SRP.N;
            //    $zkBox.conf.SRP.g = options.SRP.g;
            //    $zkBox.conf.SRP.k = options.SRP.k;
            //};
            
            //if (typeof options.AuthCookie == 'object')
            //{
            //    if (typeof (options.AuthCookie.Expires) != "undefined") $zkBox.conf.AuthCookie.Expires = options.AuthCookie.Expires;
            //    if (typeof (options.AuthCookie.Path) != "undefined") $zkBox.conf.AuthCookie.Path = options.AuthCookie.Path;
            //    if (typeof (options.AuthCookie.Secure) != "undefined") $zkBox.conf.AuthCookie.Secure = options.AuthCookie.Secure;
            //}

            $zkBox.conf.AuthCookie.Path = $zkBox.jquery.url.setUrl($zkBox.conf.ProxyUrl).attr("path");
        }

        // using AES on 256 bits
        GibberishAES.size(256)

        $zkBox.utils.cache.initialize();

        $zkBox.jquery.debug($zkBox.conf.Log);
        $zkBox.crypto.srp.initialize(
            $zkBox.conf.SRP.N,
            $zkBox.conf.SRP.g,
            $zkBox.conf.SRP.k);
    }
}


$zkBox.jquery =
{
}


$zkBox.data =
{
}

$zkBox.callbacks =
{
}

$zkBox.cache =
{
}

$zkBox.cache.core =
{
}

$zkBox.cache.transactions =
{
}

$zkBox.cache.transactions.core =
{
}

$zkBox.cache.batch =
{
}

$zkBox.cache.batch.core =
{
}

$zkBox.maintenance =
{
}

$zkBox.maintenance.core =
{
}

$zkBox.applications =
{
}

$zkBox.applications.core =
{
}

$zkBox.users =
{
}

$zkBox.users.core =
{
}

$zkBox.objects =
{
}

$zkBox.objects.crypto =
{
}

$zkBox.objects.core =
{
}

$zkBox.objects.combined =
{
}

$zkBox.transactions =
{
}

$zkBox.transactions.core =
{
}

$zkBox.transactions.management =
{
}

$zkBox.transactions.objects =
{
}

$zkBox.batch =
{
}

$zkBox.batch.core =
{
}

$zkBox.batch.management =
{
}

$zkBox.batch.objects =
{
}; 
 
 
 
/************************************************ 
**** zkBox.jquery.dump.js ***** 
************************************************/ 
/*
    jquery.dump for zkBox
    Originale source url: http://plugins.jquery.com/project/Dump
*/


/**
* jquery.dump.js
* @author Torkild Dyvik Olsen
* @version 1.0
* 
* A simple debug function to gather information about an object.
* Returns a nested tree with information.
* 
*/


$zkBox.jquery.dump = function(object)
{

    var trim = function(str)
    {
        return ltrim(rtrim(str));
    };

    var ltrim = function(str)
    {
        return str.replace(new RegExp("^[\\s]+", "g"), "");
    };

    var rtrim = function(str)
    {
        return str.replace(new RegExp("[\\s]+$", "g"), "");
    };

    var recursion = function(obj, level)
    {
        if (!level) level = 0;
        var dump = '', p = '';
        for (i = 0; i < level; i++) p += "\t";

        t = type(obj);
        switch (t)
        {
            case "string":
                return '"' + obj + '"';
                break;
            case "number":
                return obj.toString();
                break;
            case "boolean":
                return obj ? 'true' : 'false';
            case "date":
                return "Date: " + obj.toLocaleString();
            case "array":
                dump += 'Array ( \n';
                $.each(obj, function(k, v)
                {
                    dump += p + '\t' + k + ' => ' + recursion(v, level + 1) + '\n';
                });
                dump += p + ')';
                break;
            case "object":
                dump += 'Object { \n';
                $.each(obj, function(k, v)
                {
                    dump += p + '\t' + k + ': ' + recursion(v, level + 1) + '\n';
                });
                dump += p + '}';
                break;
            case "jquery":
                dump += 'jQuery Object { \n';
                $.each(obj, function(k, v)
                {
                    dump += p + '\t' + k + ' = ' + recursion(v, level + 1) + '\n';
                });
                dump += p + '}';
                break;
            case "regexp":
                return "RegExp: " + obj.toString();
            case "error":
                return obj.toString();
            case "document":
            case "domelement":
                dump += 'DOMElement [ \n'
                  + p + '\tnodeName: ' + obj.nodeName + '\n'
                  + p + '\tnodeValue: ' + obj.nodeValue + '\n'
                  + p + '\tinnerHTML: [ \n';
                $.each(obj.childNodes, function(k, v)
                {
                    if (k < 1) var r = 0;
                    if (type(v) == "string")
                    {
                        if (v.textContent.match(/[^\s]/))
                        {
                            dump += p + '\t\t' + (k - (r || 0)) + ' = String: ' + trim(v.textContent) + '\n';
                        } else
                        {
                            r--;
                        }
                    } else
                    {
                        dump += p + '\t\t' + (k - (r || 0)) + ' = ' + recursion(v, level + 2) + '\n';
                    }
                });
                dump += p + '\t]\n'
                  + p + ']';
                break;
            case "function":
                var match = obj.toString().match(/^(.*)\(([^\)]*)\)/im);
                match[1] = trim(match[1].replace(new RegExp("[\\s]+", "g"), " "));
                match[2] = trim(match[2].replace(new RegExp("[\\s]+", "g"), " "));
                return match[1] + "(" + match[2] + ")";
            case "window":
            default:
                dump += 'N/A: ' + t;
                break;
        }

        return dump;
    }

    var type = function(obj)
    {
        var type = typeof (obj);

        if (type != "object")
        {
            return type;
        }

        switch (obj)
        {
            case null:
                return 'null';
            case window:
                return 'window';
            case document:
                return 'document';
            case window.event:
                return 'event';
            default:
                break;
        }

        if (obj.jquery)
        {
            return 'jquery';
        }

        switch (obj.constructor)
        {
            case Array:
                return 'array';
            case Boolean:
                return 'boolean';
            case Date:
                return 'date';
            case Object:
                return 'object';
            case RegExp:
                return 'regexp';
            case ReferenceError:
            case Error:
                return 'error';
            case null:
            default:
                break;
        }

        switch (obj.nodeType)
        {
            case 1:
                return 'domelement';
            case 3:
                return 'string';
            case null:
            default:
                break;
        }

        return 'Unknown';
    }

    return recursion(object);
}
; 
 
 
 
/************************************************ 
**** zkBox.jquery.url.js ***** 
************************************************/ 
/* ===========================================================================
 *
 * JQuery URL Parser
 * Version 1.0
 * Parses URLs and provides easy access to information within them.
 *
 * Author: Mark Perkins
 * Author email: mark@allmarkedup.com
 *
 * For full documentation and more go to http://projects.allmarkedup.com/jquery_url_parser/
 *
 * ---------------------------------------------------------------------------
 *
 * CREDITS:
 *
 * Parser based on the Regex-based URI parser by Steven Levithan.
 * For more information (including a detailed explaination of the differences
 * between the 'loose' and 'strict' pasing modes) visit http://blog.stevenlevithan.com/archives/parseuri
 *
 * ---------------------------------------------------------------------------
 *
 * LICENCE:
 *
 * Released under a MIT Licence. See licence.txt that should have been supplied with this file,
 * or visit http://projects.allmarkedup.com/jquery_url_parser/licence.txt
 *
 * ---------------------------------------------------------------------------
 * 
 * EXAMPLES OF USE:
 *
 * Get the domain name (host) from the current page URL
 * jQuery.url.attr("host")
 *
 * Get the query string value for 'item' for the current page
 * jQuery.url.param("item") // null if it doesn't exist
 *
 * Get the second segment of the URI of the current page
 * jQuery.url.segment(2) // null if it doesn't exist
 *
 * Get the protocol of a manually passed in URL
 * jQuery.url.setUrl("http://allmarkedup.com/").attr("protocol") // returns 'http'
 *
 */

$zkBox.jquery.url = function()
{
	var segments = {};
	
	var parsed = {};
	
	/**
    * Options object. Only the URI and strictMode values can be changed via the setters below.
    */
  	var options = {
	
		url : window.location, // default URI is the page in which the script is running
		
		strictMode: false, // 'loose' parsing by default
	
		key: ["source","protocol","authority","userInfo","user","password","host","port","relative","path","directory","file","query","anchor"], // keys available to query 
		
		q: {
			name: "queryKey",
			parser: /(?:^|&)([^&=]*)=?([^&]*)/g
		},
		
		parser: {
			strict: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*):?([^:@]*))?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,  //less intuitive, more accurate to the specs
			loose:  /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*):?([^:@]*))?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/ // more intuitive, fails on relative paths and deviates from specs
		}
		
	};
	
    /**
     * Deals with the parsing of the URI according to the regex above.
 	 * Written by Steven Levithan - see credits at top.
     */		
	var parseUri = function()
	{
		str = decodeURI( options.url );
		
		var m = options.parser[ options.strictMode ? "strict" : "loose" ].exec( str );
		var uri = {};
		var i = 14;

		while ( i-- ) {
			uri[ options.key[i] ] = m[i] || "";
		}

		uri[ options.q.name ] = {};
		uri[ options.key[12] ].replace( options.q.parser, function ( $0, $1, $2 ) {
			if ($1) {
				uri[options.q.name][$1] = $2;
			}
		});

		return uri;
	};

    /**
     * Returns the value of the passed in key from the parsed URI.
  	 * 
	 * @param string key The key whose value is required
     */		
	var key = function( key )
	{
		if ( ! parsed.length )
		{
			setUp(); // if the URI has not been parsed yet then do this first...	
		} 
		if ( key == "base" )
		{
			if ( parsed.port !== null && parsed.port !== "" )
			{
				return parsed.protocol+"://"+parsed.host+":"+parsed.port+"/";	
			}
			else
			{
				return parsed.protocol+"://"+parsed.host+"/";
			}
		}
	
		return ( parsed[key] === "" ) ? null : parsed[key];
	};
	
	/**
     * Returns the value of the required query string parameter.
  	 * 
	 * @param string item The parameter whose value is required
     */		
	var param = function( item )
	{
		if ( ! parsed.length )
		{
			setUp(); // if the URI has not been parsed yet then do this first...	
		}
		return ( parsed.queryKey[item] === null ) ? null : parsed.queryKey[item];
	};

    /**
     * 'Constructor' (not really!) function.
     *  Called whenever the URI changes to kick off re-parsing of the URI and splitting it up into segments. 
     */	
	var setUp = function()
	{
		parsed = parseUri();
		
		getSegments();	
	};
	
    /**
     * Splits up the body of the URI into segments (i.e. sections delimited by '/')
     */
	var getSegments = function()
	{
		var p = parsed.path;
		segments = []; // clear out segments array
		segments = parsed.path.length == 1 ? {} : ( p.charAt( p.length - 1 ) == "/" ? p.substring( 1, p.length - 1 ) : path = p.substring( 1 ) ).split("/");
	};
	
	return {
		
	    /**
	     * Sets the parsing mode - either strict or loose. Set to loose by default.
	     *
	     * @param string mode The mode to set the parser to. Anything apart from a value of 'strict' will set it to loose!
	     */
		setMode : function( mode )
		{
			strictMode = mode == "strict" ? true : false;
			return this;
		},
		
		/**
	     * Sets URI to parse if you don't want to to parse the current page's URI.
		 * Calling the function with no value for newUri resets it to the current page's URI.
	     *
	     * @param string newUri The URI to parse.
	     */		
		setUrl : function( newUri )
		{
			options.url = newUri === undefined ? window.location : newUri;
			setUp();
			return this;
		},		
		
		/**
	     * Returns the value of the specified URI segment. Segments are numbered from 1 to the number of segments.
		 * For example the URI http://test.com/about/company/ segment(1) would return 'about'.
		 *
		 * If no integer is passed into the function it returns the number of segments in the URI.
	     *
	     * @param int pos The position of the segment to return. Can be empty.
	     */	
		segment : function( pos )
		{
			if ( ! parsed.length )
			{
				setUp(); // if the URI has not been parsed yet then do this first...	
			} 
			if ( pos === undefined )
			{
				return segments.length;
			}
			return ( segments[pos] === "" || segments[pos] === undefined ) ? null : segments[pos];
		},
		
		attr : key, // provides public access to private 'key' function - see above
		
		param : param // provides public access to private 'param' function - see above
		
	};
	
}();; 
 
 
 
/************************************************ 
**** zkBox.jquery.ajaxq.js ***** 
************************************************/ 
/*
    jquery.ajaxq for zkBox
    Originale source url: http://plugins.jquery.com/project/ajaxq
*/


/*
 * jQuery AjaxQ - AJAX request queueing for jQuery
 *
 * Version: 0.0.1
 * Date: July 22, 2008
 *
 * Copyright (c) 2008 Oleg Podolsky (oleg.podolsky@gmail.com)
 * Licensed under the MIT (MIT-LICENSE.txt) license.
 *
 * http://plugins.jquery.com/project/ajaxq
 * http://code.google.com/p/jquery-ajaxq/
 */

$zkBox.jquery.ajaxq = function(queue, options)
{
	// Initialize storage for request queues if it's not initialized yet
    if (typeof document.zkBoxAjaxQ == "undefined") document.zkBoxAjaxQ = { q: {}, r: null };

	// Initialize current queue if it's not initialized yet
    if (typeof document.zkBoxAjaxQ.q[queue] == "undefined") document.zkBoxAjaxQ.q[queue] = [];
	
	if (typeof options != "undefined") // Request settings are given, enqueue the new request
	{
		// Copy the original options, because options.complete is going to be overridden

		var optionsCopy = {};
		for (var o in options) optionsCopy[o] = options[o];
		options = optionsCopy;
		
		// Override the original callback

		var originalCompleteCallback = options.complete;

		options.complete = function (request, status)
		{
			// Dequeue the current request
		    document.zkBoxAjaxQ.q[queue].shift();
		    document.zkBoxAjaxQ.r = null;
			
			// Run the original callback
			//if (originalCompleteCallback) originalCompleteCallback (request, status);
			if (originalCompleteCallback) originalCompleteCallback.apply(this, [request, status]);

			// Run the next request from the queue
			if (document.zkBoxAjaxQ.q[queue].length > 0) document.zkBoxAjaxQ.r = jQuery.ajax(document.zkBoxAjaxQ.q[queue][0]);
		};

		// Enqueue the request
		document.zkBoxAjaxQ.q[queue].push(options);

		// Also, if no request is currently running, start it
		if (document.zkBoxAjaxQ.q[queue].length == 1) document.zkBoxAjaxQ.r = jQuery.ajax(options);
	}
	else // No request settings are given, stop current request and clear the queue
	{
	    if (document.zkBoxAjaxQ.r)
		{
		    document.zkBoxAjaxQ.r.abort();
		    document.zkBoxAjaxQ.r = null;
		}

		document.zkBoxAjaxQ.q[queue] = [];
	}
}; 
 
 
 
/************************************************ 
**** zkBox.jquery.debug.js ***** 
************************************************/ 
/*
    jquery.debug for zkBox
    Originale source url: http://trainofthoughts.org/repo/getfile?f=jquery/jquery.debug.js&v=40
*/


/*
**  jquery.debug.js -- jQuery plugin for debugging
**  Copyright (c) 2007 Ralf S. Engelschall <rse@engelschall.com> 
**  Licensed under GPL <http://www.gnu.org/licenses/gpl.txt>
**
**  $LastChangedDate$
**  $LastChangedRevision$
*/

    
/* boolean status whether debugging is enabled */
$zkBox.jquery._debug$ = false;

/* method for getting and setting debug status */
$zkBox.jquery.debug = function(onoff)
{
    var old_value = ($zkBox.jquery._debug$ == true ? true : false);
    $zkBox.jquery._debug$ = (onoff ? true : false);
    return old_value;
},

/* method for logging an object or message */
$zkBox.jquery.logs = function(message)
{
    if ($zkBox.jquery._debug$ == true)
    {
        if (typeof window.console.debug !== "undefined")
        {
            window.console.debug(message);
        }
        else
        {
            if (typeof window.console.log !== "undefined")
            {
                window.console.log(message);
            }
            else
            {
                // do not log?
            }
        }
    }
}

/* minimum Firebug emulation (see http://getfirebug.com/firebug/firebugx.js) */
$(document).ready(function()
{
    if (typeof window.console === "undefined")
    {
        /* minimum conversion of arbitrary object to text representation */
        function object2text(obj)
        {
            var text = null;
            if (typeof obj === "undefined")
                text = "[undefined]";
            else if (typeof obj === "boolean")
                text = (obj ? "true" : "false");
            else if (typeof obj === "number")
                text = "" + obj;
            else if (typeof obj === "string")
                text = obj;
            else if (typeof obj === "function")
                text = obj;
            else if (typeof obj === "object")
            {
                if (typeof obj.nodeType !== "undefined")
                {
                    if (obj.nodeType == 1)
                    { /* W3C DOM element node */
                        text = '&lt;';
                        text += obj.nodeName.toLowerCase();
                        for (var i = 0; i < obj.attributes.length; i++)
                            text += ' ' + obj.attributes[i].nodeName.toLowerCase() +
                                    '="' + obj.attributes[i].nodeValue + '"';
                        text += '&gt;';
                    }
                    else if (obj.nodeType == 2) /* W3C DOM attribute node */
                        text = obj.nodeName + '="' + obj.nodeValue;
                    else if (obj.nodeType == 3) /* W3C DOM text node */
                        text = obj.nodeValue;
                }
                else if (typeof obj.toJSONString !== "undefined")
                    text = obj.toJSONString();
                else if (typeof obj.toString !== "undefined")
                    text = obj.toString();
            }
            if (text == null)
                text = "[unknown]";
            return text;
        };

        /* create the custom console <div> node */
        $(document).ready(function()
        {
            $("body")
                .append('<div id="jQueryDebug"><ol></ol></div>');
            $("#jQueryDebug")
                .css("display", "none")
                .css("fontFamily", "monospace")
                .css("backgroundColor", "#ffffff")
                .css("color", "#000000")
                .css("padding", "10px 0px 10px 0px")
                .css("border", "4px solid #666699")
                .css("margin", "4px 4px 4px 4px");
            $("#jQueryDebug ol")
                .css("margin", "0px 0px 0px 0px")
                .css("paddingRight", "20px");
        });

        /* attach a logging function to each of the usual Firebug methods */
        var names = [
            "log", "debug", "info", "warn", "error", "assert",
            "dir", "dirxml", "group", "groupEnd", "time", "timeEnd",
            "count", "trace", "profile", "profileEnd"
        ];
        var logfunc = function(msg)
        {
            $('#jQueryDebug')
                .css("display", $zkBox.jquery._debug$ == true ? "block" : "none");
            $('#jQueryDebug ol')
                .append('<li>' + object2text(msg) + '</li>').css;
            $("#jQueryDebug ol li")
                .css("borderBottom", "1px solid #cccccc")
                .css("padding", "1px 10px 1px 10px");
        };
        window.console = {};
        for (var i = 0; i < names.length; i++)
            window.console[names[i]] = logfunc;

        /* indicate that we are the one who is providing the Firebug interface */
        window.console["jQueryDebug"] = true;
    }
});

; 
 
 
 
/************************************************ 
**** zkBox.jquery.cookie.js ***** 
************************************************/ 
/*
    jquery.debug for zkBox
    Originale source url: http: //plugins.jquery.com/project/cookie
*/


/**
 * Cookie plugin
 *
 * Copyright (c) 2006 Klaus Hartl (stilbuero.de)
 * Dual licensed under the MIT and GPL licenses:
 * http://www.opensource.org/licenses/mit-license.php
 * http://www.gnu.org/licenses/gpl.html
 *
 */

/**
 * Create a cookie with the given name and value and other optional parameters.
 *
 * @example $.cookie('the_cookie', 'the_value');
 * @desc Set the value of a cookie.
 * @example $.cookie('the_cookie', 'the_value', { expires: 7, path: '/', domain: 'jquery.com', secure: true });
 * @desc Create a cookie with all available options.
 * @example $.cookie('the_cookie', 'the_value');
 * @desc Create a session cookie.
 * @example $.cookie('the_cookie', null);
 * @desc Delete a cookie by passing null as value. Keep in mind that you have to use the same path and domain
 *       used when the cookie was set.
 *
 * @param String name The name of the cookie.
 * @param String value The value of the cookie.
 * @param Object options An object literal containing key/value pairs to provide optional cookie attributes.
 * @option Number|Date expires Either an integer specifying the expiration date from now on in days or a Date object.
 *                             If a negative value is specified (e.g. a date in the past), the cookie will be deleted.
 *                             If set to null or omitted, the cookie will be a session cookie and will not be retained
 *                             when the the browser exits.
 * @option String path The value of the path atribute of the cookie (default: path of page that created the cookie).
 * @option String domain The value of the domain attribute of the cookie (default: domain of page that created the cookie).
 * @option Boolean secure If true, the secure attribute of the cookie will be set and the cookie transmission will
 *                        require a secure protocol (like HTTPS).
 * @type undefined
 *
 * @name $.cookie
 * @cat Plugins/Cookie
 * @author Klaus Hartl/klaus.hartl@stilbuero.de
 */

/**
 * Get the value of a cookie with the given name.
 *
 * @example $.cookie('the_cookie');
 * @desc Get the value of a cookie.
 *
 * @param String name The name of the cookie.
 * @return The value of the cookie.
 * @type String
 *
 * @name $.cookie
 * @cat Plugins/Cookie
 * @author Klaus Hartl/klaus.hartl@stilbuero.de
 */
$zkBox.jquery.cookie = function(name, value, options)
{
    if (typeof value != 'undefined') { // name and value given, set cookie
        options = options || {};
        if (value === null) {
            value = '';
            options.expires = -1;
        }
        var expires = '';
        if (options.expires && (typeof options.expires == 'number' || options.expires.toUTCString)) {
            var date;
            if (typeof options.expires == 'number') {
                date = new Date();
                date.setTime(date.getTime() + (options.expires * 24 * 60 * 60 * 1000));
            } else {
                date = options.expires;
            }
            expires = '; expires=' + date.toUTCString(); // use expires attribute, max-age is not supported by IE
        }
        // CAUTION: Needed to parenthesize options.path and options.domain
        // in the following expressions, otherwise they evaluate to undefined
        // in the packed version for some reason...
        var path = options.path ? '; path=' + (options.path) : '';
        var domain = options.domain ? '; domain=' + (options.domain) : '';
        var secure = options.secure ? '; secure' : '';
        document.cookie = [name, '=', encodeURIComponent(value), expires, path, domain, secure].join('');
    } else { // only name given, get cookie
        var cookieValue = null;
        if (document.cookie && document.cookie != '') {
            var cookies = document.cookie.split(';');
            for (var i = 0; i < cookies.length; i++) {
                var cookie = jQuery.trim(cookies[i]);
                // Does this cookie string begin with the name we want?
                if (cookie.substring(0, name.length + 1) == (name + '=')) {
                    cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
                    break;
                }
            }
        }
        return cookieValue;
    }
};; 
 
 
 
/************************************************ 
**** zkBox.conf.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.conf =
{
    ShowError: function(err)
    {
        // Errors:
        // 10 Unexpected ajax object returned
        // 11 Failure when calling method

        var s = "zkBox error: ";
        switch (err)
        {
            case 10:
                s += "Unexpected ajax object returned";
                break;

            case 11:
                s += "Failure when calling method";
                break;

            default:
                s += "Unhandled error";
                break;
        }

        s += " [" + err + "]";
        if ($zkBox.logging) $zkBox.utils.log(s);
    },

    //ProxyUrl: "https://api.dev.zkbox.com/rest",
    ProxyUrl: "",

    Log: false,
    
    Cache: true,

    SRP:
    {
        // this should be kept in sync with Bitground.Common.Helpers.Crypto.SRP.NHex/gHex/kHex
        N: "D4C7F8A2B32C11B8FBA9581EC4BA4F1B04215642EF7355E37C0FC0443EF756EA2C6B8EEB755A1C723027663CAA265EF785B8FF6A9B35227A52D86633DBDFCA43",
        g: "2",
        k: "738B8451178420B53F6603BD0B44B8BB50B80D1EFE3100E8E30C05EC7119502CA3974FA7FDFFADF0BABD20CBFA405A56BEAD38686811835997319EEDEE6B15C6"
    },
    
    AuthCookie:
    {
        Name: "X-zkBox-UserAuth",
        Expires: null,          // expire when browser exits
        Path: "/missing_path",  // the cookie's path
        
        Secure: true            // cookie sent on HTTPS only
    }
}
; 
 
 
 
/************************************************ 
**** zkBox.status.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.status =
{
    user: "",
    pass: "",

    // Authentication parameters
    userId: "",
    userVersion: "",
    sessionId: "",

    // The session's key used to sign the requests (input for HMAC-SHA512)
    sessionKey: "",

    cryptoKey: "",

    // The crypto key used to encrypt the data (input for AES256)
    cryptoKeyRaw: [],

    isAuthenticated: false,

    updateAuth: function()
    {
        this.cryptoKeyRaw = base16_decode(this.cryptoKey);
        if ($zkBox.logging) $zkBox.utils.log('Setting cryptoKeyRaw = [' + this.cryptoKeyRaw + ']');

        this.isAuthenticated = true;
    },

    updateUserAuth: function(userAuth)
    {
        if (null == userAuth)
        {
            if ($zkBox.logging) $zkBox.utils.log('Clearing the ' + $zkBox.conf.AuthCookie.Name + ' cookie');
            $zkBox.jquery.cookie(
                $zkBox.conf.AuthCookie.Name,
                null,
                {
                    path: $zkBox.conf.AuthCookie.Path,
                    secure: $zkBox.conf.AuthCookie.Secure
                });
        }
        else
        {
            var val = this.userId + this.userVersion + this.sessionId + userAuth;
            if ($zkBox.logging) $zkBox.utils.log('Setting the ' + $zkBox.conf.AuthCookie.Name + ' cookie to "' + val + '" (Expires="' + $zkBox.conf.AuthCookie.Expires + '", Path="' + $zkBox.conf.AuthCookie.Path + '", Secure="' + $zkBox.conf.AuthCookie.Secure + '")');
            $zkBox.jquery.cookie(
                $zkBox.conf.AuthCookie.Name,
                val,
                {
                    expires: $zkBox.conf.AuthCookie.Expires,
                    path: $zkBox.conf.AuthCookie.Path,
                    secure: $zkBox.conf.AuthCookie.Secure
                });
        }
    },

    clearAuth: function()
    {
        this.user = "";
        this.pass = "";

        //Authentication parameters
        this.userId = "";
        this.userVersion = "";
        this.sessionId = "";
        this.sessionKey = "";
        this.cryptoKey = "";

        this.cryptoKeyRaw = [];

        this.isAuthenticated = false;

        // clear the cache
        $zkBox.utils.cache.clearAllItems();

        if ($zkBox.logging) $zkBox.utils.log('Clearing the ' + $zkBox.conf.AuthCookie.Name + ' cookie');
        $zkBox.jquery.cookie(
            $zkBox.conf.AuthCookie.Name,
            null,
            {
                path: $zkBox.conf.AuthCookie.Path,
                secure: $zkBox.conf.AuthCookie.Secure
            });
    }
}
; 
 
 
 
/************************************************ 
**** zkBox.utils.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.utils =
{
    log: function(msg, params)
    {
        var message = "";
        if (arguments.length <= 1)
        {
            message = msg;
        }
        else
        {
            params = $.makeArray(arguments).slice(1);
            message = this.format(msg, params);
        }

        toLocalFormat = function(date, theFormat)
        {
            var aVar = {
                MM: date.getMonth() + 1,
                DD: date.getDate(),
                YYYY: date.getFullYear(),
                hh: date.getHours(),
                mm: date.getMinutes(),
                ss: date.getSeconds(),
                ap: "AM"
            }
            if (theFormat.match(/\bap\b/i))
            {
                if (aVar.hh > 12)
                {
                    aVar.hh -= 12;
                    aVar.ap = "PM";
                }
            }
            for (v in aVar)
            {
                theFormat = theFormat.replace(v, f2(aVar[v]));
            }
            function f2(theVal)
            {
                if (theVal < 10) return "0" + theVal;
                return theVal;
            }
            return theFormat;
        }


        var currentTime = new Date()

        return $zkBox.jquery.logs(
            "[" + toLocalFormat(currentTime, "YYYY-MM-DD hh:mm:ss") + "] ZKBOX " +
            message);
    },

    dump: function(object)
    {
        return $zkBox.jquery.dump(object);
    },

    msg: function(source, params)
    {

        if (arguments.length == 1)
        {
            return this.format(source, "");
        }
        else
        {
            return this.format(source, params);
        }
    },

    format: function(source, params)
    {
        if (arguments.length == 1)
            return function()
            {
                var args = $.makeArray(arguments);
                args.unshift(source);
                return $.format.apply(this, args);
            };
        if (arguments.length > 2 && params.constructor != Array)
        {
            params = $.makeArray(arguments).slice(1);
        }
        if (params.constructor != Array)
        {
            params = [params];
        }
        $.each(params, function(i, n)
        {
            source = source.replace(new RegExp("\\{" + i + "\\}", "g"), n);
        });
        return source;
    },

    toJSON: function(object)
    {
        return JSON.stringify(object);
    },

    evalJSON: function(text)
    {
        return JSON.parse(text);
    },

    clone: function(obj)
    {
        // TODO_LATER: perhaps improve this
        return this.evalJSON(this.toJSON(obj));
    },

    isArray: function(obj)
    {
        // taken from jQuery library
        return Object.prototype.toString.call(obj) === "[object Array]";
    },

    runFunction: function(fnName, fn)
    {
        return function(msg)
        {
            var text = "[none]";

            if (typeof (msg) != "undefined")
            {
                text = $zkBox.utils.dump(msg);
            }

            if (typeof (fn) === "function")
            {
                if ($zkBox.logging) $zkBox.utils.log(fnName + ": " + text);
                fn(msg);
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log(fnName + " (default): " + text);
            }
        };
    },

    runFunctionOnly: function(fnName, fn)
    {
        return function(msg)
        {
            if (typeof (fn) === "function")
            {
                fn(msg);
            }
            else
            {
            }
        };
    },

    uniqueCounter: 0,
    uniqueAlphabet: "abcdefghijklmnopqrstuvwxyz0123456789",

    getUnique: function()
    {
        var uniq = '';

        // 4 chars from random
        {
            var rnd = Math.floor(1679616 * Math.random()); // 1679616 = 36^4

            var i;
            for (i = 0; i < 4; i++)
            {
                uniq += this.uniqueAlphabet.charAt(rnd % 36);
                rnd = Math.floor(rnd / 36);
            }
        }

        // 8 chars from ticks
        {
            var ticks = new Date().getTime();

            var i;
            for (i = 0; i < 8; i++)
            {
                uniq += this.uniqueAlphabet.charAt(ticks % 36);
                ticks = Math.floor(ticks / 36);
            }
        }

        // 4 chars from counter
        {
            var cnt = this.uniqueCounter;
            this.uniqueCounter++;

            var i;
            for (i = 0; i < 4; i++)
            {
                uniq += this.uniqueAlphabet.charAt(cnt % 36);
                cnt = Math.floor(cnt / 36);
            }
        }

        return uniq;
    },

    removeItemFromArray: function(ar, item)
    {
        var found = false;
        var j = 0;
        while (j < ar.length)
        {
            if (ar[j] == item)
            {
                found = true;
                ar.splice(j, 1);
            }
            else
            {
                j++;
            }
        }
        return found;
    }
};
; 
 
 
 
/************************************************ 
**** zkBox.utils.cache.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.utils.cache =
{
    cache: null,

    initialize: function()
    {
        this.cache = new zkBoxCache();
    },

    setItem: function(key, value)
    {
        if ($zkBox.logging) $zkBox.utils.log('Cache.setItem("{0}")="{1}"', key, $zkBox.utils.dump(value));
        this.cache.setItem(key, value);
    },

    getItem: function(key)
    {
        var ret = $zkBox.utils.clone(this.cache.getItem(key)); // clone every returned objects from the cache; TODO_LATER: change/optimize this?
        if ($zkBox.logging) $zkBox.utils.log('Cache.getItem("{0}") will return "{1}"', key, $zkBox.utils.dump(ret));
        return ret;
    },

    removeItem: function(key)
    {
        if ($zkBox.logging) $zkBox.utils.log('Cache.removeItem("{0}")', key);
        this.cache.removeItem(key);
    },

    clearAllItems: function()
    {
        if ($zkBox.logging) $zkBox.utils.log('Cache.clearAllItems()');
        this.cache.clear();
    },

    containsItem: function(key)
    {
        var value = this.cache.getItem(key);
        var ret = false;
        if (null != value)
        {
            ret = true;
        }
        if ($zkBox.logging) $zkBox.utils.log('Cache.containsItem("{0}") will return "{1}"', key, ret);
        return ret;
    }
}
; 
 
 
 
/************************************************ 
**** zkBox.cache.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns true if the operation succeeded, null otherwise
$zkBox.cache.AddObject = function(ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> AddObject (ob="{0}")]',
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.AddObject: ob.objectId was not provided";
        
        return $zkBox.cache.core.AddObject(ob);
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.UpdateObject = function(ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> UpdateObject (ob="{0}")]',
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.UpdateObject: ob.objectId was not provided";
        
        return $zkBox.cache.core.UpdateObject(ob);
    }
    
    return null;
}



// returns the object if it's in the cache, null otherwise
$zkBox.cache.GetObjectCheck = function(objectId)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> GetObjectCheck (objectId="{0}")]',
            $zkBox.utils.dump(objectId));

        if (!objectId) throw "Cache.GetObjectCheck: objectId was not provided";
        
        return $zkBox.cache.core.GetObjectCheck(objectId);
    }

    return null;
}

// returns true if the operation succeeded, null otherwise
$zkBox.cache.GetObjectUpdate = function(ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> GetObjectUpdate (ob="{0}")]',
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.GetObjectCheck: objectId was not provided";
        
        return $zkBox.cache.core.GetObjectUpdate(ob);
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.DeleteObject = function(ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> DeleteObject (ob="{0}")]',
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.DeleteObject: ob.objectId was not provided";

        return $zkBox.cache.core.DeleteObject(ob);
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.GetObjectsListCheck = function(ctx)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> GetObjectsListCheck (ctx="{0}")]',
            $zkBox.utils.dump(ctx));

        ctx.checkExecuted = true;
        $zkBox.cache.core.GetObjectsListInit(ctx);

        return $zkBox.cache.core.GetObjectsListCheck(ctx);
    }

    return null;
}

// returns true if the operation succeeded, null otherwise
$zkBox.cache.GetObjectsListUpdate = function(ctx)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> GetObjectsListUpdate (ctx="{0}")]',
            $zkBox.utils.dump(ctx));

        if (!ctx.checkExecuted)
        {
            $zkBox.cache.core.GetObjectsListInit(ctx);
        }
        
        return $zkBox.cache.core.GetObjectsListUpdate(ctx);
    }

    return null;
}


; 
 
 
 
/************************************************ 
**** zkBox.cache.core.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//TODO_LATER: make the key prefix 'objects-type' (and the similar others) shorter?

$zkBox.cache.core.AddObject = function(ob)
{
    ob = jQuery.extend(
    {
        objectId: null,
        type: null
    }, ob);

    var objectId = ob.objectId;
    var type = ob.type;

    // set the single object into the cache
    $zkBox.utils.cache.setItem('object-' + objectId, ob);

    // set all objects by type
    {
        var objectIdsOfType = $zkBox.utils.cache.getItem('objects-type-' + type);
        if (null != objectIdsOfType)
        {
            objectIdsOfType.push(objectId);
            $zkBox.utils.cache.setItem('objects-type-' + type, objectIdsOfType);
        }
    }

    // set all objects
    {
        var objectIds = $zkBox.utils.cache.getItem('objects-all');
        if (null != objectIds)
        {
            objectIds.push(objectId);
            $zkBox.utils.cache.setItem('objects-all', objectIds);
        }
    }

    return true;
}



$zkBox.cache.core.UpdateObject = function(ob)
{
    ob = jQuery.extend(
    {
        objectId: null
    }, ob);

    var objectId = ob.objectId;

    // update the single object's data into the cache if the object already exists
    var obc = $zkBox.utils.cache.getItem('object-' + objectId);
    if (null != obc)
    {
        obc.data = ob.data;
        $zkBox.utils.cache.setItem('object-' + objectId, obc);
    }

    return true;
}



$zkBox.cache.core.GetObjectCheck = function(objectId)
{
    var ob = $zkBox.utils.cache.getItem('object-' + objectId);
    if (null != ob)
    {
        if ($zkBox.logging) $zkBox.utils.log('Cache hit: GetObject("{0}") will return "{1}"', objectId, $zkBox.utils.dump(ob));
        return ob;
    }

    return null;
}

$zkBox.cache.core.GetObjectUpdate = function(ob)
{
    ob = jQuery.extend(
    {
        objectId: null
    }, ob);

    var objectId = ob.objectId;

    // set the single object into the cache
    $zkBox.utils.cache.setItem('object-' + objectId, ob);

    return true;
}



$zkBox.cache.core.DeleteObject = function(ob)
{
    ob = jQuery.extend(
    {
        objectId: null,
        type: null
    }, ob);

    var objectId = ob.objectId;
    var type = ob.type;

    // delete the single object
    $zkBox.utils.cache.removeItem('object-' + objectId);

    // all objects by type
    {
        var objectIdsOfType = $zkBox.utils.cache.getItem('objects-type-' + type);
        if (null != objectIdsOfType)
        {
            var found = $zkBox.utils.removeItemFromArray(objectIdsOfType, objectId);

            if (true == found)
            {
                if ($zkBox.logging) $zkBox.utils.log('We removed the item from the objects-type list. The new value is: {0}', $zkBox.utils.dump(objectIdsOfType));
                $zkBox.utils.cache.setItem('objects-type-' + type, objectIdsOfType);
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We do not have the item in the objects-type list. The objectId is "{0}" and the list is: {1}', objectId, $zkBox.utils.dump(objectIdsOfType));
            }
        }
    }

    // all objects
    {
        var objectIds = $zkBox.utils.cache.getItem('objects-all');
        if (null != objectIds)
        {
            var found = $zkBox.utils.removeItemFromArray(objectIds, objectId);

            if (true == found)
            {
                if ($zkBox.logging) $zkBox.utils.log('We removed the item from the objects-all list. The new value is: {0}', $zkBox.utils.dump(objectIds));
                $zkBox.utils.cache.setItem('objects-all', objectIds);
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We do not have the item in the objects-all list. The objectId is "{0}" and the list is: {1}', objectId, $zkBox.utils.dump(objectIds));
            }
        }
    }
    
    return true;
}



$zkBox.cache.core.GetObjectsListInit = function(ctx)
{
    ctx.requestType = 0;
    ctx.postprocessRequest = true;
    ctx.existingObjectsList = null;
    ctx.newObjectsList = null;

    // compute the request type
    {
        if (null != ctx.type)
        {
            // we assume we want objects by type
            ctx.requestType = 2; // all objects by type
        }
        else
        {
            if (null != ctx.objectsList)
            {
                // we assume we want a list of objects
                ctx.requestType = 1; // objects list by ids
            }
            else
            {
                // we'll assume we want all objects
                ctx.requestType = 3; // all objects
            }
        }
        //if ($zkBox.logging) $zkBox.utils.log("requestType={0}", ctx.requestType);
    }

    // objects list by ids
    if (1 == ctx.requestType)
    {
        ctx.existingObjectsList = new Array();
        ctx.newObjectsList = new Array();
    }
}

$zkBox.cache.core.GetObjectsListCheck = function(ctx)
{
    if ($zkBox.logging) $zkBox.utils.log('Cache.Core.GetObjectsListCheck (ctx="{0}")]',
            $zkBox.utils.dump(ctx));

    // objects list by ids
    if (1 == ctx.requestType)
    {
        // we are checking to see which objects we already have in cache
        for (var i = 0, olength = ctx.objectsList.length; i < olength; i++)
        {
            var objectId = ctx.objectsList[i];
            var ob = $zkBox.utils.cache.getItem('object-' + objectId);
            if (null != ob)
            {
                ctx.existingObjectsList.push(ob);
            }
            else
            {
                ctx.newObjectsList.push(objectId);
            }
        }
        if ($zkBox.logging) $zkBox.utils.log("existingObjectsList: {0}", $zkBox.utils.dump(ctx.existingObjectsList));
        if ($zkBox.logging) $zkBox.utils.log("newObjectsList: {0}", $zkBox.utils.dump(ctx.newObjectsList));
        ctx.objectsList = ctx.newObjectsList;

        if (ctx.newObjectsList.length <= 0)
        {
            if ($zkBox.logging) $zkBox.utils.log('We already have all the objects in cache');
            // we already have the objects list
            ctx.ret = ctx.existingObjectsList;
            ctx.executeRequest = false;
            ctx.postprocessRequest = false;
        }
        else
        {
            if ($zkBox.logging) $zkBox.utils.log('We already have {0} objects in cache. We will request only {1} objects', ctx.existingObjectsList.length, ctx.newObjectsList.length);
        }
    }

    if ((2 == ctx.requestType)  // all objects by type
        ||
        (3 == ctx.requestType)) // all objects
    {
        var objectIdsOfType;
        if (2 == ctx.requestType)
        {
            // all objects by type
            objectIdsOfType = $zkBox.utils.cache.getItem('objects-type-' + ctx.type);
        }
        else
        {
            // all objects
            objectIdsOfType = $zkBox.utils.cache.getItem('objects-all');
        }

        if (null != objectIdsOfType)
        {
            if ($zkBox.logging) $zkBox.utils.log('We already have the list of objects (of this type): {0}', $zkBox.utils.dump(objectIdsOfType));

            ctx.ret = new Array();
            for (var i = 0, olength = objectIdsOfType.length; i < olength; i++)
            {
                var objectId = objectIdsOfType[i];
                var ob = $zkBox.utils.cache.getItem('object-' + objectId);
                if (null != ob)
                {
                    ctx.ret.push(ob);
                }
                else
                {
                    if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We have the object "{0}" listed in the objects (of this type) list, but the object was not found in the cache. The list is: {1}', objectId, objectIdsOfType);
                }
            }
            ctx.executeRequest = false;
            ctx.postprocessRequest = false;
        }
    }

    return true;
}

$zkBox.cache.core.GetObjectsListUpdate = function(ctx)
{
    // if we need postprocessing...
    if (true == ctx.postprocessRequest)
    {
        // objects list by ids
        if (1 == ctx.requestType)
        {
            if ($zkBox.logging) $zkBox.utils.log('Add the {0} returned objects to the cache', ctx.ret.length);
            for (var i = 0, rlength = ctx.ret.length; i < rlength; i++)
            {
                var ob = ctx.ret[i];
                var objectId = ob.objectId;
                $zkBox.utils.cache.setItem('object-' + objectId, ob);
            }

            if ($zkBox.logging) $zkBox.utils.log('Add {0} existing cached objects to the return list', ctx.existingObjectsList.length);
            for (var i = 0, elength = ctx.existingObjectsList.length; i < elength; i++)
            {
                ctx.ret.push(ctx.existingObjectsList[i]);
            }
        }

        if ((2 == ctx.requestType)  // all objects by type
            ||
            (3 == ctx.requestType)) // all objects
        {
            if ($zkBox.logging) $zkBox.utils.log('Add the {0} returned objects to the cache', ctx.ret.length);
            var objectIdsOfType = new Array();
            for (var i = 0, rlength = ctx.ret.length; i < rlength; i++)
            {
                var ob = ctx.ret[i];
                var objectId = ob.objectId;
                $zkBox.utils.cache.setItem('object-' + objectId, ob);
                objectIdsOfType.push(objectId);
            }

            // set the list of objects (of this type)
            if (2 == ctx.requestType)
            {
                // all objects by type
                $zkBox.utils.cache.setItem('objects-type-' + ctx.type, objectIdsOfType);
            }
            else
            {
                // all objects
                $zkBox.utils.cache.setItem('objects-all', objectIdsOfType);
            }
        }
    }
    return true;
}

; 
 
 
 
/************************************************ 
**** zkBox.cache.transactions.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns true if the operation succeeded, null otherwise
$zkBox.cache.transactions.AddObject = function(transactionId, ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.AddObject (transactionId="{0}" ob="{1}")]',
            transactionId,
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.Transactions.AddObject: ob.objectId was not provided";

        if (transactionId)
        {
            // we are in a transaction, so we'll queue the cache add request
            return $zkBox.cache.transactions.core.QueueOperation(
                transactionId,
                "AddObject",
                $zkBox.cache.core.AddObject,
                ob);
        }
        else
        {
            // the operation is not in a transaction, so make the change into the cache directly
            return $zkBox.cache.core.AddObject(ob);
        }
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.transactions.UpdateObject = function(transactionId, ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.UpdateObject (transactionId="{0}" ob="{1}")]',
            transactionId,
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.Transactions.UpdateObject: ob.objectId was not provided";

        if (transactionId)
        {
            // we are in a transaction, so we'll queue the cache update request
            return $zkBox.cache.transactions.core.QueueOperation(
                transactionId,
                "UpdateObject",
                $zkBox.cache.core.UpdateObject,
                ob);
        }
        else
        {
            // the operation is not in a transaction, so make the change into the cache directly
            return $zkBox.cache.core.UpdateObject(ob);
        }
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.transactions.GetObjectUpdate = function(transactionId, ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.GetObjectUpdate (transactionId="{0}" ob="{1}")]',
            transactionId,
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.Transactions.GetObjectUpdate: ob.objectId was not provided";

        if (transactionId)
        {
            // we are in a transaction, so we'll queue the cache update request
            return $zkBox.cache.transactions.core.QueueOperation(
                transactionId,
                "GetObjectUpdate",
                $zkBox.cache.core.GetObjectUpdate,
                ob);
        }
        else
        {
            // the operation is not in a transaction, so make the change into the cache directly
            return $zkBox.cache.core.GetObjectUpdate(ob);
        }
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.transactions.DeleteObject = function(transactionId, ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.DeleteObject (transactionId="{0}" ob="{1}")]',
            transactionId,
            $zkBox.utils.dump(ob));

        if (!ob.objectId) throw "Cache.Transactions.DeleteObject: ob.objectId was not provided";

        if (transactionId)
        {
            // we are in a transaction, so we'll queue the cache delete request
            return $zkBox.cache.transactions.core.QueueOperation(
                transactionId,
                "DeleteObject",
                $zkBox.cache.core.DeleteObject,
                ob);
        }
        else
        {
            // the operation is not in a transaction, so make the change into the cache directly
            return $zkBox.cache.core.DeleteObject(ob);
        }
    }

    return null;
}



// returns true if the operation succeeded, null otherwise
$zkBox.cache.transactions.GetObjectsListUpdate = function(transactionId, ob)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.GetObjectsListUpdate (transactionId="{0}" ob="{1}")]',
            transactionId,
            $zkBox.utils.dump(ob));

        if (transactionId)
        {
            // we are in a transaction, so we'll queue the cache update request
            return $zkBox.cache.transactions.core.QueueOperation(
                transactionId,
                "GetObjectsListUpdate",
                $zkBox.cache.core.GetObjectsListUpdate,
                ob);
        }
        else
        {
            // the operation is not in a transaction, so make the change into the cache directly
            return $zkBox.cache.core.GetObjectsListUpdate(ob);
        }
    }

    return null;
}
; 
 
 
 
/************************************************ 
**** zkBox.cache.transactions.core.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.cache.transactions.core.operations = new Array();

// queue an cache change request; return true if successful, null otherwise
$zkBox.cache.transactions.core.QueueOperation = function(transactionId, operation, fn, params)
{
    if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.Core.QueueOperation (transactionId="{0}" operation="{1}" fn="{2}" params="{3}")]',
            transactionId,
            operation,
            $zkBox.utils.dump(fn),
            $zkBox.utils.dump(params));

    if (!transactionId) throw "Cache.Transactions.Core.QueueOperation: transactionId was not provided";
    if (!operation) throw "Cache.Transactions.Core.QueueOperation: operation was not provided";

    if (!$zkBox.cache.transactions.core.operations[transactionId])
    {
        $zkBox.cache.transactions.core.operations[transactionId] = new Array();
    }

    $zkBox.cache.transactions.core.operations[transactionId].push(
    {
        fn: fn,
        operation: operation,
        params: params
    });

    return true;
}


// make the changes into the cache for a given transaction; return true if succeded, null otherwise
$zkBox.cache.transactions.core.CommitTransaction = function(transactionId)
{
    if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.Core.CommitTransaction (transactionId="{0}")]',
            transactionId);

    if (!transactionId) throw "Cache.Transactions.Core.CommitTransaction: transactionId was not provided";

    var ret = null;

    if ($zkBox.cache.transactions.core.operations[transactionId])
    {
        var tran = $zkBox.cache.transactions.core.operations[transactionId];
        if ($zkBox.logging) $zkBox.utils.log('Cache.Transactions.Core.CommitTransaction: We have {0} operations to commit', tran.length);

        ret = true;
        for (var i = 0; i < tran.length; i++)
        {
            var op = tran[i];
            if ($zkBox.logging) $zkBox.utils.log('Cache.Transactions.Core.CommitTransaction: commiting operation #{0} to the cache: "{1}"',
                i,
                $zkBox.utils.dump(op));

            tempret = op.fn(op.params);
            if (!tempret)
            {
                if ($zkBox.logging) $zkBox.utils.log('Cache.Transactions.Core.CommitTransaction: Failure while commiting to the cache an operation. Abort the cache updates.');
                ret = null;
                break;
            }
        }
    }
    else
    {
        // the transaction was not found
        if ($zkBox.logging) $zkBox.utils.log('Cache.Transactions.Core.CommitTransaction: transactionId was not found');
    }

    $zkBox.cache.transactions.core.operations[transactionId] = null;
    
    if ($zkBox.logging) $zkBox.utils.log('Cache.Transactions.Core.CommitTransaction will return {0}',
        ret);
    return ret;
}


// remove the queued cache changes for a given transaction; return true if succeded, null otherwise
$zkBox.cache.transactions.core.RollbackTransaction = function(transactionId)
{
    if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Transactions.Core.RollbackTransaction (transactionId="{0}")]',
            transactionId);

    if (!transactionId) throw "Cache.Transactions.Core.RollbackTransaction: transactionId was not provided";

    $zkBox.cache.transactions.core.operations[transactionId] = null;

    return true;
}
; 
 
 
 
/************************************************ 
**** zkBox.cache.batch.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns true if the operation succeeded, null otherwise
$zkBox.cache.batch.ExecuteBatch = function(options, ctx)
{
    if ($zkBox.conf.Cache === true)
    {
        if ($zkBox.logging) $zkBox.utils.log('[CACHE -> Batch.ExecuteBatch (options="{0}" ctx="{1}")]',
            $zkBox.utils.dump(options),
            $zkBox.utils.dump(ctx));

        var transactionId;
        var shouldCommit;
        // compute the transaction id and if we should commit it
        {
            if (true == options.atStartBeginTransaction)
            {
                // we have a transaction at the start

                if (true == options.atEndCommitTransaction)
                {
                    // we are commiting the transaction at the end, so the transaction is internal on the server
                    // assume a transaction id for the client

                    transactionId = "TRAN" + $zkBox.utils.getUnique();
                    shouldCommit = true;
                }
                else
                {
                    // the transaction is open, so we should have it in the return
                    transactionId = ctx.transactionId;
                    shouldCommit = false;
                }
            }
            else
            {
                // we're not starting a transaction

                if (true == options.atEndCommitTransaction)
                {
                    // but we are commiting it, so the transaction id perhaps was provided
                    transactionId = options.transactionId
                    shouldCommit = true;
                }
                else
                {
                    // and we're not closing any transaction
                    transactionId = null;
                    shouldCommit = false;
                }
            }
        }

        var operationsCache = ctx.operationsCache;
        var results = ctx.results;

        for (var i = 0; i < operationsCache.length; i++)
        {
            var operation = operationsCache[i];
            var result = results[i];

            var opTransactionId = transactionId;
            if (operation.transactionId)
            {
                opTransactionId = operation.transactionId;
            }

            switch (operation.operation)
            {
                case "AddObject":
                    {
                        var ob =
                        {
                            objectId: result.objectId,
                            type: operation.type,
                            data: operation.data
                        };
                        $zkBox.cache.transactions.AddObject(
                            opTransactionId,
                            ob);
                    }
                    break;

                case "UpdateObject":
                    {
                        var ob =
                        {
                            objectId: operation.objectId,
                            data: operation.data
                        };
                        $zkBox.cache.transactions.UpdateObject(
                            opTransactionId,
                            ob);
                    }
                    break;

                case "GetObject":
                    {
                        $zkBox.cache.transactions.GetObjectUpdate(
                            opTransactionId,
                            result);
                    }
                    break;

                case "DeleteObject":
                    {
                        var ob =
                        {
                            objectId: operation.objectId
                        };
                        $zkBox.cache.transactions.DeleteObject(
                            opTransactionId,
                            ob);
                    }
                    break;

                case "GetObjectsList":
                    {
                        var ctx =
                        {
                            type: operation.type,
                            objectsList: operation.objectsList,
                            ret: result
                        };
                        $zkBox.cache.transactions.GetObjectsListUpdate(
                            opTransactionId,
                            ctx);
                    }
                    break;

                case "GetObjectsListIDs":
                    // GetObjectsListIDs is not using caching (TODO_LATER: use caching?)
                    break;

                default:
                    throw "Cache.Batch: Unhandled operation type '" + operation.operation + "'";
            }
        }

        if (shouldCommit)
        {
            // reflect the changes into the cache; ignore the result from the cache update
            $zkBox.cache.transactions.core.CommitTransaction(transactionId);
        }

        return true;
    }

    return null;
}
; 
 
 
 
/************************************************ 
**** zkBox.cache.batch.core.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// no core operations for Batch
; 
 
 
 
/************************************************ 
**** zkBox.crypto.js ***** 
************************************************/ 
// srp4net 1.0
// SRP for .NET - A JavaScript/C# .NET library for implementing the SRP authentication protocol
// http://code.google.com/p/srp4net/
// Copyright 2010, Sorin Ostafiev (http://www.ostafiev.com/)
// License: GPL v3 (http://www.gnu.org/licenses/gpl-3.0.txt)


$zkBox.crypto =
{
    sha512: sha.create("SHA-512"),

    hash: function(message)
    {
        // hash a message
        // outputs a 128 hex chars string (512bits)
        return base16_encode(this.sha512.hash(str2utf8(message)));
    },

    hashHex: function(hexmessage)
    {
        // hash a hex string
        // outputs a 128 hex chars string (512bits)
        return base16_encode(this.sha512.hash(base16_decode(((0 == hexmessage.length % 2) ? '' : '0') + hexmessage)));
    },

    //encode: function(clearText, key)
    //{
    //    // encode a string using a key as string
    //    // outputs a base64 string
    //    var encoded = GibberishAES.enc(clearText, key);
    //    return encoded;
    //},
    //
    //decode: function(encodedText, key)
    //{
    //    // decode a string using a key as string
    //    // input is base64 and the output is the original string
    //    var decoded = GibberishAES.dec(encodedText, key);
    //    return decoded
    //},
    
    encodeRaw: function(clearText, keyRaw)
    {
        // encode a string using a key in raw format
        // outputs a base64 string
        var encoded = GibberishAES.encRaw(clearText, keyRaw);
        return encoded;
    },

    decodeRaw: function(encodedText, keyRaw)
    {
        // decode a string using a key in raw format
        // input is base64 and the output is the original string
        var decoded = GibberishAES.decRaw(encodedText, keyRaw);
        return decoded
    },

    cryptokey: function(pass)
    {
        // generate a crypto key from a pass (for inputing into the AES)
        // input is a 128 hex chars (512bits) string and the output is a 32 chars (256bits) string
        // pass is already a hash, so we'll just reduce from 512 bits to 256 bits
        if (128 != pass.length)
        {
            if ($zkBox.logging) $zkBox.utils.log("Unexpected pass.length '" + pass.length + "'");
            return pass;
        }

        // pass the password through a hmac hash 50 times using
        // the previous value as input key at the next iteration
        var pnext = this.hashHex(pass);
        var i;
        var ptmp;
        for (i = 0; i < 50; i++)
        {
            ptmp = this.hmachash(pass, pnext);
            pass = pnext;
            pnext = ptmp;
        }
        pass = pnext;

        var ar = base16_decode(pass);
        if (64 != ar.length)
        {
            if ($zkBox.logging) $zkBox.utils.log("Unexpected ar.length '" + ar.length + "'");
            return pass;
        }

        var ret = new Array();
        for (i = 0; i < 32; i++)
        {
            ret[i] = ar[i] ^ ar[32 + i];
        }

        var strret = char2str(ret)
        if (32 != strret.length)
        {
            if ($zkBox.logging) $zkBox.utils.log("Unexpected strret.length '" + strret.length + "'");
            return pass;
        }

        return strret;
    },

    hmachash: function(key, message)
    {
        return hex_hmac_sha512(key, message).toUpperCase();
    },

    getNonce: function()
    {
        return $zkBox.utils.getUnique().toUpperCase();
    }
};
; 
 
 
 
/************************************************ 
**** zkBox.crypto.srp.js ***** 
************************************************/ 
// srp4net 1.0
// SRP for .NET - A JavaScript/C# .NET library for implementing the SRP authentication protocol
// http://code.google.com/p/srp4net/
// Copyright 2010, Sorin Ostafiev (http://www.ostafiev.com/)
// License: GPL v3 (http://www.gnu.org/licenses/gpl-3.0.txt)

// Description: Secure Remote Password protocol implementation in JavaScript and C#
// RFC: http://www.ietf.org/rfc/rfc2945.txt
// Details: http://srp.stanford.edu/ndss.html
// Implementation: SRP variant 6a with SHA2 digest (instead of SHA1 from the original specification) on 512 bits
// Author: Sorin Ostafiev (http://www.ostafiev.com/)
// Libraries used:
//      - BSD: jsSHA.js - A JavaScript implementation of the SHA family of hashes, as defined in FIPS PUB 180-2
//              originally written by Brian Turek http://jssha.sourceforge.net/
//              changed by Atsushi Oka (http://ats.oka.nu/titaniumcore/js/crypto/jsSHA.js, several functions taken from Paul Johnson)
//              slightly changed by Sorin Ostafiev (http://www.ostafiev.com/)
//      - LGPL: SHA.js - SHA class that calculates various SHA hash values
//              by Atsushi Oka (http://ats.oka.nu/titaniumcore/js/crypto/SHA.js)
//              special thanks to Mr. Brian Turek (http://sourceforge.net/users/caligatio)
//      - BSD: sha512.js - A JavaScript implementation of the Secure Hash Algorithm, SHA-512, as defined in FIPS 180-2
//              Copyright Anonymous Contributor, Paul Johnston 2000 - 2009
//              Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
//      - Public: BigInt.js - Big Integer Library
//              by Leemon Baird (http: //www.leemon.com/crypto/BigInt.js)
//      - LGPL: binary.js - Tools for creating, modifying binary data
//              by Atsushi Oka (http://ats.oka.nu/titaniumcore/js/tools/binary.js)
//      - MIT/GPL - jquery-1.3.2.js - jQuery - a new kind of JavaScript Library
//              by the jQuery team (http://docs.jquery.com/Contributors)

$zkBox.crypto.srp =
{
    N: '',
    g: '',
    k: '',

    Nbits: 0,

    one: '',
    two: '',
    three: '',

    bigint2hex: function(i)
    {
        // convert from a bigint number to a hex string
        return bigInt2str(i, 16);
    },

    hex2bigint: function(s)
    {
        // convert from a hex string to a bigint number
        return str2bigInt(s, 16, 1);
    },


    compareTo: function(big1, big2)
    {
        // -1 if big1 < big2
        //  1 if big1 > big2
        //  0 if equal
        if (greater(big1, big2))
        {
            return 1;
        }
        else
        {
            if (equals(big1, big2))
            {
                return 0;
            }
        }
        return -1;
    },

    to128: function(s)
    {
        // pad a string with zeroes up to length of 128
        var i;
        var l = 128 - s.length;
        for (i = 0; i < l; i++)
        {
            s = '0' + s;
        }
        return s;
    },

    H: function(x)
    {
        // hash a message
        return $zkBox.crypto.hash(x);
    },

    HHex: function(x)
    {
        // hash a hex string
        return $zkBox.crypto.hashHex(x);
    },

    CheckUser: function(username, fnProgress, fnSuccess, fnError)
    {
        // Check if a given user exists
        // username = the username entered in the user interface
        if (!this.initialized) throw "SRP not initialized";

        fnProgress('Checking for user');
        var login = this.HashUser(username);
        username = '';

        //fnProgress('Calling the Users/CheckUser server\'s method');
        fnProgress('Checking the user');
        if ($zkBox.logging) $zkBox.utils.log("login: " + login);
        // call the server's Users/CheckUser method
        $zkBox.model.Call(false, false, "PUT user/check", {
            login: login
        },
        function(data)
        {
            fnProgress('The user was checked');
            fnSuccess(data);
        },
        function(error)
        {
            fnProgress('Failure while checking the user');
            fnError(error);
        });
    },

    HashUser: function(username)
    {
        return this.H("!@#<32}|{_$+)EW:>fWS@@!=39dje%^#$RF']]ew3" + username + "shaSHA@");
    },

    HashPass: function(password)
    {
        return this.H("\":}[\|weFC@de';{{3$$  vdRF2w^5V|\\/.w32" + password + "sshHAA!%");
    },

    BeginUpdateUser: function(login, newpassword, fnProgress, fnSuccess, fnError)
    {
        if (!this.initialized) throw "SRP not initialized";

        fnProgress('SRP begin update user started');

        var newpass = this.HashPass(newpassword);
        newpassword = '';
        if ($zkBox.logging) $zkBox.utils.log('newpass:' + newpass);

        fnProgress('Salting and hashing the input');
        // s - login's salt
        // s = random number
        var s = randBigInt(this.Nbits, 0); //TODO_LATER: use TRNG
        var sHex = this.to128(this.bigint2hex(s));

        fnProgress('Compute the private key');
        // x - a private key derived from the password and salt
        // x = H(s || H(username) || H(password))
        var x;
        {
            var x1 = this.hex2bigint(this.HHex(
                sHex +
                login +
                newpass));

            if (this.compareTo(x1, this.N) < 0) { x = x1; }
            else { x = mod(x1, sub(this.N, this.one)); }
        }

        fnProgress('Generate the server\'s verifier');
        // v - the server's verifier
        // v = g^x (mod N)
        var v = powMod(this.g, x, this.N);
        var vHex = this.to128(this.bigint2hex(v));

        fnProgress('Calling the Users/BeginUpdateUser server\'s method');
        if ($zkBox.logging) $zkBox.utils.log("    s: " + sHex);
        if ($zkBox.logging) $zkBox.utils.log("    v: " + vHex);
        // call the server's Users/BeginUpdateUser method
        $zkBox.model.Call(false, true, "POST user/update", {
            s: sHex,
            v: vHex
        },
        function(data)
        {
            fnProgress('The user update was successfully started');
            fnProgress('Decoding the response');

            if ($zkBox.logging) $zkBox.utils.log("data: " + $zkBox.utils.dump(data));

            fnSuccess(
                login,
                newpass,
                data.newUserVersion);
        },
        function(error)
        {
            fnProgress('Failure during the SRP begin update user');
            fnError(error);
        });
    },

    CommitUpdateUser: function(newUserVersion, fnProgress, fnSuccess, fnError)
    {
        if (!this.initialized) throw "SRP not initialized";

        fnProgress('SRP commit update user started');

        fnProgress('Calling the Users/CommitUpdateUser server\'s method');
        if ($zkBox.logging) $zkBox.utils.log("newUserVersion: " + newUserVersion);
        // call the server's Users/CommitUpdateUser method
        $zkBox.model.Call(false, true, "PUT user/update/" + newUserVersion, {},
        function(data)
        {
            fnProgress('The user update was successfully commited');
            fnProgress('Decoding the response');

            if ($zkBox.logging) $zkBox.utils.log("data: " + $zkBox.utils.dump(data));

            fnSuccess();
        },
        function(error)
        {
            fnProgress('Failure during the SRP commit update user');
            fnError(error);
        });
    },

    RollbackUpdateUser: function(newUserVersion, fnProgress, fnSuccess, fnError)
    {
        if (!this.initialized) throw "SRP not initialized";

        fnProgress('SRP rollback update user started');

        fnProgress('Calling the Users/RollbackUpdateUser server\'s method');
        if ($zkBox.logging) $zkBox.utils.log("newUserVersion: " + newUserVersion);
        // call the server's Users/RollbackUpdateUser method
        $zkBox.model.Call(false, true, "DELETE user/update/" + newUserVersion, {},
        function(data)
        {
            fnProgress('The user update was successfully rolledback');
            fnProgress('Decoding the response');

            if ($zkBox.logging) $zkBox.utils.log("data: " + $zkBox.utils.dump(data));

            fnSuccess();
        },
        function(error)
        {
            fnProgress('Failure during the SRP rollback update user');
            fnError(error);
        });
    },    

    DeleteUser: function(login, fnProgress, fnSuccess, fnError)
    {
        if (!this.initialized) throw "SRP not initialized";

        fnProgress('SRP delete user started');

        fnProgress('Calling the Users/DeleteUser server\'s method');
        // call the server's Users/DeleteUser method
        $zkBox.model.Call(false, true, "DELETE user", {},
        function(data)
        {
            fnProgress('The user was successfully deleted');
            fnProgress('Decoding the response');

            if ($zkBox.logging) $zkBox.utils.log("data: " + $zkBox.utils.dump(data));

            fnSuccess();
        },
        function(error)
        {
            fnProgress('Failure during the SRP delete user');
            fnError(error);
        });
    },

    LogoutUser: function(fnProgress, fnSuccess, fnError)
    {
        if (!this.initialized) throw "SRP not initialized";

        fnProgress('SRP logout user started');

        fnProgress('Calling the Users/LogoutUser server\'s method');
        // call the server's Users/LogoutUser method
        $zkBox.model.Call(false, true, "PUT user/logout", {},
        function(data)
        {
            fnProgress('The user was successfully logged out');
            fnProgress('Decoding the response');

            if ($zkBox.logging) $zkBox.utils.log("data: " + $zkBox.utils.dump(data));

            fnSuccess();
        },
        function(error)
        {
            fnProgress('Failure during the SRP logout user');
            fnError(error);
        });
    },

    AddUser: function(username, password, fnProgress, fnSuccess, fnError)
    {
        // Creating a new user
        // username = the username entered in the user interface
        // password = the password entered in the user interface
        if (!this.initialized) throw "SRP not initialized";

        fnProgress('SRP user creation started');
        // login = H(username)
        var login = this.HashUser(username);
        username = '';

        // pass = H(password)
        var pass = this.HashPass(password);
        password = '';
        if ($zkBox.logging) $zkBox.utils.log('passw: ' + pass);

        fnProgress('Salting and hashing the input');
        // s - login's salt
        // s = random number
        var s = randBigInt(this.Nbits, 0); //TODO_LATER: use TRNG
        var sHex = this.to128(this.bigint2hex(s));

        fnProgress('Compute the private key');
        // x - a private key derived from the password and salt
        // x = H(s || H(username) || H(password))
        var x;
        {
            var x1 = this.hex2bigint(this.HHex(
                sHex +
                login +
                pass));
            if (this.compareTo(x1, this.N) < 0) { x = x1; }
            else { x = mod(x1, sub(this.N, this.one)); }
        }

        fnProgress('Generate the server\'s verifier');
        // v - the server's verifier
        // v = g^x (mod N)
        var v = powMod(this.g, x, this.N);
        var vHex = this.to128(this.bigint2hex(v));

        fnProgress('Calling the Users/AddUser server\'s method');
        if ($zkBox.logging) $zkBox.utils.log("login: " + login);
        if ($zkBox.logging) $zkBox.utils.log("    s: " + sHex);
        if ($zkBox.logging) $zkBox.utils.log("    v: " + vHex);
        // call the server's Users/AddUser method
        $zkBox.model.Call(false, false, "POST user", {
            login: login,
            s: sHex,
            v: vHex
        },
        function(data)
        {
            fnProgress('The user was successfully created');
            fnProgress('Decoding the response');

            if ($zkBox.logging) $zkBox.utils.log("data: " + $zkBox.utils.dump(data));

            fnSuccess(
                login,
                pass);
        },
        function(error)
        {
            fnProgress('Failure during the SRP user creation');
            fnError(error);
        });
    },

    AuthenticateUser: function(dataIsHashed, username, password, fnProgress, fnSuccess, fnError)
    {
        // Authenticate an user
        // username = the username entered in the user interface
        // password = the password entered in the user interface
        if (!this.initialized) throw "SRP not initialized";

        fnProgress('SRP authentication started');

        var err = 0;
        var login;
        var pass;

        if (dataIsHashed)
        {
            login = username;
            pass = password;
        }
        else
        {
            // user = H(username)
            login = this.HashUser(username);
            username = '';

            // pass = H(password)
            pass = this.HashPass(password);
            password = '';
        }

        if ($zkBox.logging) $zkBox.utils.log('login: ' + login);
        if ($zkBox.logging) $zkBox.utils.log('passw: ' + pass);

        var hex2bigint = this.hex2bigint;

        // AuthStep1
        var a;
        var A;
        var AHex;
        var uniq;
        var sHex;
        var BHex;
        var u;

        {
            fnProgress('Generate the ephemeral private key');
            // a - ephemeral private key
            // a = random between 2 and N-1
            {
                a = randBigInt(this.Nbits, 0);
                if (this.compareTo(a, this.N) >= 0) { a = mod(a, sub(this.N, this.one)); }
                if (this.compareTo(a, this.two) < 0) { a = this.two; }
            }

            fnProgress('Generate the public key');
            // A - public key
            // A = g^a (mod N)
            A = powMod(this.g, a, this.N);
            AHex = this.to128(this.bigint2hex(A));

            //fnProgress('Calling the Users/AuthStep1 server\'s method');
            fnProgress('Initiating the secure channel');
            if ($zkBox.logging) $zkBox.utils.log("login: " + login);
            if ($zkBox.logging) $zkBox.utils.log("    A: " + AHex);
            // call the server's Users/AuthStep1 method
            $zkBox.model.Call(false, false, "PUT user/auth/step1", {
                login: login,
                A: AHex
            },
            function(data)
            {
                fnProgress('Decoding the response');
                uniq = data.uniq;
                sHex = data.s;
                BHex = data.B;
                u = hex2bigint(data.u);
                if ($zkBox.logging) $zkBox.utils.log(" uniq: " + uniq);
                if ($zkBox.logging) $zkBox.utils.log("    s: " + sHex);
                if ($zkBox.logging) $zkBox.utils.log("    B: " + BHex);
            },
            function(error)
            {
                fnProgress('Failure during the SRP authentication');
                fnError(error);
                err = error;
            });
        }

        // AuthStep2
        var KHex;
        var m1Hex;
        var userId;
        var userVersion;
        var sessionId;
        var m2server;
        if (0 == err)
        {
            var B = this.hex2bigint(BHex)

            // x - private key derived from password and the salt
            // x = H(s || H(username) || H(P))
            var x;
            {
                var x1 = this.hex2bigint(this.HHex(
                    sHex +
                    login +
                    pass));
                if (this.compareTo(x1, this.N) < 0) { x = x1; }
                else { x = mod(x1, sub(this.N, this.one)); }
            }

            fnProgress('Computing SRP common exponential value');
            // S - common exponential value
            // S = (B - k * g^x) ^ (a + u * x) (mod N)
            var S = powMod(mod(sub(add(B, mult(this.N, this.k)), mult(powMod(this.g, x, this.N), this.k)), this.N), add(mult(x, u), a), this.N);

            fnProgress('Computing the strong cryptographically session key');
            // K - the strong cryptographically session key
            // K = H(S)
            var K = this.hex2bigint(this.HHex(this.bigint2hex(S)));
            KHex = this.to128(this.bigint2hex(K));
            if ($zkBox.logging) $zkBox.utils.log("    K: " + KHex);

            fnProgress('Client signing the login session');
            // m1 - client's proof that it has the correct key
            // m1 = H(A, B, K)
            var m1 = this.hex2bigint(this.HHex(
                AHex +
                BHex +
                KHex));
            m1Hex = this.to128(this.bigint2hex(m1));

            //fnProgress('Calling the Users/AuthStep2 server\'s method');
            fnProgress('Establishing the secure connection');
            if ($zkBox.logging) $zkBox.utils.log("login: " + login);
            if ($zkBox.logging) $zkBox.utils.log(" uniq: " + uniq);
            if ($zkBox.logging) $zkBox.utils.log("   m1: " + m1Hex);
            // call the server's Users/AuthStep2 method
            $zkBox.model.Call(false, false, "PUT user/auth/step2", {
                login: login,
                uniq: uniq,
                m1: m1Hex
            },
            function(data)
            {
                fnProgress('Decoding the response');
                userId = data.userId;
                userVersion = data.userVersion;
                sessionId = data.sessionId;

                // m2server - server's proof that it has the correct key
                m2server = hex2bigint(data.m2);

                if ($zkBox.logging) $zkBox.utils.log("useri: " + userId);
                if ($zkBox.logging) $zkBox.utils.log("userv: " + userVersion);
                if ($zkBox.logging) $zkBox.utils.log("sessi: " + sessionId);
                if ($zkBox.logging) $zkBox.utils.log("m2ser: " + data.m2);
            },
            function(error)
            {
                fnProgress('Failure during the SRP authentication');
                fnError(error);
                err = error;
            });
        }

        if (0 == err)
        {
            fnProgress('Checking server\'s signature');
            // m2 - expected server's proof as computed by the client
            // m2 = H(A, m1, K)
            var m2 = this.hex2bigint(this.HHex(
                AHex +
                m1Hex +
                KHex));
            if ($zkBox.logging) $zkBox.utils.log("   m2: " + this.to128(this.bigint2hex(m2)));

            if (0 == this.compareTo(m2, m2server))
            {
                fnProgress('SRP authentication succeded. Computing the crypto key.');
                // the server is trusted
                //if ($zkBox.logging) $zkBox.utils.log('client: the server is trusted');

                var ckey = $zkBox.crypto.cryptokey(pass);
                //if ($zkBox.logging) $zkBox.utils.log(" ckey: " + $zkBox.utils.dump(ckey)); // the key as string
                var cryptokey = base16_encode(str2char(ckey)); // this is displayable
                fnProgress("The crypto key was computed. The secure session was established.");

                if ($zkBox.logging) $zkBox.utils.log("sessionKey: " + KHex);
                if ($zkBox.logging) $zkBox.utils.log(" cryptokey: " + cryptokey);
                //if ($zkBox.logging) $zkBox.utils.log("Dcryptokey: " + base16_decode(cryptokey));
                //if ($zkBox.logging) $zkBox.utils.log("Ucryptokey: " + GibberishAES.enc_utf8(ckey));
                //if ($zkBox.logging) $zkBox.utils.log("U2cryptoke: " + str2utf8(ckey));
                //if ($zkBox.logging) $zkBox.utils.log("Xcryptokey: " + $zkBox.utils.dump(GibberishAES.s2a(ckey)));

                fnSuccess(
                    login,
                    pass,
                    userId,
                    userVersion,
                    sessionId,
                    KHex,
                    cryptokey);
            }
            else
            {
                fnProgress('Failure during the SRP authentication');
                if ($zkBox.logging) $zkBox.utils.log('client: the server is NOT trusted');
                fnError(101); // Invalid username or password
            }
        }
    },

    initialized: false,

    initialize: function(NHex, gHex, kHex)
    {
        this.one = this.hex2bigint("1");
        this.two = this.hex2bigint("2");
        this.three = this.hex2bigint("3");

        this.N = this.hex2bigint(NHex);
        this.Nbits = bitSize(this.N);
        this.g = this.hex2bigint(gHex);
        this.k = this.hex2bigint(kHex);

        this.initialized = true;
    }
};
; 
 
 
 
/************************************************ 
**** zkBox.data.client.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.data.client =
{
    // local data layer (e.g. Google Gears)
    // persist the data locally

    Call: function(
        isAsynchronous,
        userIsAuthenticated,
        action,
        data,
        fnSuccess,
        fnError)
    {
        // TODO_LATER: implement the local data layer (e.g. Google Gears)
        throw "$zkBox.data.client.Call said: UNIMPLEMENTED";
    }
}
; 
 
 
 
/************************************************ 
**** zkBox.data.server.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.data.server =
{
    // server data layer
    // persist the data on the server

    Call: function(
        isAsynchronous,
        userIsAuthenticated,
        action,
        data,
        fnSuccess,
        fnError)
    {
        if (isAsynchronous) // if is asynchronous
        {
            // put it in the execution queue
            return this.WSq(userIsAuthenticated, action, data, fnSuccess, fnError);
        }
        else
        {
            // if is synchronous, run it directly
            return this.WS(userIsAuthenticated, action, data, fnSuccess, fnError);
        }
    },


    WS: function(userIsAuthenticated, action, data, fnSuccess, fnError)
    {
        try
        {
            return $.ajax(this.WScoreGetOptions(userIsAuthenticated, action, data, fnSuccess, fnError));
        }
        catch (ex)
        {
            if ($zkBox.logging) $zkBox.utils.log("Exception occurred in WS: " + ex);
            fnError(1);
        }
    },

    WSq: function(userIsAuthenticated, action, data, fnSuccess, fnError)
    {
        try
        {
            // using action as queue's name
            return $zkBox.jquery.ajaxq(action, this.WScoreGetOptions(userIsAuthenticated, action, data, fnSuccess, fnError));
        }
        catch (ex)
        {
            if ($zkBox.logging) $zkBox.utils.log("Exception occurred in WSq: " + ex);
            fnError(1);
        }
    },

    isEmpty: function(obj)
    {
        for (var i in obj)
        {
            return false;
        }
        return true;
    },

    WScoreGetOptions: function(userIsAuthenticated, action, data, fnSuccess, fnError)
    {
        var method = "";
        var resource = "";
        var pos = action.indexOf(" ");

        if (pos <= 0)
        {
            // no action? this shouldn't happen
            method = action;
        }
        else
        {
            method = action.substr(0, pos);
            resource = escape("/" + action.substr(1 + pos));
        }

        //var method = action.split(" ", 2)[0];
        //var resource = "/" + action.split(" ", 2)[1];
        //if ($zkBox.logging) $zkBox.utils.log("action: '{0}'", action);
        //if ($zkBox.logging) $zkBox.utils.log("method: '{0}'", method);
        //if ($zkBox.logging) $zkBox.utils.log("resource: '{0}'", resource);

        if (!(("POST" == method) || ("PUT" == method) || ("DELETE" == method) || ("GET" == method)))
        {
            throw ("Method can be only GET, POST, PUT, DELETE, but '" + method + "' was received");
        }

        var fullresource = $zkBox.conf.ProxyUrl + resource;
        if ($zkBox.logging) $zkBox.utils.log("Ajax call: [" + method + "] " + fullresource);
        var errmsg;

        var hasData = (("POST" == method) || ("PUT" == method));
        var strdata = (this.isEmpty(data)) ? (hasData ? "{}" : null) : $zkBox.utils.toJSON(data);
        var contenttype = hasData ? "application/json; charset=utf-8" : "";
        var datatype = "json";


        if (true == userIsAuthenticated)
        {
            //sign the request

            if ('' != $zkBox.status.sessionKey)
            {
                var nonce = $zkBox.crypto.getNonce();
                if ($zkBox.logging) $zkBox.utils.log("method: " + method);

                var dataToSign = method + "\n" +
                         resource + "\n" +
                         nonce + "\n" +
                         contenttype + "\n" +
                         ((null == strdata) ? "" : strdata) + "\n";

                var signature = $zkBox.crypto.hmachash($zkBox.status.sessionKey, dataToSign)
                if ($zkBox.logging) $zkBox.utils.log("signing key: '" + $zkBox.status.sessionKey + "'");
                if ($zkBox.logging) $zkBox.utils.log("dataToSign length: " + dataToSign.length);
                if ($zkBox.logging) $zkBox.utils.log("dataToSign:\n" + dataToSign);
                if ($zkBox.logging) $zkBox.utils.log("nonce: '" + nonce + "'");
                if ($zkBox.logging) $zkBox.utils.log("signature: '" + signature + "'");
                var userAuth = nonce + signature;

                $zkBox.status.updateUserAuth(userAuth);
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log("An authenticated method was called, but the user is not authenticated");
            }
        }
        else
        {
            $zkBox.status.updateUserAuth(null);
        }


        return {
            type: method,
            contentType: contenttype,
            //contentType: "application/x-www-form-urlencoded",
            async: false,
            url: fullresource,
            data: strdata,
            dataType: datatype,

            success: function(msg)
            {
                if ((typeof (msg) !== 'undefined') && (msg) && (typeof (msg.error) !== 'undefined'))
                {
                    if (msg.error === 0) // success
                    {
                        //if ($zkBox.logging) $zkBox.utils.log("Success: " + $zkBox.utils.dump(msg.data));
                        fnSuccess(msg.data);
                    }
                    else // some logical error occured
                    {
                        if ($zkBox.logging) $zkBox.utils.log("Logical error occured: " + msg.error);
                        fnError(msg.error);
                    }
                }
                else
                {
                    errmsg = $zkBox.utils.msg(
                        'Failure when calling the "{0}" action. The server returned "' + $zkBox.utils.dump(msg) + '"',
                        action);
                    if ($zkBox.logging) $zkBox.utils.log(errmsg);
                    $zkBox.conf.ShowError(10);
                }

            },

            error: function(xhr, msg, error)
            {
                var errmsg = $zkBox.utils.msg(
                    'Failure when calling the "{0}" action',
                    action);
                if ($zkBox.logging) $zkBox.utils.log(errmsg);

                var errcode;
                try
                {
                    var err = eval("(" + xhr.responseText + ")");
                    errmsg = err.message;
                    errcode = err.error;
                    if ($zkBox.logging) $zkBox.utils.log('Error occurred. Server returned: code = "' + errcode + '", message = "' + errmsg + "'");
                }
                catch (ex)
                {
                    if ($zkBox.logging) $zkBox.utils.log('Exception occured while evaluating the response');
                }

                $zkBox.conf.ShowError(11); // Failure when calling method
                fnError(11);
            }
        };
    }
}
; 
 
 
 
/************************************************ 
**** zkBox.model.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.model =
{
    // Model: bl and ajax

    // it's dispatching between the local and the server storage

    Call: function(
        isAsynchronous,
        userIsAuthenticated,
        action,
        data,
        fnSuccess,
        fnError)
    {
        // TODO_LATER: dispatch here between local and server storage
        return $zkBox.data.server.Call(
            isAsynchronous,
            userIsAuthenticated,
            action,
            data,
            fnSuccess,
            fnError);
    }
}
; 
 
 
 
/************************************************ 
**** zkBox.model.errors.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.model.errors =
{
    /*
    
    // TODO_LATER: implement model.errors?

    show: function(err)
    {
        alert(this.translate(err));
    },

    translate: function(err, args)
    {
        var e = this.errors[err];
        if (typeof (e) === 'undefined')
        {
            e = 'Unknown error';
        }

        return $.msg(e + ' [' + err + ']', args);

    },

    errors: new Object({

        0: 'Success',


        10: 'Unexpected ajax object returned',
        11: 'Unexpected ajax call response',
        12: 'Failure when calling the {0} method',
        13: 'The view page {0} is invalid',
        14: 'The page {0} is invalid',
        15: 'The controller page {0} is invalid',

        100: 'The user already exists',
        101: 'Invalid username or password',

        999: ''
    })
    
    */
}
; 
 
 
 
/************************************************ 
**** zkBox.callbacks.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.callbacks.fnStart = null;
$zkBox.callbacks.fnStop = null;

$zkBox.callbacks.fnProgress = null;
$zkBox.callbacks.fnSuccess = null;
$zkBox.callbacks.fnError = null;

$zkBox.SetCallbacks = function(fnStart, fnStop, fnProgress, fnSuccess, fnError)
{
    $zkBox.callbacks.fnStart = fnStart;
    $zkBox.callbacks.fnStop = fnStop;

    $zkBox.callbacks.fnProgress = fnProgress;
    $zkBox.callbacks.fnSuccess = fnSuccess;
    $zkBox.callbacks.fnError = fnError;
}

; 
 
 
 
/************************************************ 
**** zkBox.maintenance.core.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// zkBox Maintenance
////////  core operations  ////////

$zkBox.maintenance.core.StartMaintenance = function(fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[MAINTENANCE -> StartMaintenance]');

    // clear the cache
    $zkBox.utils.cache.clearAllItems();

    $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
}


$zkBox.maintenance.core.StopMaintenance = function(fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[MAINTENANCE -> StopMaintenance]');

    // clear the cache
    $zkBox.utils.cache.clearAllItems();

    $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
}
; 
 
 
 
/************************************************ 
**** zkBox.maintenance.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//////////////////////////////////
////////  api operations  ////////
//////////////////////////////////

// returns true if the operation succeeded, false otherwise
$zkBox.StartMaintenance = function()
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("StartMaintenance");

    $zkBox.maintenance.core.StartMaintenance(
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("StartMaintenance");

    return ret;
}

// returns true if the operation succeeded, false otherwise
$zkBox.StopMaintenance = function()
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("StopMaintenance");

    $zkBox.maintenance.core.StopMaintenance(
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("StopMaintenance");

    return ret;
}
; 
 
 
 
/************************************************ 
**** zkBox.applications.core.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// zkBox Applications
////////  core operations  ////////

$zkBox.applications.core.GetInfo = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[APPLICATIONS -> GetInfo] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
    }, params);

    $zkBox.utils.runFunction("fnProgress", fnProgress)('Retrieving the application info');
    $zkBox.model.Call(false, false, "GET application/info", {},
        function(data)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('The application info was retrieved');
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)(data);
        },
        function(error)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while getting the application info');
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
}
; 
 
 
 
/************************************************ 
**** zkBox.applications.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//////////////////////////////////
////////  api operations  ////////
//////////////////////////////////

// returns the application info, false the operation failed
$zkBox.GetApplicationInfo = function()
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("GetApplicationInfo");

    $zkBox.applications.core.GetInfo(
        {},
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = data.exists;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("GetApplicationInfo");
    
    return ret;
}
; 
 
 
 
/************************************************ 
**** zkBox.users.core.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// zkBox Users
////////  core operations  ////////

$zkBox.users.core.CheckUser = function(user, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> CheckUser] (user="{0}")',
        user);

    $zkBox.crypto.srp.CheckUser(user,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        $zkBox.utils.runFunction("fnSuccess", fnSuccess),
        $zkBox.utils.runFunction("fnError", fnError));
};

$zkBox.users.core.AddUser = function(user, password, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> AddUser] (user="{0}", password="{1}")',
        user,
        password);

    $zkBox.crypto.srp.AddUser(user, password,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(user, pass)
        {
            $zkBox.users.core.AuthenticateUserCore(true, user, pass, fnProgress, fnSuccess, fnError);
        },
        $zkBox.utils.runFunction("fnError", fnError));
};

$zkBox.users.core.LogoutUser = function(fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> LogoutUser]');
    $zkBox.utils.runFunction("fnProgress", fnProgress)("Logging out and clearing the current session");

    $zkBox.crypto.srp.LogoutUser(
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function()
        {
            $zkBox.status.clearAuth();
            $zkBox.utils.runFunction("fnProgress", fnProgress)("Logout finished");
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
        },
        $zkBox.utils.runFunction("fnError", fnError));
};

$zkBox.users.core.UpdateUser = function(user, oldpassword, newpassword, fnStep, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> UpdateUser] (user="{0}", oldpassword="{1}", newpassword="{2}")',
        user,
        oldpassword,
        newpassword);

    if ($zkBox.crypto.srp.HashPass(oldpassword) != $zkBox.status.pass)
    {
        $zkBox.utils.runFunction("fnError", fnError)(101); // Invalid username or password
        return;
    }
    oldpassword = '';

    if ($zkBox.crypto.srp.HashUser(user) != $zkBox.status.user)
    {
        $zkBox.utils.runFunction("fnError", fnError)(101); // Invalid username or password
        return;
    }
    user = '';


    // TODO_LATER: optimize the change password function: run batch operations
    // ...when requesting objects, do it in a batch; we have a method to get the objects by id
    // ...say something like "give these 10 objects: id1, ... , id10", then a batch call for the next 10.. and so on
    // ...implement also SetObject to work in batch (to get an array of objects to update)


    $zkBox.crypto.srp.BeginUpdateUser($zkBox.status.user, newpassword,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(user, newpass, newUserVersion)
        {
            var shouldCommit = false;
            var shouldRollback = true;

            $zkBox.objects.core.GetObjectsListIDs({},
                $zkBox.utils.runFunction("fnProgress", fnProgress),
                function(objectsIDs)
                {
                    // computer the new crypto key
                    var newckey = $zkBox.crypto.cryptokey(newpass);
                    var newCryptoKey = base16_encode(str2char(newckey));
                    newckey = '';
                    var newCryptoKeyRaw = base16_decode(newCryptoKey);
                    newCryptoKey = '';

                    var hasErrors = false;
                    for (var i = 0; i < objectsIDs.length; i++)
                    {
                        if (typeof (fnStep) === "function")
                        {
                            fnStep(i + 1, objectsIDs.length);
                        }

                        var objectId = objectsIDs[i];
                        if ($zkBox.logging) $zkBox.utils.log("Reencrypting object " + objectId);

                        var object;
                        $zkBox.objects.core.GetObject(
                            {
                                objectId: objectId
                            },
                            $zkBox.utils.runFunction("fnProgress", fnProgress),
                            function(ob)
                            {
                                object = ob;
                            },
                            $zkBox.utils.runFunction("fnError", fnError));

                        if (object)
                        {
                            if ($zkBox.logging) $zkBox.utils.log("Object {0} was successfully retrieved", objectId);

                            var objectset = false;
                            $zkBox.objects.core.SetObject(
                                newCryptoKeyRaw,
                                {
                                    newUserVersion: newUserVersion,
                                    objectId: objectId,
                                    type: object.type,
                                    data: object.data
                                },
                                $zkBox.utils.runFunction("fnProgress", fnProgress),
                                function()
                                {
                                    objectset = true;
                                },
                                $zkBox.utils.runFunction("fnError", fnError));

                            if (true == objectset)
                            {
                                // the object was set
                            }
                            else
                            {
                                hasErrors = true;
                                if ($zkBox.logging) $zkBox.utils.log("Failure while setting the object {0}", objectId);
                                break;
                            }
                        }
                        else
                        {
                            hasErrors = true;
                            if ($zkBox.logging) $zkBox.utils.log("Failure while retrieving the object {0}", objectId);
                            break;
                        }
                    }

                    newCryptoKeyRaw = [];

                    if (false == hasErrors)
                    {
                        shouldCommit = true;
                        shouldRollback = false;
                    }
                },
                $zkBox.utils.runFunction("fnError", fnError));

            if (true == shouldCommit)
            {
                var commited = false;
                $zkBox.crypto.srp.CommitUpdateUser(newUserVersion,
                    $zkBox.utils.runFunction("fnProgress", fnProgress),
                    function()
                    {
                        commited = true;
                    },
                    $zkBox.utils.runFunction("fnError", fnError));

                if (true == commited)
                {
                    if ($zkBox.logging) $zkBox.utils.log("The user was successfully updated");
                    $zkBox.status.pass = newpass;
                    $zkBox.status.updateAuth();
                    $zkBox.users.core.AuthenticateUserCore(true, user, newpass, fnProgress, fnSuccess, fnError);
                }
            }

            if (true == shouldRollback)
            {
                var rolledback = false;
                $zkBox.crypto.srp.RollbackUpdateUser(newUserVersion,
                    $zkBox.utils.runFunction("fnProgress", fnProgress),
                    function()
                    {
                        rolledback = true;
                    },
                    $zkBox.utils.runFunction("fnError", fnError));

                if (true == rolledback)
                {
                    if ($zkBox.logging) $zkBox.utils.log("The user update was rolledback");
                }
            }
        },
        $zkBox.utils.runFunction("fnError", fnError));
};

$zkBox.users.core.DeleteUser = function(user, password, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> DeleteUser] (user="{0}", password="{1}")',
        user,
        password);

    if ($zkBox.crypto.srp.HashPass(password) != $zkBox.status.pass)
    {
        $zkBox.utils.runFunction("fnError", fnError)(101); // Invalid username or password
        return;
    }
    password = '';

    if ($zkBox.crypto.srp.HashUser(user) != $zkBox.status.user)
    {
        $zkBox.utils.runFunction("fnError", fnError)(101); // Invalid username or password
        return;
    }
    user = '';

    $zkBox.crypto.srp.DeleteUser($zkBox.status.user,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function()
        {
            $zkBox.status.clearAuth();
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
        },
        $zkBox.utils.runFunction("fnError", fnError));
};


$zkBox.users.core.AuthenticateUser = function(user, password, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[USERS -> AuthenticateUser] (user="{0}", password="{1}")',
        user,
        password);
    $zkBox.users.core.AuthenticateUserCore(false, user, password, fnProgress, fnSuccess, fnError);
}

$zkBox.users.core.AuthenticateUserCore = function(dataIsHashed, user, password, fnProgress, fnSuccess, fnError)
{
    $zkBox.crypto.srp.AuthenticateUser(dataIsHashed, user, password,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(user, pass, userId, userVersion, sessionId, sessionKey, cryptoKey)
        {
            $zkBox.status.user = user;
            $zkBox.status.pass = pass;
            $zkBox.status.userId = userId;
            $zkBox.status.userVersion = userVersion;
            $zkBox.status.sessionId = sessionId;
            $zkBox.status.sessionKey = sessionKey;
            $zkBox.status.cryptoKey = cryptoKey;
            $zkBox.status.updateAuth();
            $zkBox.utils.cache.clearAllItems();
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
        },
        function(error)
        {
            $zkBox.status.clearAuth();
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
};
; 
 
 
 
/************************************************ 
**** zkBox.users.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//////////////////////////////////
////////  api operations  ////////
//////////////////////////////////

// returns true if the user exists, false otherwise or if the operation failed
$zkBox.CheckUser = function(user)
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("CheckUser");

    $zkBox.users.core.CheckUser(user,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = data.exists;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("CheckUser");

    return ret;
}

// returns true if the operation succeded, false otherwise
$zkBox.AddUser = function(user, password)
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("AddUser");

    $zkBox.users.core.AddUser(user, password,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("AddUser");

    return ret;
}

// returns true if the authentication succeded, false otherwise or if the operation failed
$zkBox.AuthenticateUser = function(user, password)
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("AuthenticateUser");

    $zkBox.users.core.AuthenticateUser(user, password,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("AuthenticateUser");
    
    return ret;
}

// returns true if the operation succeded, false otherwise
$zkBox.LogoutUser = function()
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("LogoutUser");

    $zkBox.users.core.LogoutUser(
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("LogoutUser");
    
    return ret;
}

// returns true if the operation succeded, false otherwise
$zkBox.UpdateUser = function(user, oldpassword, newpassword, fnStep)
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("UpdateUser");

    $zkBox.users.core.UpdateUser(user, oldpassword, newpassword, fnStep,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("UpdateUser");
    
    return ret;
}

// returns true if the operation succeded, false otherwise
$zkBox.DeleteUser = function(user, password)
{
    var ret = false;

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("DeleteUser");

    $zkBox.users.core.DeleteUser(user, password,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("DeleteUser");
    
    return ret;
}

// returns true if an user is successfully authenticated; false otherwise
$zkBox.IsAuthenticated = function()
{
    return $zkBox.status.isAuthenticated;
}
; 
 
 
 
/************************************************ 
**** zkBox.objects.crypto.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


$zkBox.objects.crypto.EncryptCore = function(cryptoKeyRaw, ob, fnProgress, fnSuccess, fnError)
{
    // encrypts an object

    if (typeof cryptoKeyRaw === "undefined")
    {
        fnError("The encryption key is missing. We cannon encrypt without the key.");
        return;
    }

    // light check to see if we have the key (32 * 8 = 256 bits)
    if (32 != cryptoKeyRaw.length)
    {
        fnError("The encryption key is invalid.");
        return;
    }

    fnProgress('Encrypting the data');

    if (typeof ob !== "undefined")
    {
        //if (null != ob)
        {
            if ($zkBox.logging) $zkBox.utils.log("Encrypting object: " + $zkBox.utils.dump(ob));

            try
            {
                var datajson = $zkBox.utils.toJSON(ob);
                if ($zkBox.logging) $zkBox.utils.log("JSON-ized object is: " + datajson);

                var encdata = $zkBox.crypto.encodeRaw(
                    datajson,
                    cryptoKeyRaw);

                if ($zkBox.logging) $zkBox.utils.log("Encrypted object: " + encdata);

                fnProgress('The data was encrypted');

                fnSuccess(encdata);
            }
            catch (ex)
            {
                if ($zkBox.logging) $zkBox.utils.log('Failure while encrypting the object: ' + ex);
                fnProgress('Failure while encrypting the object');
                fnError('Failure while encrypting the object');
            }

        }
        //else
        //{
        //    fnError("Cannot encrypt a null object");
        //}
    }
    else
    {
        fnError("Cannot encrypt an undefined object");
    }
}

$zkBox.objects.crypto.Encrypt = function(ob, fnProgress, fnSuccess, fnError)
{
    return $zkBox.objects.crypto.EncryptCore(
        $zkBox.status.cryptoKeyRaw,
        ob,
        fnProgress,
        fnSuccess, 
        fnError);
}

$zkBox.objects.crypto.Decrypt = function(encdata, fnProgress, fnSuccess, fnError)
{
    // decrypts an object

    if (typeof $zkBox.status.cryptoKeyRaw === "undefined")
    {
        fnError("The decryption key is missing. We cannon decrypt without the key.");
        return;
    }

    // light check to see if we have the key (32 * 8 = 256 bits)
    if (32 != $zkBox.status.cryptoKeyRaw.length)
    {
        fnError("The decryption key is invalid.");
        return;
    }

    fnProgress('Decrypting the data');

    if (typeof encdata !== "undefined")
    {
        //if (null != ob)
        {
            if ($zkBox.logging) $zkBox.utils.log("Decrypting string: " + encdata);

            try
            {
                var decdata = $zkBox.crypto.decodeRaw(
                    encdata,
                    $zkBox.status.cryptoKeyRaw);
                if ($zkBox.logging) $zkBox.utils.log("Decrypted data: " + decdata);


                var ob = $zkBox.utils.evalJSON(decdata);
                if ($zkBox.logging) $zkBox.utils.log("Resulted object: " + $zkBox.utils.dump(ob));

                fnProgress('The data was decrypted');

                fnSuccess(ob);
            }
            catch (ex)
            {
                if ($zkBox.logging) $zkBox.utils.log('Failure while decrypting the data:' + ex);
                fnProgress('Failure while decrypting the data');
                fnError('Failure while decrypting the data');
            }
        }
        //else
        //{
        //    fnError("Cannot encrypt a null object");
        //}
    }
    else
    {
        fnError("Cannot decrypt an undefined string");
    }
}
; 
 
 
 
/************************************************ 
**** zkBox.objects.core.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// zkBox Objects
////////  core operations  ////////

$zkBox.objects.core.AddObject = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> AddObject] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        transactionId: null,
        type: null,
        data: null
    }, params);

    var transactionId = params.transactionId;
    var type = params.type;
    var data = params.data;

    $zkBox.objects.crypto.Encrypt(data,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(encdata)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Adding the object');

            var resource = ((null == transactionId) ? "" : "transaction/" + transactionId + "/") + ((null == type) ? "object" : ("object/type/" + type));
            $zkBox.model.Call(false, true, "POST " + resource, {
                data: encdata
            },
                function(data)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was added');
                    $zkBox.utils.runFunction("fnSuccess", fnSuccess)(data);
                },
                function(error)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while adding the object');
                    $zkBox.utils.runFunction("fnError", fnError)(error);
                });
        },
        $zkBox.utils.runFunction("fnError", fnError));
}

$zkBox.objects.core.UpdateObject = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> UpdateObject] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null,
        data: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;
    var data = params.data;

    $zkBox.objects.crypto.Encrypt(data,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(encdata)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Updating the object');

            var resource = ((null == transactionId) ? "" : "transaction/" + transactionId + "/") + "object/" + objectId;
            $zkBox.model.Call(false, true, "PUT " + resource, {
                data: encdata
            },
                function(data)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was updated');
                    $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
                },
                function(error)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while updating the object');
                    $zkBox.utils.runFunction("fnError", fnError)(error);
                });
        },
        $zkBox.utils.runFunction("fnError", fnError));
}

$zkBox.objects.core.SetObject = function(cryptoKeyRaw, params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> SetObject] ({0}, {1})',
        $zkBox.utils.dump(cryptoKeyRaw),
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        newUserVersion: null,
        objectId: null,
        type: null,
        data: null
    }, params);

    var newUserVersion = params.newUserVersion;
    var objectId = params.objectId;
    var type = params.type;
    var data = params.data;

    $zkBox.objects.crypto.EncryptCore(cryptoKeyRaw, data,
        $zkBox.utils.runFunction("fnProgress", fnProgress),
        function(encdata)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Setting the object');

            $zkBox.model.Call(false, true, "PUT " + "user/update/" + newUserVersion + "/object/" + objectId, 
                {
                    type: type,
                    data: encdata
                },
                function(data)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was set');
                    $zkBox.utils.runFunction("fnSuccess", fnSuccess)();
                },
                function(error)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while setting the object');
                    $zkBox.utils.runFunction("fnError", fnError)(error);
                });
        },
        $zkBox.utils.runFunction("fnError", fnError));
}

$zkBox.objects.core.GetObject = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> GetObject] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;

    $zkBox.utils.runFunction("fnProgress", fnProgress)('Retrieving the object');
    $zkBox.model.Call(false, true, "GET object/" + objectId, {},
        function(data)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was retrieved');

            var encdata = data.data;
            $zkBox.objects.crypto.Decrypt(encdata,
                $zkBox.utils.runFunction("fnProgress", fnProgress),
                function(ob)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was decrypted');
                    $zkBox.utils.runFunction("fnSuccess", fnSuccess)({
                        objectId: data.objectId,
                        type: data.type,
                        data: ob
                    });
                },
                $zkBox.utils.runFunction("fnError", fnError));
        },
        function(error)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while retrieving the object');
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
}

$zkBox.objects.core.DeleteObject = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> DeleteObject] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;

    $zkBox.utils.runFunction("fnProgress", fnProgress)('Deleting the object');
    var resource = ((null == transactionId) ? "" : "transaction/" + transactionId + "/") + "object/" + objectId;
    $zkBox.model.Call(false, true, "DELETE " + resource, {},
        function(data)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('The object was deleted');
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)({
                type: data.type
            });
        },
        function(error)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while deleting the object');
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
}

$zkBox.objects.core.GetObjectsList = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> GetObjectsList] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        type: null,
        objectsList: null
    }, params);

    var type = params.type;
    var objectsList = params.objectsList;

    $zkBox.utils.runFunction("fnProgress", fnProgress)('Retrieving the objects list');

    var action;
    var sendData = {};
    if (null != type)
    {
        // get objects of a given type
        action = "GET objects/type/" + type;
    }
    else
    {
        if ($zkBox.utils.isArray(objectsList))
        {
            action = "PUT objects/list";
            sendData = objectsList;
        }
        else
        {
            // get all objects
            action = "GET objects";
        }
    }

    $zkBox.model.Call(false, true, action, sendData,
        function(data)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('The objects list was retrieved');
            if ($zkBox.logging) $zkBox.utils.log("We've got the following list: {0}", $zkBox.utils.dump(data));

            var objects = data.objects;
            var cnt = objects.length;

            if (0 < cnt)
            {
                if ($zkBox.logging) $zkBox.utils.log("There are {0} item(s) to decrypt", cnt);
                $zkBox.utils.runFunction("fnProgress", fnProgress)("There are " + cnt + " item(s) to decrypt");

                var hasErrors = false;
                for (var i = 0; i < cnt; i++)
                {
                    var ob = objects[i];
                    var encdata = ob.data;
                    $zkBox.objects.crypto.Decrypt(encdata,
                        $zkBox.utils.runFunction("fnProgress", fnProgress),
                        function(data)
                        {
                            objects[i] = {
                                objectId: ob.objectId,
                                type: ob.type,
                                data: data
                            };
                            $zkBox.utils.runFunction("fnProgress", fnProgress)('The object #' + i + ' was decrypted');

                        },
                        function(err)
                        {
                            hasErrors = true;
                            if ($zkBox.logging) $zkBox.utils.log('Failure while decripting the object #' + i + ' (objectid="' + ob.objectId + '")');
                            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while decripting the object #' + i + '');
                            i = cnt;
                            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while decrypting the objects list');
                            $zkBox.utils.runFunction("fnError", fnError)(err);
                        });
                }

                if (false == hasErrors)
                {
                    $zkBox.utils.runFunction("fnProgress", fnProgress)('The objects list was decrypted');
                    $zkBox.utils.runFunction("fnSuccess", fnSuccess)(objects);
                }
            }
            else
            {
                $zkBox.utils.runFunction("fnProgress", fnProgress)("There are no items to decrypt");
                if ($zkBox.logging) $zkBox.utils.log("There are no items to decrypt");
                $zkBox.utils.runFunction("fnSuccess", fnSuccess)(objects);
            }
        },
        function(error)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while retrieving the objects list');
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
}

$zkBox.objects.core.GetObjectsListIDs = function(params, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> GetObjectsListIDs] ({0})',
        $zkBox.utils.dump(params));

    params = jQuery.extend(
    {
        type: null
    }, params);

    var type = params.type;

    $zkBox.utils.runFunction("fnProgress", fnProgress)('Retrieving the objects list IDs');

    var action;
    if (null != type)
    {
        // get objects IDs of a given type
        action = "GET objects/ids/type/" + type;
    }
    else
    {
        // get all objects IDs
        action = "GET objects/ids";
    }

    $zkBox.model.Call(false, true, action, {},
        function(data)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('The objects list IDs was retrieved');
            if ($zkBox.logging) $zkBox.utils.log("We've got the following list IDs: {0}", $zkBox.utils.dump(data));

            var objectsIDs = data.objectsIDs;

            $zkBox.utils.runFunction("fnProgress", fnProgress)('The objects list IDs was received');
            $zkBox.utils.runFunction("fnSuccess", fnSuccess)(objectsIDs);
        },
        function(error)
        {
            $zkBox.utils.runFunction("fnProgress", fnProgress)('Failure while retrieving the objects list IDs');
            $zkBox.utils.runFunction("fnError", fnError)(error);
        });
}
; 
 
 
 
/************************************************ 
**** zkBox.objects.combined.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


////////  combined operations  ////////

// set a single object; returns null if fail, true otherwise
$zkBox.objects.combined.SetSingleObject = function(
    ctx,
    params,
    fnGetObjectsList,
    fnUpdateObject, 
    fnDeleteObject)
{
    params = jQuery.extend(
    {
        transactionId: null,
        type: null,
        data: null
    }, params);

    var transactionId = params.transactionId;
    var type = params.type;
    var data = params.data;

    if (null == type)
    {
        //if ($zkBox.logging) $zkBox.utils.log('SetSingleObject was called, but the type is missing.');
        throw 'SetSingleObject was called, but the type is missing.';
        return null;
    }

    var ret = fnGetObjectsList(ctx, { transactionId: transactionId, type: type });

    if (null != ret)
    {
        if (1 == ret.length)
        {
            var objectId = ret[0].objectId;
            ret = fnUpdateObject(
            ctx,
            {
                transactionId: transactionId,
                objectId: objectId,
                data: data
            });
        }
        else
        {
            if (0 == ret.length)
            {
                if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We got zero objects of the requested type.');
                ret = null;
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We got more than one object. However, we will update the first one.');

                // delete the objects that are in excess
                try
                {
                    if ($zkBox.logging) $zkBox.utils.log('Trying to delete the excess objects');
                    var i;
                    for (i = 1; i < ret.length; i++)
                    {
                        var obId = ret[i].objectId;
                        if ($zkBox.logging) $zkBox.utils.log('Trying to delete the excess object id: "' + obId + '"');
                        if (fnDeleteObject(ctx, { transactionId: transactionId, objectId: obId }))
                        {
                            if ($zkBox.logging) $zkBox.utils.log('Excess object id: "' + obId + '" was successfully deleted');
                        }
                        else
                        {
                            if ($zkBox.logging) $zkBox.utils.log('Failure while trying to delete the excess object id: "' + obId + '"');
                        }
                    }
                }
                catch (ex)
                {
                    if ($zkBox.logging) $zkBox.utils.log("Exception occurred in SetSingleObject when deleting the excess objects: " + ex);
                }

                var objectId = ret[0].objectId;
                ret = fnUpdateObject(
                ctx,
                {
                    transactionId: transactionId,
                    objectId: objectId,
                    data: data
                });
            }
        }
    }

    return ret;
}

// get an single object or default; returns null if fail
$zkBox.objects.combined.GetSingleObject = function(
    ctx,
    params,
    fnGetObjectsList,
    fnGetObject,
    fnAddObject,
    fnDeleteObject)
{
    params = jQuery.extend(
    {
        transactionId: null,
        type: null,
        dataDefault: null
    }, params);

    var transactionId = params.transactionId;
    var type = params.type;
    var dataDefault = params.dataDefault;

    if (null == type)
    {
        //if ($zkBox.logging) $zkBox.utils.log('GetSingleObject was called, but the type is missing.');
        throw 'GetSingleObject was called, but the type is missing.';
        return null;
    }

    var ret = fnGetObjectsList(ctx, { transactionId: transactionId, type: type });

    if (null != ret)
    {
        if (1 == ret.length)
        {
            var objectId = ret[0].objectId;
            ret = fnGetObject(ctx, { transactionId: transactionId, objectId: objectId });

            if (null != ret)
            {
                // we've got the object id from the storage, so return the data
                ret = ret.data;
            }
        }
        else
        {
            if (0 == ret.length)
            {
                if ($zkBox.logging) $zkBox.utils.log('There is no object of this type. We will add it.');

                ret = fnAddObject(
                ctx,
                {
                    transactionId: transactionId,
                    type: type,
                    data: dataDefault
                });
                if (null != ret)
                {
                    if ($zkBox.logging) $zkBox.utils.log('We will now return the default value.');
                    ret = dataDefault;
                }
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('An unexpected error has occurred. We got more than one object. However, we will return the first one.');

                // delete the objects that are in excess
                try
                {
                    if ($zkBox.logging) $zkBox.utils.log('Trying to delete the excess objects');
                    var i;
                    for (i = 1; i < ret.length; i++)
                    {
                        var obId = ret[i].objectId;
                        if ($zkBox.logging) $zkBox.utils.log('Trying to delete the excess object id: "' + obId + '"');
                        if (fnDeleteObject(ctx, { transactionId: transactionId, objectId: obId }))
                        {
                            if ($zkBox.logging) $zkBox.utils.log('Excess object id: "' + obId + '" was successfully deleted');
                        }
                        else
                        {
                            if ($zkBox.logging) $zkBox.utils.log('Failure while trying to delete the excess object id: "' + obId + '"');
                        }
                    }
                }
                catch (ex)
                {
                    if ($zkBox.logging) $zkBox.utils.log("Exception occurred in GetSingleObject when deleting the excess objects: " + ex);
                }

                var objectId = ret[0].objectId;
                ret = fnGetObject(ctx, { transactionId: transactionId, objectId: objectId });

                if (null != ret)
                {
                    // we've got the object id from the storage, so return the data
                    ret = ret.data;
                }
            }
        }
    }

    return ret;
}
; 
 
 
 
/************************************************ 
**** zkBox.objects.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//////////////////////////////////
////////  api operations  ////////
//////////////////////////////////


/** caching

    we have the following cache entries:
    for objects:
        - object-[objectId]     e.g.:   object-0123456789ABCDEF

    for getobjectslist:
        - by type               e.g.:   objects-type-[some type goes here]
        - by list
        - all                   e.g.:   objects-all

**/


// returns the objectId if the operation succeeded, null otherwise
$zkBox.AddObject = function(params)
{
    var ret = null;

    params = jQuery.extend(
    {
        type: null,
        data: null
    }, params);

    var type = params.type;
    var data = params.data;

    if (!data) throw "Objects.AddObject: data was not provided";

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("AddObject");

    $zkBox.objects.core.AddObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = data.objectId;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("AddObject");

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then add the object to the cache
        $zkBox.cache.AddObject(
        {
            objectId: ret,
            type: type,
            data: data
        });
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Objects.AddObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// return true if the operation succeeded, null otherwise
$zkBox.UpdateObject = function(params)
{
    var ret = null;

    params = jQuery.extend(
    {
        objectId: null,
        data: null
    }, params);

    var objectId = params.objectId;
    var data = params.data;

    if (!objectId) throw "Objects.UpdateObject: objectId was not provided";
    if (!data) throw "Objects.UpdateObject: data was not provided";

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("UpdateObject");

    $zkBox.objects.core.UpdateObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("UpdateObject");

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then update the object into the cache
        $zkBox.cache.UpdateObject(
        {
            objectId: objectId,
            data: data
        });
    }
    ////////////

    ret = (null == ret) ? null : true;

    if ($zkBox.logging) $zkBox.utils.log("Objects.UpdateObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// returns the object if the operation succeeded, null otherwise
$zkBox.GetObject = function(params)
{
    var ret = null;

    params = jQuery.extend(
    {
        objectId: null
    }, params);

    var objectId = params.objectId;

    if (!objectId) throw "Objects.GetObject: objectId was not provided";

    //// caching
    var ob = $zkBox.cache.GetObjectCheck(objectId);
    if (null != ob)
    {
        return ob;
    }
    ////////////

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("GetObject");

    $zkBox.objects.core.GetObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(ob)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(ob);
            ret = ob;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("GetObject");

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then set the object into the cache
        $zkBox.cache.GetObjectUpdate(ret);
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Objects.GetObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// return data (object's type) if the operation succeeded, null otherwise
$zkBox.DeleteObject = function(params)
{
    var ret = null;

    params = jQuery.extend(
    {
        objectId: null
    }, params);

    var objectId = params.objectId;

    if (!objectId) throw "Objects.DeleteObject: objectId was not provided";

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("DeleteObject");

    $zkBox.objects.core.DeleteObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = data;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("DeleteObject");

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then delete the object from the cache
        $zkBox.cache.DeleteObject(
        {
            objectId: objectId,
            type: ret.type
        });
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Objects.DeleteObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// returns the objects list if the operation succeeded, null otherwise
$zkBox.GetObjectsList = function(params)
{
    params = jQuery.extend(
    {
        type: null,
        objectsList: null
    }, params);

    var ctx =
    {
        type: params.type,
        objectsList: params.objectsList,
        executeRequest: true,
        ret: null
    }

    //// caching
    $zkBox.cache.GetObjectsListCheck(ctx);
    ////////////

    if (ctx.executeRequest)
    {
        $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("GetObjectsList");

        $zkBox.objects.core.GetObjectsList(
            {
                type: ctx.type,
                objectsList: ctx.objectsList
            },
            function(msg)
            {
                $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
            },
            function(objects)
            {
                $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(objects);
                ctx.ret = objects;
            },
            function(error)
            {
                $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
            });

        $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("GetObjectsList");
    }
    else
    {
        if ($zkBox.logging) $zkBox.utils.log('There is no need to make the request');
    }

    //// caching
    if (null != ctx.ret)
    {
        // if the operation succeeded, do something according to the operation
        $zkBox.cache.GetObjectsListUpdate(ctx);
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Objects.GetObjectsList: will return {0}", $zkBox.utils.dump(ctx.ret));
    return ctx.ret;
}

// returns the objects list IDs if the operation succeeded, null otherwise
$zkBox.GetObjectsListIDs = function(params)
{
    var ret = null;

    params = jQuery.extend(
    {
        type: null
    }, params);

    //// caching
    // GetObjectsListIDs is not using caching (TODO_LATER: use caching?)
    ////////////

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("GetObjectsListIDs");

    $zkBox.objects.core.GetObjectsListIDs(
        {
            type: params.type
        },
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(objectsIDs)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(objectsIDs);
            ret = objectsIDs;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("GetObjectsListIDs");

    if ($zkBox.logging) $zkBox.utils.log("Objects.GetObjectsListIDs: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}



////////  combined operations  ////////

// set a single object
// returns null if fail, true otherwise
$zkBox.SetSingleObject = function(params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> SetSingleObject] ({0})',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.SetSingleObject(
        null,   // ctx
        params,
        function(ctx, params)
        {
            return $zkBox.GetObjectsList(params);
        },
        function(ctx, params)
        {
            return $zkBox.UpdateObject(params);
        },
        function(ctx, params)
        {
            return $zkBox.DeleteObject(params);
        });

    if ($zkBox.logging) $zkBox.utils.log("Objects.SetSingleObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}


// get an single object or default
// returns null if fail
$zkBox.GetSingleObject = function(params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> GetSingleObject] ({0})',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.GetSingleObject(
        null,   // ctx
        params,
        function(ctx, params)
        {
            return $zkBox.GetObjectsList(params);
        },
        function(ctx, params)
        {
            return $zkBox.GetObject(params);
        },
        function(ctx, params)
        {
            return $zkBox.AddObject(params);
        },
        function(ctx, params)
        {
            return $zkBox.DeleteObject(params);
        });

    if ($zkBox.logging) $zkBox.utils.log("Objects.GetSingleObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}
; 
 
 
 
/************************************************ 
**** zkBox.transactions.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//////////////////////////////////
////////  api operations  ////////
//////////////////////////////////

// returns the last operation's result if the transaction succeeded, null otherwise
$zkBox.RunTransaction = function()
{
    if ($zkBox.logging) $zkBox.utils.log('[TRANSACTIONS -> RunTransaction (arguments="{0}")]',
        $zkBox.utils.dump(arguments));

    var ret = null;
    var operations = arguments;

    if (operations.length < 1)
    {
        if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failure because there are no operations to run');
        return ret;
    }

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("RunTransaction");

    var ctx = null;
    var tranId = $zkBox.transactions.Begin(ctx);

    if (null != tranId)
    {
        var fnAddObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                type: null,
                data: null
            }, params);

            return $zkBox.transactions.AddObject(ctx, params);
        };
        var fnUpdateObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                objectId: null,
                data: null
            }, params);

            return $zkBox.transactions.UpdateObject(ctx, params);
        };
        var fnGetObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                objectId: null
            }, params);

            return $zkBox.transactions.GetObject(ctx, params);
        };
        var fnDeleteObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                objectId: null
            }, params);

            return $zkBox.transactions.DeleteObject(ctx, params);
        };
        var fnGetObjectsList = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                type: null,
                objectsList: null
            }, params);

            return $zkBox.transactions.GetObjectsList(ctx, params);
        };
        var fnGetObjectsListIDs = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                type: null
            }, params);

            return $zkBox.transactions.GetObjectsListIDs(ctx, params);
        };
        var fnSetSingleObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                type: null,
                data: null
            }, params);

            return $zkBox.transactions.SetSingleObject(ctx, params);
        };
        var fnGetSingleObject = function(params)
        {
            params = jQuery.extend(
            {
                transactionId: tranId,
                type: null,
                dataDefault: null
            }, params);

            return $zkBox.transactions.GetSingleObject(ctx, params);
        };



        // the transaction was created
        var failed = false;
        var opidx = 0;
        var oplength = operations.length;
        var results = new Array();
        var prevResult;

        while ((!failed) && (opidx < oplength))
        {
            // if we didn't fail and there are still operations...
            // execute the operation

            // construct the transaction context object
            var tran =
            {
                tranId: tranId,
                results: results,
                prevResult: prevResult,
                AddObject: fnAddObject,
                UpdateObject: fnUpdateObject,
                GetObject: fnGetObject,
                DeleteObject: fnDeleteObject,
                GetObjectsList: fnGetObjectsList,
                GetObjectsListIDs: fnGetObjectsListIDs,
                SetSingleObject: fnSetSingleObject,
                GetSingleObject: fnGetSingleObject
            }

            if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Running operation #{0}...', opidx);
            var opret = operations[opidx](tran); // pass the transaction context as parameter

            if (opret)
            {
                // the operation succeeded, go to the next operation
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Operation #{0} finished with success', opidx);
                results.push(opret); // add this operation's result to the results list
                prevResult = opret; // set the previous operation result
                opidx++; // move to the next operation
            }
            else
            {
                // the operation failed; abort
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Operation #{0} failed', opidx);
                failed = true;
            }
        }

        if (!failed)
        {
            // the operations were successfull; try to commit the transaction
            if ($zkBox.logging) $zkBox.utils.log('RunTransaction: The operations were successful; we will commit the transaction...');

            var commit = $zkBox.transactions.Commit(ctx, tranId);
            if (true == commit)
            {
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: The transaction was committed');
                ret = prevResult; // everything went OK, return the last operation's result
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failure while committing the transaction');
            }
        }
        else
        {
            // there was an issue while executing an operation; try to rollback the transaction
            if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failure at operation #{0}; we will rollback the transaction...', opidx);

            var rollback = $zkBox.transactions.Rollback(ctx, tranId);

            if (true == rollback)
            {
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: The transaction was rolled back');
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failure while rolling back the transaction');
            }
        }
    }
    else
    {
        if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failure while creating the transaction');
    }

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("RunTransaction");

    if (ret)
    {
        if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Success. Will return "{0}"', $zkBox.utils.dump(ret));
    }
    else
    {
        if ($zkBox.logging) $zkBox.utils.log('RunTransaction: Failed');
    }

    return ret;
}
; 
 
 
 
/************************************************ 
**** zkBox.transactions.core.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// zkBox Transactions
////////  core operations  ////////

$zkBox.transactions.core.Begin = function(fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[TRANSACTIONS -> Begin]');

    fnProgress('Starting a new transaction');

    $zkBox.model.Call(false, true, "POST transaction", {},
        function(data)
        {
            fnProgress('The transaction was created');

            fnSuccess(data);
        },
        function(error)
        {
            fnProgress('Failure while creating the transaction');
            fnError(error);
        });
}


$zkBox.transactions.core.Commit = function(transactionId, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[TRANSACTIONS -> Commit (transactionId="{0}")]',
        transactionId);

    fnProgress('Committing the transaction');

    $zkBox.model.Call(false, true, "PUT transaction/" + transactionId, {},
        function(data)
        {
            fnProgress('The transaction was committed');

            fnSuccess(data);
        },
        function(error)
        {
            fnProgress('Failure while committing the transaction');
            fnError(error);
        });
}


$zkBox.transactions.core.Rollback = function(transactionId, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[TRANSACTIONS -> Rollback (transactionId="{0}")]',
        transactionId);

    fnProgress('Rolling back the transaction');

    $zkBox.model.Call(false, true, "DELETE transaction/" + transactionId, {},
        function(data)
        {
            fnProgress('The transaction was rolled back');

            fnSuccess(data);
        },
        function(error)
        {
            fnProgress('Failure while rolling back the transaction');
            fnError(error);
        });
}; 
 
 
 
/************************************************ 
**** zkBox.transactions.management.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns the transactionId if the operation succeeded, null otherwise
$zkBox.transactions.Begin = function(ctx)
{
    var ret = null;

    $zkBox.transactions.core.Begin(
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = data.transactionId;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });


    return ret;
}

// returns true the operation succeeded, null otherwise
$zkBox.transactions.Commit = function(ctx, transactionId)
{
    var ret = null;

    $zkBox.transactions.core.Commit(transactionId,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    if (true == ret)
    {
        // reflect the changes into the cache; ignore the result from the cache update
        $zkBox.cache.transactions.core.CommitTransaction(transactionId);
    }

    return ret;
}


// returns true the operation succeeded, null otherwise
$zkBox.transactions.Rollback = function(ctx, transactionId)
{
    var ret = null;

    $zkBox.transactions.core.Rollback(transactionId,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    if (true == ret)
    {
        // reflect the changes into the cache; ignore the result
        $zkBox.cache.transactions.core.RollbackTransaction(transactionId);
    }

    return ret;
}
; 
 
 
 
/************************************************ 
**** zkBox.transactions.objects.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns the objectId if the operation succeeded, null otherwise
$zkBox.transactions.AddObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
    {
        transactionId: null,
        type: null,
        data: null
    }, params);

    var transactionId = params.transactionId;
    var type = params.type;
    var data = params.data;

    if (!data) throw "Transactions.Objects.AddObject: data was not provided";

    $zkBox.objects.core.AddObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = data.objectId;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then add/queue the object to the cache
        $zkBox.cache.transactions.AddObject(
            transactionId,
            {
                objectId: ret,
                type: type,
                data: data
            });
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.AddObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// return true if the operation succeeded, null otherwise
$zkBox.transactions.UpdateObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null,
        data: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;
    var data = params.data;

    if (!objectId) throw "Transactions.Objects.UpdateObject: objectId was not provided";
    if (!data) throw "Transactions.Objects.UpdateObject: data was not provided";

    $zkBox.objects.core.UpdateObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function()
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then update/queue the object into the cache
        $zkBox.cache.transactions.UpdateObject(
            transactionId,
            {
                objectId: objectId,
                data: data
            });
        
    }
    ////////////

    ret = (null == ret) ? null : true;

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.UpdateObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// returns the object if the operation succeeded, null otherwise
$zkBox.transactions.GetObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;

    if (!objectId) throw "Transactions.Objects.GetObject: objectId was not provided";

    //// caching
    var ob = $zkBox.cache.GetObjectCheck(objectId); // GetObjectCheck is not transactional
    if (null != ob)
    {
        return ob;
    }
    ////////////

    $zkBox.objects.core.GetObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(ob)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(ob);
            ret = ob;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then set the object into the cache
        $zkBox.cache.transactions.GetObjectUpdate(
            transactionId,
            ret);
    }
    ////////////
    
    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.GetObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// return data if the operation succeeded, null otherwise
$zkBox.transactions.DeleteObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
    {
        transactionId: null,
        objectId: null
    }, params);

    var transactionId = params.transactionId;
    var objectId = params.objectId;

    if (!objectId) throw "Transactions.Objects.DeleteObject: objectId was not provided";

    $zkBox.objects.core.DeleteObject(params,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)();
            ret = data;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then delete the object from the cache
        $zkBox.cache.transactions.DeleteObject(
            transactionId,
            {
                objectId: objectId,
                type: ret.type
            });
        
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.DeleteObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// returns the objects list if the operation succeeded, null otherwise
$zkBox.transactions.GetObjectsList = function(ctx, params)
{
    params = jQuery.extend(
    {
        transactionId: null,
        type: null,
        objectsList: null
    }, params);

    var transactionId = params.transactionId;

    var ctx =
    {
        type: params.type,
        objectsList: params.objectsList,
        executeRequest: true,
        ret: null
    }

    //// caching
    $zkBox.cache.GetObjectsListCheck(ctx); // GetObjectsListCheck is not transactional
    ////////////

    if (ctx.executeRequest)
    {
        $zkBox.objects.core.GetObjectsList(
            {
                type: ctx.type,
                objectsList: ctx.objectsList
            },
            function(msg)
            {
                $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
            },
            function(objects)
            {
                $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(objects);
                ctx.ret = objects;
            },
            function(error)
            {
                $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
            });
    }
    else
    {
        if ($zkBox.logging) $zkBox.utils.log('There is no need to make the request');
    }

    //// caching
    if (null != ctx.ret)
    {
        // if the operation succeeded, do something according to the operation
        $zkBox.cache.transactions.GetObjectsListUpdate(
            transactionId,
            ctx);
    }
    ////////////

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.GetObjectsList: will return {0}", $zkBox.utils.dump(ctx.ret));
    return ctx.ret;
}

// returns the objects list IDs if the operation succeeded, null otherwise
$zkBox.transactions.GetObjectsListIDs = function(ctx, params)
{
    var ret = null;
    
    params = jQuery.extend(
    {
        transactionId: null,
        type: null
    }, params);

    var transactionId = params.transactionId;

    //// caching
    // GetObjectsListIDs is not using caching (TODO_LATER: use caching?)
    ////////////

    $zkBox.objects.core.GetObjectsListIDs(
        {
            type: params.type
        },
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(objectsIDs)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(objectsIDs);
            ret = objectsIDs;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.GetObjectsListIDs: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}


////////  combined operations  ////////

// set a single object; returns null if fail, true otherwise
$zkBox.transactions.SetSingleObject = function(ctx, params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> TransactionSetSingleObject] ({0}")',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.SetSingleObject(
        ctx,
        params,
        $zkBox.transactions.GetObjectsList,
        $zkBox.transactions.UpdateObject,
        $zkBox.transactions.DeleteObject);

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.SetSingleObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}

// get a single object or default; returns null if fail
$zkBox.transactions.GetSingleObject = function(ctx, params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> TransactionGetSingleObject] ({0})',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.GetSingleObject(
        ctx,
        params,
        $zkBox.transactions.GetObjectsList,
        $zkBox.transactions.GetObject,
        $zkBox.transactions.AddObject,
        $zkBox.transactions.DeleteObject);

    if ($zkBox.logging) $zkBox.utils.log("Transactions.Objects.GetSingleObject: will return {0}", $zkBox.utils.dump(ret));
    return ret;
}
; 
 
 
 
/************************************************ 
**** zkBox.batch.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


//////////////////////////////////
////////  api operations  ////////
//////////////////////////////////

// returns the list of the operations results; returns true if the batch succeeded, null otherwise
$zkBox.RunBatch = function()
{
    if ($zkBox.logging) $zkBox.utils.log('[BATCH -> RunBatch (arguments="{0}")]',
        $zkBox.utils.dump(arguments));

    var ret = null;

    var sections = arguments;

    if (sections.length < 2)
    {
        if ($zkBox.logging) $zkBox.utils.log('RunBatch: Failure because there are no sections to run');
        return ret;
    }

    $zkBox.utils.runFunctionOnly("fnStart", $zkBox.callbacks.fnStart)("RunBatch");

    var options = sections[0];
    options = jQuery.extend(
    {
        transactionId: null,
        atStartBeginTransaction: true,
        atEndCommitTransaction: true,
        onErrorRollbackTransaction: true
    }, options);

    var operations = [];
    var operationsCache = [];
    var ctx =
    {
        operations: operations,
        operationsCache: operationsCache,
        results: null,
        transactionId: null
    };

    var fnAddObject = function(params)
    {
        return $zkBox.batch.AddObject(ctx, params);
    };
    var fnUpdateObject = function(params)
    {
        return $zkBox.batch.UpdateObject(ctx, params);
    };
    var fnGetObject = function(params)
    {
        return $zkBox.batch.GetObject(ctx, params);
    };
    var fnDeleteObject = function(params)
    {
        return $zkBox.batch.DeleteObject(ctx, params);
    };

    var fnGetObjectsList = function(params)
    {
        return $zkBox.batch.GetObjectsList(ctx, params);
    };

    var fnGetObjectsListIDs = function(params)
    {
        return $zkBox.batch.GetObjectsListIDs(ctx, params);
    };

    var fnSetSingleObject = function(params)
    {
        params = jQuery.extend(
        {
            //transactionId: null,
            type: null,
            data: null
        }, params);

        return $zkBox.batch.SetSingleObject(ctx, params);
    };

    var fnGetSingleObject = function(params)
    {
        params = jQuery.extend(
        {
            //transactionId: null,
            type: null,
            dataDefault: null
        }, params);

        return $zkBox.batch.GetSingleObject(ctx, params);
    };



    // the transaction was created
    var failed = false;
    var sectidx = 1; // the first argument are the options, so start from the 2nd (idx=1) argument
    var sectlength = sections.length;
    var results = new Array();
    var prevResult;

    while ((!failed) && (sectidx < sectlength))
    {
        // if we didn't fail and there are still sections...
        // execute the section

        // construct the transaction context object
        var batch =
        {
            transactionId: options.transactionId,
            atStartBeginTransaction: options.atStartBeginTransaction,
            atEndCommitTransaction: options.atEndCommitTransaction,
            onErrorRollbackTransaction: options.onErrorRollbackTransaction,

            results: results,
            prevResult: prevResult,
            AddObject: fnAddObject,
            UpdateObject: fnUpdateObject,
            GetObject: fnGetObject,
            DeleteObject: fnDeleteObject,
            GetObjectsList: fnGetObjectsList,
            GetObjectsListIDs: fnGetObjectsListIDs,
            SetSingleObject: fnSetSingleObject,
            GetSingleObject: fnGetSingleObject
        }

        if ($zkBox.logging) $zkBox.utils.log('RunBatch: Running section #{0}...', sectidx);
        var sectret = sections[sectidx](batch); // pass the batch context as parameter

        if (sectret)
        {
            // the section succeeded, go to the next section
            if ($zkBox.logging) $zkBox.utils.log('RunBatch: Section #{0} finished with success', sectidx);
            results.push(sectret); // add this section's result to the results list
            prevResult = sectret; // set the previous section result
            sectidx++; // move to the next section
        }
        else
        {
            // the section failed; abort
            if ($zkBox.logging) $zkBox.utils.log('RunBatch: Section #{0} failed', sectidx);
            failed = true;
        }
    }

    if (!failed)
    {
        // the sections were executed successfull; now try to execute the batch

        var tempret = null;
        if ((!operations) || (operations.length < 1))
        {
            if ($zkBox.logging) $zkBox.utils.log('RunBatch: Failure because there are no operations to run');
        }
        else
        {
            var execbatch =
            {
                transactionId: options.transactionId,
                atStartBeginTransaction: options.atStartBeginTransaction,
                atEndCommitTransaction: options.atEndCommitTransaction,
                onErrorRollbackTransaction: options.onErrorRollbackTransaction,
                operations: operations
            };

            tempret = $zkBox.batch.ExecuteBatch(execbatch);
        }

        //if ($zkBox.logging) $zkBox.utils.log('RunBatch: tempret: "{0}"', $zkBox.utils.dump(tempret));
        //if ($zkBox.logging) $zkBox.utils.log('RunBatch: tempret.results: "{0}"', $zkBox.utils.dump(tempret.results));
        //if ($zkBox.logging) $zkBox.utils.log('RunBatch: tempret.results.length: "{0}"', $zkBox.utils.dump(tempret.results.length));
        //if ($zkBox.logging) $zkBox.utils.log('RunBatch: operations.length: "{0}"', $zkBox.utils.dump(operations.length));

        if (tempret)
        {
            if ((tempret.results) && (tempret.results.length == operations.length))
            {
                ctx.results = tempret.results;
                ret = new Array();
                for (var i = 0; i < tempret.results.length; i++)
                {
                    var obret = $zkBox.batch.DecryptObject(ctx, i);
                    ret.push(obret);
                }
                ctx.results = ret;

                if (tempret.transactionId)
                {
                    ctx.transactionId = tempret.transactionId;
                    ret.transactionId = tempret.transactionId;
                }
            }
            else
            {
                if ($zkBox.logging) $zkBox.utils.log('RunBatch: Call was executed ok, but the response was unexpected (batch operations and batch results differ in length). The operations were "{0}" and the return was "{1}"', $zkBox.utils.dump(operations), $zkBox.utils.dump(tempret));
            }
        }
    }

    //// caching
    if (null != ret)
    {
        // if the operation succeeded, then update the objects into the cache if needed
        $zkBox.cache.batch.ExecuteBatch(
            options,
            ctx);

        if ($zkBox.logging) $zkBox.utils.log('RunBatch: Success. Will return "{0}"', $zkBox.utils.dump(ret));
    }
    ////////////

    $zkBox.utils.runFunctionOnly("fnStop", $zkBox.callbacks.fnStop)("RunBatch");

    if (ret)
    {
        if ($zkBox.logging) $zkBox.utils.log('RunBatch: Success. Will return "{0}"', $zkBox.utils.dump(ret));
    }
    else
    {
        if ($zkBox.logging) $zkBox.utils.log('RunBatch: Failed');
    }

    return ret;
}; 
 
 
 
/************************************************ 
**** zkBox.batch.core.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// zkBox Batch
////////  core operations  ////////

$zkBox.batch.core.ExecuteBatch = function(batch, fnProgress, fnSuccess, fnError)
{
    if ($zkBox.logging) $zkBox.utils.log('[BATCH -> ExecuteBatch ({0})]',
        $zkBox.utils.dump(batch));

    fnProgress('Executing the batch');

    $zkBox.model.Call(false, true, "PUT batch", batch,
        function(data)
        {
            fnProgress('The batch was executed');

            fnSuccess(data);
        },
        function(error)
        {
            fnProgress('Failure while executing the batch');
            fnError(error);
        });
}
; 
 
 
 
/************************************************ 
**** zkBox.batch.management.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns the result if the operation succeeded, null otherwise
$zkBox.batch.ExecuteBatch = function(batch)
{
    var ret = null;

    $zkBox.batch.core.ExecuteBatch(batch,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(data)
        {
            $zkBox.utils.runFunctionOnly("fnSuccess", $zkBox.callbacks.fnSuccess)(data);
            ret = data;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });


    return ret;
}
; 
 
 
 
/************************************************ 
**** zkBox.batch.objects.js ***** 
************************************************/ 
// zkBox JavaScript client
// http://www.zkbox.com/developers/client/javascript
// 
// Copyright 2010, Sorin Ostafiev
// http://www.ostafiev.com/
//
// Released under GPL version 3 license
// http://www.gnu.org/licenses/gpl-3.0.txt


// returns true
$zkBox.batch.AddObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "AddObject",
            //transactionId: null,
            type: null,
            data: null
        }, params);

    var type = params.type;
    var data = params.data;

    if (!data) throw "Batch.Objects.AddObject: data was not provided";

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);
    
    $zkBox.objects.crypto.Encrypt(data,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(encdata)
        {
            var ob = $zkBox.utils.clone(params);
            ob.data = encdata;
            operations.push(ob);

            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    return ret;
}

// returns true
$zkBox.batch.UpdateObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "UpdateObject",
            //transactionId: null,
            objectId: null,
            data: null
        }, params);

    var objectId = params.objectId;
    var data = params.data;

    if (!objectId) throw "Batch.Objects.UpdateObject: objectId was not provided";
    if (!data) throw "Batch.Objects.UpdateObject: data was not provided";

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);

    $zkBox.objects.crypto.Encrypt(data,
        function(msg)
        {
            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
        },
        function(encdata)
        {
            var ob = $zkBox.utils.clone(params);
            ob.data = encdata;
            operations.push(ob);
            
            ret = true;
        },
        function(error)
        {
            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
        });

    return ret;
}

// returns true
$zkBox.batch.GetObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "GetObject",
            //transactionId: null,
            objectId: null
        }, params);

    var objectId = params.objectId;

    if (!objectId) throw "Batch.Objects.GetObject: objectId was not provided";

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);

    // we'll do the call nomatter if the object is already in the local cache
    operations.push(params);
    ret = true;

    return ret;
}

// returns true
$zkBox.batch.DeleteObject = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "DeleteObject",
            //transactionId: null,
            objectId: null
        }, params);

    var objectId = params.objectId;

    if (!objectId) throw "Batch.Objects.DeleteObject: objectId was not provided";

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);

    operations.push(params);
    ret = true;
    
    return ret;
}

// returns true
$zkBox.batch.GetObjectsList = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "GetObjectsList",
            //transactionId: null,
            type: null,
            objectsList: null
        }, params);

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);

    operations.push(params);
    ret = true;
    
    return ret;
}

// returns true
$zkBox.batch.GetObjectsListIDs = function(ctx, params)
{
    var ret = null;

    params = jQuery.extend(
        {
            operation: "GetObjectsListIDs",
            //transactionId: null
            type: null
        }, params);

    var operations = ctx.operations;
    var operationsCache = ctx.operationsCache;

    operationsCache.push(params);

    operations.push(params);
    ret = true;

    return ret;
}



////////  combined operations  ////////

// set a single object; returns null if fail, true otherwise
$zkBox.batch.SetSingleObject = function(ctx, params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> BatchSetSingleObject] ({0}")',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.SetSingleObject(
        ctx,
        params,
        $zkBox.transactions.GetObjectsList,     // execute reads directly
        $zkBox.batch.UpdateObject,              // cumulate the changes into the batch
        $zkBox.batch.DeleteObject);             // cumulate the changes into the batch
    
    return ret;
}

// get a single object or default; returns null if fail
$zkBox.batch.GetSingleObject = function(ctx, params)
{
    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> BatchGetSingleObject] ({0})',
        $zkBox.utils.dump(params));

    var ret = $zkBox.objects.combined.GetSingleObject(
        ctx,
        params,
        $zkBox.transactions.GetObjectsList,     // execute reads directly
        $zkBox.transactions.GetObject,          // execute reads directly
        $zkBox.transactions.AddObject,          // execute reads directly
        $zkBox.transactions.DeleteObject);      // execute reads directly
    
    return ret;
}





////////  decrypting the results  ////////

// returns the decrypted object from the position idx from the results
$zkBox.batch.DecryptObject = function(ctx, idx)
{
    var ret = null;

    var operation = $zkBox.utils.clone(ctx.operations[idx]);
    var result = $zkBox.utils.clone(ctx.results[idx]);

    if ($zkBox.logging) $zkBox.utils.log('[OBJECTS -> DecryptObject] (idx={0} operation={1} result={2}")',
        idx,
        $zkBox.utils.dump(operation),
        $zkBox.utils.dump(result));

    switch (operation.operation)
    {
        case "AddObject":
            // there is nothing to decrypt
            ret = result.data;
            break;

        case "UpdateObject":
            // there is nothing to decrypt
            ret = result.data;
            break;

        case "GetObject":
            // decrypt the object's data
            {
                var encdata = result.data.data;
                $zkBox.objects.crypto.Decrypt(encdata,
                    function(msg)
                    {
                        $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
                    },
                    function(ob)
                    {
                        ret = result.data;
                        ret.data = ob;
                    },
                    function(error)
                    {
                        $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
                    });
            }
            break;

        case "DeleteObject":
            // there is nothing to decrypt
            ret = result.data;
            break;

        case "GetObjectsList":
            // decrypt the objects' data
            {
                ret = result.data.objects;
                var i;
                for (i = 0; i < ret.length; i++)
                {
                    var encdata = ret[i].data;
                    $zkBox.objects.crypto.Decrypt(encdata,
                        function(msg)
                        {
                            $zkBox.utils.runFunctionOnly("fnProgress", $zkBox.callbacks.fnProgress)(msg);
                        },
                        function(ob)
                        {
                            ret[i].data = ob;
                        },
                        function(error)
                        {
                            $zkBox.utils.runFunctionOnly("fnError", $zkBox.callbacks.fnError)(error);
                        });
                }
            }
            break;

        case "GetObjectsListIDs":
            // there is nothing to decrypt
            ret = result.data;
            break;

        default:
            throw "Batch.Objects: Unhandled operation type '" + operation.operation + "'";
    }

    if ($zkBox.logging) $zkBox.utils.log('DecryptObject: will return: {0}',
        $zkBox.utils.dump(ret));

    return ret;
}
; 
 
 
; 
 
 
